package com.asda.android.adbanner


import android.content.Context
import android.os.Handler
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.admonetization.AdConfig.asdaService
import com.asda.android.admonetization.AdConfig.authentication
import com.asda.android.admonetization.AdConfig.isCriteoEnabled
import com.asda.android.admonetization.AdConfig.sitespectAbTestingEngine
import com.asda.android.admonetization.criteo.*
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.constants.AdConstants.DELAYED_CALL_TIME
import com.asda.android.admonetization.formatter.BannerIdsFromZonesFormatter
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.base.interfaces.IAdManager
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.cms.model.Zone
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import java.math.BigDecimal
import java.util.*
import kotlin.collections.HashMap
import kotlin.collections.List
import kotlin.collections.set

object AdBannerManager : IAdManager {
    const val SHARED_PREF_SITESPECT = "sitespect_abtesting_module"
    const val SITESPECT_USER_ID = "DYN_USER_ID"

    override fun onBannerItemClick(context: Context, tag: Any?, shelfListItem: ShelfListItem) {
        // This is for non-banner, very first item (Native tracking) for Criteo
        val preference = context.getSharedPreferences(SHARED_PREF_SITESPECT, Context.MODE_PRIVATE)
        if (isCriteoEnabled && shelfListItem.banner == null) {  // native tracking for non-banner items
            AdTrackingServiceQuery().bannerNativeEvent(shelfListItem, tag.toString(), "abp")
        }
        asdaService.sendClickEvent(shelfListItem.clickAddUrl)
        if (shelfListItem.personalizedRecommendation &&
              authentication.isLoggedIn &&
              !preference.getString(SITESPECT_USER_ID, "").isNullOrEmpty()) {
            sitespectAbTestingEngine.trackMetrics(SitespectP13NMetrics.VARIATION_GROUP, shelfListItem.metricId, 1,
                  RestUtils.poundStringToBigDecimal(shelfListItem.priceToDisplay), SharedPreferencesUtil.isP13nRecommendationEnforced(context)).subscribe()
        }
    }


    override fun bannerAbpEvent(campaign : String?, item : ShelfListItem) {
        var trackingInformation: HashMap<String, String> = HashMap()
        trackingInformation[AdConstants.RNPARAM] = AdConstants.TRACKING_ASDA_APP
        trackingInformation[AdConstants.RSPARAM] = AdMonetization.store
        trackingInformation[AdConstants.RPPARAM] = AdUtils().getRetailPageParam(campaign).toString()
        if (AdMonetization.vuid.isNullOrEmpty())
            trackingInformation[AdConstants.VFPARAM] = AdMonetization.account["RETAILIER_VISITOR_ID"].toString()   // device id
        else
            trackingInformation[AdConstants.VFPARAM] = AdMonetization.vuid

        trackingInformation[AdConstants.TAPARAM] = AdConstants.TRACKING_ABP
        trackingInformation[AdConstants.PIPARAM] = item.id.toString()
        trackingInformation[AdConstants.PLPARAM] = AdUtils().simplifyName(item.title).toString()
            /* Send the tracking details to Criteo. */
        AdTrackingServiceQuery().makeTrackingApiCall(trackingInformation)
    }

    override fun bannerAbkEvent(existingItems: BigDecimal?, newItems: BigDecimal?, banner: Merchandising.Banner?, context: Context?) {
        if(banner == null)
            return
        if(banner.trackingInformation.size != 0 && existingItems != null && newItems != null && context != null) {
            banner.trackingInformation[AdConstants.TAPARAM] = AdConstants.TRACKING_ABK
            banner.trackingInformation[AdConstants.VFPARAM] = AdMonetization.vuid
            // Deal with respect to the current item being added
            when (newItems?.compareTo(existingItems)) {
                1 -> {
                    if (existingItems == BigDecimal.ZERO) {
                        banner.trackingInformation[AdConstants.TEPARAM] = AdConstants.TRACKING_ABK_BTN
                    } else {
                        banner.trackingInformation[AdConstants.TEPARAM] = AdConstants.TRACKING_QTY_CHANGE
                        banner.trackingInformation[AdConstants.TNPARAM] = (newItems - existingItems).toString()
                    }
                    // Sending anivia tracking for adding item from banner.
                    trackCartUpdateFromBanner((newItems - existingItems).toString(), banner, context)
                }
                -1 -> {
                    banner.trackingInformation[AdConstants.TEPARAM] = AdConstants.TRACKING_QTY_CHANGE
                    banner.trackingInformation[AdConstants.TNPARAM] = (existingItems - newItems).toString()  // -ve value
                }
            }
            // Deal with entire cart
            /* Send the tracking details to Criteo. */
            AdCartTrackingServiceQuery().makeCartTrackingApiCall(banner.trackingInformation, Utils.buildBasketXml(context))
            Handler().postDelayed({ AdTrackingServiceQuery().sendBannerClickEvent(banner) }, DELAYED_CALL_TIME)
        }
    }

    private fun trackCartUpdateFromBanner(quantity: String, banner: Merchandising.Banner, context: Context) {
        if (banner == null) {
            return
        }
        if (DuplicateControl.isItemDuplicate(banner.trackingCode, AdConstants.WAIT_TIME_IN_SEC_BEFORE_PURGE, false)) {
            return
        }
        val itemList = ArrayList<Cart.CartItem>()
        itemList.add(ItemQuantityController.createCartItem(context, banner.bannerItem, quantity))
        val event = AsdaAnalyticsEvent(Anivia.TROLLEY_ADD_EVENT)
        event.putString(Anivia.BANNER_INFO_KEY, AdTrackingServiceQuery().buildTrackingStringForAnivia(banner))
        event.putString(Anivia.BANNER_CART_ADD_KEY, banner.trackingCode)
        Tracker.get().trackCartUpdate(event, Tracker.toCartItemInfoList(itemList), null, banner.contentid, null, Anivia.TROLLEY_ADD_EVENT, CartManager.get(context)?.getBasketId())
    }

    override fun onBannerItemClick(context: Context, tag: Any?, item: IroProductDetailsPlp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean) {
        val shelfListItem = ShelfListItem.buildShelfObject(item, specialOffersTagIsClickable, isBigIcon)
        onBannerItemClick(context, tag, shelfListItem)
    }

    override fun getBannerIdsFromZones(zone: List<Zone>?): String {
        return getBannerIdFormatter().format(zone)
    }

    private fun getBannerIdFormatter() : BannerIdsFromZonesFormatter {
        return BannerIdsFromZonesFormatter()
    }
}
