package com.asda.android.app.account.address.view

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import androidx.annotation.VisibleForTesting
import com.asda.android.R
import com.asda.android.app.account.singleprofile.view.SPCheckPostCodeDialogFragment
import com.asda.android.base.core.view.actitivies.BaseAppCompatActivity
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.PostcodeCheckResponse
import com.asda.android.singleprofile.features.account.PostcodeCheckListener

class AddAddressActivity : BaseAppCompatActivity(), BaseAddEditAddressFragment.AddressBookChangedListener, PostcodeCheckListener {
    private val SCREEN_NAME = "Book Slot : Add New Address"

    override fun onAddressBookChanged(addressBookResponse: AddressBookResponse?, addedAddress: AddressBookResponse.Address?) {
        val intent = Intent(this, AddAddressActivity::class.java)
        addressBookResponse?.let { addressBookResponse ->
            if (addressBookResponse.addressBook?.size ?: 0 > 0) {
                intent.putExtra(SELECTED_ADDRESS, addressBookResponse.addressBook?.get(0))
            }
        }
        if (!intent.hasExtra(SELECTED_ADDRESS))
            intent.putExtra(SELECTED_ADDRESS, addedAddress)
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    override fun onPostcodeCheckSuccess(addressSuggestions: Array<PostcodeCheckResponse.AddressSuggestion>?, postcode: String) {
        if (!showAddAddressScreen(
                    postcode = postcode,
                    addressSuggestions = addressSuggestions,
                    onlyAddAddress = onlyAddAddress)) {
            // If failed to show address screen, then just close the activity with RESULT_CANCELED
            setResult(Activity.RESULT_CANCELED, intent)
            finish()
        }
    }

    private var title: String? = null
    private var onlyAddAddress: Boolean = false

    override fun getFragmentContainer() = R.id.fragment_container

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.fragment_container)
        getArguments()

        title?.let {
            supportActionBar?.title = it
        } ?: run {
            supportActionBar?.title = getString(R.string.add_new_address)
        }
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        showSPCheckPostDialog()
    }

    private fun showSPCheckPostDialog() {
        val checkPostcodeDialogFragment = SPCheckPostCodeDialogFragment.newInstance()
        checkPostcodeDialogFragment.setPostcodeCheckedListener(this, SCREEN_NAME)
        val fragmentTransaction = BaseFragment.getASDAFragmentTransaction(supportFragmentManager)
        fragmentTransaction.addToBackStack(null)
        val previousDialog = supportFragmentManager.findFragmentByTag("DIALOG_CHECK_POSTCODE")
        if (previousDialog != null) {
            fragmentTransaction.remove(previousDialog)
        }
        checkPostcodeDialogFragment.show(fragmentTransaction, "DIALOG_CHECK_POSTCODE")
    }

    @VisibleForTesting
    fun getAddAddressScreenVisibility( postcode: String?) : Boolean{
        postcode?.let {
            return true
        }
        return false
    }

    @VisibleForTesting
    fun showAddAddressScreen(
          postcode: String?,
          addressSuggestions: Array<PostcodeCheckResponse.AddressSuggestion>?,
          onlyAddAddress: Boolean): Boolean {
        postcode?.let {
            val address = AddressBookResponse.Address()
            address.isDefault = true
            val addressFragment = BaseAddEditAddressFragment.getAddAddressFragment(
                  it,
                  true,
                  address,
                  addressSuggestions,
                  null,
                  null,
                  null,
                  true,
                  false,
                  AddressBookResponse.Type.ShipTo.toString(),
                  onlyAddAddress)
            addressFragment.setAddressBookChangedListener(this)
            push(addressFragment)
        }
        return getAddAddressScreenVisibility(postcode)
    }

    private fun getArguments() {
        intent?.let {
            title = it.getStringExtra(EXTRA_TITLE)
            onlyAddAddress = it.getBooleanExtra(EXTRA_ONLY_ADD_ADDRESS, false)
        }
    }

    companion object {
        const val SELECTED_ADDRESS = "selected_address"
        private const val EXTRA_TITLE = "title"
        private const val EXTRA_ONLY_ADD_ADDRESS = "only_add_address"

        fun launch(context: Context, title: String?, nickName: String?, onlyAddAddress: Boolean) {
            val intent: Intent = getIntent(context = context, title = title, onlyAddAddress = onlyAddAddress)
            context.startActivity(intent)
        }

        fun launchForResult(fragment: androidx.fragment.app.Fragment, title: String?, onlyAddAddress: Boolean, requestCode: Int) {
            fragment.context?.let {
                val intent: Intent = getIntent(context = it, title = title, onlyAddAddress = onlyAddAddress)
                fragment.startActivityForResult(intent, requestCode)
            }
        }

        private fun getIntent(context: Context, title: String?, onlyAddAddress: Boolean): Intent {
            val intent = Intent(context, AddAddressActivity::class.java)
            intent.putExtra(EXTRA_TITLE, title)
            intent.putExtra(EXTRA_ONLY_ADD_ADDRESS, onlyAddAddress)
            return intent
        }
    }
}
