package com.asda.android.app.account.address.view

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.account.Countries
import com.asda.android.app.account.Countries.Companion.getCountryName
import com.asda.android.app.account.address.AddAddressListener
import com.asda.android.app.account.address.AddressEmptyValidator
import com.asda.android.app.home.RegisterHelper
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.interfaces.ICartManager
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.AddressBookResponse.DEFAULT_COUNTRY
import com.asda.android.restapi.service.data.PostcodeCheckResponse
import com.asda.android.restapi.service.data.RegistrationInfo
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.restapi.service.RestUtils.concatenateWithComma
import com.asda.android.singleprofile.BuildConfig
import com.asda.android.singleprofile.api.utils.isValidPostcode
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.base.get
import com.asda.android.singleprofile.model.*
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.AddressViewModelFactory
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers

val TAG = "SPAddEditAddressFgmt"

private const val TYPE_HOME = "home"
private const val TYPE_FLAT = "flat"
private const val SCREEN_NAME = "Address Book"

class SPAddEditAddressFragment : BaseAddEditAddressFragment() {
    private var addressViewModel: AddressViewModel? = null

    companion object{
        const val TAG = "SPAddEditAddressFgmt"
    }


    override fun getLayout(): Int = R.layout.account_edit_address

    override fun setupView() {
        super.setupView()
        hideExtraViews()
    }

    private fun hideExtraViews() {
        mCounty?.setText(AddressBookResponse.DEFAULT_COUNTRY)
        mCounty?.visibility = View.GONE
        mCountyText?.visibility = View.GONE
        mAddressNicknameText?.visibility = View.GONE
        mAddressNickname?.visibility = View.GONE
        mPhonesContainer?.visibility = View.GONE
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        addressViewModel = ViewModelProviders.of(this, AddressViewModelFactory(get(), SingleProfile.asdaService))
                .get(AddressViewModel::class.java)
    }


    override fun loadAndEditMainAddress(mainAddress: AddressBookResponse.Address) {
        isMainAddress = mainAddress.isDefault
        addrssId = mainAddress.addressId
        showMandatoryAdressFields(mainAddress)
        mainAddress.postalCode?.let {
            setPostcodeText(it, mIsAdding)
            loadPostcodeAddresses(mPostcode, Authentication.getInstance().csrfToken)
        }
    }

    private var addrssId: String? = null

    override fun setEditAddressDetails(address: AddressBookResponse.Address) {
        isMainAddress = address.isDefault
        showMandatoryAdressFields(address)
        addrssId = address.addressId
    }

    private fun showMandatoryAdressFields(address: AddressBookResponse.Address) {
        mAddressLine1.setText(address.line1)
        mAddressLine1.visibility = View.VISIBLE
        mAddressLine2.setText(address.line2)
        mAddressLine2.visibility = View.VISIBLE
        mCity.setText(address.city)
        mCity.visibility = View.VISIBLE
        val countryName = address.country?.let { getCountryName(it)?.countryName?.toUpperCase()?.replace(" ","_")}
        countryName?.let { Countries.valueOf(it).position }?.let { mCountry.setSelection(it) }
        mEditContainer.visibility = View.VISIBLE
        mAddressContainer.visibility = View.VISIBLE

        // Quick Registration empty address
        if (AddressEmptyValidator().validate(address) || (TextUtils.isEmpty(address.line1)
                        && TextUtils.isEmpty(address.city) && TextUtils.isEmpty(address.county))) {
            mAddressSpinner.prompt = getString(R.string.choose_address)
        } else {
            mAddressSpinner.prompt = concatenateWithComma(address.address, address.city, address.county)
        }
         if ("home".equals(address.addressType.toString(), true) ||
               "house".equals(address.addressType.toString(), true)) {
            mPropertyType.check(R.id.radio_house)
        } else {
            mPropertyType.check(R.id.radio_flat)
        }
        mDeliveryInstructions.setText(address.deliveryInstructions)
    }

    override fun loadPostcodeAddresses(postCode: String?, token: String?) {
        if (!TextUtils.isEmpty(postCode)) {
            val context = activity
            Log.d(TAG, "loadPostcodeAddresses from fragment " + hashCode())
            val dialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR,
                    getString(R.string.add_address_progress_postcode_check), context)
            dialog?.setCancelable(false)

            addressViewModel?.getSuggestAddressState(postCode!!)
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribe(object : ResourceObserver<AddressSuggestUiState>() {
                        override fun onComplete() {
                            dispose()
                        }

                        override fun onNext(uiState: AddressSuggestUiState) {
                            if (uiState.isLoading) {
                                dialog?.show()
                            } else {
                                try {
                                    dialog?.dismiss()
                                } catch (ignored: Throwable) {
                                }

                                uiState.addressResponse?.let {
                                    if (RegisterHelper.dontDeliver(uiState.addressResponse)) {
                                        val response = it
                                        setPostcodeNotFoundMessage(response.postcodeDeliveryMessage)
                                    } else {
                                        uiState.addressResponse?.let {
                                            mAddressSuggestions = it.addresses
                                            fillAddressSpinner(mIsAdding)
                                            setPostcodeNotFoundMessage(null)
                                        } ?: run {
                                            DialogHelper.displayErrorDialog(R.string.error_invalid_postcode, context, SCREEN_NAME)
                                        }
                                    }
                                } ?: run {
                                    if (BuildConfig.DEBUG) {
                                        Toast.makeText(getContext(), "Network Timedout try again!!", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        }

                        override fun onError(e: Throwable) {
                            try {
                                dialog?.dismiss()
                            } catch (ignored: Exception) {
                            }
                            AsdaDialogHelper.displayErrorDialog(null, -1, context, "Add Address : Enter Postcode")
                        }

                    })
        }
    }

    override fun fillContent() {
        arguments?.let {
            if (TextUtils.isEmpty(it.getString(KEY_POSTCODE))) {
                mIsAdding = true
                handleNoMainAddressCase()
            } else {
                super.fillContent()
            }
        } ?: kotlin.run {
            Log.e(TAG, "Error loading SP Address content")
        }
    }

    private fun handleGenericError(context: Context?, dialog: Dialog?, o: PostcodeCheckResponse?, i: Int) {
        mLoadingAddressSuggestions = false
        try {
            dialog?.dismiss()
        } catch (ignored: Exception) {
        }
        AsdaDialogHelper.displayErrorDialog(o, i, context, SCREEN_NAME)
    }

    override fun fillAddingAddress(suggestion: PostcodeCheckResponse.AddressSuggestion?) {
        super.fillAddingAddress(suggestion)
        hideExtraViews()
    }

    override fun prepopulateFields() {
        super.prepopulateFields()
        hideExtraViews()
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun getActionBarTitle(): String? {
        arguments?.let {
            if (it.getBoolean(KEY_IS_ADDING)) {
                return getString(R.string.sp_account_add_address)
            } else {
                return getString(R.string.sp_account_edit_address)
            }
        } ?: run {
            return getString(R.string.sp_account_add_address)
        }
    }

    override fun checkAndCommitAddress(onlyAddAddress:Boolean) {
        clearErrorLabels(mAddressLine1Text, mCityText, mAddressNicknameText, mMobilePhoneNumberText, mAddressLine2Text, mCountyText, mDaytimePhoneNumberText, mEveningPhoneNumberText)
        hideExtraViews()
        val registrationInfo = RegistrationInfo(RegistrationInfo.TYPE_ADD_ADDRESS)
        registrationInfo.setPostCode(mPostcode)
        registrationInfo.setAddress(formatAddress())
        registrationInfo.setTownOrCity(mCity.text.toString().trim { it <= ' ' })
        registrationInfo.setCounty(mCounty.text.toString().trim { it <= ' ' })
        registrationInfo.setAddressType(if (mPropertyType.checkedRadioButtonId == R.id.radio_house) TYPE_HOME else TYPE_FLAT)
        registrationInfo.setAddressNickname(mAddressNickname.text.toString().trim { it <= ' ' })
        if (mExistingAddressNickname != null) {
            registrationInfo.setExistingAddressNickname(mExistingAddressNickname)
        }
        registrationInfo.setDeliveryInstructions(mDeliveryInstructions.text.toString().trim { it <= ' ' })
        registrationInfo.setMobileNumber(mMobilePhoneNumber.text.toString().trim { it <= ' ' })
        registrationInfo.setDayNumber(mDaytimePhoneNumber.text.toString().trim { it <= ' ' })
        registrationInfo.setEveningNumber(mEveningPhoneNumber.text.toString().trim { it <= ' ' })


        val arguments = arguments
        arguments?.let {
            val existingNicknames = arguments.getStringArrayList(KEY_NICKNAMES)
            val existingPostcodes = arguments.getStringArrayList(KEY_EXISTING_POSTCODES)
            val readiness = arguments.getBooleanArray(KEY_READINESS)
            var readinessList: MutableList<Boolean>? = null
            if (readiness != null && readiness.size > 0) {
                readinessList = ArrayList(readiness.size)
                for (value in readiness) {
                    readinessList.add(value)
                }
            }
            registrationInfo.setExistingAddressNicknames(existingNicknames, existingPostcodes, readinessList)
        }

        if (registrationInfo.verifyAddressOnly(true)) {
            commitAddress(onlyAddAddress)
        } else {
            showErrors(registrationInfo.errors)
        }
    }

    private fun commitAddress(onlyAddAddress: Boolean) {

        val focusedView = mView.findFocus()
        focusedView?.let {
            ViewUtil.hideKeyboard(focusedView)
        }
        val pd = CustomProgressDialog.create(context)
        pd.setCancelable(false)
        pd.setMessage(resources.getString(R.string.add_address_progress_adding))
        pd.isIndeterminate = true
        pd.show()

        val callback = object : AddAddressListener {

            override fun onFailure(error: Int, data: AddressBookResponse, attemptedAddress: AddressBookResponse.Address?) {
                try {
                    pd.dismiss()
                } catch (ignored: Exception) {
                }

                if (!handleServicesError(data)) {
                    AsdaDialogHelper.displayErrorDialog(data, error, context, SCREEN_NAME)
                }
            }

            override fun onSuccess(addressBookResponse: AddressBookResponse, addedAddress: AddressBookResponse.Address?) {
                activity?.let {
                    addressBookResponse.addresses?.let {
                        if (it.isEmpty()) {
                            AsdaDialogHelper.displayErrorDialog(addressBookResponse, addressBookResponse.statusCodeNumber, context, SCREEN_NAME)
                        } else if (context?.let { it1 -> FeatureSettings.isCXOMigrationEnabled(it1) } == true) {
                            finishCommit(addedAddress, addressBookResponse, pd, context)
                        } else {
                            //todo: vmk this needs to be verified. This is failing
                            //cart must be refreshed to make address edits visible in other parts of app
                            val cartManager = CartManager.get(context)
                            if (mCartListenerAfterAddingAddress != null) {
                                cartManager?.removeCartListener(mCartListenerAfterAddingAddress)
                            }
                            SingleProfile.getProfileResponse()?.addresses = it
                            cartManager?.addCartListener(object : ICartManager.CartListener() {

                                override fun onChanged() {
                                    finishAddressAdding()
                                    val event = AsdaAnalyticsEvent(Anivia.ADD_DELIVERY_ADDRESS_EVENT)
                                    if (!TextUtils.isEmpty(mPostcode) && mPostcode.length > 3) {
                                        event.putString(Anivia.ADD_POSTCODE_KEY, mPostcode.substring(0, 3))
                                    }
                                    Tracker.get().trackEvent(event)
                                }

                                private fun finishAddressAdding() {
                                    CartManager.get(context)?.removeCartListener(this)
                                    finishCommit(addedAddress, addressBookResponse, pd, context)
                                }

                                override fun onFailure() {
                                    finishAddressAdding()
                                }
                            })
                            context?.let { context -> cartManager.refreshCart(context) }
                        }
                    } ?: run {
                        AsdaDialogHelper.displayErrorDialog(addressBookResponse, addressBookResponse.statusCodeNumber, context, SCREEN_NAME)
                    }
                }
            }
        }

        SingleProfile.authentication.profileResponse?.profileId?.let {
            val profileId = it
            val addressViewModel = ViewModelProviders.of(this, AddressViewModelFactory(get(), SingleProfile.asdaService))
                    .get(AddressViewModel::class.java)
            val body = AddressBookResponse()
            body.profileId = profileId
            val addresses = arrayOfNulls<AddressBookResponse.Address>(1)

            var isAddingCard: Boolean? = null
            arguments?.getBoolean(EXTRA_IS_ADD_CARD, false)?.let {
                isAddingCard = it
            }
            var setAsMain = shouldSetAddressAsMain(onlyAddAddress, isAddingCard)
            var countryCode = DEFAULT_COUNTRY
            mCountry?.selectedItem?.toString()?.toUpperCase()?.replace(" ", "_")?.let { country ->
                countryCode = Countries.valueOf(country).countryCode
            }
            addresses[0] = AddressBookResponse.Address(
                    mAddressLine1.text.toString(),
                    mAddressLine2.text.toString(),
                    " ", /*line3 no present in UI will be added in next phase*/
                    mCity.text.toString(),
                    " ", /*state no present in UI will be added in next phase*/
                    countryCode,
                    mPostcode,
                    mType, //shipto or billto todo: vmk change from where it is launched
                    setAsMain,
                    if (mPropertyType.checkedRadioButtonId == R.id.radio_house) AddressBookResponse.AddressType.House.toString() else AddressBookResponse.AddressType.Flat.toString(),
                    mDeliveryInstructions.text.toString(),
                    mAddressLine1.text.toString() + ", " + mCity.text + ", " + mPostcode
            ).apply {
                if (!mIsAdding) {
                    this.addressId = addrssId
                }
            }

            body.addressBook = addresses
            body.statusCodeNumber = 0
            var isSetMain = (!onlyAddAddress && (isMainAddress || mIsAdding))

            // if only add address, do not set this as main address
            if (onlyAddAddress) {
                isMainAddress = false
            }

            addressViewModel.addOrUpdateAddress(body, mIsAdding)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(object : Observer<AddressUiState> {
                        override fun onSubscribe(d: Disposable) {
                            safeAdd(d)
                        }

                        override fun onNext(state: AddressUiState) {
                            when (state) {
                                AddressLoading -> {

                                }

                                is AddressLoadedState -> {
                                    mLoadingAddressSuggestions = false
                                    try {
                                        SingleProfile.getProfileResponse()?.addresses = state.addressResponse.addressBook
                                        pd.dismiss()
                                    } catch (e: Exception) {
                                        if (Utils.isDebug()) {
                                            Log.e(TAG, "Error: $state")
                                        }
                                    }
                                    callback.onSuccess(state.addressResponse, addresses[0])
                                }

                                is AddressErrorState -> {
                                    if (Utils.isDebug()) {
                                        Log.e(TAG, "Error: $state")
                                    }
                                    //todo vmk pass the failures correctly
                                    handleGenericError(context, pd, null, -1)
                                }
                                IsBillingAddress -> {
                                }
                            }.let {}
                        }

                        override fun onError(e: Throwable) {
                            if (Utils.isDebug()) {
                                Log.e(TAG, "Error: $e")
                            }
                            handleGenericError(context, pd, null, -1)
                        }

                        override fun onComplete() {

                        }
                    })
        } ?: run {
            if (Utils.isDebug()) {
                Log.e(TAG, "profileId is null")
            }
            handleGenericError(context, pd, null, -1)
        }
    }

    @VisibleForTesting
    fun shouldSetAddressAsMain(onlyAddAddress: Boolean, isAddingCard: Boolean?): Boolean {
        // If onlyAddAddress, this should override any locig - we don't want to set this as main address.
        // This is coming from bookslot screen, address selection (where we do not change any address permanently).
        // We only take the selected or newly added address (do not add as main here in that case).
        if (onlyAddAddress) {
            return false
        }

        var setAsMain = isMainAddress || mIsAdding
        if (!isValidPostcode(mPostcode)) {
            setAsMain = false
        } else {
            isAddingCard?.let {
                if (it == true) {
                    setAsMain = false
                }
            }
        }
        return setAsMain
    }
}
