package com.asda.android.app.account.address.view

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import com.asda.android.R
import com.asda.android.base.core.view.actitivies.BaseAppCompatActivity
import com.asda.android.restapi.service.data.AddressBookResponse

class SelectAddressActivity : BaseAppCompatActivity(), SelectAddressPresenter.OnAddressChangedListener {

    override fun onAddressSelected(selectedAddress: AddressBookResponse.Address?) {
        val intent = Intent(this, SelectAddressActivity::class.java)
        intent.putExtra(SELECTED_ADDRESS, selectedAddress)
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    override fun onAddressChanged(addedAddress: AddressBookResponse.Address?) {
        // Do nothing
    }

    private var title: String? = null
    private var nickName: String? = null
    private var onlySelection: Boolean = false

    override fun getFragmentContainer() = R.id.fragment_container

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.fragment_container)
        getArguments()

        title?.let {
            supportActionBar?.title = it
        }?: run {
            supportActionBar?.title = getString(R.string.choose_address)
        }
        supportActionBar?.setDisplayHomeAsUpEnabled(true)


        val fragment = SelectAddressPresenter.newInstance(nickName, onlySelection)
        fragment.setOnAddressChangedListener(this)
        fragment?.let {
            push(it)
        }
    }

    private fun getArguments() {
        intent?.let {
            title = it.getStringExtra(EXTRA_TITLE)
            nickName = it.getStringExtra(EXTRA_NICK_NAME)
            onlySelection = it.getBooleanExtra(EXTRA_ONLY_SELECTION, false)
        }
    }

    companion object {
        const val SELECTED_ADDRESS = "selected_address"
        private const val EXTRA_TITLE = "title"
        private const val EXTRA_NICK_NAME = "nick_name"
        private const val EXTRA_ONLY_SELECTION = "only_selection"

        fun launch(context: Context, title:String?, nickName: String?, onlySelection:Boolean) {
            val intent:Intent = getIntent(context = context, title = title, nickName = nickName, onlySelection = onlySelection)
            context.startActivity(intent)
        }

        fun launchForResult(fragment: androidx.fragment.app.Fragment, title:String?, nickName: String?, onlySelection:Boolean, requestCode:Int) {
            fragment.context?.let {
                val intent:Intent = getIntent(context = it, title = title, nickName = nickName, onlySelection = onlySelection)
                fragment.startActivityForResult(intent, requestCode)
            }
        }

        private fun getIntent(context: Context, title:String?, nickName: String?, onlySelection:Boolean): Intent {
            val intent = Intent(context, SelectAddressActivity::class.java)
            intent.putExtra(EXTRA_TITLE, title)
            intent.putExtra(EXTRA_NICK_NAME, nickName)
            intent.putExtra(EXTRA_ONLY_SELECTION, onlySelection)
            return intent
        }
    }
}
