package com.asda.android.app.account.parser

import com.asda.android.base.core.parser.IParser
import com.asda.android.app.account.view.ChangePasswordFragment
import com.asda.android.app.account.view.EditPersonalDetailsFragment
import com.asda.android.app.account.view.FeedbackFragment
import com.asda.android.app.account.view.VouchersFragment
import com.asda.android.app.account.model.FragmentInfoParserModel
import com.asda.android.app.checkout.OrderSummaryPresenter
import com.asda.android.app.dp.view.DPDetailsFragment
import com.asda.android.app.orders.OrderFragment
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.detail.view.SettingsFragment
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.singleprofile.view.SPAddressesFragment
import com.asda.android.app.account.singleprofile.view.instance
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.singleprofile.features.account.view.PersonalDetailsFragment
import com.asda.android.singleprofile.features.account.view.PhoneBookFragment
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment
import com.asda.android.base.core.web.model.WebViewModel
import com.asda.android.base.core.web.view.WebViewFragment
import com.asda.android.restapi.service.base.AsdaServiceSettings.getAsdaWebUrl

/**
 * Creates fragment based on tag from input
 */
class AccountFragmentTagParser : IParser<FragmentInfoParserModel, BaseFragment?> {
    private val platform = "&platform=android"
    private val personalDetailURL = getAsdaWebUrl() + "account?web_view=personal_details" + platform
    private val signInDetailURL = getAsdaWebUrl() + "account?web_view=signin_details" + platform
    private val permissionCenterURL = getAsdaWebUrl() + "account?enable_permission=true&web_view=permission_centre" + platform

    companion object {
        const val SIGN_IN_DETAILS_FRAGMENT_TAG = "account_sign_in_details"
        const val PERMISSION_CENTER_FRAGMENT_TAG = "account_permission_center"
    }

    override fun parse(input: FragmentInfoParserModel): BaseFragment? {
        return when (input.tag) {
            OrderFragment.TAG -> OrderFragment()
            DPDetailsFragment.TAG -> DPDetailsFragment()
            SPAddressesFragment.TAG -> instance()
            EditPersonalDetailsFragment.TAG -> EditPersonalDetailsFragment.newInstance()
            VouchersFragment.TAG -> VouchersFragment.newInstance()
            ChangePasswordFragment.TAG -> ChangePasswordFragment.newInstance()
            SettingsFragment.TAG -> SettingsFragment()
            RecurringSlotDetailsFragment.TAG -> RecurringSlotDetailsFragment.newInstance()
            PersonalDetailsFragment.TAG -> {
                if (input.phoneNumberLoggedIn) {
                    WebViewFragment.newInstance(WebViewModel(PersonalDetailsFragment.TAG, personalDetailURL, input.title))
                } else PersonalDetailsFragment.newInstance()
            }
            PhoneBookFragment.TAG -> PhoneBookFragment.newInstance(OrderSummaryPresenter.TAG)
            SPPaymentCardsFragment.TAG -> SPPaymentCardsFragment.newInstance()
            FeedbackFragment.TAG -> FeedbackFragment.newInstance(FeedbackFragment.SURVEY_TYPE_OPT_INT_KEY, FeedbackFragment.ENTITY_TYPE_OPT_IN_KEY, -1, "")
            SPAddEditPaymentCardFragment.TAG -> SPAddEditPaymentCardFragment.newInstance(input.card, input.card == null, false, SPAddEditPaymentCardFragment.PURCHASE_TYPE_BASKET)
            SIGN_IN_DETAILS_FRAGMENT_TAG -> {
                if (input.phoneNumberLoggedIn) {
                    WebViewFragment.newInstance(WebViewModel(SIGN_IN_DETAILS_FRAGMENT_TAG, signInDetailURL, input.title))
                } else null
            }
            PERMISSION_CENTER_FRAGMENT_TAG -> {
                if (input.permissionCenterEnabled) {
                    WebViewFragment.newInstance(WebViewModel(PERMISSION_CENTER_FRAGMENT_TAG, permissionCenterURL, input.title, input.privacyPolicy))
                } else null
            }
            else -> null
        }

    }
}
