package com.asda.android.app.account.parser

import com.asda.android.app.account.parser.AccountFragmentTagParser.Companion.PERMISSION_CENTER_FRAGMENT_TAG
import com.asda.android.base.core.parser.IParser
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.singleprofile.features.account.view.PersonalDetailsFragment
import com.asda.android.base.core.web.constants.EventOwner
import com.asda.android.base.core.web.constants.WebConstants.ACTION_CANCEL
import com.asda.android.base.core.web.model.WebViewEventModel

/**
 * Returns the tag of a fragment which will be removed as result of provided webviewevent model
 */
class FragmentRemoveParser : IParser<BaseStateResponse<WebViewEventModel>, String?> {
    private val personalDetailsPageName = "personal_details"
    private val permissionCenterPageName = "permission_centre"

    override fun parse(input: BaseStateResponse<WebViewEventModel>): String? {
        input.getData()?.let {
            if (input.getState() == State.SUCCESS && it.eventOwner == EventOwner.EVENT_OWNER_USER && it.action == ACTION_CANCEL) {
                when (it.pageName) {
                    personalDetailsPageName, PersonalDetailsFragment.TAG -> return PersonalDetailsFragment.TAG
                    permissionCenterPageName, PERMISSION_CENTER_FRAGMENT_TAG -> return PERMISSION_CENTER_FRAGMENT_TAG
                }
            }
        }
        return null
    }

}
