package com.asda.android.app.account.singleprofile.view


import android.Manifest
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.text.Editable
import android.text.Spannable
import android.text.TextUtils
import android.text.TextUtils.isEmpty
import android.text.style.ForegroundColorSpan
import android.text.style.RelativeSizeSpan
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import android.widget.*
import androidx.annotation.StringRes
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.PV_PAYMENT_CARDS
import com.asda.android.analytics.constants.Anivia.PV_PAYMENT_CARD_EDIT
import com.asda.android.analytics.constants.Anivia.SECTION_ACCOUNT
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.AsdaEngineeringAnalyticsEvent
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.UserAccountDetails
import com.asda.android.app.account.address.view.BaseAddEditAddressFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.view.GroupFourDigitsEditText
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.getASDAFragmentTransaction
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.cxo.CartManager
import com.asda.android.cxo.repository.CheckoutRepository
import com.asda.android.restapi.constants.ExternalUrls
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.CreditCardsListResponse.CreditCardType
import com.asda.android.restapi.service.data.checkout.*
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.base.get
import com.asda.android.singleprofile.features.account.PostcodeCheckListener
import com.asda.android.singleprofile.model.AddressErrorState
import com.asda.android.singleprofile.model.AddressLoadedState
import com.asda.android.singleprofile.model.AddressLoading
import com.asda.android.singleprofile.model.CreateCardInitializationHolder
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.PaymentCardsViewModel
import com.asda.android.utils.view.BaseEditFragment
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import com.jakewharton.rxbinding2.widget.RxAdapterView
import com.jakewharton.rxbinding2.widget.RxTextView
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.Permissions
import com.asda.android.base.core.view.FloatLabel
import io.card.payment.CardIOActivity
import io.reactivex.BackpressureStrategy
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Function3
import io.reactivex.subscribers.DefaultSubscriber
import io.reactivex.subscribers.DisposableSubscriber
import pangea.getEncryptCreditCardService
import java.text.ParseException
import java.util.*
import java.util.regex.Pattern

class SPAddEditPaymentCardFragment : BaseEditFragment(), PostcodeCheckListener, BaseAddEditAddressFragment.AddressBookChangedListener {
    val PURCHASE_TYPE_DP = "dppurchase"
    val PURCHASE_TYPE_BASKET = "basket"
    val CODE_NO_CHANGE = -1
    val PURCHASE_TYPE_DP_RENEW = "dprenew"

    private val DIALOG_CHECK_POSTCODE = 1
    val SCAN_REQUEST_CODE = 1001

    private val SCREEN_NAME = "Payment : Add Card"

    private val NUMBER_OF_YEARS = io.card.payment.CreditCard.EXPIRY_MAX_FUTURE_YEARS

    private lateinit var mAddressesLoadingView: View
    private lateinit var mCardHolderName: EditText
    private lateinit var mCardHolderNameText: FloatLabel
    private lateinit var mExpiryDateText: FloatLabel
    private lateinit var mExpiryYearSpinner: Spinner
    private lateinit var mExpiryMonthSpinner: Spinner
    private lateinit var mCardNumber: GroupFourDigitsEditText
    private lateinit var mCardIcon: ImageView
    private lateinit var mCardNumberText: FloatLabel
    private lateinit var mAddressText: FloatLabel
    private lateinit var mAddressSpinner: Spinner
    private lateinit var mSingleUseSave: Button

    private var mIsAdding: Boolean = false
    private var mCreditCard: PaymentDetailsResponse.PaymentCards? = null
    private var mListener: OnCardAddedListener? = null
    private var mCXOListener: OnCXOCardAddedListener? = null
    private var mCardDataScanned: Boolean = false
    private var hasError = false
    private var mSingleUseEnabled = true
    private var mPurchaseType: String? = null
    private var mCurrentExpiryYear: Int = 0
    private var mCurrentExpiryMonth: Int = 0

    private var mCardHolderEditableNameText: Editable? = null

    private lateinit var mCardNumberObservable: Flowable<CharSequence>
    private lateinit var mExpiryMonthSpinnerObservable: Flowable<Int>
    private lateinit var mExpiryYearSpinnerObservable: Flowable<Int>


    private var mOnSaveStateChange: OnSaveStateChange? = null

    private val selectedMonth: Int
        get() = mExpiryMonthSpinner.selectedItemPosition + 1

    private var paymentCardsViewModel: PaymentCardsViewModel? = null

    private var addressViewModel: AddressViewModel? = null

    fun setOnCardAddedListener(cardListener: OnCardAddedListener?) {
        mListener = cardListener
    }

    fun setOnCXOCardAddedListener(cardListener: OnCXOCardAddedListener?) {
        mCXOListener = cardListener
    }

    interface OnCardAddedListener {
        fun onCardAdded(creditCardResponse: AsdaResponse, isSingleUser: Boolean, cardNumber: String?)
    }

    interface OnCXOCardAddedListener {
        fun onCXOCardAdded(checkoutResponse: CheckoutResponse?, cardNickName: String?)
    }

    interface OnSaveStateChange {
        fun onChange(codeChange: Int)
    }

    fun setOnSaveStateChange(onSaveStateChange: OnSaveStateChange) {
        mOnSaveStateChange = onSaveStateChange
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mSingleUseEnabled = bundle.getBoolean(KEY_ENABLE_SINGLE_USE, true)
            mIsAdding = bundle.getBoolean(KEY_IS_ADDING, true)
            mCreditCard = bundle.getParcelable(KEY_CREDIT_CARD)
            mPurchaseType = bundle.getString(KEY_PURCHASE_TYPE)
            mCurrentExpiryMonth = bundle.getInt(KEY_EXPIRY_MONTH)
            mCurrentExpiryYear = bundle.getInt(KEY_EXPIRY_YEAR)
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(outState, "SPAddEditPaymentCardFragment")
        outState.putBoolean(KEY_ENABLE_SINGLE_USE, mSingleUseEnabled)
        outState.putParcelable(KEY_CREDIT_CARD, mCreditCard)
        outState.putBoolean(KEY_IS_ADDING, mIsAdding)
        outState.putString(KEY_PURCHASE_TYPE, mPurchaseType)
        outState.putInt(KEY_EXPIRY_YEAR, mCurrentExpiryYear)
        outState.putInt(KEY_EXPIRY_MONTH, mCurrentExpiryMonth)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        mOnSaveStateChange?.onChange(CODE_NO_CHANGE)
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed(): Boolean {
        mOnSaveStateChange?.onChange(CODE_NO_CHANGE)
        return super.onBackPressed()
    }


    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.account_edit_payment_card, fragmentContainer, false)
        setupAddEditCard(view)
        paymentCardsViewModel = PaymentCardsViewModel(get())
        return view
    }

    private fun setupView(view: View) {
        mCardHolderName = view.findViewById<View>(R.id.name_value) as EditText
        mCardHolderNameText = view.findViewById<View>(R.id.name_value_text) as FloatLabel
        mCardHolderNameText.attach(mCardHolderName)
        mCardNumber = view.findViewById<View>(R.id.card_number_value) as GroupFourDigitsEditText
        mCardNumber.setDontGroupAfterDigitsCount(16)
        mCardIcon = view.findViewById<View>(R.id.card_icon) as ImageView
        mCardNumberText = view.findViewById<View>(R.id.card_number_text) as FloatLabel
        mCardNumberText.attach(mCardNumber)
        mExpiryDateText = view.findViewById<View>(R.id.expiry_date_text) as FloatLabel
        mExpiryDateText.show()
        mExpiryMonthSpinner = view.findViewById<View>(R.id.expiry_date_month_spinner) as Spinner
        mExpiryYearSpinner = view.findViewById<View>(R.id.expiry_date_year_spinner) as Spinner
        mAddressText = view.findViewById<View>(R.id.address_text) as FloatLabel
        mAddressesLoadingView = view.findViewById(R.id.billing_address_loading)
        mAddressSpinner = view.findViewById<View>(R.id.address_spinner) as Spinner
        mSingleUseSave = view.findViewById<View>(R.id.save_use_once) as Button

        if (mSingleUseEnabled) {
            mSingleUseSave.visibility = View.VISIBLE
            mSingleUseSave.setOnClickListener { _ ->
                if (hasError) {
                    //do nothing as error is being displayed already
                } else if (wereFieldsChanged() && !hasError) {
                    verifyAndCommitCard(true)
                } else if (mIsAdding && TextUtils.isEmpty(mCardNumber.rawNumberString)) {
                    mCardNumberText.let {
                        displayErrorMessage(mCardNumber, it, R.string.error_no_card_number, true)
                    }
                } else {
                    pop()
                }
            }
        } else {
            mSingleUseSave.visibility = View.GONE
        }

        mCardHolderName.setOnEditorActionListener { v, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_NEXT || actionId == EditorInfo.IME_ACTION_DONE) {
                ViewUtil.hideKeyboard(v)
            }
            false
        }

        val mastercardLink = view.findViewById<View>(R.id.mastercard_link_text) as TextView
        mastercardLink.setOnClickListener { ViewUtils.openWebPage(ExternalUrls.MASTERCARD_SECURE_CODE, activity, SCREEN_NAME) }
        val visaLink = view.findViewById<View>(R.id.visa_link_text) as TextView
        visaLink.setOnClickListener { ViewUtils.openWebPage(ExternalUrls.VERIFIED_BY_VISA, activity, SCREEN_NAME) }
        val amexLink = view.findViewById<View>(R.id.amex_link_text) as TextView
        amexLink.setOnClickListener { ViewUtils.openWebPage(ExternalUrls.AMEX_SAFEKEY, activity, SCREEN_NAME) }

        mCardNumberObservable = RxTextView.textChanges(mCardNumber)
              .toFlowable(BackpressureStrategy.LATEST)

        mExpiryMonthSpinnerObservable = observeStartMonthSpinner(mExpiryMonthSpinner).skip(1)
              .toFlowable(BackpressureStrategy.LATEST)

        mExpiryYearSpinnerObservable = observeStartYearSpinner(mExpiryYearSpinner).skip(1)
              .toFlowable(BackpressureStrategy.LATEST)

        setupCardScanButton(view)
        setupSaveButton(view)
        styleHeaderText(view)

    }

    private fun setupCardScanButton(view: View) {
        val scanButton = ViewUtil.findViewById<TextView>(view, R.id.scan_button)
        scanButton.setOnClickListener {
            if (Permissions.isCameraPermissionGranted(context)) {
                startCardIOActivity()
            } else if (isAdded) {
                requestPermissions(arrayOf(Manifest.permission.CAMERA), Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_CAMERA)
            }
        }
    }

    private fun startCardIOActivity() {
        val scanIntent = Intent(activity, CardIOActivity::class.java)
        scanIntent.putExtra(CardIOActivity.EXTRA_REQUIRE_EXPIRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SCAN_EXPIRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SUPPRESS_CONFIRMATION, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SUPPRESS_MANUAL_ENTRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_HIDE_CARDIO_LOGO, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_KEEP_APPLICATION_THEME, true)

        startActivityForResult(scanIntent, SCAN_REQUEST_CODE)
    }

    private fun styleHeaderText(view: View) {
        val header = ViewUtil.findViewById<TextView>(view, R.id.header_text)
        val spannable = header.text as Spannable
        // 13 is "Free delivery" string end index
        spannable.setSpan(RelativeSizeSpan(1.43f), 0, 13, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        context?.let {
            spannable.setSpan(ForegroundColorSpan(ContextCompat.getColor(it, R.color.asda_dark_green)), 0, 13, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        }
        ViewUtils.applyAsdaFont(header, 0, 13, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
    }

    private fun fillContent() {
        loadBillingAddresses()
        if (!mIsAdding) {
            mCardHolderName.setText(mCreditCard?.nameOnCard)
            mCardNumber.enableLetterSupport(true)
            mCardNumber.setText(mCreditCard?.card)
            mCardNumber.isEnabled = false
            setExpiryDate(mCreditCard?.expiryDate)
        } else {
            initExpiryDateSpinners(-1, NUMBER_OF_YEARS)
        }

        mCardNumberObservable.subscribe(object : DefaultSubscriber<CharSequence>() {
            override fun onNext(charSequence: CharSequence) {
                Log.d(TAG, "Card Number Observable OnNext")
                val type: CreditCardType = if (charSequence.toString().startsWith("x") || charSequence.toString().startsWith("*")) {
                    mCreditCard?.creditCardType.let {
                        getCardTypeFromString(it)
                    }
                } else {
                    getCardType(charSequence.toString().replace("\\s".toRegex(), ""))
                }

                mCardIcon.setImageResource(type.cardIcon)
            }

            override fun onError(t: Throwable) {
                Log.d(TAG, "Card Number Observable onError" + t.message)

            }

            override fun onComplete() {
                Log.d(TAG, "Card Number Observable onComplete")
            }
        })
    }

    private fun initExpiryDateSpinners(startYear: Int, numberOfYears: Int) {
        val calendar = Utils.getCurrentTimeInUK()
        mExpiryYearSpinner.adapter = createYearSpinnerAdapter(if (startYear > -1) startYear else calendar.get(Calendar.YEAR), numberOfYears)
        mExpiryMonthSpinner.adapter = createMonthSpinnerAdapter(activity)

        if (mCurrentExpiryMonth != -1 && mCurrentExpiryMonth < mExpiryMonthSpinner.count) {
            mExpiryMonthSpinner.setSelection(mCurrentExpiryMonth)
        }

        if (mCurrentExpiryYear != -1 && mCurrentExpiryYear < mExpiryYearSpinner.count) {
            mExpiryYearSpinner.setSelection(mCurrentExpiryYear)
        }
    }

    private fun setExpiryDate(expiryDate: String?) {
        val currentYear = Utils.getCurrentTimeInUK().get(Calendar.YEAR)
        try {
            val expiryDateParsed = Utils.parseCreditCardDate(expiryDate)
            val yearSelector: Int
            val numberOfYears: Int
            if (currentYear > expiryDateParsed.year) {
                yearSelector = currentYear - expiryDateParsed.year - 1
                numberOfYears = NUMBER_OF_YEARS + currentYear - expiryDateParsed.year
            } else {
                yearSelector = expiryDateParsed.year - currentYear
                numberOfYears = NUMBER_OF_YEARS
            }
            val startYear = mCreditCard?.let {
                if (it.expired) {
                    expiryDateParsed.year
                } else {
                    -1
                }
            } ?: -1

            initExpiryDateSpinners(startYear, numberOfYears)

            if (mCurrentExpiryMonth == -1) {
                mExpiryMonthSpinner.setSelection(expiryDateParsed.month - 1)
            }
            mExpiryYearSpinner.let {
                if (mCurrentExpiryYear == -1 && yearSelector < it.count) {
                    mExpiryYearSpinner.setSelection(yearSelector)
                }
            }
        } catch (e: ParseException) {
            Log.e(TAG, "Error parsing card expiry date")
        }

    }

    private fun createYearSpinnerAdapter(beginYear: Int, numberOfYears: Int): ArrayAdapter<Int> {
        val years = arrayOfNulls<Int>(numberOfYears)
        for (i in years.indices) {
            years[i] = beginYear + i
        }

        val adapter = ArrayAdapter<Int>(activity!!, android.R.layout.simple_spinner_item, years)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        return adapter
    }

    override fun onSaveButtonClick() {
        if (hasError) {
            //do nothing as error is being displayed already
        } else if (wereFieldsChanged() && !hasError) {
            verifyAndCommitCard(false)
        } else if (mIsAdding && TextUtils.isEmpty(mCardNumber.rawNumberString)) {
            displayErrorMessage(mCardNumber, mCardNumberText, R.string.error_no_card_number, true)
        } else {
            pop()
        }
    }

    override fun wereFieldsChanged(): Boolean {
        if (mIsAdding) {
            return !TextUtils.isEmpty(mCardNumber.text.toString().trim { it <= ' ' }) ||
                  mExpiryMonthSpinner.selectedItemPosition > -1 ||
                  mExpiryYearSpinner.selectedItemPosition > -1 ||
                  mAddressSpinner.selectedItemPosition != 0
        } else {
            val considerExpiryYear = mExpiryYearSpinner.selectedItem != null
            return mCardHolderName.text.toString().trim { it <= ' ' } != mCreditCard!!.cardHolderName ||
                  considerExpiryYear && isDifferentDate(mCreditCard!!.expirationDate, selectedMonth, mExpiryYearSpinner.selectedItem as Int) ||
                  mAddressSpinner.selectedItem != null && mCreditCard!!.billingAddress != null && mAddressSpinner.selectedItem.toString() != mCreditCard!!.billingAddress!!.toString()
        }
    }

    private fun isDifferentDate(date: String?, comparingMonth: Int, comparingYear: Int): Boolean {
        try {
            val expiryDateParsed = Utils.parseCreditCardDate(date)
            return expiryDateParsed == null || expiryDateParsed.month != comparingMonth || expiryDateParsed.year != comparingYear
        } catch (e: ParseException) {
            Log.d(TAG, "parse error", e)
            return true
        }

    }

    override fun getInternalTag(): String? {
        return TAG
    }


    private fun loadBillingAddresses() {
        val profileId = SingleProfile.getProfileResponse()?.profileId ?: ""
        if (profileId.isNotEmpty()) {
            addressViewModel = AddressViewModel(get())
            val d = addressViewModel?.getDisplayableAddressState(profileId)
                  ?.observeOn(AndroidSchedulers.mainThread())
                  ?.subscribe {
                      when (it) {
                          AddressLoading -> mAddressSpinner.isEnabled = false
                          is AddressErrorState -> {
                              mAddressSpinner.isEnabled = true
                              populateAddresses(Array(0, { AddressBookResponse.Address() }))
                          }
                          is AddressLoadedState -> {
                              mAddressSpinner.isEnabled = true
                              if (it.addressResponse.addresses?.isNotEmpty() == true) {
                                  populateAddresses(it.addressResponse.addresses)
                              } else {
                                  populateAddresses(Array(0, { AddressBookResponse.Address() }))
                              }
                          }
                      }
                  }
            safeAdd(d)
        }
    }

    private fun populateAddresses(addresses: Array<AddressBookResponse.Address>) {
        val newAddressList = ArrayList<AddressBookResponse.Address>()
        val addAddress = AddressBookResponse.Address()
        addAddress.line1 = resources.getString(R.string.add_new_address)
        newAddressList.addAll(addresses)

        newAddressList.add(addAddress)

        if (newAddressList.size == 1) {
            val selectorAddress = AddressBookResponse.Address()
            newAddressList.add(0, selectorAddress)
        }

        val adapter = ArrayAdapter<AddressBookResponse.Address>(activity!!, android.R.layout.simple_spinner_item, newAddressList)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        mAddressSpinner.adapter = adapter
        mAddressSpinner.visibility = View.VISIBLE
        mAddressesLoadingView.visibility = View.GONE
        mAddressText.show()
        if (!mIsAdding && mCreditCard?.address != null) {
            for (i in addresses.indices) {
                if (Utils.equals(addresses[i].addressId, mCreditCard?.address?.addressId)) {
                    mAddressSpinner.setSelection(i)
                    break
                }
            }
        } else {
            mAddressSpinner.setSelection(0)
        }
        mAddressSpinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                if (adapter.count == position + 1) {
                    mAddressSpinner.setSelection(0)
                    showDialog(DIALOG_CHECK_POSTCODE)
                }
            }

            override fun onNothingSelected(parent: AdapterView<*>) {

            }
        }
    }

    override fun showDialog(type: Int) {
        val ft = activity?.supportFragmentManager?.getASDAFragmentTransaction()
        val previousDialog = activity?.supportFragmentManager?.findFragmentByTag("dialog")
        previousDialog?.let {
            ft?.remove(it)
        }
        ft?.addToBackStack(null)

        when (type) {
            DIALOG_CHECK_POSTCODE -> {
                val postcodeDialogFrag = SPCheckPostCodeDialogFragment.newInstance()
                postcodeDialogFrag.setPostcodeCheckedListener(this, SCREEN_NAME)
                ft?.let { postcodeDialogFrag.show(it, "dialog") }
            }
            else -> super.showDialog(type)
        }
    }

    private fun verifyAndCommitCard(singleUse: Boolean) {
        clearErrorLabels(mCardHolderNameText, mExpiryDateText, mCardNumberText)
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                if (singleUse) {
                    addCXOCard()
                } else {
                    addCard(false, mIsAdding, true)
                }
            } else {
                addCard(singleUse, mIsAdding, false)
            }
        }
    }

    /**
     * Observable for the selected position in spinner
     *
     * @param mExpiryMonthSpinner
     * @return Observable for Expiry Month Spinner
     */
    fun observeStartMonthSpinner(mExpiryMonthSpinner: Spinner): Observable<Int> {
        return RxAdapterView.itemSelections<SpinnerAdapter>(mExpiryMonthSpinner)
    }

    fun observeStartYearSpinner(mExpiryYearSpinner: Spinner): Observable<Int> {
        return RxAdapterView.itemSelections<SpinnerAdapter>(mExpiryYearSpinner)
    }

    /**
     * Validate credit card form input, Combine 5 different Observables that track input changes at
     * Credit Card Number, Card Holder Name, Expiry Month Spinner, Expiry Year Spinner, and Card
     * Nick name. Once an event change is triggered from all 4 inputs, the result is combined and
     * form is evaluated for validity.
     */
    private fun combineLatestEvents() {
        val disposableObserver = object : DisposableSubscriber<Boolean>() {
            override fun onNext(formValid: Boolean) {
                if (formValid) {
                    clearErrorLabels(mCardHolderNameText, mExpiryDateText, mCardNumberText)
                    hasError = false
                    Log.e(TAG, "Form is $formValid")
                } else {
                    Log.e(TAG, "Form Valid $formValid")
                    hasError = true
                }
            }

            override fun onError(t: Throwable) {
                Log.e(TAG, "Form has Errors", t)
            }

            override fun onComplete() {
                Log.e(TAG, " Form onComplete")
            }
        }

        Flowable.combineLatest<CharSequence, Int, Int, Boolean>(
              mCardNumberObservable,
              mExpiryMonthSpinnerObservable,
              mExpiryYearSpinnerObservable,
              Function3<CharSequence, Int, Int, Boolean> { cardNumber, mExpiryMonth, mExpiryYear ->
                  mCurrentExpiryYear = mExpiryYear
                  mCurrentExpiryMonth = mExpiryMonth
                  var cardNumberValid = if (mIsAdding) {
                      !isEmpty(cardNumber) && isValidCardNumber(cardNumber.toString()) && isValidCardType(cardNumber.toString())
                  } else {
                      !isEmpty(cardNumber)
                  }

                  if (!cardNumberValid) {
                      displayErrorMessage(mCardNumber, mCardNumberText,
                            R.string.error_invalid_card_number, true)
                      hasError = true
                  }

                  val expiryMonthValid = mExpiryMonth != -1
                  if (!expiryMonthValid) {
                      displayErrorMessage(mExpiryMonthSpinner, mExpiryDateText,
                            R.string.error_no_expire_date, true)
                      hasError = true
                  }

                  val expiryYearValid = mExpiryYear != -1
                  if (!expiryYearValid) {
                      displayErrorMessage(mExpiryYearSpinner, mExpiryDateText,
                            R.string.error_no_expire_date, true)
                      hasError = true
                  }

                  (cardNumberValid && expiryMonthValid
                        && expiryYearValid)
              }
        ).subscribe(disposableObserver)
        safeAdd(disposableObserver)
    }


    private fun createBaseCreditCard(editCard: PaymentDetailsResponse.PaymentCards?): PaymentDetailsResponse.PaymentCards? {
        val cardNumber = mCardNumber.rawNumberString
        val cardHolderName = mCardHolderName.editableText.toString().trim { it <= ' ' }
        val billingAddress = mAddressSpinner.selectedItem as? AddressBookResponse.Address

        val cardType = getCardType(cardNumber)

        if (!hasError) {
            if (mExpiryMonthSpinner.selectedItemPosition == -1 || mExpiryYearSpinner.selectedItemPosition == -1) {
                mExpiryDateText.let {
                    displayErrorMessage(mExpiryMonthSpinner, it, R.string.error_no_expire_date, true)
                }
                hasError = true
            }
        }

        val expiryMonth = if (hasError) {
            0
        } else {
            mExpiryMonthSpinner.selectedItemPosition + 1
        }

        val expiryYear = if (hasError) {
            0
        } else {
            mExpiryYearSpinner.selectedItem ?: -1
        } as Int

        if (billingAddress == null) {
            return null
        }

        return if (hasError) {
            null
        } else {
            val shortYear = if (expiryYear.toString().length == 4) {
                expiryYear.toString().substring(2)
            } else {
                expiryYear.toString()
            }
            val expiryDate = getString(R.string.credit_expiration_base, expiryMonth.toString(), shortYear)

            val initializationHolder = CreateCardInitializationHolder()
            initializationHolder.cardNumber = cardNumber
            initializationHolder.cardHolderName = cardHolderName
            initializationHolder.billingAddressId = billingAddress.addressId ?: ""
            initializationHolder.expiryMonth = expiryMonth
            initializationHolder.expiryYear = expiryYear
            initializationHolder.expiryDate = expiryDate
            initializationHolder.cardBrand = cardType.name
            initializationHolder.isMainCard = true
            initializationHolder.editCard = editCard
            PaymentCardsViewModel.createCreditCard(initializationHolder)
        }
    }

    private fun addCXOCard() {
        val paymentCard = createBaseCreditCard(mCreditCard)
        val pd = CustomProgressDialog.create(context)
        val calendar = Utils.getCurrentTimeInUK()
        paymentCard?.let { card ->
            when {
                card.expiryYear < calendar.get(Calendar.YEAR) || card.expiryYear == calendar.get(Calendar.YEAR) && card.expiryMonth < calendar.get(Calendar.MONTH) -> {
                    displayErrorMessage(mExpiryMonthSpinner, mExpiryDateText, R.string.add_card_error_expired, true)
                }
                else -> {
                    val disposable = getEncryptCreditCardService(context)
                          ?.flatMap { encryptCreditCardService -> encryptCreditCardService.rxEncryptCc(card.card) }
                          ?.doOnNext { encryptedCc ->
                              card.card = encryptedCc.encryptedCc
                              card.integrityCheck = encryptedCc.integrityCheck
                              card.ppEncryptedCC = encryptedCc.encryptedCc
                              card.ppIntegrityCheck = encryptedCc.integrityCheck
                              card.keyId = encryptedCc.keyId
                              card.phase = encryptedCc.phase
                              card.isDefault = false
                              card.encryptedCvv = encryptedCc.encryptedCvv
                              card.ppEncryptedCvv = encryptedCc.encryptedCvv
                          }
                          ?.flatMap {
                              CartManager.instance?.cartId?.let { id ->
                                  CheckoutRepository().addOneTimeUseCard(id, createCXOCardRequest(card))?.toObservable()
                              }
                          }
                          ?.observeOn(AndroidSchedulers.mainThread())
                          ?.subscribe({ checkoutResponse ->
                              try {
                                  pd.dismiss()
                              } catch (ignored: Exception) {
                              }
                              mCXOListener?.onCXOCardAdded(checkoutResponse, checkoutResponse?.data?.purchaseOrder?.paymentDetails?.get(0)?.cardDisplayName)
                              mOnSaveStateChange?.onChange(CODE_STATE_CHANGED)
                          }) { throwable ->
                              try {
                                  pd.dismiss()
                              } catch (ignored: Exception) {
                              }
                              if (throwable is RxFailure) {
                                  AsdaDialogHelper.displayErrorDialog(throwable.asdaResponse, throwable.integer, context, SCREEN_NAME)
                              } else {
                                  try {
                                      val dialog = DialogFactory.createAlertDialog(null, throwable.message, getContext(), SCREEN_NAME)
                                      dialog.show()
                                  } catch (ignore: Throwable) {
                                      Log.w(TAG, ignore)
                                  }
                              }
                          }
                    safeAdd(disposable)
                    pd.show()
                }
            }
        }
    }

    private fun createCXOCardRequest(card: PaymentDetailsResponse.PaymentCards): AddCreditCardRequest {
        val paymentCard = Payment(card.startMonth.toString(), card.startYear.toString(), card.issueNumber ?: "0", card.nameOnCard, card.creditCardType, card.expiryMonth.toString()
              , card.expiryYear.toString(), card.last4Digits, card.ppEncryptedCC, card.encryptedCvv, card.integrityCheck, card.keyId, card.phase)
        val address = mAddressSpinner.selectedItem as AddressBookResponse.Address
        val billingAddress = BillingAddress(address.line1, address.line2, address.postalCode, address.city,
              address.county, address.country)
        return AddCreditCardRequest(CardData(billingAddress, paymentCard))
    }

    private fun addCard(singleUse: Boolean, addCard: Boolean, isCXOMigrationEnabled: Boolean) {
        val context = activity
        val card = createBaseCreditCard(if (mIsAdding) {
            null
        } else {
            mCreditCard
        })

        if (card != null) {
            val calendar = Utils.getCurrentTimeInUK()

            if (card.expiryYear < calendar.get(Calendar.YEAR) || card.expiryYear == calendar.get(Calendar.YEAR) && card.expiryMonth < calendar.get(Calendar.MONTH)) {
                displayErrorMessage(mExpiryMonthSpinner, mExpiryDateText, R.string.add_card_error_expired, true)
            } else {
                SingleProfile.getProfileResponse()?.profileId?.let {
                    val pd = CustomProgressDialog.create(context)
                    pd.setCancelable(false)
                    pd.setMessage(resources.getString(R.string.adding_card))
                    pd.isIndeterminate = true

                    val profileId = SingleProfile.getProfileResponse()?.profileId
                    val creditCard = PaymentDetailsResponse()
                    creditCard.profileId = profileId;
                    creditCard.cards = Array(1) { PaymentDetailsResponse.PaymentCards() }
                    creditCard.cards!![0] = card

                    if (!addCard) {
                        val disposable = paymentCardsViewModel?.updateCreditCards(creditCard)?.observeOn(AndroidSchedulers.mainThread())
                              ?.subscribe({ response ->
                                  try {
                                      pd.dismiss()
                                  } catch (ignored: Exception) {
                                  }

                                  if (response.isSuccess) {
                                      Toast.makeText(context, R.string.card_added, Toast.LENGTH_SHORT).show()
                                      trackCardEvent(false)
                                      pop()
                                      mListener?.onCardAdded(response, singleUse, mCardNumber.rawNumberString)

                                      mOnSaveStateChange?.onChange(CODE_STATE_CHANGED)
                                  } else {
                                      context?.let { DialogHelper.displayErrorDialog(R.string.failed_to_verify_card, it, SCREEN_NAME) }
                                  }
                              }) { throwable ->
                                  try {
                                      pd.dismiss()
                                  } catch (ignored: Exception) {
                                  }

                                  if (throwable is RxFailure) {
                                      AsdaDialogHelper.displayErrorDialog(throwable.asdaResponse, throwable.integer, context, SCREEN_NAME)
                                  } else {
                                      try {
                                          val dialog = DialogFactory.createAlertDialog(null, throwable.message, getContext(), SCREEN_NAME)
                                          dialog.show()
                                      } catch (ignore: Throwable) {
                                          Log.w(TAG, ignore)
                                      }

                                  }
                              }
                        safeAdd(disposable)
                    } else {
                        val disposable = getEncryptCreditCardService(context)
                              ?.flatMap { encryptCreditCardService -> encryptCreditCardService.rxEncryptCc(card.card) }
                              ?.doOnNext { encryptedCc ->
                                  card.card = encryptedCc.encryptedCc
                                  card.integrityCheck = encryptedCc.integrityCheck
                                  card.ppEncryptedCC = encryptedCc.encryptedCc
                                  card.ppIntegrityCheck = encryptedCc.integrityCheck
                                  card.keyId = encryptedCc.keyId
                                  card.phase = encryptedCc.phase
                                  card.isDefault = !singleUse
                                  card.encryptedCvv = encryptedCc.encryptedCvv
                                  card.ppEncryptedCvv = encryptedCc.encryptedCvv
                              }
                              ?.flatMap { encryptedCc ->
                                  if (singleUse) {
                                      val cardType = getCardTypeFromString(card.cardBrand)
                                      val pangeaCard = CreditCard.create(cardType, card.card, card.nameOnCard, card.billingAddressId, card.expiryMonth, card.expiryYear, "")
                                      val loginResponse = SingleProfile.getLoginResponse()
                                      val token = SingleProfile.authentication.token
                                      pangeaCard.keyId = encryptedCc.keyId
                                      pangeaCard.ppEncryptedCC = encryptedCc.encryptedCc
                                      pangeaCard.ppEncryptedCvv = encryptedCc.encryptedCvv
                                      pangeaCard.ppIntegrityCheck = encryptedCc.integrityCheck
                                      pangeaCard.phase = encryptedCc.phase
                                      pangeaCard.billingAddressId = card.address?.addressId

                                      SingleProfile.asdaService.pangeaAddCreditCard(pangeaCard, loginResponse, token, singleUse, mPurchaseType, false)
                                  } else {
                                      paymentCardsViewModel?.addCreditCards(creditCard)
                                  }
                              }
                              ?.observeOn(AndroidSchedulers.mainThread())
                              ?.subscribe({ response ->
                                  try {
                                      pd.dismiss()
                                  } catch (ignored: Exception) {
                                  }

                                  val isSuccess: Boolean = if (response is AddCardResponse) {
                                      "0" == response.statusCode;
                                  } else {
                                      response.isSuccess
                                  }
                                  if (isSuccess) {
                                      Toast.makeText(context, R.string.card_added, Toast.LENGTH_SHORT).show()
                                      trackCardEvent(false)
                                      pop()

                                      if (isCXOMigrationEnabled) {
                                           if (response is PaymentDetailsResponse) {
                                               mCXOListener?.onCXOCardAdded(null, response.cards?.get(0)?.cardId)
                                           }
                                      }
                                      mListener?.onCardAdded(response, singleUse, mCardNumber.rawNumberString)

                                      mOnSaveStateChange?.onChange(CODE_STATE_CHANGED)
                                  } else {
                                      BaseAsdaDialogHelper.displayErrorDialog(response, -1, context!!, SCREEN_NAME)
                                  }
                              }) { throwable ->
                                  try {
                                      pd.dismiss()
                                  } catch (ignored: Exception) {
                                  }

                                  if (throwable is RxFailure) {
                                      AsdaDialogHelper.displayErrorDialog(throwable.asdaResponse, throwable.integer, context, SCREEN_NAME)
                                  } else {
                                      try {
                                          val dialog = DialogFactory.createAlertDialog(null, throwable.message, getContext(), SCREEN_NAME)
                                          dialog.show()
                                      } catch (ignore: Throwable) {
                                          Log.w(TAG, ignore)
                                      }

                                  }
                              }
                        safeAdd(disposable)
                        pd.show()
                    }
                }

            }
        }
    }

    private fun trackCardEvent(edit: Boolean) {
        Tracker.get().trackEvent(AsdaEngineeringAnalyticsEvent(Anivia.ADD_EDIT_PAYMENT_CARD_EVENT)
              .putBoolean(Anivia.CARD_EDITED_KEY, edit)
              .putBoolean(Anivia.CARD_SCANNED_KEY, mCardDataScanned))
    }

    private fun displayErrorMessage(toRequestFocus: View?, errorLabel: FloatLabel, @StringRes errorMsgId: Int, requestFocus: Boolean) {
        errorLabel.setError(errorMsgId)
        if (requestFocus) {
            toRequestFocus?.requestFocus()
            toRequestFocus?.let {
                view?.scrollTo(it.left, it.top - 80)
            }
        }
    }

    private fun clearErrorLabels(vararg labels: FloatLabel?) {
        for (label in labels) {
            label?.clearError()
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            startCardIOActivity()
        } else {
            context?.let {
                AlertDialog.Builder(it).setMessage(R.string.no_permission_for_camera)
                      .setPositiveButton(R.string.ok, null).show()
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        if (requestCode == SCAN_REQUEST_CODE && data != null) {
            val scanResult = data.getParcelableExtra<io.card.payment.CreditCard>(CardIOActivity.EXTRA_SCAN_RESULT)
            populateScannedCardFields(scanResult)
        }
    }

    private fun populateScannedCardFields(scanResult: io.card.payment.CreditCard?) {
        if (scanResult != null) {
            if (mIsAdding) {
                mCardNumber.setText(scanResult.cardNumber)
            }

            if (!TextUtils.isEmpty(scanResult.cardholderName)) {
                mCardHolderName.setText(scanResult.cardholderName)
            }

            if (scanResult.isExpiryValid) {
                val currentYear = Utils.getCurrentTimeInUK().get(Calendar.YEAR)
                initExpiryDateSpinners(currentYear, NUMBER_OF_YEARS)
                mExpiryMonthSpinner.setSelection(scanResult.expiryMonth - 1)
                mExpiryYearSpinner.setSelection(scanResult.expiryYear - currentYear)
            } else if (scanResult.expiryMonth != 0 && scanResult.expiryYear != 0) {
                // Expiry date detected but card is expired
                mExpiryDateText.let {
                    displayErrorMessage(mExpiryMonthSpinner, it, R.string.add_card_error_expired, true)
                }
            }

            mCardDataScanned = true
        }
    }

    fun setupAddEditCard(view: View) {
        initView(view)
    }

    fun initView(view: View) {
        setupView(view)
        fillContent()
        combineLatestEvents()
    }

    override fun onStart() {
        super.onStart()
        if (arguments != null && arguments!!.getBoolean(KEY_IS_ADDING)) {
            val pageViewEvent = PageViewEvent(PV_PAYMENT_CARDS, SECTION_ACCOUNT)
            Tracker.get().trackPageView(pageViewEvent)
        } else {
            val pageViewEvent = PageViewEvent(PV_PAYMENT_CARD_EDIT, SECTION_ACCOUNT)
            Tracker.get().trackPageView(pageViewEvent)
        }
    }

    override fun getActionBarTitle(): String? {
        return if (arguments != null && arguments!!.getBoolean(KEY_IS_ADDING)) {
            getString(R.string.account_new_card)
        } else {
            getString(R.string.account_edit_card)
        }
    }

    override fun onPostcodeCheckSuccess(addressSuggestions: Array<PostcodeCheckResponse.AddressSuggestion>?, postcode: String) {
        //TODO Check to add address
        var mainAddress: AddressBookResponse.Address? = null
        mCreditCard?.let {
            val billingAddress = it.address
            billingAddress?.let {
                mainAddress = AddressBookResponse.Address()
                mainAddress?.address = billingAddress.address
                mainAddress?.isDefault = true
            }
        }
        val addressFragment = BaseAddEditAddressFragment.getAddAddressFragment(postcode, true,
              mainAddress, addressSuggestions, null, null,
              null, false,
              true, AddressBookResponse.Type.ShipTo.toString(), false)
        addressFragment.setAddressBookChangedListener(this)
        push(addressFragment)
    }

    override fun onAddressBookChanged(addressBookResponse: AddressBookResponse?, addedAddress: AddressBookResponse.Address?) {
        if (isAdded) {
            UserAccountDetails.getInstance().setAddressBookResponse(addressBookResponse)
            addressBookResponse?.addressBook?.let {
                populateAddresses(it)
            }
            mAddressSpinner.setSelection(0)
        }
    }


    /**
     * Helper method to get the CreditCardType when the server returns masked card number with a String card type.
     *
     * @param cardType
     * @return [CreditCardType]
     */
    private fun getCardTypeFromString(cardType: String?): CreditCardType {
        when (cardType) {
            "AMEX" -> return CreditCardType.AMEX
            "Visa" -> return CreditCardType.VISA
            "MasterCard" -> return CreditCardType.MASTERCARD
            "ASDA Money Credit" -> return CreditCardType.ASDA
            "Visa Electron" -> return CreditCardType.VISAELECTRON
            "Delta" -> return CreditCardType.DELTA
            "Maestro" -> return CreditCardType.MAESTRO
            else -> return CreditCardType.UNKNOWN
        }
    }

    fun isValidCardType(cardNumber: String?): Boolean {
        var cardNumber: String? = cardNumber ?: return false
        cardNumber = cardNumber!!.replace(" ", "")
        val cardType = getCardType(cardNumber)
        if (CreditCardType.TYPE_VISA.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 13 || cardNumber.length == 16 || cardNumber.length == 19
        } else if (CreditCardType.TYPE_MASTER_CARD.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 16
        } else if (CreditCardType.TYPE_ASDA_MONEY_CREDIT.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 16
        } else if (CreditCardType.TYPE_MAESTRO.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length >= 13 && cardNumber.length <= 19 //some 12 card Maestros exist, we are ignoring them
        } else if (CreditCardType.TYPE_DELTA.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 16  //same as VISA
        } else if (CreditCardType.TYPE_VISA_ELECTRON.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 16
        } else if (CreditCardType.TYPE_AMEX.equals(cardType.cardtype!!, ignoreCase = true)) {
            return cardNumber.length == 15
        }
        return false
    }

    /**
     * http://en.wikipedia.org/wiki/Luhn_algorithm
     *
     * @param cardNumber card number to check. May have spaces.
     * @return true if the card number looks valid
     */
    fun isValidCardNumber(cardNumber: String?): Boolean {
        var cardNumberLocal = cardNumber
        if (cardNumber != null) {
            cardNumberLocal = cardNumber.replace(" ", "")
            if (cardNumberLocal.length >= 13) {
                val cc = IntArray(cardNumberLocal.length)
                for (i in 0 until cardNumberLocal.length) {
                    try {
                        cc[i] = Character.digit(cardNumberLocal[i], 10)
                    } catch (nfe: NumberFormatException) {
                        return false
                    }

                }
                var sum = 0
                val length = cc.size
                for (j in 0 until length) {
                    // get digits in reverse order 
                    var digit = cc[length - j - 1]

                    // every 2nd number multiply with 2
                    if (j % 2 == 1) {
                        digit *= 2
                    }
                    sum += if (digit > 9) digit - 9 else digit
                }
                return sum % 10 == 0
            }
        }
        return false
    }

    private fun createMonthSpinnerAdapter(context: Context?): ArrayAdapter<String> {
        val months = resources.getStringArray(R.array.months)
        val adapter = ArrayAdapter<String>(context!!, android.R.layout.simple_spinner_item, months)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        return adapter
    }

    private fun matchesAnyPrefix(text: String, vararg prefixes: String): Boolean {

        for (prefix in prefixes) {
            if (text.startsWith(prefix)) {
                return true
            }
        }
        return false
    }

    fun getCardType(cardNumber: String?): CreditCardType {
        /*
     * References
     * ==========
     * https://en.wikipedia.org/wiki/Bank_card_number
     * http://en.wikipedia.org/wiki/List_of_Issuer_Identification_Numbers
     *
     * The Bank Identification Number (BIN) are the first six digits of a
     * card number and they allow to know which institution (Visa, etc) and
     * bank have issued the card
     *
     * Regex matching ref: https://confluence.walmart.com/pages/viewpage.action?spaceKey=ASDA&title=Card+Type+Validation
     */
        var pattern: Pattern?
        if (cardNumber == null) {
            return CreditCardType.UNKNOWN
        }

        // More restrictive rules shall be placed first
        val asdaPrefixes = arrayOf("529932", "541647", "557712", "529507")
        pattern = Pattern.compile("(^52993258)")
        var matcher = pattern!!.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *asdaPrefixes) || matcher.find()) {
            return CreditCardType.ASDA
        }

        val visaElectronPrefixes = arrayOf("417500", "4026", "4405", "4508", "4844", "4913", "4917")
        if (matchesAnyPrefix(cardNumber, *visaElectronPrefixes)) {
            return CreditCardType.VISAELECTRON
        }

        val maestroPrefixes = arrayOf("503396", "633300", "5018", "5020", "5038", "5893", "6304", "67")
        pattern = Pattern.compile("^(?:5[0678]\\d\\d|6304|6390|67\\d\\d)\\d{8,15}$")
        val p1 = Pattern.compile("^(5[06-8]|6\\d)\\d{14}(\\d{2,3})?$")
        matcher = pattern!!.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *maestroPrefixes) || matcher.find() ||
              p1.matcher(cardNumber).find()) {
            return CreditCardType.MAESTRO
        }

        val deltaPrefixes = arrayOf("453978", "446200")
        if (matchesAnyPrefix(cardNumber, *deltaPrefixes)) {
            return CreditCardType.DELTA
        }

        val masterCardPrefixes = arrayOf("51", "52", "53", "54", "55")
        pattern = Pattern.compile("(^(5[0-5]|2[2-6]|27[0-1]|2720))")
        matcher = pattern!!.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *masterCardPrefixes) || matcher.find()) {
            return CreditCardType.MASTERCARD
        }

        val amexPrefixes = arrayOf("34", "37")
        if (matchesAnyPrefix(cardNumber, *amexPrefixes)) {
            return CreditCardType.AMEX
        }

        val visaPrefixes = arrayOf("4")
        return if (matchesAnyPrefix(cardNumber, *visaPrefixes)) {
            CreditCardType.VISA
        } else CreditCardType.UNKNOWN
    }

    companion object {
        const val TAG = "SPAddEditPaymentCard"

        const val PURCHASE_TYPE_DP = "dppurchase"
        const val PURCHASE_TYPE_DP_RENEW = "dprenew"
        const val PURCHASE_TYPE_BASKET = "basket"

        const val CODE_STATE_CHANGED = 1
        const val CODE_NO_CHANGE = -1

        const val KEY_CREDIT_CARD = "parcelable"
        const val KEY_IS_ADDING = "adding"
        const val KEY_ENABLE_SINGLE_USE = "single_use"
        const val KEY_PURCHASE_TYPE = "purchase_type"
        const val KEY_EXPIRY_YEAR = "expiry_year"
        const val KEY_EXPIRY_MONTH = "expiry_month"

        fun newInstance(paymentCard: PaymentDetailsResponse.PaymentCards?, isAdding: Boolean, enableSingleUse: Boolean, purchaseType: String?): SPAddEditPaymentCardFragment {
            val fragment = SPAddEditPaymentCardFragment()
            val args = Bundle()
            args.putParcelable(KEY_CREDIT_CARD, paymentCard)
            args.putBoolean(KEY_IS_ADDING, isAdding)
            args.putBoolean(KEY_ENABLE_SINGLE_USE, enableSingleUse)
            args.putString(KEY_PURCHASE_TYPE, purchaseType)
            args.putInt(KEY_EXPIRY_MONTH, -1)
            args.putInt(KEY_EXPIRY_YEAR, -1)
            fragment.arguments = args

            return fragment
        }

    }
}
