package com.asda.android.app.account.singleprofile.view

import com.asda.android.base.interfaces.ICartManager
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.lifecycle.ViewModelProviders
import com.asda.android.analytics.constants.Anivia.PV_ADDRESS_AND_PHONE_NUMBERS
import com.asda.android.analytics.constants.Anivia.SECTION_ACCOUNT
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.address.view.BaseAddEditAddressFragment
import com.asda.android.app.bookslotv3.service.BookSlotRepository
import com.asda.android.app.main.RemoteConfigController
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.fragments.DisplayListFragment
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.PostcodeCheckResponse
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.R
import com.asda.android.singleprofile.features.account.PostcodeCheckListener
import com.asda.android.singleprofile.view.AddressAdapter
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.base.get
import com.asda.android.singleprofile.model.*
import com.asda.android.singleprofile.view.openWarning
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.AddressViewModelFactory
import com.asda.android.utils.Utils
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.DisposableObserver
import io.reactivex.observers.ResourceObserver


fun instance(): SPAddressesFragment {
    return SPAddressesFragment()
}

private const val ASDA_ADDRESS_IN_USE = "ASDA_ADDRESS_IN_USE"
private const val ADDRESS_USED_IN_PREFERENCE = "ADDRESS_USED_IN_PREFERENCE"

class SPAddressesFragment : DisplayListFragment(), AddressAdapter.AddressAdapterClickListener, PostcodeCheckListener, BaseAddEditAddressFragment.AddressBookChangedListener {

    private val SCREEN_NAME = "Address Book"
    private var addressViewModel: AddressViewModel? = null
    private var addressListAdapter: AddressAdapter? = null
    private val DIALOG_CHECK_POSTCODE = 1
    private var mMainAddress: AddressBookResponse.Address? = null
    private var bookSlotRepository: BookSlotRepository? = null

    private var cartListener: ICartManager.CartListener? = object : ICartManager.CartListener() {
        override fun onChanged() {
           // KillSwitchController.validateStore(activity)t log
            RemoteConfigController.validateStore(activity)
        }
    }

    companion object {
        const val TAG = "SPAddressesFragment"
    }

    override fun onSetMainAddressClicked(address: AddressBookResponse.Address?, position: Int) {
        val context = activity
        val pd = CustomProgressDialog.create(context)
        pd.setCancelable(false)
        pd.setMessage(resources.getString(com.asda.android.R.string.setting_main_address))
        pd.isIndeterminate = true

        SingleProfile.authentication.profileResponse?.let {
            val addressBookBody = AddressBookResponse()
            address?.isDefault = true
            addressBookBody.addresses = arrayOf(address)
            addressBookBody.addresses
            addressBookBody.profileId = it.profileId
            val disposable = addressViewModel?.addOrUpdateAddress(addressBookBody, false)
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribeWith(object : ResourceObserver<AddressUiState>() {
                        override fun onComplete() {

                        }

                        override fun onNext(state: AddressUiState) {
                            when (state) {
                                AddressLoading -> {
                                    pd.show()
                                }
                                is AddressLoadedState -> {
                                    try {
                                        pd.dismiss()
                                    } catch (e: Exception) {

                                    }
                                    if (state.addressResponse.addresses == null || state.addressResponse.addresses?.size == 0) {
                                        BaseAsdaDialogHelper.displayErrorDialog(state.addressResponse, state.addressResponse.statusCodeNumber, context, SCREEN_NAME)
                                    } else {
                                        state.addressResponse.let {
                                            state.addressResponse.addresses?.let {
                                                mRecyclerView?.visibility = View.VISIBLE
                                                addressListAdapter?.setAddressResponse(it)
                                                mRecyclerView?.adapter = addressListAdapter
                                                mRecyclerView?.refreshDrawableState()
                                                //Updating the latest addresses in Single Profile addresses.
                                                SingleProfile.getProfileResponse()?.addresses = it

                                            }
                                        } ?: run {
                                            if (Utils.isDebug() || Utils.isDevBuild()) {
                                                Toast.makeText(context, "Network Timedout try again!!", Toast.LENGTH_SHORT).show()
                                            } else {

                                            }
                                        }
                                    }
                                }
                                is AddressErrorState -> {
                                    try {
                                        pd.dismiss()
                                    } catch (e: Exception) {

                                    }
                                    BaseAsdaDialogHelper.displayErrorDialog(null, -1, context, SCREEN_NAME);

                                    if (Utils.isDebug() || Utils.isDevBuild()) {
                                        Log.e(TAG, "addressState -> $state")
                                    } else {

                                    }
                                }
                                IsBillingAddress -> {
                                    //Will be used only for delete Address
                                }
                            }
                        }

                        override fun onError(e: Throwable) {

                        }

                    })
            safeAdd(disposable)
        }
        address?.let {
            bookSlotRepository = BookSlotRepository()
            val disposable = bookSlotRepository?.makeServiceEligibilityCall(it.postCode, false)
                  ?.subscribe({
                      Utils.setExpressFlag(it)
                  }, {
                      Log.e(TAG, "ServiceElegibility -> $it")
                  })
            safeAdd(disposable)
        }
    }

    override fun onDeleteAddressClicked(address: AddressBookResponse.Address) {
        val context = activity
        val pd = CustomProgressDialog.create(context)
        pd.setCancelable(false)
        pd.setMessage(resources.getString(com.asda.android.R.string.delete_address_progress))
        pd.isIndeterminate = true

        SingleProfile.authentication.profileResponse?.profileId?.let {
            val profileId = it
            address.addressId?.let {
                val addressID = it
                address.type?.let {
                    val disposable = addressViewModel?.deleteAddress(addressID, profileId, it)
                            ?.observeOn(AndroidSchedulers.mainThread())
                            ?.subscribeWith(object : ResourceObserver<AddressUiState>() {
                                override fun onComplete() {

                                }

                                override fun onNext(state: AddressUiState) {
                                    when (state) {
                                        AddressLoading -> {
                                            pd.show()
                                        }
                                        is AddressLoadedState -> {
                                            try {
                                                pd.dismiss()
                                            } catch (e: Exception) {

                                            }
                                            if (state.addressResponse.addresses == null || state.addressResponse.addresses?.size == 0) {
                                                if (ASDA_ADDRESS_IN_USE.equals(state.addressResponse.errorCode)) {
                                                    openLastAddressWarning()
                                                } else if (ADDRESS_USED_IN_PREFERENCE == state.addressResponse.errorCode) {
                                                    openWarning(activity, getString(R.string.delete_address_permission_center_warning_title), getString(R.string.delete_address_permission_center_warning_message))
                                                } else {
                                                    if (state.addressResponse.isSuccess) {
                                                        mRecyclerView?.visibility = View.VISIBLE
                                                        addressListAdapter?.setAddressResponse(null)
                                                        mRecyclerView?.adapter = addressListAdapter
                                                        mRecyclerView?.refreshDrawableState()
                                                    } else {
                                                        BaseAsdaDialogHelper.displayErrorDialog(state.addressResponse, state.addressResponse.statusCodeNumber, context, SCREEN_NAME)
                                                    }
                                                }
                                            } else {
                                                state.addressResponse.let {
                                                    state.addressResponse.addresses?.let {
                                                        mRecyclerView?.visibility = View.VISIBLE
                                                        addressListAdapter?.setAddressResponse(it)
                                                        mRecyclerView?.adapter = addressListAdapter
                                                        mRecyclerView?.refreshDrawableState()

                                                    }
                                                } ?: run {
                                                    if (Utils.isDebug() || Utils.isDevBuild()) {
                                                        Log.e(TAG, "addressResponse is null")
                                                    }
                                                }
                                            }
                                        }
                                        is AddressErrorState -> {
                                            try {
                                                pd.dismiss()
                                            } catch (e: Exception) {

                                            }
                                            BaseAsdaDialogHelper.displayErrorDialog(null, -1, context, SCREEN_NAME);
                                            if (Utils.isDebug() || Utils.isDevBuild()) {
                                                Log.e(TAG, "addressState -> $state")
                                            } else {

                                            }
                                        }
                                        IsBillingAddress -> {
                                            try {
                                                pd.dismiss()
                                            } catch (e: Exception) {

                                            }
                                            openLastAddressWarning()
                                        }
                                    }
                                }

                                override fun onError(e: Throwable) {

                                }

                            })
                    safeAdd(disposable)
                }
            }
        }
    }

    override fun onEditAddressClicked(address: AddressBookResponse.Address) {
        address.postalCode?.let {
            //todo: vmk type to be corrected from caller flow
            val editAddressFragment = BaseAddEditAddressFragment.getEditAddressFragment(address, it, AddressBookResponse.Type.ShipTo.toString())
            editAddressFragment.setAddressBookChangedListener(this)
            push(editAddressFragment)
        }
    }

    override fun onAddressBookChanged(addressBookResponse: AddressBookResponse?, addedAddress: AddressBookResponse.Address?) {
        addressListAdapter.let {
            addressBookResponse?.let {
                addressListAdapter?.setAddressResponse(it.addressBook)
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val pageViewEvent = PageViewEvent(PV_ADDRESS_AND_PHONE_NUMBERS, SECTION_ACCOUNT)
        Tracker.get().trackPageView(pageViewEvent)
        addressViewModel = ViewModelProviders.of(this, AddressViewModelFactory(get(), SingleProfile.asdaService))
                .get(AddressViewModel::class.java)
        context?.let {
            addressListAdapter = AddressAdapter(it, Array(0, { AddressBookResponse.Address() }), this)
        }
        cartListener?.let { CartManager.get(context).addCartListener(it) }
    }

    override fun fillContent() {
        SingleProfile.authentication.profileResponse?.profileId?.let {
            val disposable = addressViewModel?.getDisplayableAddressState(it)
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribeWith(object : DisposableObserver<AddressUiState>() {
                        override fun onNext(state: AddressUiState) {
                            Log.e(TAG, "addressState -> $state")

                            when (state) {
                                AddressLoading -> mLoadingView?.visibility = View.VISIBLE

                                is AddressLoadedState -> {
                                    mLoadingView?.visibility = View.GONE
                                    if (state.addressResponse.addresses == null || state.addressResponse.addresses?.size == 0) {
                                        if (!state.addressResponse.isSuccess) {
                                            BaseAsdaDialogHelper.displayErrorDialog(state.addressResponse, state.addressResponse.statusCodeNumber, context, SCREEN_NAME)
                                        } else {
                                        }
                                    } else {
                                        state.addressResponse.let {
                                            state.addressResponse.addresses?.let {
                                                mRecyclerView?.visibility = View.VISIBLE
                                                addressListAdapter?.setAddressResponse(it)
                                                mRecyclerView?.adapter = addressListAdapter
                                                mRecyclerView?.refreshDrawableState()
                                            }
                                        } ?: run {
                                            if (Utils.isDevBuild() || Utils.isDebug()) {
                                                Toast.makeText(context, "Network Timedout try again!!", Toast.LENGTH_SHORT).show()
                                            }
                                        }
                                    }
                                }

                                is AddressErrorState -> {
                                    mNoContentTextView?.setText(R.string.oops_error)
                                    mNoContentTextView?.visibility = View.VISIBLE
                                    mLoadingView?.visibility = View.GONE
                                    BaseAsdaDialogHelper.displayErrorDialog(null, -1, context, SCREEN_NAME);
                                    if (Utils.isDevBuild() || Utils.isDebug()) {
                                        Log.e(TAG, "addressState -> $state")
                                    } else {

                                    }
                                }
                                IsBillingAddress -> {
                                    //Will be used only for delete Address
                                }
                            }.let {}
                        }

                        override fun onError(e: Throwable) {
                            mNoContentTextView?.setText(R.string.oops_error)
                            mNoContentTextView?.visibility = View.VISIBLE
                            mLoadingView?.visibility = View.GONE
                            BaseAsdaDialogHelper.displayErrorDialog(null, -1, context, SCREEN_NAME);
                            if (Utils.isDevBuild() || Utils.isDebug()) {
                                Log.e(TAG, "addressState -> $e")
                            }
                        }

                        override fun onComplete() {

                        }
                    })
            safeAdd(disposable)
        }
    }

    override fun processActionBarClick() {
        showDialog(DIALOG_CHECK_POSTCODE)
    }

    override fun showDialog(type: Int) {
        val ft = activity?.supportFragmentManager?.beginTransaction()
        val previousDialog: androidx.fragment.app.Fragment?
        ft?.addToBackStack(null)

        when (type) {
            DIALOG_CHECK_POSTCODE -> {
                val postcodeDialogFrag = SPCheckPostCodeDialogFragment.newInstance()
                postcodeDialogFrag.setPostcodeCheckedListener(this, SCREEN_NAME)
                previousDialog = activity?.supportFragmentManager?.findFragmentByTag("DIALOG_CHECK_POSTCODE")
                if (previousDialog != null) {
                    ft?.remove(previousDialog)
                }
                ft?.let { postcodeDialogFrag.show(it, "DIALOG_CHECK_POSTCODE") }
            }
        }
    }

    private fun openLastAddressWarning() {
        openWarning(activity)
    }

    override fun onDestroy() {
        super.onDestroy()
        CartManager.get(context).removeCartListener(cartListener)
    }

    override fun getInternalTag(): String? = TAG

    override fun getActionBarTitle(): String? = getString(R.string.sp_account_setting_addresses_title)

    override fun onPostcodeCheckSuccess(addressSuggestions: Array<out PostcodeCheckResponse.AddressSuggestion>?, postcode: String?) {
        postcode?.let {
            //todo: vmk Need to pass type from caller
            val addressFragment: BaseAddEditAddressFragment = BaseAddEditAddressFragment.getAddAddressFragment(it, true, mMainAddress, addressSuggestions, null, null, null, true,
                    false,
                    AddressBookResponse.Type.ShipTo.toString(),
                  false)
            addressFragment.setAddressBookChangedListener(this)
            push(addressFragment)
        } ?: run {
            BaseAsdaDialogHelper.displayErrorDialog(null, -1, context, SCREEN_NAME);
            if (Utils.isDevBuild() || Utils.isDebug()) {
                Log.e(TAG, "postcode in null")
            }
        }

    }
}
