package com.asda.android.app.account.singleprofile.view

import android.os.Bundle
import android.text.TextUtils
import android.widget.EditText
import android.widget.Toast
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.app.account.address.view.BaseAddEditAddressFragment
import com.asda.android.app.account.view.CheckPostcodeDialogFragment
import com.asda.android.app.home.RegisterHelper
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.singleprofile.BuildConfig
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.base.get
import com.asda.android.singleprofile.model.AddressSuggestUiState
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.AddressViewModelFactory
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable


class SPCheckPostCodeDialogFragment : CheckPostcodeDialogFragment() {

    private var addressViewModel: AddressViewModel? = null
    private var disposable: Disposable? = null

    companion object {
        fun newInstance() = SPCheckPostCodeDialogFragment()
        val TAG = "PCheckPostCodeDialog"
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        addressViewModel = ViewModelProviders.of(this, AddressViewModelFactory(get(), SingleProfile.asdaService))
                .get(AddressViewModel::class.java)
    }

    override fun loadPostcodeAddresses(postcodeValue: EditText?) {
        val postCode = postcodeValue?.getEditableText().toString().trim({ it <= ' ' })
        val context = activity

        if (!TextUtils.isEmpty(postCode)) {
            val dialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR,
                    getString(R.string.add_address_progress_postcode_check), context)

            addressViewModel?.getSuggestAddressState(postCode)
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribe(object : Observer<AddressSuggestUiState> {
                        override fun onComplete() {
                            disposable?.let {
                                if (it.isDisposed) {
                                    it.dispose()
                                }
                            }

                        }

                        override fun onSubscribe(d: Disposable) {
                            disposable = d
                        }

                        override fun onNext(uiState: AddressSuggestUiState) {
                            if (uiState.isLoading) {
                                dialog?.show()
                            } else {
                                try {
                                    dialog?.dismiss()
                                } catch (ignored: Throwable) {
                                }

                                uiState.addressResponse?.let {
                                    if (RegisterHelper.dontDeliverSP(it)) {
                                        BaseAddEditAddressFragment.setPostcodeNotFoundMessage(context?.getString(R.string.we_do_not_currently_offer_hd, postCode))
                                    }
                                    it.addresses?.size?.let {
                                        if (it > 0) {
                                            mPostcodeCheckedListener?.onPostcodeCheckSuccess(uiState.addressResponse?.addresses, postCode)
                                        } else {
                                            mPostcodeCheckedListener?.onPostcodeCheckSuccess(null, postCode)
                                        }
                                        BaseAddEditAddressFragment.setPostcodeNotFoundMessage(null)
                                    } ?: run {
                                        mPostcodeCheckedListener?.onPostcodeCheckSuccess(null, postCode)
                                    }
                                } ?: run {
                                    if (BuildConfig.DEBUG) {
                                        Toast.makeText(getContext(), "Network Timedout try again!!", Toast.LENGTH_LONG).show()
                                    }
                                }
                            }
                        }

                        override fun onError(e: Throwable) {
                            try {
                                dialog?.dismiss()
                            } catch (ignored: Exception) {
                            }
                            AsdaDialogHelper.displayErrorDialog(null, -1, context, "Add Address : Enter Postcode")
                        }

                    })
        }
    }
}
