package com.asda.android.app.account.view

import android.content.Context
import android.content.DialogInterface
import android.graphics.Typeface
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.*
import android.view.accessibility.AccessibilityEvent
import android.widget.TextView
import androidx.core.content.res.ResourcesCompat
import androidx.core.view.AccessibilityDelegateCompat
import androidx.core.view.ViewCompat
import com.asda.android.BuildConfig
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.restapi.feedback.FeedbackAPI
import com.asda.android.restapi.feedback.model.FeedbackFormResponse
import com.asda.android.restapi.constants.ExternalUrls
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import com.jaredrummler.android.device.DeviceName
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.feedback_ratingbar.view.*
import java.util.*
import kotlin.collections.HashMap


class FeedbackFragment : FeaturedFragment() {
    private var sendMenu: MenuItem? = null
    private lateinit var rootView: View
    private var selectedRatingPosition: Int = -1
    private val colors = IntArray(10)
    private val textColor = IntArray(10)
    private val COLOR_KEY = R.id.rating_bar_color_key
    private val TEXT_COLOR_KEY = R.id.rating_bar_text_color_key
    private val ratingBars = ArrayList<TextView>()
    private var createdDate: String? = null
    private var createdId: String? = null
    private var profileId: String? = null
    private var payload: FeedbackFormResponse.Payload? = FeedbackFormResponse.Payload()
    private var feedback: FeedbackFormResponse.Feedback? = FeedbackFormResponse.Feedback()
    private var feedbackRequestData: FeedbackFormResponse? = FeedbackFormResponse()

    private val ANONYMOUS = "anonymous"
    private var entityId:String = ANONYMOUS

    private fun initColors() {
        for (i in 0..1) {
            colors[i] = ResourcesCompat.getColor(resources, R.color.red_30, null)
            textColor[i] = ResourcesCompat.getColor(resources, R.color.white, null)
        }

        for (i in 2..7) {
            colors[i] = ResourcesCompat.getColor(resources, R.color.warning_color, null)
            textColor[i] = ResourcesCompat.getColor(resources, R.color.black, null)
        }

        for (i in 8..9) {
            colors[i] = ResourcesCompat.getColor(resources, R.color.refund_green, null)
            textColor[i] = ResourcesCompat.getColor(resources, R.color.white, null)
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        rootView = inflater.inflate(R.layout.feedback_ratingbar, container, false)
        setHasOptionsMenu(true)
        initColors()
        initEntityAndProfileIds()

        if (arguments?.get(SURVEY_TYPE) != "opt-in") {
            rootView.tap_rate.setText(R.string.rate_your_experience)
        }
        rootView.rating_privacy_link.let {
            it.text = Utils.fromHtml(getString(R.string.feedback_privacy_link))
            it.setOnClickListener(PrivateLinkOnClickListner())
        }

        initRatingBar()

        selectedRatingPosition = arguments?.getInt(PRE_RATING) ?: -1
        if (isValidRatingPosition(selectedRatingPosition)) {
            var preSelectedRating = ratingBars[selectedRatingPosition]
            preSelectedRating.setBackgroundColor(preSelectedRating.getTag(COLOR_KEY) as Int)
        }

        initFeedback()
        return rootView
    }

    private fun initEntityAndProfileIds() {
        profileId = Authentication.getInstance().userViewResponse?.profileId
        if (profileId == null) {
            profileId = ANONYMOUS
        }
        entityId = profileId + "__" + UUID.randomUUID().toString()
    }

    // This makes a network call to fetch feedback form values (the result will be used in subsequent post).
    // On successful fetch, it will post the rating if valid.
    private fun initFeedback() {
        val networkAPI = ServiceGenerator.createService(getBaseUrl(context), "", FeedbackAPI::class.java)
        if (networkAPI == null) {
            return
        }

        val getFeedbackObservable = networkAPI.fetchFeedbackForm(arguments?.getString(ENTITY_TYPE), entityId)
        getFeedbackObservable
                .flatMap { feedbackFormResponse ->
                    // Save feedbackFormResponse
                    createdDate = feedbackFormResponse.payload?.createdDate
                    createdId = feedbackFormResponse.payload?.createdId
                    payload = feedbackFormResponse.payload
                    feedbackRequestData = feedbackFormResponse

                    if (isValidRatingPosition(selectedRatingPosition)) {
                        return@flatMap getPostFeedbackObserver(false)
                    } else {
                        return@flatMap Observable.just(null)
                    }
                }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe( {feedbackPayload ->
                        if (feedbackPayload != null) {
                            Log.i(TAG, "Posted the rating")
                        } else {
                            Log.i(TAG, "Posted the rating")
                        }
                    }, { error->
                        error.printStackTrace()
                    }
                )
    }

    private fun initRatingBar() {
        ratingBars.clear()
        ratingBars.add(rootView.rating_bar_one)
        ratingBars.add(rootView.rating_bar_two)
        ratingBars.add(rootView.rating_bar_three)
        ratingBars.add(rootView.rating_bar_four)
        ratingBars.add(rootView.rating_bar_five)
        ratingBars.add(rootView.rating_bar_six)
        ratingBars.add(rootView.rating_bar_seven)
        ratingBars.add(rootView.rating_bar_eight)
        ratingBars.add(rootView.rating_bar_nine)
        ratingBars.add(rootView.rating_bar_ten)
        for (i in 0..ratingBars.size - 1) {
            ratingBars[i].setTag(COLOR_KEY, colors[i])
            ratingBars[i].setTag(TEXT_COLOR_KEY, textColor[i])
            ratingBars[i].tag = i
            ViewCompat.setAccessibilityDelegate(ratingBars[i], object : AccessibilityDelegateCompat() {
                override fun onPopulateAccessibilityEvent(host: View, event: AccessibilityEvent) {
                    super.onPopulateAccessibilityEvent(host, event)
                    if (event.eventType == AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED) {
                        if (isValidRatingPosition(selectedRatingPosition)) {
                                ratingBars[i].announceForAccessibility(getString(R.string.a11y_feedback_change_rating, (selectedRatingPosition+1).toString()))
                        }
                        else {
                            ratingBars[i].announceForAccessibility(getString(R.string.a11y_feedback_focus_rating))
                        }
                    }
                }
            })
            ratingBars[i].setOnClickListener(ratingClickListener)
        }
    }

    private inner class PrivateLinkOnClickListner : View.OnClickListener {
        override fun onClick(p0: View?) {
            ViewUtils.openWebPage(ExternalUrls.FEEDBACK_PRIVACY_LINK, activity, SCREEN_NAME)
        }
    }

    private fun isValidRatingPosition(ratingPos: Int): Boolean {
        return (ratingPos > -1 && ratingPos < 10)
    }

    val ratingClickListener = View.OnClickListener { view ->
        // Unselect the previous number (if there was selection).
        var prevSelectedView: TextView?
        if (isValidRatingPosition(selectedRatingPosition)) {
            prevSelectedView = ratingBars[selectedRatingPosition]
            prevSelectedView.background = ResourcesCompat.getDrawable(resources, R.drawable.rating_view_black_border, null)
            prevSelectedView.setTypeface(null,Typeface.NORMAL)
            prevSelectedView.setTextColor(ResourcesCompat.getColor(resources, R.color.black, null))
        }

        // Now update the selectedRatingPosition and select the newly tapped number
        selectedRatingPosition = -1
        if (view.tag is Int) {
            selectedRatingPosition = view.tag as Int //bump index 0 as rating 1 star
        }
        if (isValidRatingPosition(selectedRatingPosition)) {
            view.setBackgroundColor(view.getTag(COLOR_KEY) as Int)
            view.announceForAccessibility(getString(R.string.a11y_feedback_select))
            prevSelectedView = ratingBars[selectedRatingPosition]
            prevSelectedView.setTypeface(null, Typeface.BOLD)
            prevSelectedView.setTextColor(view.getTag(TEXT_COLOR_KEY) as Int)

            showSendButton(true)
        } else {
            showSendButton(false)
        }

        postFeedback(false)

    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return resources.getString(R.string.feed_back)
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        menu.clear()
        inflater.inflate(R.menu.feedback_send_button, menu)
        sendMenu = menu.findItem(R.id.action_send)
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        if (isValidRatingPosition(selectedRatingPosition)) {
            showSendButton(true)
        } else {
            showSendButton(false)
        }
    }

    private fun showSendButton(show: Boolean) = sendMenu?.setVisible(show)

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.action_send -> {
                if (!isValidRatingPosition(selectedRatingPosition) && rootView.hint_review.text.isEmpty()) {
                    DialogFactory.createAlertDialog(getString(R.string.error), getString(R.string.select_rating_or_write_Review), context, null).show()
                    return false
                }
                postFeedback(true)
                return true
            }
        }
        return false
    }

    override fun onBackPressed(): Boolean {
        return if (hasModifiedForm()) {

            val msg = getString(R.string.discard_review_dialog)
            val negativeText = getString(R.string.cancel)
            val positiveText = getString(R.string.discard)

            val clickListener = DialogInterface.OnClickListener { _, _ ->
                pop()
            }

            com.asda.android.base.core.view.DialogFactory.createAlertDialog(msg, positiveText, clickListener, negativeText, activity, SCREEN_NAME).show()

            return true
        } else {
            super.onBackPressed()
        }
    }

    private fun hasModifiedForm(): Boolean {
        if (!TextUtils.isEmpty(rootView.hint_review.text.toString())) {
            return true
        }
        return false
    }

    private fun getRequiredParamMap(surveryCompleted:Boolean): HashMap<String?, String?> {
        var requiredParamMap = HashMap<String?, String?>()
        requiredParamMap.put("submit", surveryCompleted.toString())
        requiredParamMap.put("surveyType", arguments?.get(SURVEY_TYPE) as String?)
        requiredParamMap.put("platform", "android")
        requiredParamMap.put("deviceType", if (RestUtils.isTablet(context)) "tablet" else "phone")
        requiredParamMap.put("deviceName", DeviceName.getDeviceName())
        requiredParamMap.put("appVersion", BuildConfig.VERSION_NAME)
        requiredParamMap.put("osVersion", android.os.Build.VERSION.RELEASE)
        requiredParamMap.put("customerId", profileId)
        requiredParamMap.put("sessionId", Tracker.get().sessionId)
        return requiredParamMap
    }

    private fun postFeedback(submit: Boolean) {
        getPostFeedbackObserver(submit)?.let {
            it.subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({ _ ->
                        if (activity != null && submit) {
                            DialogFactory.createAlertDialog(getString(R.string.submitted), getString(R.string.thanks_for_feedback), context, DialogInterface.OnClickListener { _, _ ->
                                pop()
                            }).show()
                        }
                    }) { error ->
                    if (Utils.isDevBuild()) {
                        Log.e(TAG, "Failed to post feedback, error = \"" + error.message + "\"")
                    }
                    if (activity != null && submit) {
                        DialogFactory.createAlertDialog(getString(R.string.failed_to_submit), getString(R.string.try_again_later), context, DialogInterface.OnClickListener { _, i ->
                            dismissDialog(i)
                        }).show()
                    }
                }
        }
    }

    private fun getPostFeedbackObserver(submit: Boolean): Observable<FeedbackFormResponse.Payload>? {
        var observable:Observable<FeedbackFormResponse.Payload>? = null

        if (TextUtils.isEmpty(rootView.hint_review.text) && !isValidRatingPosition(selectedRatingPosition)) {
            if (Utils.isDevBuild()) {
                Log.e(TAG, "Both text is empty and and rating is not selected, so not posting the review.")
            }
            return null
        }

        var requiredParamMap: HashMap<String?, String?> = getRequiredParamMap(submit)
        if (arguments?.get(SURVEY_TYPE) as String? == SURVEY_TYPE_OPT_INT_KEY) {
            payload?.feedback?.text_mobile_optIn_survey_comments = rootView.hint_review.text.toString()
        } else {
            payload?.feedback?.text_mobile_order_confirmation_survey_comments = rootView.hint_review.text.toString()
            requiredParamMap.put("orderId", arguments?.get(ORDER_ID) as String?)
        }

        if (isValidRatingPosition(selectedRatingPosition)) {
            var rating: Int = selectedRatingPosition + 1
            payload?.feedback?.data_rating = rating.toString()
        }

        //submit rating bar and edit text reviews
        //TODO - Create instance for CustomerFeedback
        val networkAPI = ServiceGenerator.createService(getBaseUrl(context), "", FeedbackAPI::class.java)
        networkAPI?.let {
            observable = it.postFeedbackForm(arguments?.getString(ENTITY_TYPE), entityId, feedbackRequestData, requiredParamMap)
        }
        return observable
    }

    private fun getBaseUrl(context: Context?): String {
        return SharedPreferencesUtil.getAsdaFeedbackUrl(context)
    }

    override fun getScreenName(): String {
        return "Feedback Screen"
    }

    companion object {
        const val TAG = "FeedbackFragment"
        const val SURVEY_TYPE = "surveyType"
        const val ENTITY_TYPE = "entityType"
        const val PRE_RATING = "preRating"
        const val SCREEN_NAME = "feed back screen"
        const val ORDER_ID = "orderId"
        const val ENTITY_TYPE_OPT_IN_KEY = "asda_uk_grocery_mobile_optIn"
        const val SURVEY_TYPE_OPT_INT_KEY = "opt-in"

        fun newInstance(surveyType: String?, entityType: String?, preRating: Int, orderId: String?): FeedbackFragment {
            val args = Bundle()
            args.putString(SURVEY_TYPE, surveyType)
            args.putString(ENTITY_TYPE, entityType)
            args.putInt(PRE_RATING, preRating)
            args.putString(ORDER_ID, orderId)
            val fragment = FeedbackFragment()
            fragment.arguments = args
            return fragment
        }
    }
}
