package com.asda.android.app.account.viewmodel

import android.app.Application
import android.content.Context
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.SECTION_ACCOUNT
import com.asda.android.analytics.ITracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.datasource.GetFragmentTitleDataSource
import com.asda.android.app.account.model.FragmentInfoParserModel
import com.asda.android.app.account.parser.AccountFragmentTagParser
import com.asda.android.app.account.parser.FragmentRemoveParser
import com.asda.android.app.account.parser.OpenUrlParser
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.features.login.validator.PhoneNumberLoginFeatureFlagValidator
import com.asda.android.singleprofile.features.permissioncenter.validator.PermissionCenterFeatureFlagValidator
import com.asda.android.base.core.web.datasource.WebEventCallback
import com.asda.android.base.core.web.model.WebViewEventModel

class BaseAccountViewModel(application: Application) : AndroidViewModel(application) {
    private val accountFragmentTagParser = AccountFragmentTagParser()
    private val fragmentRemoveParser = FragmentRemoveParser()
    private val titleDataSource = GetFragmentTitleDataSource()
    val openUrlParser = OpenUrlParser()
    val webEventLiveData: LiveData<BaseStateResponse<WebViewEventModel>> = WebEventCallback.event
    @VisibleForTesting
    val phoneNumberLoginFeatureFlagValidator = PhoneNumberLoginFeatureFlagValidator()
    @VisibleForTesting
    val permissionCenterFeatureFlagValidator = PermissionCenterFeatureFlagValidator()

    fun parse(input: FragmentInfoParserModel): BaseFragment? {
        input.phoneNumberLoggedIn = isPhoneNumberLoginEnabled()
        input.title = titleDataSource.execute(getApplication() as Context, input.tag)
        input.permissionCenterEnabled = isPermissionCenterEnabled()
        input.privacyPolicy = SharedPreferencesUtil.getPrivacyPolicyUrl(getApplication() as Context)
        return accountFragmentTagParser.parse(input)
    }

    fun trackPageViewEvent() {
        getTracker()?.trackPageView(PageViewEvent(Anivia.PV_MY_ACCOUNT, SECTION_ACCOUNT))
    }

    fun getWebPageOwner(response: BaseStateResponse<WebViewEventModel>): String? {
        return fragmentRemoveParser.parse(response)
    }

    fun isPhoneNumberLoginEnabled(): Boolean {
        return phoneNumberLoginFeatureFlagValidator.validate(getApplication() as Context)
    }

    fun isPermissionCenterEnabled(): Boolean {
        return permissionCenterFeatureFlagValidator.validate(getApplication() as Context)
    }

    @VisibleForTesting
    fun getTracker(): ITracker? {
        return SingleProfile.tracker
    }
}
