package com.asda.android.app.beforeyougo.formatter

import com.asda.android.base.core.formatter.IFormatter
import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.info.BYGPageViewInfo
import com.asda.android.app.beforeyougo.model.BygP13NInfoFormatterModel
import com.asda.android.app.beforeyougo.model.BygP13NInfoModel
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.restapi.service.data.recipes.RecipeViewResponse

class BygP13NInfoFormatter : IFormatter<BygP13NInfoFormatterModel, BygP13NInfoModel> {
    private val separator = ","
    private val recipeIdsFormatter = RecipeIdsFormatter()
    override fun format(input: BygP13NInfoFormatterModel): BygP13NInfoModel {
        val bygTrackerInfo = mutableListOf<BYGPageViewInfo>()
        input.results?.let {
            var cinIds: String? = null
            var recipeIds: String? = null

            for ((moduleNumber, bygResult) in it.withIndex()) {
                bygResult.items?.let {
                    cinIds = getCinIds(it)

                }
                bygResult.recipes?.let {
                    recipeIds = getRecipeIds(it)
                }

                val bygPageViewInfo = BYGPageViewInfo(moduleNumber.toString(), cinIds, bygResult.strategyId, bygResult.strategyMessage, input.placement, recipeIds)
                bygTrackerInfo.add(bygPageViewInfo)
            }
        }
        return BygP13NInfoModel(bygTrackerInfo.toTypedArray(), input.hasPersonalization)
    }

    @VisibleForTesting
    fun getCinIds(items: Array<PersonalizationResponse.Items>): String {
        return items.mapNotNull { item -> item.cin }.distinct().joinToString(separator)
    }

    @VisibleForTesting
    fun getRecipeIds(items: Array<RecipeViewResponse.Recipe>): String {
        return recipeIdsFormatter.format(items)
    }
}
