package com.asda.android.app.beforeyougo.formatter

import com.asda.android.base.core.formatter.IFormatter
import android.content.res.Configuration
import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.beforeyougo.model.BygPageViewInfoModel
import com.asda.android.app.beforeyougo.model.PageViewEventModel
import com.asda.android.restapi.service.base.IAuthentication
import com.asda.android.restapi.service.data.UserViewResponse
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils

class BygPageViewInfoFormatter : IFormatter<PageViewEventModel, BygPageViewInfoModel> {
    private val separator = ","
    override fun format(input: PageViewEventModel): BygPageViewInfoModel {
        var bannerIds: String? = null
        input.pageContent?.let {
            try {
                bannerIds = Utils.getBannerIdsFromContentLoaderItem(it)
            } catch (exception: Exception) {
                bannerIds = null
            }

        }
        val orientation = if (input.orientation == Configuration.ORIENTATION_PORTRAIT) "portrait" else "landscape"
        val userViewResp: UserViewResponse? = getAuthentication().userViewResponse
        return BygPageViewInfoModel(getPageViewEvent(
              Anivia.PV_BEFORE_YOU_GO,
              Anivia.SECTION_CHECKOUT,
              bannerIds,
              orientation,
              userViewResp,
              getRecipeIds(input)))
    }

    @VisibleForTesting
    fun getAuthentication(): IAuthentication {
        return Authentication.getInstance()
    }

    @VisibleForTesting
    fun getRecipeIds(input: PageViewEventModel): String? {
        var recipeIds: String? = null
        input.results?.forEach { results ->
            results.recipes?.let {
                recipeIds = it.mapNotNull { item -> item.recipeId }.distinct().joinToString(separator)
            }
        }
        return recipeIds
    }

    @VisibleForTesting
    fun getPageViewEvent(
          pageName: String,
          section: String,
          bannerIds: String?,
          orientation: String?,
          userViewResp: UserViewResponse?,
          recipeIds: String?): PageViewEvent {

        val pageViewEvent = PageViewEvent(pageName, section)

        if (!orientation.isNullOrEmpty()) {
            pageViewEvent.putString("orientation", orientation)
        }

        val customerType = if (Authentication.isBusinessCustomer(userViewResp)) Authentication.CUSTOMER_TYPE_BUSINESS else Authentication.CUSTOMER_TYPE_CONSUMER
        if (customerType.isNotEmpty()) {
            pageViewEvent.putString(Anivia.CUSTOMER_TYPE_KEY, customerType)
        }

        if (!bannerIds.isNullOrEmpty()) {
            pageViewEvent.putString(Anivia.BANNER_INFO_KEY, bannerIds)
        }

        if (!recipeIds.isNullOrEmpty()) {
            pageViewEvent.putString(Anivia.P13N_RECIPES, recipeIds)
        }

        userViewResp?.let {
            if (!it.profileId.isNullOrEmpty()) {
                pageViewEvent.putString(Anivia.CUSTOMER_ID_KEY, it.profileId)
            }

            if (!it.storeId.isNullOrEmpty()) {
                pageViewEvent.putString(Anivia.STOREID_KEY, it.storeId)
            }
        }


        return pageViewEvent
    }
}
