package com.asda.android.app.beforeyougo.repository

import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.P13NImpressionEvent
import com.asda.android.analytics.info.BYGPageViewInfo
import com.asda.android.app.beforeyougo.model.BygP13NInfoModel

class BygP13NImpressionTrackerRepository: BaseRepository() {
    fun execute(p13NInfoModel: BygP13NInfoModel): Boolean {
        var isSuccess = false
        try {
            p13NInfoModel.bygPageViewInfos?.forEach { bygPageViewInfo ->
                if (isValid(p13NInfoModel) && isValidEvent(bygPageViewInfo)) {
                    getTracker()?.trackEvent(P13NImpressionEvent(bygPageViewInfo,
                          Anivia.PV_BEFORE_YOU_GO,
                          Anivia.SECTION_CHECKOUT, null, null, null, Anivia.SECTION_CHECKOUT))
                    isSuccess = true
                }
            }
        } catch (ex: Exception) {
            isSuccess = false
        }
        return isSuccess
    }

    @VisibleForTesting
    fun isValid(p13NInfoModel: BygP13NInfoModel): Boolean {
        return p13NInfoModel.hasP13nProducts && p13NInfoModel.bygPageViewInfos != null
    }

    @VisibleForTesting
    fun isValidEvent(bygPageViewInfo: BYGPageViewInfo): Boolean {
        return !bygPageViewInfo.p13nProducts.isNullOrEmpty() || !bygPageViewInfo.p13nRecipes.isNullOrEmpty()
    }
}
