package com.asda.android.app.beforeyougo.repository

import androidx.annotation.VisibleForTesting
import com.asda.android.app.beforeyougo.datasource.IBygDataSource
import com.asda.android.app.beforeyougo.model.APIErrorEventModel
import com.asda.android.app.beforeyougo.model.BygEventInfo
import com.asda.android.app.beforeyougo.model.BygP13NInfoModel
import com.asda.android.app.beforeyougo.model.BygPageViewInfoModel

/**
 * Performs background operations e.g track pageView, p13nImpression
 */
class BygRemoteRepository : IBygDataSource {
    @VisibleForTesting
    val p13NImpressionTrackerRepository = BygP13NImpressionTrackerRepository()
    @VisibleForTesting
    val pageViewTrackerRepository = BygPageViewTrackerRepository()
    @VisibleForTesting
    val errorTrackerRepository = BygErrorTrackerRepository()

    override fun trackEvent(eventInfo: BygEventInfo) {
        if (eventInfo is BygPageViewInfoModel) {
            pageViewTrackerRepository.execute(eventInfo.pageViewEvent)
        } else if (eventInfo is BygP13NInfoModel) {
            p13NImpressionTrackerRepository.execute(eventInfo)
        } else if (eventInfo is APIErrorEventModel) {
            errorTrackerRepository.execute(eventInfo)
        }
    }
}
