package com.asda.android.app.beforeyougo.viewmodel

import androidx.annotation.VisibleForTesting
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.asda.android.app.beforeyougo.formatter.*
import com.asda.android.app.beforeyougo.model.BygEventInfo
import com.asda.android.app.beforeyougo.parser.RecordsPerPageParser
import com.asda.android.app.beforeyougo.parser.VariationGroupParser
import com.asda.android.app.beforeyougo.repository.BygRemoteRepository
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.restapi.service.data.cms.Content
import com.asda.android.restapi.service.data.direct.PersonalizationResponse

class BeforeYouGoViewModel : ViewModel() {

    private val contents = mutableListOf<Content>()
    val itemIdsFormatter = CartItemIdsFormatter()
    val recipeIdsFormatter = CartRecipeIdsFormatter()
    val personalizationContentFormatter = PersonalizationContentFormatter()
    val bygPageViewInfoFormatter = BygPageViewInfoFormatter()
    val bygP13NInfoFormatter = BygP13NInfoFormatter()
    val variationGroupParser = VariationGroupParser()
    val recordsPerPageParser = RecordsPerPageParser()
    var placement: String? = null
    @VisibleForTesting
    val dataSource: BygRemoteRepository = BygRemoteRepository()
    val pageStateResponse: MutableLiveData<BaseStateResponse<Int>> = MutableLiveData()
    var hasPersonalization = false

    val cxoRecipeIdsFormatter = CxoCartRecipeIdsFormatter()

    fun track(eventInfo: BygEventInfo?) {
        eventInfo?.let {
            dataSource.trackEvent(it)
        }
    }


    fun updateContents(personalizationResponse: PersonalizationResponse?) {
        personalizationResponse?.results?.let {
            it.forEach { response ->
                run {
                    response.items?.let {
                        val bygResults = PersonalizationResponse.BYGResults(response)
                        bygResults.placement = placement
                        contents.add(bygResults)
                    }

                    response.recipes?.let { _ ->
                        var insertRecipeIndex = contents.indexOfFirst { item -> item is PersonalizationResponse.BYGResults && !item.items.isNullOrEmpty() }
                        insertRecipeIndex = if (insertRecipeIndex < 0) {
                            0
                        } else {
                            insertRecipeIndex
                        }
                        val bygResults = PersonalizationResponse.BYGResults(response)
                        bygResults.placement = placement
                        contents.add(insertRecipeIndex, bygResults)
                    }
                }
            }
        }
    }

    fun clearContents() {
        contents.clear()
    }

    fun getContents(): Array<Content> {
        return contents.toTypedArray()
    }
}
