package com.asda.android.app.bookslot

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.bookslotv3.view.fragment.BookSlotFragment
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.base.core.view.adapter.HeaderRecyclerViewAdapter
import com.asda.android.restapi.constants.ShelfListAdapterConstants.TYPE_ITEM
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.data.RecurringSlotsViewResponse
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*

class EditRecurringSlotsPresenter : FeaturedFragment() {
    private var mContext: Activity? = null
    private var mRecurringSlots: RecurringSlotsViewResponse? = null
    private var mView: View? = null
    private var mAdapter: RecurringSlotsAdapter? = null
    private var mEmptyView: View? = null

    private val mRecurringSlotsListener = object : RecurringSlotsViewResponse.Listener() {

        override fun onRecurringSlotsReceived(recurringSlotsViewResponse: RecurringSlotsViewResponse) {
            mRecurringSlots = recurringSlotsViewResponse
            if (mAdapter != null) {
                mAdapter?.refreshItems()
                if (mAdapter?.itemCount == 0) {
                    pop()
                }
            }

        }
    }

    init {
        pageViewEvent = PageViewEvent(Anivia.PV_BOOKSLOT_EDIT_RECURRING_SLOTS, Anivia.SECTION_BOOK_SLOT)//UKAPPSAN109.1
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as Activity
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mRecurringSlots = bundle.getParcelable(RECURRING_SLOTS)
        }
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        if (mView == null) {
            mView = inflater.inflate(R.layout.edit_recurring_slots, fragmentContainer, false)
            val recyclerView = mView?.findViewById<RecyclerView>(R.id.recycler_view)
            recyclerView?.layoutManager = LinearLayoutManager(mContext)
            mEmptyView = mView?.findViewById(R.id.empty_view)
            mAdapter = RecurringSlotsAdapter()
            mAdapter?.setFooterView(inflater.inflate(R.layout.edit_recurring_slots_footer, recyclerView, false))
            recyclerView?.adapter = mAdapter

        }
        return mView!!
    }

    override fun onResume() {
        super.onResume()
        RecurringSlotsViewResponse.addListener(mRecurringSlotsListener)
    }

    override fun onPause() {
        super.onPause()
        RecurringSlotsViewResponse.removeListener(mRecurringSlotsListener)
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mContext?.getString(R.string.recurring_slot)
    }

    private inner class RecurringSlotsAdapter : HeaderRecyclerViewAdapter() {
        private val mItems = ArrayList<RecurringSlotsViewResponse.RecurringSlotDetails>()

        init {
            refreshItems()
        }

        override fun getItemCount(): Int {
            return super.getItemCount() + mItems.size
        }

        fun getItem(position: Int): RecurringSlotsViewResponse.RecurringSlotDetails {
            return mItems[position]
        }

        fun refreshItems() {
            mItems.clear()
            if (mRecurringSlots != null && mRecurringSlots?.recurringSlotDetails != null) {
                mRecurringSlots?.recurringSlotDetails?.let {
                    for (slotDetails in it) {
                        if (slotDetails != null && !TextUtils.isEmpty(slotDetails.id) && !slotDetails.isCancelled) {
                            mItems.add(slotDetails)
                        }
                    }
                }
            }
            mEmptyView?.visibility = if (mItems.isEmpty()) View.VISIBLE else View.GONE
            notifyDataSetChanged()
        }

        override fun getItemId(position: Int): Long {
            return 0
        }

        override fun getItemViewType(position: Int): Int {
            if (isHeaderPosition(position)) {
                return TYPE_LIST_HEADERS - position
            } else if (isFooterPosition(position)) {
                return TYPE_LIST_FOOTER
            }
            return TYPE_ITEM
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder? {
            val holder = super.onCreateViewHolder(parent, viewType)
            return holder
                  ?: BaseViewHolder(ViewUtil.inflate(mContext, R.layout.recurring_slot_entry, parent), TYPE_ITEM)
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            if ((holder as BaseViewHolder).viewType == TYPE_ITEM) {
                val view = holder.itemView
                val item = getItem(position)
                val clickAndCollect = CartManager.getSlotTypeByDeliveryOption(item.fulfillmentType) == CartManager.SLOT_TYPE_CC
                ViewUtil.setText(R.id.title, view, if (clickAndCollect) R.string.order_summary_cc_details_header else R.string.order_summary_delivery_details_header)
                val deliveryTimeLabelId = if (clickAndCollect) R.string.order_summary_your_cc_slot else R.string.order_summary_your_delivery
                ViewUtil.setText(R.id.date, view, Utils.makeTwoColorText(mContext, mContext?.getString(deliveryTimeLabelId),
                      Utils.slotIntervalToDescription(mContext, item.date), R.color.black, R.color.text_grey))
                ViewUtil.setText(R.id.frequency_value, view, Utils.makeTwoColorText(mContext, mContext?.getString(R.string.recurring_slot_entry_frequency),
                      Utils.getFrequencyLabel(mContext, item.frequencyInDays), R.color.black, R.color.text_grey))
                ViewUtil.setText(R.id.order_by_date_value, view, Utils.makeTwoColorText(mContext, mContext?.getString(R.string.recurring_slot_entry_order_by_date),
                      mContext?.getString(R.string.cutoff_date, SimpleDateFormat.getDateInstance(DateFormat.LONG).format(Utils.parseIso8610Timestamp(item.orderByDate))), R.color.black, R.color.text_grey))

                val cancelButton = view.findViewById<View>(R.id.cancel)
                if (item.isCancelled) {
                    cancelButton.visibility = View.GONE
                } else {
                    cancelButton.visibility = View.VISIBLE
                    cancelButton.setOnClickListener { v ->
                        if (!TextUtils.isEmpty(item.id)) {
                            val progressDialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR, mContext?.getString(R.string.cancelling), mContext)
                            progressDialog?.show()
                            AsdaServiceFactory.get().cancelRecurringSlot(item.id,
                                  Authentication.getInstance().csrfToken,
                                  object : NetworkCallback<RecurringSlotsViewResponse>() {

                                      override fun onFailure(error: Int?, data: RecurringSlotsViewResponse) {
                                          if (isAdded && mContext != null) {
                                              progressDialog.dismiss()
                                              AsdaDialogHelper.displayErrorDialog(data, error, mContext, SCREEN_NAME)
                                          }
                                      }

                                      override fun onSuccess(data: RecurringSlotsViewResponse) {
                                          if (isAdded && mContext != null) {
                                              mContext?.let {
                                                  CartManager.get(mContext).refreshCart(it)
                                              }
                                              progressDialog.dismiss()
                                              RecurringSlotsViewResponse.broadcast(data)
                                              if (mContext is DeliveryImpactActivity && activity != null) {
                                                  activity?.setResult(ActivityRequestCodes.RESULT_CODE_RECURRING_SLOT_SUCCESS)
                                                  activity?.finish()
                                              } else
                                                  pop()
                                          }
                                      }
                                  }
                            )
                        }
                    }
                }
                val skipButton = view.findViewById<View>(R.id.skip)
                if (TextUtils.isEmpty(item.stopId)) {
                    skipButton.visibility = View.GONE
                } else {
                    skipButton.visibility = View.VISIBLE
                    skipButton.setOnClickListener {
                        if (!TextUtils.isEmpty(item.id)) {
                            val progressDialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR, mContext?.getString(R.string.skipping), mContext)
                            progressDialog?.show()
                            AsdaServiceFactory.get().skipRecurringSlot(item.id, item.stopId, Authentication.getInstance().csrfToken,
                                  object : NetworkCallback<RecurringSlotsViewResponse>() {

                                      override fun onFailure(error: Int?, data: RecurringSlotsViewResponse) {
                                          if (isAdded && mContext != null) {
                                              progressDialog.dismiss()
                                              AsdaDialogHelper.displayErrorDialog(data, error, mContext, SCREEN_NAME)
                                          }
                                      }

                                      override fun onSuccess(data: RecurringSlotsViewResponse) {
                                          if (isAdded && mContext != null) {
                                              progressDialog.dismiss()
                                              val cartManager = CartManager.get(mContext)

                                              data.recurringSlotDetails?.let { recurringSlotDetails ->
                                                  if (recurringSlotDetails.isNotEmpty() && recurringSlotDetails[0] != null) {
                                                      cartManager?.setSlotWasBooked(CartManager.getSlotTypeByDeliveryOption(recurringSlotDetails[0].fulfillmentType))
                                                  }
                                              }

                                              mContext?.let {
                                                  cartManager?.refreshCart(it)
                                              }
                                              RecurringSlotsViewResponse.broadcast(data)
                                              DialogFactory.createAlertDialog(mContext?.getString(R.string.slot_skipped), mContext, BookSlotFragment.SCREEN_NAME).show()
                                          }
                                      }
                                  }
                            )
                        }
                    }
                }
            }
        }
    }

    companion object {
        private const val SCREEN_NAME = "Edit Recurring Slot"
        const val TAG = "EditRecSlotsPresenter"
        private const val RECURRING_SLOTS = "recurring_slots"

        fun newInstance(recurringSlots: RecurringSlotsViewResponse?): EditRecurringSlotsPresenter {
            val fragment = EditRecurringSlotsPresenter()
            val args = Bundle()
            args.putParcelable(RECURRING_SLOTS, recurringSlots)
            fragment.arguments = args
            return fragment
        }
    }

}
