package com.asda.android.app.bookslot

import android.app.Activity
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import android.widget.ImageView

import androidx.viewpager.widget.PagerAdapter
import androidx.viewpager.widget.ViewPager

import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.view.ui.BaseAppCompatActivity
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils

class LearnAboutCNCActivity : BaseAppCompatActivity(), View.OnClickListener, ViewPager.OnPageChangeListener, Animation.AnimationListener {
    private var mAnimStarted: Boolean = false
    private var mButtonsContainer: View? = null
    private var mAnimation: Animation? = null
    private var mSeenFirstPagesInThisActivity: Boolean = false

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Utils.addNewRefToLeakDetector(TAG, this)
        setTitleAndBackArrow(R.string.about_cnc_title)
        setContentView(R.layout.learn_about_cnc)
        val viewPager = findViewById<ViewPager>(R.id.messages_pager)
        viewPager.adapter = Adapter()
        viewPager.addOnPageChangeListener(this)
        mAnimation = AnimationUtils.loadAnimation(this@LearnAboutCNCActivity, R.anim.slide_in_up)
        mAnimation?.setAnimationListener(this)
    }

    override fun onAnimationStart(animation: Animation) {
        mAnimStarted = true
    }

    override fun onAnimationEnd(animation: Animation) {
        mAnimStarted = false
        if (mButtonsContainer != null) {
            mButtonsContainer?.visibility = View.VISIBLE
        }
    }

    override fun onAnimationRepeat(animation: Animation) {

    }

    override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {}

    override fun onPageSelected(position: Int) {
        val indicators = findViewById<ViewGroup>(R.id.indicators)
        for (i in 0 until indicators.childCount) {
            (indicators.getChildAt(i) as ImageView).setImageResource(if (i == position) R.drawable.slide_dot_on else R.drawable.slide_dot_off)
        }
        mButtonsContainer = findViewById(R.id.buttons_container)
        if (position == 2) {
            startAnimation()
        } else {
            if (mButtonsContainer != null) {
                mButtonsContainer?.visibility = View.INVISIBLE
            }
            mSeenFirstPagesInThisActivity = true
        }
        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                .putString(Anivia.OVERLAY_COPY_KEY, "about c&c : screen " + (position + 1)))//UKAPPSAN6 ASDAA-5513
    }

    private fun startAnimation() {
        if (!mAnimStarted && mButtonsContainer != null) {
            mButtonsContainer?.clearAnimation()
            mButtonsContainer?.startAnimation(mAnimation)
        }
    }

    override fun onPageScrollStateChanged(state: Int) {

    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.button_not_now -> setResult(Activity.RESULT_CANCELED)
            R.id.button_book_slot -> setResult(Activity.RESULT_OK)
        }
        SharedPreferencesUtil.setCNCLearnt(this)
        finish()
    }

    private inner class Adapter : PagerAdapter() {
        internal val PAGES = intArrayOf(R.layout.learn_about_cnc_page_1, R.layout.learn_about_cnc_page_2, R.layout.learn_about_cnc_page_3)
        private val mLayoutInflater: LayoutInflater = LayoutInflater.from(this@LearnAboutCNCActivity)

        override fun getCount(): Int {
            return PAGES.size
        }

        override fun isViewFromObject(view: View, `object`: Any): Boolean {
            return view === `object`
        }

        override fun instantiateItem(container: ViewGroup, position: Int): Any {
            val view = mLayoutInflater.inflate(PAGES[position], container, false)
            container.addView(view)
            mButtonsContainer = view.findViewById(R.id.buttons_container)
            if (mButtonsContainer != null) {
                mButtonsContainer?.findViewById<View>(R.id.button_not_now)?.setOnClickListener(this@LearnAboutCNCActivity)
                mButtonsContainer?.findViewById<View>(R.id.button_book_slot)?.setOnClickListener(this@LearnAboutCNCActivity)
                if (!mSeenFirstPagesInThisActivity) {
                    mButtonsContainer?.visibility = View.VISIBLE//after recreate onPageSelected invokes first
                }
            }
            return view
        }

        override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
            container.removeView(`object` as View)
        }
    }

    companion object {
        private val TAG = "LearnAboutCNC"
    }
}
