package com.asda.android.app.bookslot

import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.ImageSpan
import android.util.Log
import android.view.*
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.annotation.VisibleForTesting
import androidx.annotation.VisibleForTesting.PRIVATE
import androidx.appcompat.app.AlertDialog
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.bookslotv3.view.activity.BookSlotActivity
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.recurringslot.RecurringSlotInfoFragment
import com.asda.android.app.recurringslot.SlotSubscriptionManager
import com.asda.android.app.recurringslot.SlotSubscriptionRepository
import com.asda.android.app.recurringslot.module.view.SlotSuggestionFragment
import com.asda.android.app.recurringslot.module.view.SlotSuggestionFragment.Companion.ERROR_SLOT_RECURRING_SLOTS_UNAVAILABLE
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.*
import com.asda.android.base.core.listener.OnSingleClickListener
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.base.interfaces.ICartManager
import com.asda.android.restapi.constants.ExternalUrls
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.data.BookSlotResponse
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.CheckoutResponse
import com.asda.android.restapi.service.data.RecurringSlotsViewResponse
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.restapi.utils.SubscriptionUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.ViewUtils
import io.reactivex.observers.ResourceSingleObserver
import java.util.*

class SlotBookedConfirmationPresenter : FeaturedFragment() {

    private var mContext: Activity? = null
    private var mAvailableSlots: BookSlotResponse.RecurringSlotInfo? = null
    private val mAniviaEventShowFrequencies: PageViewEvent = PageViewEvent(Anivia.PV_BOOKSLOT_SELECT_RECURRING_SLOT, Anivia.SECTION_BOOK_SLOT)
    private var mBookedSlotType: Int = 0
    private var mView: View? = null
    private var mSuccessSubtitleView: TextView? = null
    private var bookedUntilTextView: TextView? = null
    private var deliveryDetailsHeader: TextView? = null
    private var expressSlotHeader: TextView? = null
    private var bookSlotButtonsHeader: View? = null
    private var deliveryDetailsView: View? = null
    private var bookedRecurringSlotFreqContainer: View? = null
    private var recurringSlotSwitchContainer: View? = null
    private var recurringSlotLoader: View? = null
    private var bookButton: View? = null
    private var rememberSection: View? = null
    private var cartListener: ICartManager.CartListener? = null
    private var subscriptionEligibilityInfo: SubscriptionEligibilityInfo? = null
    private val PRIORITY_PASS_RELEASE_DATE = "24 Aug 2020"
    private var slotStartDate = ""
    private var slotEndDate = ""

    override fun onStart() {
        super.onStart()
        //book slot page view event
        val pageViewEvent = PageViewEvent(Anivia.BOOK_SLOT_CONFIRMATION, Anivia.SECTION_BOOK_SLOT)
        Tracker.get().trackPageView(pageViewEvent)
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mAvailableSlots = bundle.getParcelable(AVAILABLE_SLOTS)
            mBookedSlotType = bundle.getInt(BOOKED_DELIVERY_METHOD)
            slotStartDate = bundle.getString(EXTRA_START_DATE) ?: ""
            slotEndDate = bundle.getString(EXTRA_END_DATE) ?: ""
            if (bundle.containsKey(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO)) {
                subscriptionEligibilityInfo = bundle.getParcelable(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO)
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as Activity
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        if (mView == null) {
            mView = inflater.inflate(R.layout.slot_booked_confirmation, fragmentContainer, false)
            addCartListener()
            setupViews()
            setupRecurringSlotSection()

            mView?.findViewById<View>(R.id.recurring_slot_help)?.setOnClickListener { _ -> push(RecurringSlotInfoFragment()) }

            context?.let { context ->
                if (FeatureSettings.isCXOMigrationEnabled(context)) {
                    setupWithCXOCart()
                    addCXOCartListener()
                } else {
                    setupWithCart()
                }

                setupRecurringSlotPeak(context)
            }
        }
        return mView!!
    }

    private fun setupRecurringSlotPeak(context: Context) {

        val rsPeakMessage = mView?.findViewById<TextView>(R.id.rs_peak_season_message)
        val rsLearnMorePeak = mView?.findViewById<TextView>(R.id.rs_learn_more_peak_season)

        SlotUtils().getPeakSeasonConfig(context)?.let { _config ->
            _config.message?.let { _message ->
                if (_message.isNotEmpty() && _message.isNotBlank()) {
                    rsPeakMessage?.visibility = View.VISIBLE
                    rsPeakMessage?.text = Utils.fromHtml(_message)

                    _config.faqUrl?.let { _faqUrl ->
                        if (_faqUrl.isNotEmpty() && _faqUrl.isNotBlank()) {
                            rsLearnMorePeak?.visibility = View.VISIBLE
                            rsLearnMorePeak?.setOnClickListener {
                                ViewUtils.openWebPage(_faqUrl, context, TAG)
                            }
                        }
                    }
                }
            }
        }
    }

    private fun addCXOCartListener() {
        com.asda.android.cxo.CartManager?.instance?.updatedCart?.observe(viewLifecycleOwner,
              androidx.lifecycle.Observer<com.asda.android.restapi.service.data.cart.Cart> { cart ->
                  var bookedTillTime: String?
                  if (cart?.fulfillmentDetails?.slotInfo?.isRecurringSlot == true && cart.fulfillmentDetails?.slotInfo?.startDateTime?.isNotEmpty() == true) {
                      cart.fulfillmentDetails?.slotInfo?.startDateTime?.let {
                          bookedTillTime = SubscriptionUtil.getSubscriptionExpiryDate(it,
                                DateFormats.EEE_DD_MMM_YYYY,
                                DateFormats.HH_MM_AA_EEE_DD_MMM,
                                TimeZoneFormat.UK,
                                TimeZoneFormat.UK)
                          bookedUntilTextView?.text = mContext?.getString(getSlotConfirmationMessage(cart, false, Utils.isExpressEnabled(mContext)), bookedTillTime)
                      }
                  }
              })
    }

    private fun setupViews() {
        recurringSlotSwitchContainer = mView?.findViewById<View>(R.id.recurring_slot_switch_container)
        bookSlotButtonsHeader = mView?.findViewById<View>(R.id.recurring_slot_frequency_selector_header)
        bookButton = mView?.findViewById<View>(R.id.recurring_slot_book_button)
        recurringSlotLoader = mView?.findViewById<View>(R.id.recurring_slot_loader)
        rememberSection = mView?.findViewById<View>(R.id.remember_section)
        rememberSection?.visibility = View.GONE
        mSuccessSubtitleView = mView?.findViewById(R.id.success_slot_booked)
        ViewUtils.applyAsdaFont(mSuccessSubtitleView)
        bookedUntilTextView = mView?.findViewById<TextView>(R.id.book_slot_confirmation_until)
        deliveryDetailsHeader = mView?.findViewById<TextView>(R.id.delivery_details_header)
        expressSlotHeader = mView?.findViewById<TextView>(R.id.express_slot_header)
        deliveryDetailsView = mView?.findViewById<View>(R.id.delivery_details)
        bookedRecurringSlotFreqContainer = mView?.findViewById<View>(R.id.booked_frequency_info_container)
        bookedRecurringSlotFreqContainer?.visibility = View.GONE
        deliveryDetailsView?.findViewById<View>(R.id.carat)?.visibility = View.GONE
        deliveryDetailsView?.setBackgroundDrawable(null)
        mView?.findViewById<View>(R.id.booked_until_text)?.visibility = View.GONE
        mView?.findViewById<View>(R.id.booked_until_value)?.visibility = View.GONE
    }

    private fun setupWithCart() {
        val clickAndCollect = mBookedSlotType == CartManager.SLOT_TYPE_CC

        val cartManager = CartManager.get(mContext)
        val cart = cartManager?.cart
        if (cart == null || TextUtils.isEmpty(cart.bookedUntilTime)) {
            bookedUntilTextView?.visibility = View.GONE
        } else {
            bookedUntilTextView?.visibility = View.VISIBLE
            bookedUntilTextView?.text = mContext?.getString(getSlotConfirmationMessage(cart, clickAndCollect, Utils.isExpressEnabled(mContext)), cart.bookedUntilTime)

        }
        if (Utils.isExpressEnabled(mContext) && cart != null && cart.isExpress != null && cart.isExpress?.equals("true", ignoreCase = true) == true) {
            deliveryDetailsHeader?.visibility = View.GONE
            expressSlotHeader?.visibility = View.VISIBLE
            val spannableStringBuilder = SpannableStringBuilder()
                  .append(mContext?.getString(R.string.single_space))
            mContext?.let {
                spannableStringBuilder.setSpan(ImageSpan(it, R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            }
            spannableStringBuilder.append(mContext?.getString(R.string.single_space)).append(mContext?.getString(R.string.slot))
            expressSlotHeader?.text = spannableStringBuilder
        } else {
            deliveryDetailsHeader?.visibility = View.VISIBLE
            expressSlotHeader?.visibility = View.GONE
        }

        mView?.let {
            if (cart != null && !TextUtils.isEmpty(cart.bookedUntilTime)) {
                deliveryDetailsHeader?.visibility = View.VISIBLE
                deliveryDetailsView?.visibility = View.VISIBLE
                deliveryDetailsView?.findViewById<View>(R.id.carat)?.visibility = View.GONE
                mView?.findViewById<View>(R.id.booked_until_text)?.visibility = View.GONE
                mView?.findViewById<View>(R.id.booked_until_value)?.visibility = View.GONE

                val titleId = if (clickAndCollect) R.string.order_summary_cc_details_header else R.string.order_summary_delivery_details_header
                ViewUtil.setText(R.id.delivery_details_header, mView, mContext?.getString(titleId))
                val deliveryTimeLabelId = if (clickAndCollect) R.string.order_summary_your_cc_slot else R.string.order_summary_your_delivery
                ViewUtil.setText(R.id.delivery_time_text, deliveryDetailsView, mContext?.getString(deliveryTimeLabelId))
                ViewUtil.setText(R.id.delivery_time_value, deliveryDetailsView, Utils.formatDeliveryTimeString(cart.slotDate, cart.deliveryStartTime, cart.deliveryEndTime, true))
                val phoneText = deliveryDetailsView?.findViewById<View>(R.id.phone_text)
                val phoneValue = ViewUtil.findViewById<TextView>(deliveryDetailsView, R.id.phone_value)
                val address: String?
                val cartExtraInfo = cartManager.cartExtraInfo
                if (cartExtraInfo != null) {
                    if (clickAndCollect && cartExtraInfo.cncStoreAddress != null && !TextUtils.isEmpty(cartExtraInfo.cncStoreAddress.city)) {
                        ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_cc_address))
                        address = cartExtraInfo.cncStoreAddress.toString()
                        phoneText?.visibility = View.VISIBLE
                        phoneValue.visibility = View.VISIBLE
                        phoneValue.text = cartExtraInfo.cncStoreAddress.phoneNumber
                    } else {
                        ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_address))
                        address = if (cartExtraInfo.deliveryAddress != null && !TextUtils.isEmpty(cartExtraInfo.deliveryAddress.address))
                            cartExtraInfo.deliveryAddress.address + ", " + cart.postCode
                        else
                            cart.postCode
                        phoneText?.visibility = View.GONE
                        phoneValue.visibility = View.GONE
                    }
                    ViewUtil.setText(R.id.address_fee_value, deliveryDetailsView, address)
                }
            } else {
                deliveryDetailsHeader?.visibility = View.GONE
                deliveryDetailsView?.visibility = View.GONE
            }

            mView?.findViewById<View>(R.id.booked_frequency_info_container)?.visibility = View.GONE

            val fromCheckout = mContext?.intent?.getBooleanExtra(ActivityRequestCodes.EXTRA_FROM_CHECKOUT, false)
            val continueButton = mView?.findViewById<View>(R.id.continue_button)
            val checkoutButton = ViewUtil.findViewById<Button>(it, R.id.checkout_button)

            // If in "amending" state, the button should say "CHECKOUT AMENDS"
            if (CartManager.get(context)?.isAmending == true) {
                checkoutButton.setText(R.string.cart_checkout_amends)
            } else {
                checkoutButton.setText(R.string.checkout)
            }

            if (fromCheckout == true) {
                continueButton?.visibility = View.GONE
                checkoutButton.visibility = View.VISIBLE//no need to check for min order here because at least one button should be visible
            } else {
                continueButton?.visibility = View.VISIBLE
                // TODO: Check with Jose.
                // Not sure if there is a story, but per delivery-surcharge project requirement we should allow checkout on surcharge enabled stores.
                val minOrderPresent = CartManager.checkIfMinOrderPresent(cart)
                val isExpressOrder = CartManager.isExpressOrder(cart)
                if (isExpressOrder) {
                    checkoutButton.visibility = View.VISIBLE
                } else {
                    checkoutButton.visibility = if (minOrderPresent) View.VISIBLE else View.GONE
                }
            }

            continueButton?.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_SHOPPING)
                    mContext?.finish()
                } else {
                    pop()
                }
            }

            checkoutButton.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    if (fromCheckout == false) {
                        mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_CHECKOUT)
                        if (CartManager.get(mContext)?.hasCheckoutErrorOccurred() == true) {
                            checkout()
                        }
                    }
                    mContext?.finish()
                } else {
                    pop()
                }
            }
        }
    }


    private fun setupWithCXOCart() {
        val cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
        val clickAndCollect = cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType != FulfillmentTypes.DELIVERY.name

        fun setBookUntilTime(cxoCart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean) {
            bookedUntilTextView?.text = mContext?.getString(getSlotConfirmationMessage(cxoCart, clickAndCollect, Utils.isExpressEnabled(mContext)),
                  cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))
            bookedUntilTextView?.visibility = if (cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime.isNullOrEmpty()) View.GONE else View.VISIBLE
        }

        fun setExpressSection(cxoCart: com.asda.android.restapi.service.data.cart.Cart?) {
            if (Utils.isExpressEnabled(mContext) && cxoCart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                deliveryDetailsHeader?.visibility = View.GONE
                expressSlotHeader?.visibility = View.VISIBLE
                val spannableStringBuilder = SpannableStringBuilder()
                      .append(mContext?.getString(R.string.single_space))
                mContext?.let {
                    spannableStringBuilder.setSpan(ImageSpan(it, R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
                }
                spannableStringBuilder.append(mContext?.getString(R.string.single_space)).append(mContext?.getString(R.string.slot))
                expressSlotHeader?.text = spannableStringBuilder
            } else {
                deliveryDetailsHeader?.visibility = View.VISIBLE
                expressSlotHeader?.visibility = View.GONE
            }
        }

        fun setAddress(cxoCart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean) {
            if (!cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime.isNullOrEmpty()) {
                deliveryDetailsHeader?.visibility = View.VISIBLE
                deliveryDetailsView?.visibility = View.VISIBLE

                val titleId = if (clickAndCollect) R.string.order_summary_cc_details_header else R.string.order_summary_delivery_details_header
                ViewUtil.setText(R.id.delivery_details_header, mView, mContext?.getString(titleId))
                val deliveryTimeLabelId = if (clickAndCollect) R.string.order_summary_your_cc_slot else R.string.order_summary_your_delivery
                ViewUtil.setText(R.id.delivery_time_text, deliveryDetailsView, mContext?.getString(deliveryTimeLabelId))
                ViewUtil.setText(R.id.delivery_time_value, deliveryDetailsView,
                      getString(R.string.slot_reserved_date_from_to,
                            cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM),
                            cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                            cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))
                )
                val phoneText = deliveryDetailsView?.findViewById<View>(R.id.phone_text)
                val phoneValue = ViewUtil.findViewById<TextView>(deliveryDetailsView, R.id.phone_value)
                val address: String?
                address = "${cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address}, ${cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode}"

                if (clickAndCollect) {
                    ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_cc_address))
                    phoneText?.visibility = View.VISIBLE
                    phoneValue.visibility = View.VISIBLE
                    phoneValue.text = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.phoneNumber
                } else {
                    ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_address))
                    phoneText?.visibility = View.GONE
                    phoneValue.visibility = View.GONE
                }
                ViewUtil.setText(R.id.address_fee_value, deliveryDetailsView, address)
            } else {
                deliveryDetailsHeader?.visibility = View.GONE
                deliveryDetailsView?.visibility = View.GONE
            }
        }

        fun setContinueAndCheckoutButtons(cxoCart: com.asda.android.restapi.service.data.cart.Cart?) {
            val fromCheckout = mContext?.intent?.getBooleanExtra(ActivityRequestCodes.EXTRA_FROM_CHECKOUT, false)
            val continueButton = mView?.findViewById<View>(R.id.continue_button)
            val checkoutButton = ViewUtil.findViewById<Button>(mView, R.id.checkout_button)

            // If in "amending" state, the button should say "CHECKOUT AMENDS"
            if (CartManager.get(context)?.isAmending == true) {
                checkoutButton.setText(R.string.cart_checkout_amends)
            } else {
                checkoutButton.setText(R.string.checkout)
            }

            if (fromCheckout == true) {
                continueButton?.visibility = View.GONE
                checkoutButton.visibility = View.VISIBLE//no need to check for min order here because at least one button should be visible
            } else {
                continueButton?.visibility = View.VISIBLE
                val minOrderPresent = checkIfMinOrderPresent(cxoCart)
                val isExpressOrder = cxoCart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type
                if (isExpressOrder) {
                    checkoutButton.visibility = View.VISIBLE
                } else {
                    checkoutButton.visibility = if (minOrderPresent) View.VISIBLE else View.GONE
                }
            }

            continueButton?.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_SHOPPING)
                    mContext?.finish()
                } else {
                    pop()
                }
            }

            checkoutButton.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    if (fromCheckout == false) {
                        mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_CHECKOUT)
                        if (CartManager.get(mContext)?.hasCheckoutErrorOccurred() == true) {
                            checkout()
                        }
                    }
                    mContext?.finish()
                } else {
                    pop()
                }
            }
        }

        setBookUntilTime(cxoCart, clickAndCollect)
        setExpressSection(cxoCart)
        setAddress(cxoCart, clickAndCollect)
        setContinueAndCheckoutButtons(cxoCart)
    }

    private fun checkIfMinOrderPresent(cxoCart: com.asda.android.restapi.service.data.cart.Cart?): Boolean {
        var result = true
        val cxoCartManager = com.asda.android.cxo.CartManager.instance
        if (cxoCartManager?.minOrderValue != null && cxoCart?.cartPriceSummary?.total != null) {
            try {
                result = (cxoCart?.cartPriceSummary?.total!!.compareTo(cxoCartManager?.minOrderValue!!.toDouble())) > 0
            } catch (e: Exception) {
                Log.w(TAG, "unable to parse cart total & minOrder '${cxoCart?.cartPriceSummary?.total!!}' '${cxoCartManager?.minOrderValue}'")
            }
        }
        return result
    }

    private fun setupRecurringSlotSection() {
        mContext?.let {
            if (FeatureSettings.isSlotSubscriptionMigratedUser(it)) {
                if (subscriptionEligibilityInfo?.eligible == true && subscriptionEligibilityInfo?.active_subscription == false && isPriorityPassCustomer()) {
                    bookButton?.visibility = View.VISIBLE
                    bookButton?.setOnClickListener(object : OnSingleClickListener(this@SlotBookedConfirmationPresenter) {
                        override fun onSingleClick(v: View) {
                            mContext?.let { context ->
                                recurringSlotLoader?.visibility = View.VISIBLE
                                bookButton?.visibility = View.GONE

                                val cxoCartSlotBookedDate = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.startDateTime

                                SlotSubscriptionRepository().getSubscriptionFrequencyOption(
                                      singleProfileId = SingleProfile.getProfileResponse()?.profileId
                                            ?: "",
                                      slotDate = cxoCartSlotBookedDate
                                            ?: "")?.subscribe(object : ResourceSingleObserver<SubscriptionFreqOptionsResponse>() {
                                    override fun onSuccess(subscriptionFrequencyResponse: SubscriptionFreqOptionsResponse) {
                                        recurringSlotLoader?.visibility = View.GONE
                                        bookButton?.visibility = View.VISIBLE

                                        val labels = arrayOfNulls<String>(subscriptionFrequencyResponse.data?.frequencies?.size
                                              ?: 0)
                                        for (i in labels.indices) {
                                            labels[i] = Utils.getFrequencyLabel(mContext, subscriptionFrequencyResponse.data?.frequencies?.get(i)?.frequency)
                                        }

                                        val builder = AlertDialog.Builder(context)
                                        builder.setTitle(R.string.choose_frequency)
                                        builder.setSingleChoiceItems(labels, -1) { dialog, which ->
                                            dialog.dismiss()
                                            subscriptionFrequencyResponse.data?.frequencies?.let { frequencies ->
                                                bookRecurringSlot(frequencies[which].frequency, labels[which] ?: "", slotStartDate, slotEndDate)
                                            }
                                        }
                                        builder.setOnDismissListener { }
                                        builder.create().show()

                                    }

                                    override fun onError(e: Throwable) {
                                        recurringSlotLoader?.visibility = View.GONE
                                        bookButton?.visibility = View.VISIBLE
                                        Toast.makeText(mContext, mContext?.getString(R.string.get_subscription_error_msg), Toast.LENGTH_LONG).show()
                                    }
                                })

                                pageViewEvent = mAniviaEventShowFrequencies
                                Tracker.get().trackPageView(mAniviaEventShowFrequencies)//UKAPPSAN109.2.1
                            }
                        }
                    })


                } else {
                    hideRecurringSlotView()
                }
            } else {
                if (mAvailableSlots != null && "true" == mAvailableSlots?.allowNewRecurringSlot && mAvailableSlots?.availableFrequencies != null && isPriorityPassCustomer()) {
                    bookSlotButtonsHeader?.visibility = View.VISIBLE
                    val labels = arrayOfNulls<String>(mAvailableSlots?.availableFrequencies?.size
                          ?: 0)
                    for (i in labels.indices) {
                        labels[i] = Utils.getFrequencyLabel(mContext, mAvailableSlots?.availableFrequencies?.get(i)?.frequency)
                    }
                    bookButton?.visibility = View.VISIBLE
                    bookButton?.setOnClickListener(object : OnSingleClickListener(this@SlotBookedConfirmationPresenter) {
                        override fun onSingleClick(v: View) {
                            mContext?.let { context ->
                                val builder = AlertDialog.Builder(context)
                                builder.setTitle(R.string.choose_frequency)
                                builder.setSingleChoiceItems(labels, -1) { dialog, which ->
                                    dialog.dismiss()
                                    mAvailableSlots?.availableFrequencies?.let {
                                        bookRecurringSlot(it[which].frequency, labels[which] ?: "", slotStartDate, slotEndDate)
                                    }
                                }
                                builder.setOnDismissListener { }
                                builder.create().show()
                                pageViewEvent = mAniviaEventShowFrequencies
                                Tracker.get().trackPageView(mAniviaEventShowFrequencies)//UKAPPSAN109.2.1
                            }
                        }
                    })
                } else {
                    hideRecurringSlotView()
                }
            }
        }
    }

    private fun hideRecurringSlotView() {
        recurringSlotSwitchContainer?.visibility = View.GONE
        bookButton?.visibility = View.GONE
        bookSlotButtonsHeader?.visibility = View.GONE
    }

    private fun isPriorityPassCustomer(): Boolean {
        if (FeatureSettings.isRecurringSlotDisabledForNormalDP(requireContext())) {
            if (!Authentication.getInstance().userViewResponse.deliveryPass?.activeDPPromotionsInProfile.isNullOrEmpty()) {
                val activeDPPromotionInProfile = Authentication.getInstance().userViewResponse.deliveryPass?.activeDPPromotionsInProfile?.get(0)
                if (activeDPPromotionInProfile?.name.equals(PRIORITY_PASS, true)) {
                    return true
                } else if (activeDPPromotionInProfile?.dpStartDate?.substring(0, 10)?.isBefore(
                            dateToCompare = PRIORITY_PASS_RELEASE_DATE,
                            dateToCompareFormat = DateFormats.DD_MMM_YYYY,
                            inputDateFormat = DateFormats.YYYY_MM_dd)
                      == true) {
                    return true
                }
            }
        } else {
            return true
        }
        return false
    }

    override fun onDestroy() {
        removeCartListener()
        super.onDestroy()
    }

    private fun checkout() {
        val callback = object : NetworkCallback<CheckoutResponse>() {

            override fun onFailure(error: Int?, data: CheckoutResponse) {
                if (AsdaServiceSettings.DEBUG) {
                    Log.e(TAG, "checkout() onFailure")
                }
            }

            override fun onSuccess(data: CheckoutResponse) {
                if (AsdaServiceSettings.DEBUG) {
                    Log.i(TAG, "checkout() onSuccess")
                }

                mContext?.finish()
            }
        }

        CartManager.get(mContext)?.checkout(callback)

    }

    @VisibleForTesting(otherwise = PRIVATE)
    fun getSlotConfirmationMessage(cart: Cart, clickAndCollect: Boolean, isConfigExpressEnabled: Boolean): Int {
        var slotConfirmationTextId = R.string.book_slot_confirmation_until
        if (clickAndCollect) {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart.isExpress != null && cart.isExpress?.equals("true", ignoreCase = true) == true) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_cc_slot_confirmation_until
            }
        } else {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart.isExpress != null && cart.isExpress?.equals("true", ignoreCase = true) == true) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_slot_confirmation_until
            }
        }
        return slotConfirmationTextId
    }

    //CXO Confirmation Message
    fun getSlotConfirmationMessage(cart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean, isConfigExpressEnabled: Boolean): Int {
        var slotConfirmationTextId = R.string.book_slot_confirmation_until

        if (clickAndCollect) {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_cc_slot_confirmation_until
            }
        } else {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_slot_confirmation_until
            }
        }
        return slotConfirmationTextId
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mContext?.getString(R.string.book_slot_header)
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        menu.clear()
        inflater.inflate(R.menu.done_button, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.action_done) {
            if (CartManager.get(mContext)?.hasCheckoutErrorOccurred() == true) {
                checkout()
            }
            mContext?.setResult(ActivityRequestCodes.RESULT_CODE_FINISH_ACTIVITY)
            mContext?.finish()
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed(): Boolean {
        if (mContext is DeliveryImpactActivity) {
            mContext?.setResult(ActivityRequestCodes.RESULT_REFRESH_BOOKED_SLOT)
            mContext?.finish()
        }
        return super.onBackPressed()
    }

    private fun bookRecurringSlot(spanId: String?, label: String, slotStartDate: String?, slotEndDate: String?) {
        context?.let {
            if (FeatureSettings.isSlotSubscriptionMigratedUser(it) && FeatureSettings.isCXOMigrationEnabled(it)) {
                createSubscription(spanId, label)
            } else if (!FeatureSettings.isSlotSubscriptionMigratedUser(it) && FeatureSettings.isCXOMigrationEnabled(it)) {
                createSubscriptionForATGUsers(spanId, label)
            } else {
                createRecurringSlots(spanId, label, slotStartDate, slotEndDate)
            }
        }
    }

    private fun createSubscription(spanId: String?, label: String) {
        SingleProfile.getProfileResponse()?.profileId?.let { singleProfileId ->
            val pd = CustomProgressDialog.create(mContext)
            pd.setMessage(mContext?.getString(R.string.booking_slot_progress))
            pd.show()

            SlotSubscriptionRepository().createSubscription(singleProfileId,
                  getSubscriptionCreationRequest(spanId))?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                override fun onSuccess(subscriptionCreation: SubscriptionCreation) {
                    validateSuccessSubscription(subscriptionCreation, spanId, label, singleProfileId, false, pd)
                }

                override fun onError(e: Throwable) {
                    createSubscriptionOnError(pd)
                }

            })
        }
    }

    private fun validateSuccessSubscription(subscriptionCreation: SubscriptionCreation, spanId: String?, label: String, singleProfileId: String, isATG: Boolean, progressDialog: ProgressDialog) {
        if (subscriptionCreation.errors?.isNotEmpty() == true && !subscriptionCreation.errors?.get(0)?.code.isNullOrBlank()) {
            if (subscriptionCreation.errors?.get(0)?.code == ERROR_SLOT_RECURRING_SLOTS_UNAVAILABLE) {
                //Open suggestion fragment
                val fragment = SlotSuggestionFragment.newInstance(subscriptionCreation?.data?.subscription_info?.start_time,
                      subscriptionCreation?.data?.subscription_info?.end_time,
                      spanId, label, subscriptionCreation?.data?.suggestedSlotInfo)
                fragment.suggestionSelectListener = object : SlotSuggestionFragment.SuggestionSelectListener {
                    override fun onSuggestionSelected(subscriptionCreation: SubscriptionCreation, label: String) {

                        if (isATG) {
                            SlotSubscriptionRepository().createSubscriptionForATG(singleProfileId,
                                  subscriptionCreation)?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                                override fun onSuccess(t: SubscriptionCreation) {
                                    validateSuccessSubscription(t, spanId, label, singleProfileId, isATG, progressDialog)
                                }

                                override fun onError(e: Throwable) {
                                    createSubscriptionOnError(progressDialog)
                                }

                            })
                        } else {
                            SlotSubscriptionRepository().createSubscription(singleProfileId,
                                  subscriptionCreation)?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                                override fun onSuccess(t: SubscriptionCreation) {
                                    validateSuccessSubscription(t, spanId, label, singleProfileId, isATG, progressDialog)
                                }

                                override fun onError(e: Throwable) {
                                    createSubscriptionOnError(progressDialog)
                                }

                            })
                        }
                    }
                }
                push(fragment)
            } else {
                createSubscriptionOnError(progressDialog)
            }
        } else {
            if (isATG) {
                SlotSubscriptionManager.refreshSubscriptionEligibilityForATGUsers()
            }
            createSubscriptionOnSuccess(progressDialog, label, subscriptionCreation)
        }
    }

    private fun createSubscriptionOnSuccess(pd: ProgressDialog, label: String, subscriptionCreation: SubscriptionCreation) {
        try {
            pd.dismiss()
            val aniviaEventShowRecConfirmation = PageViewEvent(Anivia.PV_BOOKSLOT_CONFIRM_RECURRING_SLOT, Anivia.SECTION_BOOK_SLOT)//UKAPPSAN109.2.2
            aniviaEventShowRecConfirmation.putString(Anivia.RECURRING_SLOT_KEY, "yes : " + label.toLowerCase(Locale.UK))
            pageViewEvent = aniviaEventShowRecConfirmation
            Tracker.get().trackPageView(aniviaEventShowRecConfirmation)
            rememberSection?.visibility = View.VISIBLE
            bookSlotButtonsHeader?.visibility = View.GONE
            recurringSlotSwitchContainer?.visibility = View.GONE

            val clickAndCollect = mBookedSlotType == CartManager.SLOT_TYPE_CC
            val recurringSlotDateContainer = mView?.findViewById<ViewGroup>(R.id.next_recurring_slot_date)
            subscriptionCreation.data?.subscription_info
            if (subscriptionCreation.data?.subscription_info != null) {
                (recurringSlotDateContainer?.findViewById<View>(R.id.next_recurring_slot_date_label) as TextView)
                      .setText(if (clickAndCollect) R.string.next_collection_date else R.string.next_delivery_date)
                (recurringSlotDateContainer.findViewById<View>(R.id.next_recurring_slot_date_value) as TextView).text =
                      SubscriptionUtil.getFormattedDeliveryDate(subscriptionCreation.data?.subscription_info)
                recurringSlotDateContainer.visibility = View.VISIBLE
            } else {
                recurringSlotDateContainer?.visibility = View.GONE
            }
            mSuccessSubtitleView?.setText(if (clickAndCollect) R.string.success_recurring_cnc_slot_booked else R.string.success_recurring_slot_booked)
            bookedRecurringSlotFreqContainer?.visibility = View.VISIBLE
            ViewUtil.setText(R.id.frequency_value, bookedRecurringSlotFreqContainer, label)
            context?.let {
                com.asda.android.cxo.CartManager?.instance?.refreshCart()
            }
            SlotSubscriptionManager.refreshSubscriptionEligibility()
        } catch (ignored: Exception) {
            Log.e(TAG, "Something went wrong while creating subscriptions.")
        }
    }

    private fun createSubscriptionOnError(pd: ProgressDialog) {
        try {
            pd.dismiss()
            DialogFactory.createAlertDialog(context?.getString(R.string.oops_error), context?.getString(R.string.ok), null, "", context).show()
            Tracker.get().trackPageView(PageViewEvent(Anivia.PV_BOOK_SLOT_ERROR, Anivia.SECTION_BOOK_SLOT)
                  .putString(Anivia.ERROR_EXPERIENCED_KEY, "book slot : "))
        } catch (ignored: Exception) {
            Log.e(TAG, "Something went wrong while creating subscriptions.")
        }
    }

    private fun createSubscriptionForATGUsers(spanId: String?, label: String) {
        SingleProfile.getProfileResponse()?.profileId?.let { singleProfileId ->
            val pd = CustomProgressDialog.create(mContext)
            pd.setMessage(mContext?.getString(R.string.booking_slot_progress))
            pd.show()

            SlotSubscriptionRepository().createSubscriptionForATG(singleProfileId,
                  getSubscriptionCreationRequest(spanId))?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                override fun onSuccess(subscriptionCreation: SubscriptionCreation) {
                    validateSuccessSubscription(subscriptionCreation, spanId, label, singleProfileId, true, pd)
                }

                override fun onError(e: Throwable) {
                    createSubscriptionOnError(pd)
                }

            })
        }
    }

    private fun getSubscriptionCreationRequest(spanId: String?): SubscriptionCreation {
        val subscriptionCreation = SubscriptionCreation()
        subscriptionCreation.data = SubscriptionCreationData()
        val cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
        val serviceAddress = ServiceAddress(
              postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
              latitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.latitude,
              longitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.longitude,
              address_1 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address,
              address_2 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address2,
              city = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.city,
              country = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.country
        )

        subscriptionCreation.data?.subscription_info = SubscriptionInfo(
              frequency = spanId,
              group = "PT2H",
              start_date = cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
              end_date = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
              start_time = cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
              end_time = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
              postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
              fulfillment_type = cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType,
              reservation_id = cxoCart?.fulfillmentDetails?.slotInfo?.reservationId,
              service_address = serviceAddress
        )

        return subscriptionCreation
    }

    private fun createRecurringSlots(spanId: String?, label: String?, slotStartDate: String?, slotEndDate: String?, suggestedInfo: SubscriptionCreation? = null) {
        val pd = CustomProgressDialog.create(mContext)
        pd.setMessage(mContext?.getString(R.string.booking_slot_progress))
        pd.show()
        val callback = object : NetworkCallback<RecurringSlotsViewResponse>() {
            override fun onFailure(error: Int?, data: RecurringSlotsViewResponse?) {
                try {
                    pd.dismiss()
                    if (mContext != null) {
                        AsdaDialogHelper.displayErrorDialog(data, error, mContext, "Book Slot Confirmation")
                        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_BOOK_SLOT_ERROR, Anivia.SECTION_BOOK_SLOT)
                              .putString(Anivia.ERROR_EXPERIENCED_KEY, "book slot : " + if (data != null && !TextUtils.isEmpty(data.firstErrorMessage)) data.firstErrorMessage else ""))

                    }
                } catch (ignore: Exception) {
                    Log.d(TAG, "Error while: " + ignore.message)
                }
            }

            override fun onSuccess(data: RecurringSlotsViewResponse) {
                try {
                    RecurringSlotsViewResponse.broadcast(data)
                    pd.dismiss()
                    val aniviaEventShowRecConfirmation = PageViewEvent(Anivia.PV_BOOKSLOT_CONFIRM_RECURRING_SLOT, Anivia.SECTION_BOOK_SLOT)//UKAPPSAN109.2.2
                    aniviaEventShowRecConfirmation.putString(Anivia.RECURRING_SLOT_KEY, "yes : " + label?.toLowerCase(Locale.UK))
                    pageViewEvent = aniviaEventShowRecConfirmation
                    Tracker.get().trackPageView(aniviaEventShowRecConfirmation)
                    rememberSection?.visibility = View.VISIBLE
                    bookSlotButtonsHeader?.visibility = View.GONE
                    recurringSlotSwitchContainer?.visibility = View.GONE
                    val clickAndCollect = mBookedSlotType == CartManager.SLOT_TYPE_CC
                    val recurringSlotDateContainer = mView?.findViewById<ViewGroup>(R.id.next_recurring_slot_date)
                    if (data.recurringSlotDetails?.isNotEmpty() == true && data.recurringSlotDetails?.get(0) != null) {
                        (recurringSlotDateContainer?.findViewById<View>(R.id.next_recurring_slot_date_label) as TextView)
                              .setText(if (clickAndCollect) R.string.next_collection_date else R.string.next_delivery_date)
                        (recurringSlotDateContainer.findViewById<View>(R.id.next_recurring_slot_date_value) as TextView).text = Utils.slotIntervalToDescription(mContext, data.recurringSlotDetails?.get(0)?.date)
                        recurringSlotDateContainer.visibility = View.VISIBLE
                    } else {
                        recurringSlotDateContainer?.visibility = View.GONE
                    }
                    mSuccessSubtitleView?.setText(if (clickAndCollect) R.string.success_recurring_cnc_slot_booked else R.string.success_recurring_slot_booked)
                    bookedRecurringSlotFreqContainer?.visibility = View.VISIBLE
                    ViewUtil.setText(R.id.frequency_value, bookedRecurringSlotFreqContainer, label)
                    context?.let {
                        CartManager.get(it).refreshCart(it)
                    }

                    if (data.suggestedSlotInfo?.slotDays?.isNotEmpty() == true) {
                        val fragment = SlotSuggestionFragment.newInstance(slotStartDate,
                              slotEndDate,
                              spanId, label, data.suggestedSlotInfo)

                        fragment.suggestionSelectListener = object : SlotSuggestionFragment.SuggestionSelectListener {
                            override fun onSuggestionSelected(subscriptionCreation: SubscriptionCreation, label: String) {
                                createRecurringSlots(spanId, label, slotStartDate, slotEndDate, subscriptionCreation)
                            }
                        }
                        push(fragment)
                    }
                } catch (ignored: Exception) {
                    Log.e(TAG, "Error:" + ignored.message)
                }

            }
        }
        suggestedInfo?.let {
            AsdaServiceFactory.get().createSuggestedRecurringSlot(spanId, Authentication.getInstance().csrfToken,
                  slotStartDate,
                  slotEndDate,
                  it.data?.subscription_info?.suggested_start_time,
                  it.data?.subscription_info?.suggested_end_time, callback)
        } ?: run {
            AsdaServiceFactory.get().createRecurringSlot(spanId, slotStartDate, slotEndDate, Authentication.getInstance().csrfToken, callback)
        }
    }

    private fun addCartListener() {
        cartListener = object : ICartManager.CartListener() {
            override fun onChanged() {
                val cart = CartManager.get(mContext)?.cart
                var bookedTillTime: String?
                if (cart?.isRecurringSlot == true && cart.slotDate?.isNotEmpty() == true) {
                    cart.slotDate?.let {
                        bookedTillTime = SubscriptionUtil.getSubscriptionExpiryDate(it,
                              DateFormats.EEE_DD_MMM_YYYY,
                              DateFormats.HH_MM_AA_EEE_DD_MMM,
                              TimeZoneFormat.UK,
                              TimeZoneFormat.UK)
                        bookedUntilTextView?.text = mContext?.getString(getSlotConfirmationMessage(cart, false, Utils.isExpressEnabled(mContext)), bookedTillTime)
                    }
                }
            }
        }
        cartListener?.let {
            CartManager.get(context).addCartListener(it)
        }
    }

    private fun removeCartListener() {
        context?.let {
            if (cartListener != null) {
                CartManager.get(it).removeCartListener(cartListener)
            }
        }
    }


    companion object {
        const val TAG = "SlotBookedConfPresenter"
        private const val BOOKED_DELIVERY_METHOD = "booked_delivery_method"
        private const val AVAILABLE_SLOTS = "available_slots"
        private const val EXTRA_START_DATE = "extra_start_date"
        private const val EXTRA_END_DATE = "extra_end_date"
        const val PRIORITY_PASS = "priority pass"

        fun newInstance(availableSlots: BookSlotResponse.RecurringSlotInfo?,
                        bookedDeliveryMethod: Int,
                        startDate: String?,
                        endDate: String?,
                        subscriptionEligibilityInfo: SubscriptionEligibilityInfo? = null): SlotBookedConfirmationPresenter {
            val fragment = SlotBookedConfirmationPresenter()
            val args = Bundle()
            args.putInt(BOOKED_DELIVERY_METHOD, bookedDeliveryMethod)
            args.putParcelable(AVAILABLE_SLOTS, availableSlots)
            args.putString(EXTRA_START_DATE, startDate)
            args.putString(EXTRA_END_DATE, endDate)
            subscriptionEligibilityInfo?.let {
                args.putParcelable(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO, subscriptionEligibilityInfo)
            }
            fragment.arguments = args
            return fragment
        }
    }
}
