package com.asda.android.app.bookslotv3

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.restapi.app.cnc.model.SelectCNCModel
import com.asda.android.restapi.service.data.bookslotv3.AccessPoint
import com.asda.android.utils.Utils
import java.util.*

class CnCListV3Fragment : Fragment() {
    private var mLoadingView: View? = null
    private var mEmptyView: View? = null
    private var mAdapter: Adapter? = null
    private var mActivity: SelectCNCLocationActivityV3? = null

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is SelectCNCLocationActivityV3) {
            mActivity = context
        }
    }

    override fun onDetach() {
        super.onDetach()
        mActivity = null
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.locations_list, container, false)
        val list = view.findViewById<RecyclerView>(R.id.locations_list)
        list.layoutManager = LinearLayoutManager(mActivity)
        mAdapter = Adapter()
        list.adapter = mAdapter
        mLoadingView = view.findViewById(R.id.loading)
        mEmptyView = view.findViewById(R.id.no_items_found_text)
        mActivity?.onListFragmentInit()
        return view
    }

    fun showLoadingState() {
        if (mLoadingView != null) {
            mLoadingView?.visibility = View.VISIBLE
            mEmptyView?.visibility = View.GONE
        }
    }

    fun onLocationFindFail(): Boolean {
        return if (mLoadingView != null && mActivity != null) {
            if (mActivity?.loadStoresForPostcodeFromCart() == false) {
                mLoadingView?.visibility = View.GONE
                mEmptyView?.visibility = View.VISIBLE
            }
            true
        } else {
            false
        }
    }

    fun notifyDatasetChanged(cncLocations: ArrayList<AccessPoint?>) {
        if (mLoadingView != null) {
            mLoadingView?.visibility = View.GONE
            mEmptyView?.visibility = if (cncLocations.isEmpty()) View.VISIBLE else View.GONE
            mAdapter?.setItems(cncLocations)
        }
    }

    private inner class Adapter : RecyclerView.Adapter<Adapter.ViewHolder>() {
        private val mItems = ArrayList<AccessPoint?>()

        inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
            val storeName: TextView = itemView.findViewById(R.id.store_name)
            val address: TextView = itemView.findViewById(R.id.store_address)
            val collectionType: TextView = itemView.findViewById(R.id.collection_type)
            val distance: TextView = itemView.findViewById(R.id.store_distance)
            val express_logo: ImageView = itemView.findViewById(R.id.express_logo)
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            return ViewHolder(LayoutInflater.from(mActivity).inflate(R.layout.cc_list_entry, parent, false))
        }

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val place = mItems[position]
            if (place != null) {
                holder.itemView.isSelected = place.accesspoint_id != null && place.accesspoint_id == mActivity?.selectedLocationId
                val storeName = if (TextUtils.isEmpty(place.description)) mActivity?.getString(R.string.default_cnc_name) else place.description
                holder.storeName.text = storeName
                val collectionTypeDescription = place.collectionTypeDescription
                if (TextUtils.isEmpty(collectionTypeDescription) && !TextUtils.isEmpty(place.collectionTypeId)) {
                    place.collectionTypeDescription = SelectCNCLocationActivityV3.sCollectionTypeIdToDescription[place.collectionTypeId]
                }
                if (TextUtils.isEmpty(collectionTypeDescription)) {
                    holder.collectionType.visibility = View.GONE
                } else {
                    holder.collectionType.visibility = View.VISIBLE
                    holder.collectionType.text = collectionTypeDescription
                }
                holder.address.text = place.getFullAddress()
                if (place.distance != null) {
                    holder.distance.text = place.getDistanceLabel(resources)
                }
                if (Utils.isExpressEnabled(mActivity)) {
                    holder.express_logo.visibility = if (SelectCNCModel.getExpressVisibility(place) == true) View.VISIBLE else View.GONE
                }
                holder.itemView.setOnClickListener { mActivity?.selectCnCLocation(place) }
            }
        }


        override fun getItemCount(): Int {
            return mItems.size
        }

        fun setItems(cncLocations: ArrayList<AccessPoint?>) {
            mItems.clear()
            mItems.addAll(cncLocations)
            notifyDataSetChanged()
        }
    }
}
