package com.asda.android.app.bookslotv3

import android.Manifest
import android.app.Activity
import android.app.ProgressDialog
import android.content.ActivityNotFoundException
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.location.Location
import android.os.Bundle
import android.os.Handler
import android.provider.Settings
import android.util.Log
import android.view.*
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.collection.ArrayMap
import androidx.core.app.ActivityCompat
import androidx.core.view.MenuItemCompat
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.PV_CNC_LIST_VIEW
import com.asda.android.analytics.constants.Anivia.PV_CNC_MAP_VIEW
import com.asda.android.analytics.constants.Anivia.SECTION_BOOK_SLOT
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.JsonMapper
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.bookslot.ClickAndCollectCitiesSuggestionProvider
import com.asda.android.app.bookslotv3.service.BookSlotAPI
import com.asda.android.app.bookslotv3.service.BookSlotRepository
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.data.ASDAContentProvider
import com.asda.android.app.location.LocationInfoActivity
import com.asda.android.restapi.search.model.AsdaSearchData
import com.asda.android.search.data.RecentSearchDB
import com.asda.android.app.storelocator.view.AsdaStoreFinderConfigurator
import com.asda.android.app.storelocator.LocationHelper
import com.asda.android.base.core.view.ui.BaseAppCompatActivity
import com.asda.android.app.view.ExtendedSearchView
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.restapi.app.cnc.model.SelectCNCModel
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.bookslotv3.AccessPoint
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ReverseGeoCoderResult
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.CartManager
import com.asda.android.base.core.view.AsyncTask
import com.asda.android.utils.Utils
import com.google.android.gms.maps.*
import com.google.android.gms.maps.model.*
import com.asda.android.search.data.provider.LocationSuggestionProvider
import com.asda.android.restapi.search.model.SearchData
import com.asda.android.search.data.provider.SuggestionProvider
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.service.EventUtils
import com.asda.android.service.constants.EventConstants.CNC_STORE_CHANGED
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.Permissions
import com.asda.android.utils.LocationMath
import com.asda.android.utils.LocationUtils
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceObserver
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers
import java.io.IOException
import java.util.*
import kotlin.math.max
import kotlin.math.roundToInt

class SelectCNCLocationActivityV3 : BaseAppCompatActivity(), LocationHelper.ProgressListener {
    private var mSelectedLocation: String? = ""//currently selected and active CNC location
    private var myLocation: Location? = null
    private var mWaitingForMyLocation: Boolean = false
    private var mStoresLocation: LatLng? = null
    private var mStoresRadiusMeters = RADIUS_NO_STORES_HERE
    private var mListFragment: CnCListV3Fragment? = null
    private var mMapFragment: SupportMapFragment? = null
    private var mLocationHelper: LocationHelper? = null
    private val mAllFilters = TreeSet<String?> { lhs, rhs ->
        compareInts(sCollectionTypeIdToNumberOfTimesDownloaded[rhs],
              sCollectionTypeIdToNumberOfTimesDownloaded[lhs])
    }
    private var mPendingMapShow: Boolean = false
    private var mIsTransactionCommitNotAllowed: Boolean = false
    private var mGoogleMapMarkers: LinkedList<Marker>? = null
    private var mNetworkRequestInFlight: LatLng? = null
    private var mPendingNetworkRequest: LatLng? = null
    private var mSearchItem: MenuItem? = null
    private var mSearchText: String? = null
    private var storeId: String? = null
    private var postcode: String? = null
    private var fulfillmentType: String? = null
    private val mFilteredCncLocations = ArrayList<AccessPoint?>()
    private var mAllCncLocations: ArrayList<AccessPoint?>? = null
    private var mMapVisible: Boolean = false
    private var mGoogleMap: GoogleMap? = null
    private var mMapShowsWholeUK: Boolean = false
    private var mZoomToStoresOnLoad: Boolean = false
    private var mSearchProgressDialog: ProgressDialog? = null
    private var mActiveFilter: String? = ID_ALL_STORES
    private var mFilterAction: MenuItem? = null
    private var mHasGooglePlayServices: Boolean = false
    private var mButtonsPanel: View? = null
    private var mModeButton: TextView? = null
    private var mUserLocationMarker: Marker? = null
    private var mFindLocationProgressDialog: ProgressDialog? = null
    private var mLocationWasRequested: Boolean = false

    private val storesLatLng: LatLng?
        get() {
            if (mStoresLocation == null && myLocation == null) {
                return null
            }
            val lat = if (mStoresLocation == null) myLocation?.latitude else mStoresLocation?.latitude
            val lon = if (mStoresLocation == null) myLocation?.longitude else mStoresLocation?.longitude
            return if (lat != null && lon != null)
                LatLng(lat, lon)
            else null
        }

    val selectedLocationId: String?
        get() {
            if (mSelectedLocation?.isEmpty() == true) {
                for (accessPoint in mFilteredCncLocations) {
                    if (accessPoint?.fulfill_store_id != null && accessPoint.fulfill_store_id == storeId
                          && accessPoint.service_info != null && accessPoint.service_info?.fulfillment_type != null && accessPoint.service_info?.fulfillment_type == fulfillmentType
                          && accessPoint.service_address != null && accessPoint.service_address?.postcode != null && accessPoint.service_address?.postcode == postcode) {
                        mSelectedLocation = accessPoint.accesspoint_id
                    }
                }
            }
            return mSelectedLocation
        }

    private var mSearchViewAction: ExtendedSearchView? = null

    private val currentViewTypeString: String
        get() = if (mMapVisible) "map" else "list"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Utils.addNewRefToLeakDetector(TAG, this)
        setContentView(R.layout.select_cnc_location)
        initFilters()
        val ab = supportActionBar
        if (ab != null) {
            ab.setDisplayHomeAsUpEnabled(true)
            ab.setTitle(R.string.select_cc_location_title)
        } else {
            setTitle(R.string.select_cc_location_title)
        }
        getBundleValues()
        val fragmentTransaction = BaseFragment.getASDAFragmentTransaction(supportFragmentManager)
        mListFragment = CnCListV3Fragment()
        mListFragment?.let {
            fragmentTransaction.add(R.id.fragment_container, it)
            fragmentTransaction.commit()
        }
        mLocationHelper = LocationHelper(this)
        mMapVisible = false
        mLocationWasRequested = false
        mHasGooglePlayServices = Utils.isGPlayAvailable(this)
        mButtonsPanel = findViewById(R.id.buttons_container)
        val myLocationButton = mButtonsPanel?.findViewById<View>(R.id.map_header_location_button)
        myLocationButton?.setOnClickListener { v -> requestMyLocation(true) }
        mModeButton = mButtonsPanel?.findViewById(R.id.map_header_list_button)
        mModeButton?.setOnClickListener { v ->
            val selectorType = currentViewTypeString
            Tracker.get().trackEvent(AsdaAnalyticsEvent("locationInteraction")
                  .putString("viewSelected", selectorType))//UKAPPSAN203_locationInteraction
            val pageViewEvent: PageViewEvent
            if (selectorType.toLowerCase().equals("map", ignoreCase = true)) {
                pageViewEvent = PageViewEvent(PV_CNC_LIST_VIEW, SECTION_BOOK_SLOT)
            } else {
                pageViewEvent = PageViewEvent(PV_CNC_MAP_VIEW, SECTION_BOOK_SLOT)
            }
            Tracker.get().trackPageView(pageViewEvent)
            switchMapListMode()
        }
        if (mHasGooglePlayServices) {
            switchMapListMode()
            //location will be requested after map will be ready to avoid conflicts with camera move
        } else {
            requestMyLocation(true)
        }
        Handler().postDelayed({
            if (!isFinishing && !mLocationWasRequested) {
                requestMyLocation(true)//just in case onCameraChange will not be invoked
            }
        }, 2000)
    }

    private fun getBundleValues() {
        if (intent != null) {
            if (intent.hasExtra(EXTRA_SELECTED_CNC_ID)) {
                mSelectedLocation = intent.getStringExtra(EXTRA_SELECTED_CNC_ID)
            }
            if (intent.hasExtra(EXTRA_SELECTED_STORE_ID)) {
                storeId = intent.getStringExtra(EXTRA_SELECTED_STORE_ID)
            }
            if (intent.hasExtra(EXTRA_SELECTED_STORE_FULFILLMENT_TYPE)) {
                fulfillmentType = intent.getStringExtra(EXTRA_SELECTED_STORE_FULFILLMENT_TYPE)
            }
            if (intent.hasExtra(EXTRA_SELECTED_STORE_POSTCODE)) {
                postcode = intent.getStringExtra(EXTRA_SELECTED_STORE_POSTCODE)
            }
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        if (requestCode == Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION) {
            Permissions.onLocationPermissionRequestReceived(this)
            if (Permissions.isLocationPermissionGranted(this)) {
                LocalBroadcastManager.getInstance(this).sendBroadcast(Intent(Permissions.LocationPermissionGrantedEvent.ACTION))
                requestMyLocation(true)
            } else {
                Toast.makeText(this, R.string.no_permission_for_location, Toast.LENGTH_SHORT).show()
            }
        }
    }

    override fun onResumeFragments() {
        super.onResumeFragments()
        mIsTransactionCommitNotAllowed = false
        if (mPendingMapShow) {
            mPendingMapShow = false
            switchMapListMode()
        }
    }

    override fun onStart() {
        super.onStart()
    }

    override fun onPause() {
        super.onPause()
        mIsTransactionCommitNotAllowed = true
    }

    private fun initFilters() {
        mAllFilters.add(ID_ALL_STORES)

        sCollectionTypeIdToDescription[ID_ALL_STORES] = getString(R.string.filter_cnc_all)
        sCollectionTypeIdToDescription["drivethru"] = getString(R.string.filter_cnc_drive_thru)
        sCollectionTypeIdToDescription["petrol"] = getString(R.string.filter_cnc_petrol)
        sCollectionTypeIdToDescription["petrol station"] = getString(R.string.filter_cnc_petrol)
        sCollectionTypeIdToDescription["tube"] = getString(R.string.filter_cnc_tube)
        sCollectionTypeIdToDescription["internal"] = getString(R.string.filter_cnc_internal)
        sCollectionTypeIdToDescription["clickcall"] = getString(R.string.filter_cnc_clickcall)
        sCollectionTypeIdToDescription["lockers collection"] = getString(R.string.filter_cnc_lockers)
        sCollectionTypeIdToDescription["parkride"] = getString(R.string.filter_cnc_parkride)
        sCollectionTypeIdToDescription["supermarket"] = getString(R.string.filter_cnc_supermarket)
        sCollectionTypeIdToDescription["superstore"] = getString(R.string.filter_cnc_superstore)
        sCollectionTypeIdToDescription["living"] = getString(R.string.filter_cnc_living)

        sCollectionTypeIdToNumberOfTimesDownloaded[ID_ALL_STORES] = Integer.MAX_VALUE
    }

    private fun switchMapListMode() {
        if (mIsTransactionCommitNotAllowed) {
            mPendingMapShow = true
        } else {
            if (!mMapVisible) {
                if (mMapFragment == null) {
                    val position: CameraPosition
                    val location = storesLatLng
                    if (location == null) {
                        position = CameraPosition.fromLatLngZoom(LatLng(54.65, -2.9), 5.5f)
                        mMapShowsWholeUK = true
                    } else {
                        position = CameraPosition.fromLatLngZoom(location, DETAILED_ZOOM)
                    }
                    mMapFragment = SupportMapFragment.newInstance(GoogleMapOptions().camera(position))

                    mMapFragment?.getMapAsync { googleMap ->
                        mGoogleMap = googleMap
                        mGoogleMapMarkers = LinkedList()
                        googleMap.setInfoWindowAdapter(CnCLocationInfoPopupAdapter(findViewById(R.id.fragment_container)))
                        googleMap.setOnInfoWindowClickListener { marker ->
                            if (marker.snippet != null) {
                                try {
                                    selectCnCLocation(JsonMapper.get().readValue(marker.snippet, AccessPoint::class.java))
                                } catch (e: IOException) {
                                    Log.e(TAG, "unable to read marker", e)
                                }

                            }
                        }
                        googleMap.setOnCameraChangeListener { position1 ->
                            if (mStoresLocation != null && mStoresRadiusMeters > 1) {
                                mStoresLocation?.let {
                                    val distFromStoresCenterMeters = LocationMath.computeDistance(position1.target.latitude, position1.target.longitude,
                                          it.latitude, it.longitude, 'm')
                                    if (distFromStoresCenterMeters > mStoresRadiusMeters / 3) {
                                        reloadCNCLocationsForGivenLatLon(position1.target, false)
                                    }
                                }
                            } else if (!mLocationWasRequested) {
                                requestMyLocation(true)
                            }

                        }

                        if (ActivityCompat.checkSelfPermission(this@SelectCNCLocationActivityV3, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED || ActivityCompat.checkSelfPermission(this@SelectCNCLocationActivityV3, Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
                            googleMap.isMyLocationEnabled = false
                        }
                        refreshMapMarkers()
                    }
                }
            }
            mMapVisible = !mMapVisible

            updateModeButtons()
            val fragmentTransaction = BaseFragment.getASDAFragmentTransaction(supportFragmentManager)
            if (mMapVisible) {
                mListFragment?.let {
                    fragmentTransaction.hide(it)
                }
                mMapFragment?.let { mapFragment ->
                    if (mMapFragment?.isHidden == true) {
                        fragmentTransaction.show(mapFragment)
                    } else {
                        fragmentTransaction.add(R.id.fragment_container, mapFragment)
                    }
                }
            } else {
                mMapFragment?.let {
                    fragmentTransaction.hide(it)
                }

                mListFragment?.let {
                    fragmentTransaction.show(it)
                }
            }
            fragmentTransaction.commit()
        }
    }

    private fun updateMyLocationMarkerOnMap(moveMapToMyLocation: Boolean) {
        if (mGoogleMap != null) {
            if (myLocation != null) {
                val myLatLng = LatLng(myLocation?.latitude ?: 0.0, myLocation?.longitude ?: 0.0)
                if (mUserLocationMarker == null) {
                    mUserLocationMarker = mGoogleMap?.addMarker(MarkerOptions()
                          .anchor(0.5f, 0.5f)
                          .flat(true)
                          .position(myLatLng)
                          .icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_maps_indicator_current_position)))
                } else {
                    mUserLocationMarker?.position = myLatLng
                }
                if (moveMapToMyLocation) {
                    mMapShowsWholeUK = false
                    mGoogleMap?.moveCamera(CameraUpdateFactory.newLatLngZoom(myLatLng, DETAILED_ZOOM))
                } else if (mMapShowsWholeUK) {
                    mMapShowsWholeUK = false
                    mGoogleMap?.moveCamera(CameraUpdateFactory.zoomTo(DETAILED_ZOOM))
                }
            } else if (mUserLocationMarker != null) {
                mUserLocationMarker?.remove()
                mUserLocationMarker = null
            }
        }
    }

    override fun onDestroy() {
        mLocationHelper?.setProgressListener(null)
        mLocationHelper?.clearAllPendingLocationRequests()
        super.onDestroy()
    }

    private fun stopLocationDetection(foundLocation: Location?) {
        myLocation = foundLocation
        if (mWaitingForMyLocation) {
            mWaitingForMyLocation = false
            if (mFindLocationProgressDialog != null) {
                try {
                    mFindLocationProgressDialog?.dismiss()
                } catch (ignored: Exception) {
                }

                mFindLocationProgressDialog = null
                if (myLocation == null) {
                    DialogHelper.displayErrorDialog(R.string.store_finder_dialog_location_fix_timeout, this, SCREEN_NAME)
                }
            }
            updateMyLocationMarkerOnMap(true)
            myLocation?.let {
                reloadCNCLocationsForGivenLatLon(LatLng(it.latitude, it.longitude), true)
            } ?: run {
                if (mListFragment?.onLocationFindFail() == false) {
                    loadStoresForPostcodeFromCart()
                }
            }
        }
        mLocationHelper?.setProgressListener(null)
    }

    private fun reloadCNCLocationsForGivenPostcodeOrCity(searchText: String) {
        reloadCNCLocationsForGivenPostcode(ClickAndCollectCitiesSuggestionProvider.getFirstCityMatch(searchText))
    }

    internal fun reloadCNCLocationsForGivenPostcode(postcode: String?) {
        reloadCNCLocations(null, postcode)
    }

    internal fun reloadCNCLocationsForGivenLatLon(latLon: LatLng, moveMap: Boolean) {
        mStoresRadiusMeters = RADIUS_NO_STORES_HERE
        mStoresLocation = LatLng(latLon.latitude, latLon.longitude)
        mZoomToStoresOnLoad = moveMap
        if (moveMap && mGoogleMap != null) {
            mMapShowsWholeUK = false
            mGoogleMap?.moveCamera(CameraUpdateFactory.newLatLngZoom(LatLng(mStoresLocation?.latitude
                  ?: 0.0, mStoresLocation?.longitude ?: 0.0), DETAILED_ZOOM))
        }
        reloadCNCLocations(latLon, null)
    }

    private fun reloadCNCLocations(latLon: LatLng?, postcode: String?) {
        val callbackResourceObserver = object : ResourceObserver<ServiceEligibilityData>() {
            override fun onComplete() {
                dispose()
            }

            override fun onNext(data: ServiceEligibilityData) {
                onServiceEligibilitySuccess(data, latLon)
            }

            override fun onError(error: Throwable) {
                onServiceEligibilityFailure()
                dispose()
            }
        }

        if (latLon != null) {
            if (mNetworkRequestInFlight == null) {//no need to do that for manual search by postcode/town
                mNetworkRequestInFlight = latLon

                val tempPostCode = if (postcode.isNullOrEmpty()) mSearchText else postcode
                if (!tempPostCode.isNullOrEmpty()) {
                    ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", BookSlotAPI::class.java)?.getServiceEligibility(tempPostCode, latLon.latitude.toFloat(), latLon.longitude.toFloat())?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())?.safeSubscribe(callbackResourceObserver)
                } else {
                    //Get the postcode from lat lon and then make the call.
                    val latlon = latLon.latitude.toString() + "," + latLon.longitude
                    ServiceGenerator.createService("https://maps.googleapis.com/", "", BookSlotAPI::class.java)?.getAddressFromLatLon(latlng = latlon, key = getString(R.string.google_maps_api_key))?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())?.safeSubscribe(object : ResourceObserver<ReverseGeoCoderResult>() {
                        override fun onComplete() {
                            dispose()
                        }

                        override fun onNext(result: ReverseGeoCoderResult) {

                            var postCodeFound = false

                            var i = 0
                            result.address?.let {
                                while (i < it.size && !postCodeFound) {
                                    val (addressComponents) = it[i]
                                    val size = addressComponents?.size ?: 0
                                    var addressCompIndex = size - 1
                                    addressComponents?.let { iTAddressComponents ->

                                        while (addressCompIndex >= 0 && !postCodeFound) {
                                            val addressComponent = iTAddressComponents[addressCompIndex]
                                            val types = addressComponent.types
                                            val longName = addressComponent.longName

                                            if (types?.contains("postal_code") == true) {
                                                longName?.let {
                                                    ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", BookSlotAPI::class.java)?.getServiceEligibility(longName, latLon.latitude.toFloat(), latLon.longitude.toFloat())?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())?.safeSubscribe(callbackResourceObserver)
                                                    postCodeFound = true
                                                }
                                                break
                                            }
                                            addressCompIndex--
                                        }
                                    }
                                    i++
                                }
                            }
                        }

                        override fun onError(e: Throwable) {
                            dispose()
                        }
                    })
                }
            } else if (LocationMath.computeDistance(mNetworkRequestInFlight?.latitude
                        ?: 0.0, mNetworkRequestInFlight?.longitude
                        ?: 0.0, latLon.latitude, latLon.longitude, 'M') > 1) {
                Log.v(TAG, "postpone cnc request cnc for $latLon")
                mPendingNetworkRequest = latLon
            } else {
                Log.v(TAG, "ignoring cnc request cnc for $latLon")
            }
        } else if (!postcode.isNullOrBlank()) {
            BookSlotRepository().makeServiceEligibilityCall(postcode)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
                override fun onSuccess(serviceEligibilityData: ServiceEligibilityData) {
                    onServiceEligibilitySuccess(serviceEligibilityData, latLon)
                }

                override fun onError(e: Throwable) {
                    onServiceEligibilityFailure()
                }
            })
        }
    }

    private fun onServiceEligibilitySuccess(data: ServiceEligibilityData?, latLon: LatLng?) {
        mNetworkRequestInFlight = null
        dismissSearchStoresProgressDialog()
        val newCncLocations = ArrayList<AccessPoint?>()
        if (data?.data != null && data.data?.service_eligibility != null && data.data?.service_eligibility?.size ?: 0 > 0 &&
              data.data?.service_eligibility?.get(0)?.accesspoint_list != null) {

            val accessPoints = data.data?.service_eligibility?.get(0)?.accesspoint_list as ArrayList<AccessPoint?>?

            accessPoints?.let {
                for (cncStore in it) {
                    if (cncStore != null && !(cncStore.fulfill_store_id.isNullOrEmpty())
                          && cncStore.service_info != null && cncStore.service_info?.fulfillment_type != null
                          && FulfillmentTypes.DELIVERY.type != cncStore.service_info?.fulfillment_type) {
                        if (myLocation != null) {
                            try {
                                cncStore.distance = LocationMath.computeDistance(myLocation?.latitude
                                      ?: 0.0, myLocation?.longitude
                                      ?: 0.0, java.lang.Long.parseLong(cncStore.service_address?.latitude
                                      ?: "0").toDouble(), java.lang.Long.parseLong(cncStore.service_address?.longitude
                                      ?: "0").toDouble(), 'M').roundToInt()
                            } catch (e: Exception) {
                                cncStore.distance = null
                                Log.w(TAG, "failed to parse $cncStore")
                            }

                        }
                        newCncLocations.add(cncStore)
                    }
                }
            }
            if (latLon != null) {
                mStoresLocation = latLon
            } else if (newCncLocations.size > 0) {
                mStoresLocation = findMiddle(newCncLocations)
            }
            mStoresRadiusMeters = RADIUS_NO_STORES_HERE
            if (mStoresLocation != null) {
                for (location in newCncLocations) {
                    try {
                        val lat = java.lang.Double.parseDouble(location?.service_address?.latitude
                              ?: "0")
                        val lon = java.lang.Double.parseDouble(location?.service_address?.longitude
                              ?: "0")
                        mStoresRadiusMeters = max(mStoresRadiusMeters, LocationMath.computeDistance(lat, lon,
                              mStoresLocation?.latitude ?: 0.0, mStoresLocation?.longitude
                              ?: 0.0, 'm'))
                    } catch (e: Exception) {
                        Log.w(TAG, "failed to parse $location")
                    }

                }
            }
        } else {
            Log.e(TAG, "unable to find CC store")
        }
        filterAndShowTheseLocations(newCncLocations)
        mAllFilters.clear()
        mAllFilters.add(ID_ALL_STORES)
        mAllFilters.add(mActiveFilter)
        for (loc in newCncLocations) {
            if (!loc?.collectionTypeId.isNullOrBlank()) {
                if (sCollectionTypeIdToNumberOfTimesDownloaded.containsKey(loc?.collectionTypeId)) {
                    sCollectionTypeIdToNumberOfTimesDownloaded[loc?.collectionTypeId] = (sCollectionTypeIdToNumberOfTimesDownloaded[loc?.collectionTypeId]
                          ?: 0) + 1
                } else {
                    sCollectionTypeIdToNumberOfTimesDownloaded[loc?.collectionTypeId] = 1
                }

                if (sCollectionTypeIdToDescription.containsKey(loc?.collectionTypeId)) {
                    mAllFilters.add(loc?.collectionTypeId)
                } else {
                    Log.w(TAG, "Unknown collection type id found with no description: " + loc?.collectionTypeId)
                }
            }
        }
        startPendingRequest()
    }

    private fun onServiceEligibilityFailure() {
        mNetworkRequestInFlight = null
        dismissSearchStoresProgressDialog()
        //                AsdaDialogHelper.displayErrorDialog(data, error, SelectCNCLocationActivityV3.this, SCREEN_NAME); //TODO check this
        filterAndShowTheseLocations(ArrayList())
        startPendingRequest()
    }

    private fun startPendingRequest() {
        if (mPendingNetworkRequest != null) {
            reloadCNCLocations(mPendingNetworkRequest, null)
            mPendingNetworkRequest = null
        }
    }

    private fun findMiddle(mCncLocations: ArrayList<AccessPoint?>): LatLng? {
        try {
            var latSum = 0.0
            var lonSum = 0.0
            var count = 0
            for (location in mCncLocations) {
                try {
                    val lat = java.lang.Double.parseDouble(location?.service_address?.latitude
                          ?: "0")
                    val lon = java.lang.Double.parseDouble(location?.service_address?.longitude
                          ?: "0")
                    latSum += lat
                    lonSum += lon
                    count++
                } catch (e: Exception) {
                    Log.w(TAG, "failed to parse $location")
                }

            }
            return if (count == 0) null else LatLng(latSum / count, lonSum / count)
        } catch (e: Exception) {
            //just to make sure it will not crash over pacific ocean
            Log.e(TAG, "ouch", e)
        }

        return null
    }


    private fun refreshMapMarkers() {
        if (mGoogleMap != null) {
            val alreadyShownLocationIds = removeAllMarkersExcept(mFilteredCncLocations)
            mUserLocationMarker = null
            updateMyLocationMarkerOnMap(false)
            for (location in mFilteredCncLocations) {
                try {
                    if (!alreadyShownLocationIds.contains(location?.accesspoint_id)) {
                        if (Utils.isExpressEnabled(this@SelectCNCLocationActivityV3) && SelectCNCModel.getExpressVisibility(location) == true) {
                            addMarkersToMap(location, R.drawable.checked_store, R.drawable.express_store)
                        } else {
                            addMarkersToMap(location, R.drawable.checked_store, R.drawable.map_pin_filled)
                        }
                    }
                } catch (parseException: Exception) {
                    Log.w(TAG, "failed to parse $location")
                }

            }
        }
    }

    private fun addMarkersToMap(loc: AccessPoint?, map_selected: Int, map_pin: Int) {
        val alreadySelected = loc?.accesspoint_id != null && loc.accesspoint_id == selectedLocationId
        val lat = java.lang.Double.parseDouble(loc?.service_address?.latitude ?: "0.0")
        val lon = java.lang.Double.parseDouble(loc?.service_address?.longitude ?: "0.0")

        val marker = mGoogleMap?.addMarker(MarkerOptions()
              .position(LatLng(lat, lon))
              .title(loc?.description)
              .snippet(loc?.getAsJSON())
              .icon(BitmapDescriptorFactory.fromResource(if (alreadySelected) map_selected else map_pin)))
        marker?.let {
            mGoogleMapMarkers?.add(it)
        }
    }

    private fun removeAllMarkersExcept(filteredCncLocations: ArrayList<AccessPoint?>?): HashSet<String?> {
        val alreadyShownLocationIds = HashSet<String?>()
        if (filteredCncLocations != null && mGoogleMapMarkers != null) {
            val cncLocationIds = HashSet<String?>()
            for (filLoc in filteredCncLocations) {
                cncLocationIds.add(filLoc?.accesspoint_id)
            }
            val iterator = mGoogleMapMarkers?.iterator()
            while (iterator?.hasNext() == true) {
                val marker = iterator.next()
                val snippet = marker.snippet
                var tag: AccessPoint? = null
                try {
                    tag = if (snippet.isNullOrEmpty()) null else JsonMapper.get().readValue(snippet, AccessPoint::class.java)
                } catch (e: Exception) {
                    Log.e(TAG, "snippet parse error", e)
                }

                if (tag?.accesspoint_id == null || !cncLocationIds.contains(tag.accesspoint_id)) {
                    marker.remove()
                    iterator.remove()
                } else if (tag.accesspoint_id != null) {
                    alreadyShownLocationIds.add(tag.accesspoint_id)
                }
            }
        }
        return alreadyShownLocationIds
    }

    internal fun selectCnCLocation(cncStore: AccessPoint) {
        EventUtils.broadcast(this@SelectCNCLocationActivityV3, CNC_STORE_CHANGED)
        val intent = Intent()
        intent.putExtra(EXTRA_SELECTED_CNC_JSON, cncStore.getAsJSON())
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    internal fun requestMyLocation(showLocationOffPopup: Boolean) {
        mLocationWasRequested = true
        mWaitingForMyLocation = true
        mLocationHelper?.setProgressListener(this)
        if (mLocationHelper?.findMyLocationSilent(2000) == true) {
            if (mFindLocationProgressDialog == null) {
                mFindLocationProgressDialog = CustomProgressDialog.create(this)
                mFindLocationProgressDialog?.setMessage(getString(R.string.store_finder_dialog_find_location))
                mFindLocationProgressDialog?.isIndeterminate = true
                try {
                    mFindLocationProgressDialog?.show()
                } catch (e: Exception) {
                    Log.e(TAG, "unable to show dialog", e)
                }

            }
        } else {
            stopLocationDetection(null)
            if (showLocationOffPopup) {
                if (!Permissions.isLocationPermissionGranted(this@SelectCNCLocationActivityV3)) {
                    val showLocationInfo = SharedPreferencesUtil.showLocationInfo(baseContext)
                    if (showLocationInfo) {
                        startActivity(Intent(baseContext, LocationInfoActivity::class.java))
                    } else {
                        Permissions.requestLocationPermission(this@SelectCNCLocationActivityV3)
                    }
                } else {
                    try {
                        AlertDialog.Builder(this)
                              .setMessage(R.string.location_dialog_activate_source)
                              .setPositiveButton(android.R.string.yes) { dialog, which ->
                                  if (Permissions.isLocationPermissionGranted(this@SelectCNCLocationActivityV3)) {
                                      val locationSettingsIntent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
                                      locationSettingsIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                      try {
                                          startActivityForResult(locationSettingsIntent, REQUEST_LOCATION_ON)
                                      } catch (e: ActivityNotFoundException) {
                                          // If we can't launch location settings, try to launch the main settings
                                          try {
                                              val mainSettingsIntent = Intent(Settings.ACTION_SETTINGS)
                                              mainSettingsIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                              startActivityForResult(mainSettingsIntent, REQUEST_LOCATION_ON)
                                          } catch (e1: ActivityNotFoundException) {
                                              // Silently fail
                                          }

                                      }

                                  } else {
                                      Permissions.openPermissionsWindow(this@SelectCNCLocationActivityV3)
                                  }
                              }
                              .setNegativeButton(android.R.string.no, null)
                              .create().show()
                    } catch (e: Exception) {
                        Log.e(TAG, "no location enabled", e)
                    }

                }
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_LOCATION_ON) {
            requestMyLocation(false)//it doesn't work because android still says it's turned off for some time
        }
    }

    override fun onLocationFound(location: Location) {
        stopLocationDetection(location)
    }

    override fun onLastLocationFound(location: Location) {
        stopLocationDetection(location)
    }

    override fun onLocationFixTimeout() {
        stopLocationDetection(myLocation)
    }

    fun onListFragmentInit() {
        if (mWaitingForMyLocation) {
            mListFragment?.showLoadingState()
        } else {
            mListFragment?.notifyDatasetChanged(mFilteredCncLocations)
        }
    }

    fun loadStoresForPostcodeFromCart(): Boolean {
        val postcode: String? = if (CartManager.get(this)?.cartExtraInfo != null
              && CartManager.get(this)?.cartExtraInfo?.cncStoreAddress != null
              && CartManager.get(this)?.cartExtraInfo?.cncStoreAddress?.postalCode != null) {
            CartManager.get(this)?.cartExtraInfo?.cncStoreAddress?.postalCode
        } else {
            SlotUtils().getPostCodeForServiceEligibility()
        }
        return if (!postcode.isNullOrEmpty()) {
            mZoomToStoresOnLoad = mMapShowsWholeUK
            reloadCNCLocationsForGivenPostcode(if (this.postcode != null) this.postcode else postcode)
            true
        } else {
            false
        }
    }

    internal inner class CnCLocationInfoPopupAdapter(parent: ViewGroup) : GoogleMap.InfoWindowAdapter {
        private val mView: View = LayoutInflater.from(this@SelectCNCLocationActivityV3).inflate(R.layout.cnc_map_marker, parent, false)
        private val express_icon: ImageView

        init {
            express_icon = mView.findViewById(R.id.express_logo)
        }

        override fun getInfoWindow(marker: Marker): View? {
            var viewToReturn: View? = null
            try {
                if (marker.snippet != null) {
                    val tag = JsonMapper.get().readValue(marker.snippet, AccessPoint::class.java)
                    ViewUtil.setTextHideIfEmpty(R.id.title, mView, tag.description)
                    var collectionTypeDescription = tag.collectionTypeDescription
                    if (collectionTypeDescription.isNullOrEmpty() && !tag.collectionTypeId.isNullOrEmpty()) {
                        collectionTypeDescription = sCollectionTypeIdToDescription[tag.collectionTypeId]
                    }
                    ViewUtil.setTextHideIfEmpty(R.id.collection_type, mView, collectionTypeDescription)
                    ViewUtil.setTextHideIfEmpty(R.id.address, mView, tag.getFullAddress())
                    if (Utils.isExpressEnabled(this@SelectCNCLocationActivityV3)) {
                        express_icon.visibility = if (SelectCNCModel.getExpressVisibility(tag) == true) View.VISIBLE else View.GONE
                    }
                    viewToReturn = mView
                }
            } catch (e: Exception) {
                Log.e(TAG, "unable to show tag for store", e)
            }

            return viewToReturn
        }

        override fun getInfoContents(marker: Marker): View? {
            return null
        }
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.cnc_selector_menu, menu)
        mFilterAction = menu.findItem(R.id.action_filter)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                finish()
                return true
            }
            R.id.action_filter -> {
                val builder = AlertDialog.Builder(this)
                builder.setTitle(R.string.store_finder_dialog_filter_title)
                val filterIds = mAllFilters.toTypedArray()
                val filterLabels = arrayOfNulls<String>(mAllFilters.size)
                for (i in filterIds.indices) {
                    filterLabels[i] = sCollectionTypeIdToDescription[filterIds[i]]
                }
                val activeFilterPosition = listOf(*filterIds).indexOf(mActiveFilter)
                builder.setSingleChoiceItems(filterLabels, activeFilterPosition) { dialog, which ->
                    mActiveFilter = filterIds[which]
                    mFilterAction?.setIcon(if (mActiveFilter.isNullOrEmpty()) R.drawable.ftricn_filter else R.drawable.ic_filter_on_green)
                    filterAndShowTheseLocations(mAllCncLocations)
                    dialog.dismiss()

                    Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.FILTER_INTERACTION_EVENT)//UKAPPSAN204
                          .putString("filterName", filterLabels[which])
                          .putString(Anivia.LOCATION_KEY, "C&C store location $currentViewTypeString"))
                }
                builder.setNegativeButton(R.string.cancel, null)
                builder.create().show()
                return true
            }
        }

        return super.onOptionsItemSelected(item)
    }

    private fun filterAndShowTheseLocations(cncLocations: ArrayList<AccessPoint?>?) {
        mAllCncLocations = cncLocations
        mFilteredCncLocations.clear()
        if (cncLocations != null) {
            for (location in cncLocations) {
                val collectionTypeId = location?.collectionTypeId
                val collectionTypeDescription = location?.collectionTypeId
                if (!collectionTypeId.isNullOrBlank() && !collectionTypeDescription.isNullOrBlank()) {
                    sCollectionTypeIdToDescription[collectionTypeId] = location.collectionTypeDescription
                }

                if (mActiveFilter.isNullOrBlank() || mActiveFilter == location?.collectionTypeId) {
                    mFilteredCncLocations.add(location)
                }
            }
        }
        mListFragment?.notifyDatasetChanged(mFilteredCncLocations)
        refreshMapMarkers()
        val location = storesLatLng//if this point is null then it means that no stores were loaded yet
        if (mZoomToStoresOnLoad && mGoogleMap != null && mMapFragment != null && location != null) {
            mZoomToStoresOnLoad = false
            val builder = LatLngBounds.Builder()
            builder.include(location)
            val stores = TreeMap<Double, AccessPoint?>()
            var needToMoveMapToShowStores = true
            for (store in mFilteredCncLocations) {
                try {
                    val lat = java.lang.Double.parseDouble(store?.service_address?.latitude ?: "0")
                    val lon = java.lang.Double.parseDouble(store?.service_address?.longitude ?: "0")
                    val distance = LocationMath.computeDistance(location.latitude, location.longitude, lat, lon, 'M')
                    if (distance < DEFAULT_MAX_RADIUS) {
                        stores[distance] = store
                    }
                    if (distance < DEFAULT_MIN_RADIUS) {
                        needToMoveMapToShowStores = mMapShowsWholeUK
                    }
                } catch (e: Exception) {
                    Log.w(TAG, "Failed to parse location")
                }

            }
            if (needToMoveMapToShowStores) {//no stores to show near by, zoom out to show some
                var spanPointsCount = 1
                for (store in stores.values) {
                    builder.include(LatLng(java.lang.Double.parseDouble(store?.service_address?.latitude
                          ?: "0.0"), java.lang.Double.parseDouble(store?.service_address?.longitude
                          ?: "0.0")))
                    spanPointsCount++
                    if (spanPointsCount > 5) {//to show approximately this number of c&c locations around
                        break
                    }
                }
                var cameraUpdate: CameraUpdate? = null
                try {
                    if (spanPointsCount >= 3) {//my location and at least two stores around should be in the span to make it valid
                        val mapView = mMapFragment?.view
                        if (mapView != null && mapView.width > 0) {
                            val maxPaddingInPixels = (0.8 * (Math.min(mapView.width, mapView.height) / 2)).toInt()//80% of map size, ~128dp for the smallest screen (320dp)
                            val paddingDp = 48
                            val paddingInPixels = Math.min(ViewUtil.dpToPixels(paddingDp, this), maxPaddingInPixels)
                            cameraUpdate = CameraUpdateFactory.newLatLngBounds(builder.build(), paddingInPixels)
                        }
                    }
                    if (cameraUpdate != null) {
                        mGoogleMap?.animateCamera(cameraUpdate)
                    }
                } catch (e: NullPointerException) {
                    //CameraUpdateFactory can throw NPE for unknown reason
                    //see https://wmobile.atlassian.net/browse/ASDAA-2932
                    Log.e(TAG, "failed to show requested location", e)
                }

            }
        }
    }

    override fun onPreparePanel(featureId: Int, view: View?, menu: Menu): Boolean {
        mSearchItem = menu.findItem(R.id.action_search)
        mSearchItem?.let {
            mSearchViewAction = MenuItemCompat.getActionView(it) as ExtendedSearchView
        }
        val searchManager = getSystemService(Context.SEARCH_SERVICE) as android.app.SearchManager
        mSearchViewAction?.setSearchableInfo(searchManager.getSearchableInfo(ComponentName(applicationContext, SelectCNCLocationActivityV3::class.java)))
        mSearchViewAction?.setOnSuggestionClickListener(object : ExtendedSearchView.OnSuggestionClickListener {
            override fun onItemSelected(searchData: AsdaSearchData) {
                onSearchRequested(searchData)
            }

            override fun onQuerySubmitted(query: String) {
                onSearchRequested(
                      AsdaSearchData(
                            query,
                            SearchData.SearchType.SEARCH_TYPE_EXACT,
                            null
                      )
                )
            }
        })
        val suggestionProvider: SuggestionProvider<AsdaSearchData>
        if (mHasGooglePlayServices) {
            val locationSuggestionProvider = object : LocationSuggestionProvider<AsdaSearchData>(getString(R.string.google_maps_api_key)) {
                override fun createSearchData(suggestion: String): AsdaSearchData {
                    return AsdaSearchData(
                          suggestion,
                          SearchData.SearchType.SEARCH_TYPE_SUGGESTION,
                          null
                    )
                }
            }
            locationSuggestionProvider.setCountryRestriction("uk")
            locationSuggestionProvider.setReferrer(AsdaStoreFinderConfigurator.ASDA_REFERRER)
            suggestionProvider = locationSuggestionProvider
        } else {
            suggestionProvider = ClickAndCollectCitiesSuggestionProvider()
        }
        val recentSearchDB = RecentSearchDB(this, ASDAContentProvider.PATH_RECENT_STORE_SEARCHES)
        mSearchViewAction?.setSuggestionsProvider(suggestionProvider, recentSearchDB)
        mSearchViewAction?.setTextHint(getString(R.string.search_hint_store))
        updateModeButtons()
        return super.onPreparePanel(featureId, view, menu)
    }

    private fun updateModeButtons() {
        if (mHasGooglePlayServices) {
            mModeButton?.setCompoundDrawablesWithIntrinsicBounds(if (mMapVisible) R.drawable.ftricn_listview else R.drawable.ftricn_mapview, 0, 0, 0)
            mModeButton?.setText(if (mMapVisible) R.string.list_view else R.string.map_view)
        } else {
            mButtonsPanel?.visibility = View.GONE
        }
    }

    private fun onSearchRequested(searchData: AsdaSearchData) {
        finishSearchAction()
        if (mSearchProgressDialog == null) {
            mSearchProgressDialog = CustomProgressDialog.create(this)
            mSearchProgressDialog?.setMessage(getString(R.string.store_finder_dialog_searching_location))
            mSearchProgressDialog?.isIndeterminate = true
            mSearchProgressDialog?.show()
        }
        //TODO uncomment this once the service-eligibility api is ready to accept lat long and city name
        if (mHasGooglePlayServices) {
            object : AsyncTask<LocationUtils.AddressSearchResult>() {

                override fun doInBackground(): LocationUtils.AddressSearchResult {
                    mSearchText = searchData.searchText.toString().replace(",", " ")
                    return LocationUtils.getAddress(this@SelectCNCLocationActivityV3, mSearchText)
                }

                override fun onPostExecute(result: LocationUtils.AddressSearchResult) {
                    if (result.hasAddress()) {
                        val address = result.address
                        reloadCNCLocationsForGivenLatLon(LatLng(address.latitude, address.longitude), true)
                    } else {
                        dismissSearchStoresProgressDialog()
                        when (result.error) {
                            LocationUtils.AddressSearchResult.ERROR_NO_NETWORK -> DialogHelper.displayErrorDialog(R.string.network_error_message, this@SelectCNCLocationActivityV3, SCREEN_NAME)
                            LocationUtils.AddressSearchResult.ERROR_NO_ADDRESSES_FOUND -> DialogHelper.displayErrorDialog(R.string.store_finder_dialog_invalid_location, this@SelectCNCLocationActivityV3, SCREEN_NAME)
                            LocationUtils.AddressSearchResult.ERROR_SERVICE_UNAVAILABLE -> DialogHelper.displayErrorDialog(R.string.store_finder_dialog_location_service_failed, this@SelectCNCLocationActivityV3, SCREEN_NAME)
                        }
                    }
                }
            }.execute()
        } else {
            reloadCNCLocationsForGivenPostcodeOrCity(searchData.searchText.toString())
        }
    }

    private fun dismissSearchStoresProgressDialog() {
        try {
            mSearchProgressDialog?.dismiss()
            mSearchProgressDialog = null
        } catch (ignored: Exception) {
        }

    }

    internal fun finishSearchAction() {
        if (mSearchViewAction != null) {
            mSearchViewAction?.collapse()
            mSearchItem?.let {
                MenuItemCompat.collapseActionView(it)
            }
        }
    }

    override fun onBackPressed() {
        if (mSearchViewAction != null && mSearchViewAction?.isIconified == false) {
            finishSearchAction()
        }
        super.onBackPressed()
    }

    override fun onSearchRequested(): Boolean {
        if (mSearchViewAction != null && mSearchViewAction?.isIconified == true) {
            //this doesn't work at least on gingerbread because
            //search view should be inside a custom view for some weird reason. okay, I leave this as is.
            mSearchViewAction?.isIconified = false
            return true
        }
        return false
    }

    companion object {
        const val EXTRA_SELECTED_STORE_ID = "selected_store_id"
        const val EXTRA_SELECTED_STORE_FULFILLMENT_TYPE = "fulfillment_type"
        const val EXTRA_SELECTED_STORE_POSTCODE = "postcode"
        const val EXTRA_SELECTED_CNC_ID = "selected_cnc_id"
        const val EXTRA_SELECTED_CNC_JSON = "selected_cnc_json"
        private const val TAG = "SelectCnC"
        private const val SCREEN_NAME = "Select CNC Location"
        private const val DETAILED_ZOOM = 11f
        private const val DEFAULT_MAX_RADIUS = 200
        private const val DEFAULT_MIN_RADIUS = 3
        private const val RADIUS_NO_STORES_HERE = 10000.0//no stores around ~10km
        const val REQUEST_LOCATION_ON = 100
        private const val ID_ALL_STORES = ""

        internal fun compareInts(lhs: Int?, rhs: Int?): Int {
            var lhs = lhs
            var rhs = rhs
            if (lhs == null) {
                lhs = 0
            }
            if (rhs == null) {
                rhs = 0
            }
            return if (lhs < rhs) -1 else if (lhs == rhs) 0 else 1
        }

        val sCollectionTypeIdToDescription = ArrayMap<String, String>(8)
        val sCollectionTypeIdToNumberOfTimesDownloaded = ArrayMap<String, Int>(8)
    }

}
