package com.asda.android.app.bookslotv3.service

import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.bookslotv3.*
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers

class BookSlotRepository {


    fun makeReserveCall(token: String, reserveSlotRequestV3: ReserveSlotRequestV3): Single<ReserveSlotResponseV3>? {
        return getService()?.reserveBookSlot(token = token,
              reserveSlotRequestV3 = reserveSlotRequestV3)
              ?.subscribeOn(Schedulers.io())
              ?.observeOn(AndroidSchedulers.mainThread())
    }

    fun makeViewSlotCall(viewSlotRequestV3: ViewSlotRequestV3, xCatchHeader: Boolean = false): Single<ViewSlotResponseV3>? {
        return getService()?.viewSlots(xCatchHeader = xCatchHeader, viewSlotRequestData = viewSlotRequestV3)
              ?.subscribeOn(Schedulers.io())
              ?.observeOn(AndroidSchedulers.mainThread())
    }

    fun makeServiceEligibilityCall(postcode: String, includeAccessPointList: Boolean? = null): Single<ServiceEligibilityData>? {
        return getService()?.getServiceEligibility(
              postCode = postcode,
              includeAccessPointList = includeAccessPointList)
              ?.subscribeOn(Schedulers.io())
              ?.observeOn(AndroidSchedulers.mainThread())
    }

    fun getService(): BookSlotAPI? {
        return ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "",
              BookSlotAPI::class.java)
    }

}

