package com.asda.android.app.bookslotv3.subscription

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.recurringslots.SlotSubscription
import com.asda.android.restapi.utils.SubscriptionUtil.getFormattedDeliveryDate
import com.asda.android.utils.Utils

class EditSubscriptionsAdapter(private val context: Context,
                               private val listener: EditSubscriptionsListener,
                               private val subscriptions: ArrayList<Any>) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    object ItemType {
        const val FOOTER = 0
        const val SUBSCRIPTION = 1
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val layoutInflater: LayoutInflater = LayoutInflater.from(context)
        return if (viewType == ItemType.SUBSCRIPTION)
            EditSubscriptionViewHolder(layoutInflater.inflate(R.layout.edit_subscriptions_list_item, parent, false))
        else
            EditSubscriptionFooterViewHolder(layoutInflater.inflate(R.layout.edit_recurring_slots_footer, parent, false))
    }

    override fun getItemCount(): Int {
        return subscriptions.size
    }

    override fun getItemViewType(position: Int): Int {
        return if (subscriptions[position] is SlotSubscription)
            ItemType.SUBSCRIPTION
        else
            ItemType.FOOTER
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is EditSubscriptionViewHolder) {
            val subscription = subscriptions[position] as SlotSubscription
            holder.subscriptionTitle.text =
                  if (subscription.subscription_info?.fulfillment_type == FulfillmentTypes.DELIVERY.type)
                      context.getString(R.string.order_summary_delivery_details_header)
                  else context.getString(R.string.order_summary_cc_details_header)
            holder.subscriptionDeliveryDate.text = getFormattedDeliveryDate(subscription.subscription_info)
            holder.subscriptionFrequency.text = Utils.getFrequencyLabel(context, subscription.subscription_info?.frequency)
            holder.subscriptionCutOffTime.text = context.getString(R.string.cutoff_date,
                  subscription.subscription_info?.next_booking_expiry_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.MMM_DD_YYYY))

            setupSkipSlotButton(holder.skipSlotButton, subscription)
            setupCancelSlotButton(holder.cancelSlotButton, subscription)
        }
    }

    private fun setupSkipSlotButton(skipSlotButton: Button, subscription: SlotSubscription) {

        var visibility = View.GONE
        subscription.subscription_info?.subscription_bookings?.let { subscriptionBookingList ->
            if (!subscriptionBookingList[0].booking_id.isNullOrEmpty()) {
                visibility = View.VISIBLE
                skipSlotButton.setOnClickListener {
                    listener.onSubscriptionSkip(subscriptionBookingList[0].booking_id!!)
                }
            }
        }
        skipSlotButton.visibility = visibility
    }

    private fun setupCancelSlotButton(cancelSlotButton: Button, subscription: SlotSubscription) {
        cancelSlotButton.setOnClickListener {
            listener.onSubscriptionCancel(subscription)
        }
    }

    //------------------------------------- view holders ------------------------------------------

    private class EditSubscriptionViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        internal val subscriptionTitle: TextView = itemView.findViewById(R.id.subscription_title)
        internal val subscriptionDeliveryDate: TextView = itemView.findViewById(R.id.subscription_list_item_delivery_date_value)
        internal val subscriptionFrequency: TextView = itemView.findViewById(R.id.subscription_list_item_frequency_value)
        internal val subscriptionCutOffTime: TextView = itemView.findViewById(R.id.subscription_list_item_cut_off_value)
        internal val skipSlotButton: Button = itemView.findViewById(R.id.subscription_skip_slot)
        internal val cancelSlotButton: Button = itemView.findViewById(R.id.subscription_cancel_slot)

    }

    private class EditSubscriptionFooterViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        // Don't need to do anything here as footer is displayed as it is.
    }
}

