package com.asda.android.app.bookslotv3.subscription

import android.app.Application
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.app.recurringslot.SlotSubscriptionRepository
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.singleprofile.base.SingleProfile
import io.reactivex.observers.ResourceSingleObserver


class EditSubscriptionsViewModel(application: Application) : AndroidViewModel(application) {

    val TAG = "EditSubViewModel"
    var liveDataSubscriptionResponse = MutableLiveData<SubscriptionsResponse>()
    var slotSubscriptions = ArrayList<Any>()
    var shouldAddFooter = true

    val skipSubscriptionLiveData = MediatorLiveData<SubscriptionCreation>()
    val cancelSubscriptionLiveData = MediatorLiveData<SubscriptionCreation>()

    fun getSubscriptions(singleProfileId: String, includeAll: Boolean) {
        SlotSubscriptionRepository().getSubscriptions(singleProfileId, includeAll)?.subscribe(object : ResourceSingleObserver<SubscriptionsResponse>() {
            override fun onSuccess(subscriptionsResponse: SubscriptionsResponse) {
                subscriptionsResponse.data?.subscriptions?.filter { it.subscription_info?.status == SubscriptionSlotStatus.ACTIVE.name }.let { it?.let { it1 -> slotSubscriptions.addAll(it1) } }
                if (shouldAddFooter) {
                    slotSubscriptions.add(SlotSubscriptionFooter())
                    shouldAddFooter = false
                }
                liveDataSubscriptionResponse.postValue(subscriptionsResponse)
            }

            override fun onError(e: Throwable) {
                liveDataSubscriptionResponse.postValue(SubscriptionsResponse(errors = arrayListOf(Error(code = "1"))))
                Log.e(TAG, "Something went wrong while getting Subscriptions.")
            }
        })
    }

    fun onSubscriptionCancel(subscription: SlotSubscription) {
        val singleProfileId = SingleProfile.getProfileResponse()?.profileId ?: ""

        if (singleProfileId.isNullOrEmpty()) return

        SlotSubscriptionRepository().cancelSubscription(singleProfileId,
              subscription.subscription_info?.subscription_id
                    ?: "", SubscriptionCreation(SubscriptionCreationData(subscription.subscription_info)))
              ?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                  override fun onSuccess(t: SubscriptionCreation) {
                      cancelSubscriptionLiveData.postValue(t)
                  }

                  override fun onError(e: Throwable) {
                      cancelSubscriptionLiveData.postValue(SubscriptionCreation(errors = arrayListOf(Error(code = "1"))))
                  }

              })
    }

    fun onSubscriptionSkip(skipId: String) {

        val singleProfileId = SingleProfile.getProfileResponse()?.profileId ?: ""

        if (singleProfileId.isNullOrEmpty()) return

        SlotSubscriptionRepository().skipSubscription(singleProfileId, skipId)
              ?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                  override fun onSuccess(t: SubscriptionCreation) {
                      skipSubscriptionLiveData.postValue(t)
                  }

                  override fun onError(e: Throwable) {
                      skipSubscriptionLiveData.postValue(SubscriptionCreation(errors = arrayListOf(Error(code = "1"))))
                  }

              })
    }
}

