package com.asda.android.app.bookslotv3.subscription

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.convertToFormatAddingTime
import com.asda.android.restapi.service.data.recurringslots.SubscriptionBooking
import com.asda.android.restapi.service.data.recurringslots.SubscriptionInfo
import com.asda.android.utils.Utils

class SubscriptionAdapter(private val context: Context,
                          private val subscriptionInfo: SubscriptionInfo?,
                          private val subscriptions: List<SubscriptionBooking>?,
                          private val subscriptionListener: SubscriptionAdapterListener?) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    companion object {
        private const val removeTimeFromExpiry = -60000
    }

    object ItemType {
        const val SUBSCRIPTION = 1
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val layoutInflater: LayoutInflater = LayoutInflater.from(context)
        return EditSubscriptionViewHolder(layoutInflater.inflate(R.layout.subscription_list_item, parent, false))
    }

    override fun getItemCount(): Int {
        return subscriptions?.size ?: 0
    }

    override fun getItemViewType(position: Int): Int {
        return ItemType.SUBSCRIPTION
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is EditSubscriptionViewHolder) {
            subscriptions?.let { _data ->
                _data[position]?.let { _subscription ->
                    holder.subscriptionTitle.text = context.getString(R.string.rs_item_title)
                    holder.subscriptionDetails.text = Utils.fromHtml(
                          context.getString(R.string.rs_item_address,
                                _subscription.start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEE_DD_MMM_HH_MM_AA),
                                _subscription.end_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                                subscriptionInfo?.service_address?.address_1,
                                subscriptionInfo?.service_address?.city,
                                subscriptionInfo?.service_address?.postcode))

                    holder.subscriptionOrderBy.text = context.getString(R.string.rs_item_order_by, _subscription.expiry_time?.convertToFormatAddingTime(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEE_DD_MMM_HH_MM_AA, removeTimeFromExpiry))

                    setupSkipSlotButton(holder.skipSlotButton, _subscription)
                }
            }
        }
    }

    fun updateToSkipState(skippedId: String?) {
        subscriptions?.forEach { _subscriptionBooking ->
            skippedId?.let { _skippedBookingId ->
                if (_subscriptionBooking.booking_id == _skippedBookingId) {
                    _subscriptionBooking.status = SubscriptionBooking.LOCAL_SKIPPED
                }
            }
        }
        this.notifyDataSetChanged()
    }

    private fun setupSkipSlotButton(skipSlotButton: TextView, subscription: SubscriptionBooking?) {
        subscription?.let { _subscriptionBooking ->
            if (!_subscriptionBooking.booking_id.isNullOrEmpty()) {
                skipSlotButton.visibility = View.VISIBLE
                skipSlotButton.setOnClickListener {
                    _subscriptionBooking.booking_id?.let { _bookingId ->
                        if (!subscriptionInfo?.subscription_id.isNullOrEmpty()) {
                            subscriptionListener?.onSubscriptionSkip(subscriptionInfo?.subscription_id
                                  ?: "", _bookingId,
                                  _subscriptionBooking.start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEE_DD_MMM_HH_MM_AA)
                                        ?: "",
                                  _subscriptionBooking.end_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA)
                                        ?: "")
                        }
                    }
                }

                if (_subscriptionBooking.status == SubscriptionBooking.LOCAL_SKIPPED) {
                    skipSlotButton.setTextColor(ContextCompat.getColor(context, R.color.managing_shopping_list_add_new_list))
                    skipSlotButton.text = context.getString(R.string.rs_item_skipped)
                    skipSlotButton.setOnClickListener(null)
                }
            } else {
                skipSlotButton.visibility = View.GONE
            }

        }
    }

    private class EditSubscriptionViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        internal val subscriptionTitle: TextView = itemView.findViewById(R.id.subscription_title)
        internal val subscriptionDetails: TextView = itemView.findViewById(R.id.subscription_details)
        internal val subscriptionOrderBy: TextView = itemView.findViewById(R.id.subscription_order_by)
        internal val skipSlotButton: TextView = itemView.findViewById(R.id.subscription_skip_slot)
    }
}

