package com.asda.android.app.bookslotv3.utils

import android.content.Context
import android.util.Log
import com.asda.android.app.bookslot.PeakSeasonConfig
import com.asda.android.cxo.CartManager
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.UserViewResponse
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.base.SingleProfile.getProfileResponse
import com.fasterxml.jackson.databind.ObjectMapper

class SlotUtils {

    companion object {
        const val TAG = "SlotUtils"
        const val PRIORITY_PASS = "priority pass"
    }

    fun updateMinOrderValueAndDeliverySurcharge(serviceEligibilityDataResponse: ServiceEligibilityData?,
                                                cartManager: CartManager) {

        serviceEligibilityDataResponse?.data?.service_eligibility?.let { serviceEligibilityList ->
            val serviceEligibility = serviceEligibilityList[0]

            if (serviceEligibility.accesspoint_list?.isNotEmpty() == true) {
                val accessPoint = serviceEligibility.accesspoint_list!![0]
                accessPoint.minimum_purchase?.let {
                    cartManager.hdMinOrderValue = it.toString()
                }
                accessPoint.minimum_purchase_charge?.let {
                    cartManager.hdDeliverySurchargeValue = it.toString()
                }
            }

            if (serviceEligibility.nearest_collection_point != null) {
                val nearestCollectionPoint = serviceEligibility.nearest_collection_point
                nearestCollectionPoint?.minimum_purchase?.let {
                    cartManager.cncMinOrderValue = it.toString()
                }
                nearestCollectionPoint?.minimum_purchase_charge?.let {
                    cartManager.cncDeliverySurchargeValue = it.toString()
                }
            }

            if (cartManager.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.fulfillmentType.isNullOrEmpty() ||
                  cartManager.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.fulfillmentType?.equals(FulfillmentTypes.DELIVERY) == true) {
                cartManager.minOrderValue = cartManager.hdMinOrderValue
            } else {
                cartManager.minOrderValue = cartManager.cncMinOrderValue
            }
            cartManager.refreshViews()
        }
    }

    fun updateMinOrderValueAndDeliverySurcharge(serviceEligibilityDataResponse: ServiceEligibilityData?,
                                                cartManager: com.asda.android.service.base.CartManager) {

        serviceEligibilityDataResponse?.data?.service_eligibility?.let { serviceEligibilityList ->
            val serviceEligibility = serviceEligibilityList[0]

            //HOME DELIVERY
            if (serviceEligibility.accesspoint_list?.isNotEmpty() == true) {
                val accessPoint = serviceEligibility.accesspoint_list!![0]
                accessPoint.minimum_purchase?.let {
                    cartManager.cart?.minOrderValue = it.toString()
                    cartManager.cart?.deliverySurchargeExtraInfo?.homeDelivery?.minOrderValue = it.toString()
                }
                accessPoint.minimum_purchase_charge?.let {
                    cartManager.cart?.deliverySurcharge = it.toString()
                    cartManager.cart?.deliverySurchargeExtraInfo?.homeDelivery?.surcharge = it.toString()
                }
            }

            //CLICK N COLLECT
            if (serviceEligibility.nearest_collection_point != null) {
                val nearestCollectionPoint = serviceEligibility.nearest_collection_point
                if (nearestCollectionPoint?.minimum_purchase != null) {
                    cartManager.cart?.minOrderValue = nearestCollectionPoint.minimum_purchase.toString()
                    cartManager.cart?.deliverySurchargeExtraInfo?.clickAndCollect?.minOrderValue = nearestCollectionPoint.minimum_purchase.toString()
                }
                if (nearestCollectionPoint?.minimum_purchase_charge != null) {
                    cartManager.cart?.deliverySurcharge = nearestCollectionPoint.minimum_purchase_charge.toString()
                    cartManager.cart?.deliverySurchargeExtraInfo?.clickAndCollect?.surcharge = nearestCollectionPoint.minimum_purchase_charge.toString()
                }
            }
        }
    }

    fun getDefaultPostCodeFromSingleProfile(): String {
        return getSingleProfileDefaultDeliveryAddress(SingleProfile.getProfileResponse()?.addresses)?.postCode.orEmpty()
    }

    fun getSingleProfileDefaultDeliveryAddress(singleProfileAddresses: Array<AddressBookResponse.Address>?): AddressBookResponse.Address? {
        var defaultDeliveryAddress: AddressBookResponse.Address? = null

        singleProfileAddresses?.let { addresses ->
            for (address in addresses) {
                if (address.isDefault &&
                      (FulfillmentTypes.DELIVERY.type == address.type || FulfillmentTypes.SHIPTO.type == address.type) &&
                      address.line1 != AddressBookResponse.DUMMY && address.city != AddressBookResponse.DUMMY) {
                    defaultDeliveryAddress = address
                }
            }
        }
        return defaultDeliveryAddress
    }

    fun getPostCodeForServiceEligibility(): String {

        getProfileResponse()?.addresses?.let { arrayOfAddress ->

            val cncAddressList = arrayOfAddress.filter { it.type == FulfillmentTypes.CNC.type }
            if (!cncAddressList.isNullOrEmpty()) {
                return cncAddressList[0].postCode
            }

            val defaultPostCodeAddress = arrayOfAddress.filter { it.isDefault }
            if (!defaultPostCodeAddress.isNullOrEmpty()) {
                return defaultPostCodeAddress[0].postCode
            }
        }
        return ""
    }

    fun getPeakSeasonConfig(context: Context): PeakSeasonConfig? {
        val config = SharedPreferencesUtil.getRecurringSlotPeakSeasonConfig(context)
        config?.let {
            if (it.isNotEmpty() && it.isNotBlank()) {
                try {
                    val mapper = ObjectMapper()
                    val seasonConfig = mapper.readValue<PeakSeasonConfig>(it, PeakSeasonConfig::class.java)
                    seasonConfig?.let {
                        return it
                    }
                } catch (ex: Exception) {
                    Log.e(TAG, "Unable to parse peak config information.")
                }
            }
        }
        return null
    }

    fun isPriorityPassCustomer(userViewResponse: UserViewResponse?): Boolean {
        userViewResponse?.let { _userViewResponse ->
            if (!_userViewResponse?.deliveryPass?.activeDPPromotionsInProfile.isNullOrEmpty()) {
                val activeDPPromotionInProfile = _userViewResponse.deliveryPass?.activeDPPromotionsInProfile?.get(0)
                if (activeDPPromotionInProfile?.name.equals(PRIORITY_PASS, true)) {
                    return true
                }
            }
        }
        return false
    }
}
