package com.asda.android.app.bookslotv3.view

import android.content.Context
import android.util.AttributeSet
import android.view.MotionEvent
import androidx.viewpager.widget.ViewPager

/**
 * This custom view pager will help to disable the swipe action between neighboring fragments.
 */
class CustomViewPager(context: Context, attrs: AttributeSet) : ViewPager(context, attrs) {

    private var enabledPaging: Boolean = false

    init {
        this.enabledPaging = true
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        return if (this.enabledPaging) {
            super.onTouchEvent(event)
        } else false

    }

    override fun onInterceptTouchEvent(event: MotionEvent): Boolean {
        return if (this.enabledPaging) {
            super.onInterceptTouchEvent(event)
        } else false

    }

    fun setPagingEnabled(enabled: Boolean) {
        this.enabledPaging = enabled
    }
}

