package com.asda.android.app.bookslotv3.view.activity

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import com.asda.android.R
import com.asda.android.app.bookslot.PreviewBookSlotPresenter
import com.asda.android.app.bookslot.SlotBookedConfirmationPresenter
import com.asda.android.app.model.WismoCardModel.ORDER_TO_BE_AMENDED
import com.asda.android.app.orders.ods.AMEND_OPERATION
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.actitivies.AsdaAppCompatActivity
import com.asda.android.cxo.CXOUtils
import com.asda.android.cxo.deliveryimpact.view.fragment.CXOUnavailableItemsFragment
import com.asda.android.cxo.deliveryimpact.view.fragment.PREVIEW_AMEND
import com.asda.android.recipes.view.fragments.ImpactedRecipeListFragment
import com.asda.android.recipes.view.fragments.UnavailableRecipesFragment
import com.asda.android.restapi.service.data.BookSlotResponse
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.service.data.bookslotv3.DeliveryImpact
import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotCXOResponse
import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotResponseV3
import com.asda.android.restapi.service.data.cart.Cart
import com.asda.android.restapi.service.data.cart.CartResponse
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityInfo
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import io.reactivex.observers.ResourceSingleObserver
import kotlinx.android.synthetic.main.container_fragment.*

const val RESULT_ORDER = "RESULT_ORDER"
class DeliveryImpactActivity : AsdaAppCompatActivity() {

    private var cxoUnavailableFragment : CXOUnavailableItemsFragment? = null
    private var previewAmend: Boolean = false
    private var amendOperation: String = ""
    private var viewOrder: ViewOrderResponse.Order? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Utils.addNewRefToLeakDetector(TAG, this)
        setContentView(R.layout.container_fragment)
        loading_view.visibility = View.VISIBLE
        intent?.getBooleanExtra(PREVIEW_AMEND, false)?.let {
            previewAmend = it
        }
        intent?.getStringExtra(AMEND_OPERATION)?.let {
            amendOperation = it
        }
        viewOrder = intent?.getParcelableExtra(ORDER_TO_BE_AMENDED)
        pushImpactedFragment(intent)
    }

    override fun getFragmentContainer(): Int {
        return R.id.fragment_container
    }

    private fun pushImpactedFragment(intent: Intent?) {
        loading_view.visibility = View.GONE
        if (FeatureSettings.isCXOMigrationEnabled(context = this)) {
            val deliveryImpact = intent?.getParcelableExtra<DeliveryImpact?>(DELIVERY_IMPACT)
            when (intent?.getStringExtra(EXTRA_FRAGMENT_TAG)) {
                UnavailableRecipesFragment.TAG -> pushUnavailableRecipesFragmentCXO(deliveryImpact)
                ImpactedRecipeListFragment.TAG -> pushImpactedRecipesFragmentCXO(deliveryImpact)
                PreviewBookSlotPresenter.TAG -> pushUnavailableFragmentForCXO(deliveryImpact)
                SlotBookedConfirmationPresenter.TAG -> {
                    pushSlotBookedConfirmationPresenter(intent.getParcelableExtra<SubscriptionEligibilityInfo?>(SUBSCRIPTION_ELIGIBILITY_INFO),
                          intent.getStringExtra(EXTRA_START_DATE),
                          intent.getStringExtra(EXTRA_END_DATE))
                }
                RecurringSlotDetailsFragment.TAG -> {
                    pushEditRecurringSlotsPresenter()
                }
            }
        } else {
            val reserveSlotResp3 = intent?.getParcelableExtra<ReserveSlotResponseV3?>(RESERVE_SLOT_V3_RESPONSE)
            when (intent?.getStringExtra(EXTRA_FRAGMENT_TAG)) {
                UnavailableRecipesFragment.TAG -> pushUnavailableRecipesFragment(reserveSlotResp3)
                ImpactedRecipeListFragment.TAG -> pushImpactedRecipesFragment(reserveSlotResp3)
                PreviewBookSlotPresenter.TAG -> pushPreviewBookSlotPresenter(reserveSlotResp3)
                SlotBookedConfirmationPresenter.TAG -> {
                    pushSlotBookedConfirmationPresenter(intent.getParcelableExtra<SubscriptionEligibilityInfo?>(SUBSCRIPTION_ELIGIBILITY_INFO),
                          intent.getStringExtra(EXTRA_START_DATE),
                          intent.getStringExtra(EXTRA_END_DATE))
                }
                RecurringSlotDetailsFragment.TAG -> {
                    pushEditRecurringSlotsPresenter()
                }
            }
        }
    }

    private fun pushUnavailableRecipesFragment(reserveSlotResp3: ReserveSlotResponseV3?) {
        val fragment = UnavailableRecipesFragment.newInstance(reserveSlotResp3)
        fragment.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                when {
                    reserveSlotResp3?.data?.impacted_recipes?.isNotEmpty() == true -> {
                        pushImpactedRecipesFragment(reserveSlotResp3)
                    }
                    PreviewBookSlotPresenter.getShelfItems(applicationContext, reserveSlotResp3?.data?.messages).isNotEmpty() -> {
                        pushPreviewBookSlotPresenter(reserveSlotResp3)
                    }
                    else -> setResult(reserveSlotResp3)
                }
                dispose()
            }

            override fun onError(e: Throwable) {
                dispose()
            }

        })
        push(fragment, false)
    }

    private fun pushUnavailableRecipesFragmentCXO(deliveryImpact: DeliveryImpact?) {
        val fragment = UnavailableRecipesFragment.newInstance(deliveryImpact?.unavailableRecipes)
        fragment.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                when {
                    deliveryImpact?.ingredientsUnavailableRecipes?.isNotEmpty() == true -> {
                        pushImpactedRecipesFragmentCXO(deliveryImpact)
                    }
                    (deliveryImpact?.priceChangedItems?.isNotEmpty() == true || deliveryImpact?.promoChangedItems?.isNotEmpty() == true ||
                          deliveryImpact?.delistedItems?.isNotEmpty() == true) -> {
                        pushUnavailableFragmentForCXO(deliveryImpact)
                    }
                    else ->
                        setResult()
                }
                dispose()
            }

            override fun onError(e: Throwable) {
                dispose()
            }

        })
        push(fragment, false)
    }


    private fun pushImpactedRecipesFragmentCXO(deliveryImpact: DeliveryImpact?) {
        val fragment = deliveryImpact?.ingredientsUnavailableRecipes?.let { ImpactedRecipeListFragment.newInstance(it) }
        fragment?.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                when {
                    (deliveryImpact.priceChangedItems?.isNotEmpty() == true || deliveryImpact.promoChangedItems?.isNotEmpty() == true || deliveryImpact.delistedItems?.isNotEmpty() == true) -> {
                        pushUnavailableFragmentForCXO(deliveryImpact)
                    }
                    else -> setResult()
                }
                dispose()
            }

            override fun onError(e: Throwable) {
                dispose()
            }

        })
        push(fragment)
    }

    private fun pushImpactedRecipesFragment(reserveSlotResp3: ReserveSlotResponseV3?) {
        val fragment = ImpactedRecipeListFragment.newInstance(reserveSlotResp3)
        fragment.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                when {
                    PreviewBookSlotPresenter.getShelfItems(applicationContext, reserveSlotResp3?.data?.messages).isNotEmpty() -> {
                        pushPreviewBookSlotPresenter(reserveSlotResp3)
                    }
                    else -> setResult(reserveSlotResp3)
                }
                dispose()
            }

            override fun onError(e: Throwable) {
                dispose()
            }

        })
        push(fragment)
    }

    private fun pushPreviewBookSlotPresenter(reserveSlotResp3: ReserveSlotResponseV3?) {
        val fragment = PreviewBookSlotPresenter.newInstance(reserveSlotResp3)
        fragment.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                if (t) {
                    setResult(reserveSlotResp3)
                }
            }

            override fun onError(e: Throwable) {
                dispose()
            }

        })
        push(fragment)
    }

    private fun pushUnavailableFragmentForCXO(deliveryImpact: DeliveryImpact?) {
        cxoUnavailableFragment = if (previewAmend) {
            CXOUnavailableItemsFragment.newInstance(CXOUtils.getDeliveryImpactItems(deliveryImpact), true)
        } else {
            com.asda.android.cxo.CartManager.instance?.cartItemMap?.let {
                CXOUnavailableItemsFragment.newInstance(CXOUtils.getDeliveryImpactItems(deliveryImpact), it)
            }
        }


        cxoUnavailableFragment?.subscribe(object : ResourceSingleObserver<Boolean>() {
            override fun onSuccess(t: Boolean) {
                if (t) {
                    setResult()
                }
            }

            override fun onError(e: Throwable) {
                dispose()
            }
        })
        push(cxoUnavailableFragment)
        if (previewAmend) {
            setTitleAndBackArrow(R.string.changes_to_preview_title)
        } else {
            setTitleAndBackArrow(R.string.changes_to_trolley)
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
    }

    private fun pushSlotBookedConfirmationPresenter(subscriptionEligibilityInfo: SubscriptionEligibilityInfo?, startDate: String?, endDate: String?) {
        intent?.let {
            val recurringSlot: BookSlotResponse.RecurringSlotInfo? = if (!FeatureSettings.isCXOMigrationEnabled(this) && !FeatureSettings.isSlotSubscriptionMigratedUser(this))
                CartManager.get(this).cart?.recurringSlot
            else if (FeatureSettings.isCXOMigrationEnabled(this) && !FeatureSettings.isSlotSubscriptionMigratedUser(this)) {
                Authentication.getInstance().userViewResponse.recurringSlotInfo
            } else {
                CartManager.get(this).cart?.recurringSlot
            }

            val fragment = SlotBookedConfirmationPresenter.newInstance(recurringSlot, it.getIntExtra(CART_MANAGER_SLOT_TYPE, CartManager.SLOT_TYPE_DELIVERY), startDate, endDate, subscriptionEligibilityInfo)
            push(fragment)
        }
    }

    private fun pushEditRecurringSlotsPresenter() {
        push(RecurringSlotDetailsFragment.newInstance())
    }

    private fun setResult(reserveSlotResp3: ReserveSlotResponseV3?) {
        val resultIntent = Intent()
        resultIntent.putExtra(RESERVE_SLOT_V3_RESPONSE, reserveSlotResp3)
        setResult(Activity.RESULT_OK, resultIntent)
        finish()
    }

    private fun setResult() {
        val resultIntent = Intent()
        resultIntent.putExtra(RESULT_ORDER, viewOrder)
        resultIntent.putExtra(AMEND_OPERATION, amendOperation)
        setResult(Activity.RESULT_OK, resultIntent)
        finish()
    }


    companion object {
        const val TAG = "DeliveryImpactActivity"
        const val RESERVE_SLOT_V3_RESPONSE = "reserve_slot_v3_response"
        const val DELIVERY_IMPACT = "DELIVERY_IMPACT"
        const val EXTRA_FRAGMENT_TAG = "fragment_tag"
        const val CART_MANAGER_SLOT_TYPE = "cart_manager_slot_type"
        const val SUBSCRIPTION_ELIGIBILITY_INFO = "subscription_eligibility_info"
        const val REQUEST_CODE = 1
        const val EXTRA_START_DATE = "extra_start_date"
        const val EXTRA_END_DATE = "extra_end_date"

        fun newIntent(context: Context?, reserveSlotResp3: ReserveSlotResponseV3?, fragmentTag: String?): Intent {
            val intent = Intent(context, DeliveryImpactActivity::class.java)
            intent.putExtra(EXTRA_FRAGMENT_TAG, fragmentTag)
            intent.putExtra(RESERVE_SLOT_V3_RESPONSE, reserveSlotResp3)
            return intent
        }

        fun newIntent(context: Context?, deliveryImpact: DeliveryImpact?, fragmentTag: String?): Intent {
            val intent = Intent(context, DeliveryImpactActivity::class.java)
            intent.putExtra(EXTRA_FRAGMENT_TAG, fragmentTag)
            intent.putExtra(DELIVERY_IMPACT, deliveryImpact)
            return intent
        }

        fun newIntent(context: Context?, cartManagerSlotType: Int, startDate: String?, endDate: String?, fragmentTag: String?, subscriptionEligibilityInfo: SubscriptionEligibilityInfo? = null): Intent {
            val intent = Intent(context, DeliveryImpactActivity::class.java)
            intent.putExtra(EXTRA_FRAGMENT_TAG, fragmentTag)
            intent.putExtra(CART_MANAGER_SLOT_TYPE, cartManagerSlotType)
            intent.putExtra(EXTRA_START_DATE, startDate)
            intent.putExtra(EXTRA_END_DATE, endDate)
            subscriptionEligibilityInfo?.let {
                intent.putExtra(SUBSCRIPTION_ELIGIBILITY_INFO, it)
            }
            return intent
        }

        fun newIntent(context: Context?, fragmentTag: String?): Intent {
            val intent = Intent(context, DeliveryImpactActivity::class.java)
            intent.putExtra(EXTRA_FRAGMENT_TAG, fragmentTag)
            return intent
        }
    }
}
