package com.asda.android.app.bookslotv3.view.fragment

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.ImageSpan
import android.util.Log
import android.view.View
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.JsonMapper
import com.asda.android.analytics.constants.Anivia.PV_BOOK_SLOT_CNC
import com.asda.android.analytics.constants.Anivia.SECTION_BOOK_SLOT
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.bookslotv3.view.fragment.BookSlotParentFragment.Companion.SWITCH_TO_CNC_TAB
import com.asda.android.app.bookslotv3.BookSlotType.CLICK_AND_COLLECT
import com.asda.android.app.bookslotv3.SelectCNCLocationActivityV3
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.restapi.service.data.SlotResponse
import com.asda.android.restapi.service.data.bookslotv3.AccessPoint
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.utils.ExpressUtils
import com.asda.android.utils.Utils
import kotlinx.android.synthetic.main.order_summary_express_details.*
import kotlinx.android.synthetic.main.book_slot_header_dp_booked_slot.*
import kotlinx.android.synthetic.main.fragment_book_slot.*
import kotlinx.android.synthetic.main.home_card_express.*
import java.io.IOException

class BookSlotClickNCollectFragment : BookSlotFragment(CLICK_AND_COLLECT) {

    var storeId = ""
    private var storeName: String? = null

    companion object {
        const val TAG = "BookSlot-Click&Collect"
        const val IS_EXPRESS_BANNER_CLICKED = "express_banner_clicked"
        fun createInstance(slotEvents: SlotEvents,
                           expressBannerClicked: Boolean = false, maxItemReachedExpress: Boolean): BookSlotFragment {
            val fragment = BookSlotClickNCollectFragment()
            fragment.slotEvents = slotEvents
            val bundle = Bundle()
            bundle.putBoolean(IS_EXPRESS_BANNER_CLICKED, expressBannerClicked)
            bundle.putBoolean(SWITCH_TO_CNC_TAB, maxItemReachedExpress)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun onStart() {
        super.onStart()
        Tracker.get().trackPageView(PageViewEvent(PV_BOOK_SLOT_CNC, SECTION_BOOK_SLOT))
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        arguments?.let {
            if (it.containsKey(IS_EXPRESS_BANNER_CLICKED) && it.get(IS_EXPRESS_BANNER_CLICKED) == true) {
                slotEvents?.switchTabsTo(CLICK_AND_COLLECT)
                onLocationTypeAddressClicked()
            }

            if (it.containsKey(SWITCH_TO_CNC_TAB) && it.getBoolean(SWITCH_TO_CNC_TAB)) {
                slotEvents?.switchTabsTo(CLICK_AND_COLLECT)
            }
        }
    }

    override fun addLiveDataObservers() {
        super.addLiveDataObservers()
        viewModel.getServiceEligibilityLiveData().observe(this, Observer<ServiceEligibilityData> { data ->
            onServiceEligibilityDataAvailable(data)
            viewSlots()
        })
    }

    override fun inflateHeaderView(view: View) {
        //Do nothing
    }

    override fun getCurrentFragmentTitle(context: Context): String {
        return context.getString(R.string.click_and_collect)
    }

    override fun setDeliveryHeaderText() {
        book_slot_location_type.text = getString(R.string.collection_location)
    }

    override fun setExpressInfo() {
        var showExpressInfo = false;
        if (context?.let { FeatureSettings.isCXOMigrationEnabled(it) } == true) {
            val cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
            showExpressInfo = cxoCart?.cartItemSummary?.lineItemCount ?: 0 <= 0 && cxoCart?.cartItemSummary?.totalQuantity ?: 0 <= 0 && Utils.isExpressEnabled(context)
                  && Utils.getExpressFlagCNC()
        } else {
            showExpressInfo = context != null && CartManager.get(context).lineItemCount != null && CartManager.get(context).totalQuantity != null
                  && ExpressUtils.showExpressInfo(CartManager.get(context).lineItemCount, CartManager.get(context).totalQuantity) && Utils.isExpressEnabled(context)
                  && Utils.getExpressFlagCNC()
        }
        if (showExpressInfo) {
            express_info.visibility = View.VISIBLE
            val spannableStringBuilder = SpannableStringBuilder()
                  .append(context?.getString(R.string.add_items_to_cart))
                  .append(context?.getString(R.string.double_spaces))
            spannableStringBuilder.setSpan(ImageSpan(requireContext(), R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            spannableStringBuilder.append(context?.getString(R.string.single_space)).append(context?.getString(R.string.slots_available))
            express_text.text = spannableStringBuilder
        } else {
            express_info.visibility = View.GONE
        }
    }

    override fun setAddress() {
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                setAddressFromCXO()
            } else {
                setAddressFromOldCart()
            }
            viewModel.isServiceAddressFromCart = true
        }

        if (viewModel.serviceAddress == null) {
            viewModel.isServiceAddressFromCart = false
            viewModel.makeServiceEligibilityCall(SlotUtils().getPostCodeForServiceEligibility())
        }
    }

    private fun setAddressFromCXO() {
        val fulfillmentAddress = viewModel.cxoCart?.fulfillmentDetails?.fulfillmentAddress
        if (fulfillmentAddress?.postalCode != null && FulfillmentTypes.DELIVERY.type != viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType) {
            viewModel.fulfillmentType = viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType.orEmpty()

            viewModel.serviceAddress = ServiceAddress(fulfillmentAddress.postalCode,
                  fulfillmentAddress.latitude, fulfillmentAddress.longitude,
                  fulfillmentAddress.city, fulfillmentAddress.country,
                  fulfillmentAddress.address, fulfillmentAddress.address2)
            storeId = viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.storeId.orEmpty()
            viewModel.serviceAddress?.storeId = storeId
            setAddress(viewModel.serviceAddress)
        } else if (SingleProfile.getProfileResponse()?.addresses != null) {
            SingleProfile.getProfileResponse()?.addresses?.forEach { address ->
                address.type?.let { addressType ->
                    if (addressType == FulfillmentTypes.INSTOREPICKUP.type || addressType == FulfillmentTypes.POPUP.type) {
                        viewModel.fulfillmentType = addressType
                        viewModel.serviceAddress = ServiceAddress(address.postalCode, address.latitude, address.longitude, address.city, address.country, address.line1,
                              address.line2, address.addressId)
                        setAddress(viewModel.serviceAddress)
                    }
                }
            }
        }
    }

    private fun setAddressFromOldCart() {
        val cartExtraInfo = CartManager.get(context).cartExtraInfo
        val cart = CartManager.get(context).cart

        if (cartExtraInfo?.cncStoreAddress != null && cartExtraInfo.cncStoreAddress.postalCode != null && FulfillmentTypes.DELIVERY.type != cart?.fulfillmentType) {
            viewModel.fulfillmentType = cart?.fulfillmentType.orEmpty()
            viewModel.serviceAddress = ServiceAddress(cartExtraInfo.cncStoreAddress.postalCode,
                  cartExtraInfo.cncStoreAddress.latitude, cartExtraInfo.cncStoreAddress.longitude,
                  cartExtraInfo.cncStoreAddress.city, cartExtraInfo.cncStoreAddress.country,
                  cartExtraInfo.cncStoreAddress.addressLine1, cartExtraInfo.cncStoreAddress.addressLine2)
            storeId = cartExtraInfo.cncStoreAddress?.storeid.orEmpty()
            viewModel.serviceAddress?.storeId = storeId
            storeName = cartExtraInfo.cncStoreAddress?.storename
            setAddress(viewModel.serviceAddress)
        } else if (SingleProfile.getProfileResponse()?.addresses != null) {
            SingleProfile.getProfileResponse()?.addresses?.forEach { address ->
                address.type?.let { addressType ->
                    if (addressType == FulfillmentTypes.INSTOREPICKUP.type || addressType == FulfillmentTypes.POPUP.type) {
                        viewModel.fulfillmentType = addressType
                        viewModel.serviceAddress = ServiceAddress(address.postalCode, address.latitude, address.longitude, address.city, address.country, address.line1,
                              address.line2, address.addressId)
                        setAddress(viewModel.serviceAddress)
                    }
                }
            }
        }
    }

    override fun setAddress(serviceAddress: ServiceAddress?) {
        if (storeName.isNullOrBlank())
            super.setAddress(serviceAddress)
        else {
            book_slot_location_type_address?.let {
                val sb = StringBuilder()
                sb.append(storeName).append(", ")
                if (!serviceAddress?.city.isNullOrBlank())
                    sb.append(serviceAddress?.city)
                if (!serviceAddress?.postcode.isNullOrBlank())
                    sb.append(" ").append(serviceAddress?.postcode?.replace(" ", ""))
                book_slot_location_type_address.text = sb.toString()
            }
        }
    }

    override fun setUserVisibleHint(isVisibleToUser: Boolean) {
        super.setUserVisibleHint(isVisibleToUser)
        if (isViewModelInitialized() && isVisibleToUser && viewModel.serviceAddress == null) {
            onLocationTypeAddressClicked()
        }
    }

    private fun onServiceEligibilityDataAvailable(serviceEligibilityData: ServiceEligibilityData?) {
        serviceEligibilityData?.data?.service_eligibility?.let { it ->
            if (it.isNotEmpty()) {
                val serviceAddress = it[0].nearest_collection_point?.service_address
                serviceAddress?.let { serviceAddress ->
                    viewModel.serviceAddress = serviceAddress
                    storeName = it[0].nearest_collection_point?.description
                    setAddress(serviceAddress)
                }
                it[0].nearest_collection_point?.service_info?.fulfillment_type?.let { fulfillmentType ->
                    viewModel.fulfillmentType = fulfillmentType
                }
                it[0].nearest_collection_point?.fulfill_store_id?.let { fulfillStoreId ->
                    storeId = fulfillStoreId
                    viewModel.serviceAddress?.storeId = storeId
                }

            }
        }
    }

    override fun onLocationTypeAddressClicked() {
        val intent = Intent(context, SelectCNCLocationActivityV3::class.java)

        intent.putExtra(SelectCNCLocationActivityV3.EXTRA_SELECTED_STORE_ID, storeId)
        var postcode = viewModel.serviceAddress?.postcode.orEmpty()
        if (postcode.isNotBlank())
            postcode = postcode.replace(" ", "")
        intent.putExtra(SelectCNCLocationActivityV3.EXTRA_SELECTED_STORE_POSTCODE, postcode)
        intent.putExtra(SelectCNCLocationActivityV3.EXTRA_SELECTED_STORE_FULFILLMENT_TYPE, viewModel.fulfillmentType)

        startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_SELECT_CNC)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ActivityRequestCodes.REQUEST_CODE_SELECT_CNC && resultCode == Activity.RESULT_OK) {
            val json = data?.getStringExtra(SelectCNCLocationActivityV3.EXTRA_SELECTED_CNC_JSON)
            val accessPoint: AccessPoint?
            if (!TextUtils.isEmpty(json)) {
                try {
                    accessPoint = JsonMapper.get().readValue(json, AccessPoint::class.java)
                    viewModel.serviceAddress = accessPoint.service_address
                    viewModel.fulfillmentType = accessPoint.service_info?.fulfillment_type ?: ""
                    storeId = accessPoint.fulfill_store_id.orEmpty()
                    viewModel.serviceAddress?.storeId = storeId
                    viewModel.serviceAddress?.dispenseStoreId = accessPoint.dispense_store_id.orEmpty()
                    viewModel.serviceAddress?.remoteSiteId = accessPoint.remote_site_id.orEmpty()
                    storeName = accessPoint.description
                    setAddress(accessPoint.service_address)
                    resetGrid()
                    viewSlots()
                    if (context != null && Utils.isExpressEnabled(context)) {
                        updateExpressInfoHeader(accessPoint.service_info?.enable_express)
                    }
                } catch (e: IOException) {
                    Log.e(TAG, "failed to read postcode info", e)
                }
            }
        }
    }

    override fun getServiceAddressForTab(): ServiceAddress? {
        return viewModel.viewSlotData?.service_address
    }

    private fun updateExpressInfoHeader(isExpress: Boolean?) {
        isExpress?.let {
            if (it) {
                setExpressInfo()
            } else {
                express_info.visibility = View.GONE
            }
        }
    }

    override fun setAdditionalCostText() {
        additional_cost_text.text = getString(R.string.the_asda_same_day_has_additional_cost, getString(R.string.collection))
    }

    override fun handleSlotData() {
        context?.let { _context ->
            if (FeatureSettings.isCXOMigrationEnabled(_context)) {
                val slotInfo = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.fulfillmentDetails
                slotInfo?.let { _slotInfo ->
                    if ((_slotInfo?.slotInfo?.fulfillmentType == FulfillmentTypes.CNC.type || _slotInfo?.slotInfo?.fulfillmentType == FulfillmentTypes.INSTOREPICKUP.type) && _slotInfo?.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                          _slotInfo?.slotInfo?.cartSlotId?.isNotBlank() == true && _slotInfo?.slotInfo?.slotExpired != true) {
                        booked_slot_details_express?.visible()
                        booked_slot_details_stub?.gone()
                        val fulFillmentAddress = _slotInfo.fulfillmentAddress
                        fulFillmentAddress?.let { _fulFillmentAddress ->
                            express_store_name?.text = getString(R.string.express_delivery_address, _fulFillmentAddress?.address, _fulFillmentAddress?.city)
                            express_store_address?.text = _fulFillmentAddress?.postalCode

                            express_slot_date?.text = _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMMM_YYYY)
                            express_slot_time?.text = getString(R.string.express_slot_time, _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA), _slotInfo.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))
                            express_reserved_until?.text = getString(R.string.slot_reserved_until, _slotInfo.slotInfo?.bookedUntilTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))

                            if (_slotInfo.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                                context?.let { _context ->
                                    express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                                }
                                express_span?.visible()
                                express_span_bookslot?.gone()
                            } else {
                                express_span?.gone()
                            }
                        } ?: run {
                            booked_slot_details_express?.gone()
                        }
                    } else {
                        booked_slot_details_express?.gone()
                    }
                } ?: run {
                    booked_slot_details_express?.gone()
                }
            } else {
                val cart = CartManager.get(context).cart

                cart?.let { cartData ->
                    if (cartData.fulfillmentType == SlotResponse.ODS_FULFILLMENT_OPTION_IN_STORE_PICKUP || cartData.fulfillmentType == SlotResponse.FULFILLMENT_OPTION_IN_STORE_PICKUP &&
                          cartData.extraInfo?.bookedSlotId?.isNotBlank() == true && cartData.extraInfo?.bookedSlotId?.isNotEmpty() == true
                          && cartData.slotExpiredFlag?.equals("false") == true) {
                        booked_slot_details_express?.visible()
                        booked_slot_details_stub?.gone()
                        express_store_name?.text = getString(R.string.express_delivery_address, cartData.extraInfo?.cncStoreAddress?.addressLine1, cartData.extraInfo?.cncStoreAddress?.city)
                        express_store_address?.text = cartData.extraInfo?.cncStoreAddress?.postalCode

                        express_slot_date?.text = cartData.slotDate
                        express_slot_time?.text = getString(R.string.express_slot_time, cartData?.deliveryStartTime, cartData?.deliveryEndTime)
                        express_reserved_until?.text = getString(R.string.slot_reserved_until, cartData.bookedUntilTime)

                        if (cart.isExpress == "true") {
                            context?.let {_context ->
                                express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                            }
                            express_span?.visible()
                            express_span_bookslot?.gone()
                        } else {
                            express_span?.gone()
                        }
                    } else {
                        booked_slot_details_express?.gone()
                    }
                } ?: run {
                    booked_slot_details_express?.gone()
                }
            }
        }
    }

    override fun setExpressAvailable(expressAvailable: Boolean) {
        if (Utils.isExpressEnabled(context) && expressAvailable) {
            setExpressInfo()

            context?.let { _context ->
                if (FeatureSettings.isCXOMigrationEnabled(_context)) {
                    val cartItemList = com.asda.android.cxo.CartManager.instance?.getCartItems()
                    val cart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
                    cart?.let { _cart ->
                        if ((_cart.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.CNC.type || _cart.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.INSTOREPICKUP.type) && _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                              _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotBlank() == true) {
                            express_info.gone()
                            express_span_bookslot?.gone()
                        } else {
                            if (cartItemList?.isEmpty() == true) {
                                express_info.visibility = View.VISIBLE
                                express_span_bookslot?.gone()
                            } else {
                                express_info.visibility = View.GONE
                                express_span_bookslot?.visible()
                            }
                        }
                    }
                } else {
                    val cart = CartManager.get(context).cart
                    cart?.let { _cart ->
                        if (_cart.fulfillmentType == SlotResponse.ODS_FULFILLMENT_OPTION_IN_STORE_PICKUP || _cart.fulfillmentType == SlotResponse.FULFILLMENT_OPTION_IN_STORE_PICKUP &&
                              _cart.extraInfo?.bookedSlotId?.isNotBlank() == true && _cart.extraInfo?.bookedSlotId?.isNotEmpty() == true) {
                            express_info.gone()
                            express_span_bookslot?.gone()
                        } else {
                            if (_cart.numItemsInBasket != "0") {
                                express_info.visibility = View.GONE
                                express_span_bookslot?.visible()
                            } else {
                                express_info.visibility = View.VISIBLE
                                express_span_bookslot?.gone()
                            }
                        }
                    }
                }
            }
        } else {
            express_span_bookslot?.gone()
        }
    }
}
