package com.asda.android.app.bookslotv3.view.fragment

import android.app.Activity
import android.app.Activity.RESULT_OK
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.analytics.AnalyticsConfig
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.BOOK_SLOT_CONFIRMATION
import com.asda.android.app.bookslot.BannerConfig
import com.asda.android.app.bookslot.PreviewBookSlotPresenter
import com.asda.android.app.bookslot.SlotBookedConfirmationPresenter
import com.asda.android.app.bookslotv3.BookSlotGridInterface
import com.asda.android.app.bookslotv3.BookSlotType.CLICK_AND_COLLECT
import com.asda.android.app.bookslotv3.BookSlotType.HOME_DELIVERY
import com.asda.android.app.bookslotv3.adapter.BookSlotAdapter
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.bookslotv3.view.ItemDecorationGrid
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.bookslotv3.viewmodel.BookSlotViewModel
import com.asda.android.app.main.RemoteConfigController
import com.asda.android.app.model.WismoCardModel.FULFILLMENT_TYPE_CNC
import com.asda.android.app.model.WismoCardModel.FULFILLMENT_TYPE_HD
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.*
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.base.interfaces.ICartManager
import com.asda.android.coachmark.view.BubbleCoachMark
import com.asda.android.coachmark.view.CoachMark
import com.asda.android.designlibrary.view.dialog.AsdaAlertDialog
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.recipes.view.fragments.ImpactedRecipeListFragment
import com.asda.android.recipes.view.fragments.UnavailableRecipesFragment
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.AsdaResponse.ERROR_CODE_LOGIN_REQUIRED
import com.asda.android.restapi.service.data.SlotResponse
import com.asda.android.restapi.service.data.bookslotv3.*
import com.asda.android.restapi.service.data.bookslotv3.ErrorCodes
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.restapi.utils.SubscriptionUtil
import com.asda.android.restapi.utils.convertToDate
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.features.login.view.LoginSpActivity
import com.asda.android.utils.ExpressUtils
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import kotlinx.android.synthetic.main.book_slot_header_dp_booked_slot.*
import kotlinx.android.synthetic.main.booked_slot_details.*
import kotlinx.android.synthetic.main.fragment_book_slot.*

abstract class BookSlotFragment(private var currentFragmentType: String) : BaseFragment(), BookSlotGridInterface {
    interface SlotEvents {
        fun switchTabsTo(destinationSlotFragmentType: String)
    }

    companion object {
        const val TAG = "BookSlotFragment"
        var SCREEN_NAME = "Booking Delivery Slot"
    }

    var slotEvents: SlotEvents? = null
    var bookSlotAdapter: BookSlotAdapter? = null
    private lateinit var gridLayoutManager: GridLayoutManager
    private lateinit var recyclerViewOnScrollListener: RecyclerView.OnScrollListener
    private var isViewSlotCallInProgress = false
    var coachMark: CoachMark? = null

    var cartListener: ICartManager.CartListener? = null
    lateinit var viewModel: BookSlotViewModel
    fun isViewModelInitialized(): Boolean {
        return ::viewModel.isInitialized
    }


    override fun getInternalTag(): String? {
        return TAG
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null && !isViewModelInitialized()) {
            viewModel = ViewModelProviders.of(this).get(BookSlotViewModel::class.java)
            context?.let {
                viewModel.isExpressEnabled = Utils.isExpressEnabled(it)
                viewModel.isSlotEligibilityMinPurchaseEnabled = FeatureSettings.isSlotEligibilityMinPurchaseEnabled(it)
                viewModel.isCXOMigrationEnabled = FeatureSettings.isCXOMigrationEnabled(it)
                viewModel.setBannerConfig(SharedPreferencesUtil.getSlotBannerConfig(it))
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_book_slot, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        inflateHeaderView(view)
        express_info.visibility = View.GONE
        context?.let {
            getCurrentFragmentTitle(it)
        }
        getCXOCartValue()
        addCartListener()
        initAdapter()
        refreshSlots()
        addLiveDataObservers()
        addClickListeners()
        addNextAndBackButtonFunction()
    }

    override fun onResume() {
        super.onResume()
        setDeliveryHeaderText()
        setupCovid19AlertBanner()
        setAdditionalCostText()
        handleSlotData()
        setupSecondaryStoreFooter()
        setBannerConfig(viewModel.bannerConfig)
    }

    private fun addNextAndBackButtonFunction() {
        book_slot_grid_right_button.setOnClickListener {
            val firstVisibleItemPosition = gridLayoutManager.findFirstVisibleItemPosition()
            val scrollPosition = firstVisibleItemPosition + viewModel.numOfRows.times(3)
            gridLayoutManager.scrollToPositionWithOffset(scrollPosition, 0)
        }

        book_slot_grid_left_button.setOnClickListener {
            val firstVisibleItemPosition = gridLayoutManager.findFirstVisibleItemPosition()
            val scrollPosition = firstVisibleItemPosition - viewModel.numOfRows.times(3)
            gridLayoutManager.scrollToPositionWithOffset(scrollPosition, 0)
        }
    }

    private fun getCXOCartValue() {
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                viewModel.cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
                viewModel.cxoCartManager = com.asda.android.cxo.CartManager.instance
            }
            com.asda.android.cxo.CartManager.instance?.updatedCart?.observe(viewLifecycleOwner,
                  Observer<com.asda.android.restapi.service.data.cart.Cart> { cart ->
                      viewModel.cxoCart = cart
                      setBookedSlotDetails()
                  })
        }
    }

    private fun setupCovid19AlertBanner() {
        context?.let {
            if (FeatureSettings.isCovid19BannerOnBookSlotPageEnabled(it)) {
                covid_19_update_banner.visibility = View.VISIBLE
                val days = if (CartManager.get(context).activeDeliveryPass != null)
                    SharedPreferencesUtil.getCOVID19BannerPriorityPassHorizonDays(context)
                else SharedPreferencesUtil.getCOVID19BannerNONPriorityPassHorizonDays(context)

                covid_19_update_banner.text = Utils.fromHtml(resources.getString(R.string.bookslot_covid19_alert_banner, days))
            } else {
                covid_19_update_banner.visibility = View.GONE
            }
        }
    }

    open fun addLiveDataObservers() {

        addCXOReserveAPICallObserver()
        addATGReserveAPICallObserver()
        addViewSlotAPICallObserver()

        // RefreshSlots for the tab which is not active in order to refresh the booked slot details
        // on other page.
        (parentFragment as? BookSlotParentFragment)?.getBookedSlotChangedLiveData()?.observe(viewLifecycleOwner, Observer<Boolean> {
            resetGrid()
            refreshSlots()
        })
    }

    private fun addCXOReserveAPICallObserver() {
        viewModel.getCXOReserveSlotResponseLiveData().observe(viewLifecycleOwner, object : Observer<ReserveSlotCXOResponse> {
            override fun onChanged(reserveSlotCXOResponse: ReserveSlotCXOResponse?) {
                if (book_slot_page_loader == null) return
                var errorFound = false
                reserveSlotCXOResponse?.errors?.let { errors ->
                    errors.find { it.code == ErrorCodes.CART_CHANGES_TO_TROLLEY }?.let {
                        errorFound = true
                        handleDeliveryImpact(reserveSlotCXOResponse)
                    }
                    errors.find { it.code == ErrorCodes.INTERNAL_ERROR || it.code == ErrorCodes.HTTP_ERROR || it.code == "MISSING_MANDATORY_PARAM" }?.let { error ->
                        errorFound = true
                        handleReserveSlotFailure(error)
                    }
                    errors.find { it.code == ERROR_CODE_LOGIN_REQUIRED }?.let { error ->
                        errorFound = true
                        showLoginAlertDialog()
                    }
                }
                if (!errorFound) {
                    if (viewModel.deliveryImpactContinue) { // delivery_impact = continue, show bookSlotConfirm
                        onSlotBookSuccess()
                    } else {
                        handleDeliveryImpact(reserveSlotCXOResponse)
                    }
                }
            }
        })
    }

    private fun addATGReserveAPICallObserver() {
        viewModel.getReserveSlotResponseLiveData().observe(viewLifecycleOwner, object : Observer<ReserveSlotResponseV3> {
            override fun onChanged(reserveSlotResp3: ReserveSlotResponseV3?) {
                if (book_slot_page_loader == null) return
                var errorFound = false
                reserveSlotResp3?.errors?.let { errors ->
                    errors.find { it.code == ErrorCodes.INTERNAL_ERROR || it.code == ErrorCodes.HTTP_ERROR || it.code == "MISSING_MANDATORY_PARAM" }?.let { error ->
                        errorFound = true
                        handleReserveSlotFailure(error)
                    }
                }
                if (!errorFound) {
                    if (viewModel.deliveryImpactContinue) { // delivery_impact = continue, show bookSlotConfirm
                        onSlotBookSuccess(reserveSlotResp3)
                    } else {
                        reserveSlotResp3?.data?.slot_info = viewModel.slotInfoOfReserveCallRequest
                        handleReserveSlotSuccess(reserveSlotResp3)
                    }
                }
            }
        })
    }

    private fun addViewSlotAPICallObserver() {
        viewModel.geViewSlotLiveData().observe(viewLifecycleOwner, object : Observer<ViewSlotResponseV3> {
            override fun onChanged(viewSlotResponseV3: ViewSlotResponseV3?) {
                isViewSlotCallInProgress = false
                if (book_slot_grid_holder == null) return

                //ERROR
                val errorFound = handleViewSlotError(viewSlotResponseV3?.errors)

                //SUCCESS
                if (!errorFound) {
                    viewModel.checkIfItsLastPage(viewSlotResponseV3?.data)
                    viewModel.viewSlotData = viewSlotResponseV3?.data
                    showGridRetryView(shouldShow = false, isRetrying = false)
                    loadSlots(viewModel.viewSlotData)
                    if (context != null && Utils.isExpressEnabled(context) && Utils.getExpressFlag()) {
                        setExpressAvailable(isExpressAvailable(viewSlotResponseV3))
                    }
                    showLoader(false)
                }
            }
        })
    }

    private fun isExpressAvailable(data: ViewSlotResponseV3?): Boolean {
        data?.data?.let { v3Data ->
            v3Data.slot_days?.forEach { slotDay ->
                slotDay?.slots?.forEach { slot ->
                    if (slot.slot_info?.is_express_slot == true) {
                        return true
                    }
                }
            }
        }
        return false
    }

    private fun handleViewSlotError(errors: List<Error>?): Boolean {
        var errorFound = false
        errors?.let {
            if (it.isNotEmpty()) {
                errorFound = true
                trackBookSlotFailure(it[0], "v3/slot/view")
                if (viewModel.data.isNotEmpty()) {
                    viewModel.isLastPage = true
                } else {
                    showLoader(false)
                    var hideOptionalLinksOnErrorView = false
                    error_message_for_postcode_not_found.text =
                          when (it[0].code) {
                              ErrorCodes.COLLECTION_CHECK_MISSING -> context?.getString(R.string.sorry_we_do_not_offer_home_delivery_for_this_address)
                              ErrorCodes.INVALID_POSTCODE -> context?.getString(R.string.sorry_we_do_not_recognise_that_postcode)
                              ErrorCodes.VP_UNAVAILABLE_SLOT -> {
                                  hideOptionalLinksOnErrorView = true
                                  it[0].message
                              }
                              else -> {
                                  hideOptionalLinksOnErrorView = true
                                  context?.getString(R.string.something_went_wrong)
                              }
                          }
                    showGridRetryView(shouldShow = true, hideOptionalLinksOnErrorView = hideOptionalLinksOnErrorView)
                }
            }
        }
        return errorFound
    }

    private fun addClickListeners() {
        add_or_update_address_textview.setOnClickListener { onLocationTypeAddressClicked() }
        collect_your_shopping_from_us_textview.setOnClickListener { slotEvents?.switchTabsTo(CLICK_AND_COLLECT) }
    }

    private fun refreshSlots() {
        setDeliveryHeaderText()
        setBookedSlotDetails()
        setAddress()
        handleSlotData()
        viewSlots(true)
        book_slot_location_type_address.setOnClickListener { onLocationTypeAddressClicked() }
    }

    override fun onDestroyView() {
        removeCartListener()
        super.onDestroyView()
    }

    private fun addCartListener() {
        cartListener = object : ICartManager.CartListener() {
            override fun onChanged() {
                viewModel.cartExtraInfo = CartManager.get(context).cartExtraInfo
                viewModel.cart = CartManager.get(context).cart
                viewModel.cartManager = CartManager.get(context)

                if (!isAdded) {
                    return
                }
                setBookedSlotDetails()
                resetGrid()
                viewSlots()
                //  KillSwitchController.validateStore(activity)
                RemoteConfigController.validateStore(activity)

            }
        }
        cartListener?.let {
            CartManager.get(context).addCartListener(it)
        }
    }

    private fun removeCartListener() {
        context?.let {
            if (cartListener != null) {
                CartManager.get(it).removeCartListener(cartListener)
            }
        }
    }

    private fun showLoader(shouldShow: Boolean) {
        if (book_slot_page_loader == null) return
        if (shouldShow) {
            book_slot_page_loader.visibility = View.VISIBLE
        } else {
            book_slot_page_loader.visibility = View.GONE
        }
    }

    private fun isFullPageLoaderShown() = book_slot_page_loader.visibility == View.VISIBLE

    //region setBookedSlotDetails
    /**
     * Set booked slot details, check if it is Home Delivery or Click and Collect and set the address
     * accordingly.
     */
    fun setBookedSlotDetails() {
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                setBookedSlotDetailsWithCXO()
            } else {
                setBookedSlotDetailsWithATGCart()
            }
        }

        if (viewModel.shouldShowBookSlotConfirmPage) {
            openBookSlotConfirmationPage()
            viewModel.shouldShowBookSlotConfirmPage = false
        }
    }

    private fun setBookedSlotDetailsWithCXO() {
        val fulfillmentDetails = viewModel.cxoCart?.fulfillmentDetails
        fulfillmentDetails?.slotInfo?.cartSlotId?.let {
            //TODO DELETE DETAIL STUB
            //booked_slot_details_stub.visibility = View.VISIBLE

            // Check if the booked slot is Home Delivery option.
            if (SlotResponse.FULFILLMENT_OPTION_HOME_DELIVERY == fulfillmentDetails.slotInfo?.fulfillmentType) {
                delivery_image.setImageResource(R.drawable.ic_truck)
                slot_reserved_header_text.text = getString(R.string.home_delivery_slot_booked)
            } else {
                setBookedCNCInfo()
            }
            slot_reserved_address.text = getFormattedAddress(fulfillmentDetails.fulfillmentAddress?.address,
                  fulfillmentDetails.fulfillmentAddress?.address2,
                  fulfillmentDetails.fulfillmentAddress?.city,
                  fulfillmentDetails.fulfillmentAddress?.postalCode)

            val price: Double = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.cartPriceSummary?.fulfillmentCost?.cost
                  ?: 0.0
            slot_reserved_price.text = context?.resources?.getString(R.string.pound_amount, price.toFloat())

            slot_reserved_time.text = getString(R.string.slot_reserved_date_from_to,
                  fulfillmentDetails.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM),
                  fulfillmentDetails.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                  fulfillmentDetails.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))

            val reservedUntilTime = if (viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.isRecurringSlot == true) {
                SubscriptionUtil.getSubscriptionExpiryDate(viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
                      DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z,
                      DateFormats.HH_MM_AA_EEE_DD_MMM,
                      TimeZoneFormat.UTC,
                      TimeZoneFormat.UTC)
            } else {
                viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime?.getFormattedDateWithDay()
            }

            slot_reserved_until.text = getString(R.string.slot_reserved_until, reservedUntilTime)
        } ?: run {
            booked_slot_details_stub.visibility = View.GONE
        }
    }

    private fun setBookedSlotDetailsWithATGCart() {
        viewModel.cartExtraInfo = CartManager.get(context).cartExtraInfo
        viewModel.cart = CartManager.get(context).cart
        viewModel.cartManager = CartManager.get(context)

        if (booked_slot_details_stub == null) {
            return
        }

        if (!viewModel.cartExtraInfo?.bookedSlotId.isNullOrBlank()) {

            //TODO DELETE detaul_stub
            //booked_slot_details_stub.visibility = View.VISIBLE

            // Check if the booked slot is Home Delivery option.
            if (viewModel.cart?.deliveryOption.equals(SlotResponse.DELIVERY_OPTION_HOME_DELIVERY)) {
                delivery_image.setImageResource(R.drawable.ic_truck)
                slot_reserved_header_text.text = getString(R.string.home_delivery_slot_booked)
                slot_reserved_address.text = getFormattedAddress(viewModel.cart?.extraInfo?.deliveryAddress?.address,
                      viewModel.cart?.extraInfo?.deliveryAddress?.address2,
                      viewModel.cart?.extraInfo?.deliveryAddress?.city,
                      viewModel.cart?.extraInfo?.deliveryAddress?.postalCode)
            } else {
                setBookedCNCInfo()
                slot_reserved_address.text = getFormattedAddress(viewModel.cart?.extraInfo?.cncStoreAddress?.addressLine1,
                      viewModel.cart?.extraInfo?.cncStoreAddress?.addressLine2,
                      viewModel.cart?.extraInfo?.cncStoreAddress?.city,
                      viewModel.cart?.extraInfo?.cncStoreAddress?.postalCode)
            }
            slot_reserved_price.text = viewModel.cart?.deliveryPrice
            slot_reserved_time.text = getString(R.string.slot_reserved_date_from_to,
                  viewModel.cart?.slotDate, viewModel.cart?.deliveryStartTime, viewModel.cart?.deliveryEndTime)
            val reservedUntilTime = if (viewModel.cart?.isRecurringSlot == true) {
                SubscriptionUtil.getSubscriptionExpiryDate(viewModel.cart?.slotDate,
                      DateFormats.EEE_DD_MMM_YYYY,
                      DateFormats.HH_MM_AA_EEE_DD_MMM,
                      TimeZoneFormat.UK,
                      TimeZoneFormat.UK)
            } else viewModel.cart?.bookedUntilTime

            slot_reserved_until.text = getString(R.string.slot_reserved_until, reservedUntilTime)
        } else {
            booked_slot_details_stub.visibility = View.GONE
        }
    }
    //endregion setBookedSlotDetails

    open fun openBookSlotConfirmationPage() {
        val slotDates = viewModel.getBookedStartAndEndTime()
        val intent = DeliveryImpactActivity.newIntent(activity,
              if (currentFragmentType == HOME_DELIVERY)
                  CartManager.SLOT_TYPE_DELIVERY
              else CartManager.SLOT_TYPE_CC, slotDates.first, slotDates.second, SlotBookedConfirmationPresenter.TAG)
        startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
    }

    private fun setBookedCNCInfo() {
        context?.let {
            val isExpressSlotBooked = if (FeatureSettings.isCXOMigrationEnabled(it))
                ExpressUtils.isExpressSlotBooked(viewModel.cxoCart)
            else
                ExpressUtils.isExpressSlotBooked(viewModel.cart)

            if (isExpressSlotBooked) {
                slot_reserved_header_text.text = ExpressUtils.getExpressIconText(it)
                slot_reserved_until.setTextColor(ContextCompat.getColor(it, R.color.asda_red))
                slot_text?.visibility = View.VISIBLE
            } else {
                slot_reserved_until.setTextColor(ContextCompat.getColor(it, R.color.black))
                slot_reserved_header_text.text = getString(R.string.click_and_collect_slot_booked)
                slot_text?.visibility = View.GONE
            }
            delivery_image.setImageResource(R.drawable.ic_click_collect)
        }
    }

    open fun setAddress(serviceAddress: ServiceAddress?) {
        book_slot_location_type_address?.let {
            book_slot_location_type_address.text = getFormattedAddress(serviceAddress?.address_line_one,
                  serviceAddress?.address_line_two, serviceAddress?.city, serviceAddress?.postcode)
        }
    }

    private fun getFormattedAddress(addressLine1: String?, addressLine2: String?, city: String?, postalCode: String?): String {
        val sb = StringBuilder()

        if (!addressLine1.isNullOrBlank()) sb.append(addressLine1).append(", ")
        if (!addressLine2.isNullOrBlank()) sb.append(addressLine2).append(", ")
        if (!city.isNullOrBlank()) sb.append(city).append(", ")
        if (!postalCode.isNullOrBlank()) sb.append(postalCode)
        return sb.toString()
    }


    private fun loadSlots(viewSlotData: ViewSlotResponseV3.Data?) {
        viewModel.isLoading = false
        viewSlotData?.let {
            if (viewModel.isFirstPage) {
                viewModel.isFirstPage = false
                updateRecyclerViewLayoutManager(viewSlotData)
                addAutoScrollToBookedSlot()
            }
            bookSlotAdapter?.setViewingSlotsForBookedSlotAddress(viewModel.viewingSlotsForBookedSlotAddress())
            bookSlotAdapter?.setBookedStartAndEndTime(viewModel.getBookedStartAndEndTime())
            viewModel.data.addAll(getFormattedData(viewSlotData, viewModel.numOfRows))
            bookSlotAdapter?.notifyDataSetChanged()
        }
    }

    // On the first page load, this method will auto scroll the RecyclerView to the position of
    // the booked slot. Since the RecyclerView is placed inside NestedScrollView, adding a
    // addOnLayoutChangeListener on NestedScrollView works as expected.
    private fun addAutoScrollToBookedSlot() {
        book_slot_nested_scroll_view.addOnLayoutChangeListener(object : View.OnLayoutChangeListener {
            override fun onLayoutChange(v: View?, left: Int, top: Int, right: Int, bottom: Int, oldLeft: Int, oldTop: Int, oldRight: Int, oldBottom: Int) {
                book_slot_nested_scroll_view.removeOnLayoutChangeListener(this)
                var height = 0
                var heightMargin = 0
                context?.let {
                    height = it.resources.getDimension(R.dimen.row_height).toInt()
                    heightMargin = it.resources.getDimension(R.dimen.grid_item_space).times(viewModel.bookedSlotIdRowPositionInRecyclerViewGrid).toInt()
                }
                val scrollTo = height.times(viewModel.bookedSlotIdRowPositionInRecyclerViewGrid) + heightMargin
                book_slot_nested_scroll_view.smoothScrollTo(0, scrollTo)
                gridLayoutManager.scrollToPositionWithOffset(viewModel.bookedSlotIdPositionInRecyclerViewGrid, 0)
            }
        })
    }

    private fun initAdapter() {
        context?.let {
            bookSlotAdapter = BookSlotAdapter(requireContext(), viewModel.data, currentFragmentType, this)
            book_slot_grid_view.adapter = bookSlotAdapter
            if (book_slot_grid_view.itemDecorationCount == 0)
                book_slot_grid_view.addItemDecoration(ItemDecorationGrid(resources.getDimensionPixelSize(R.dimen.grid_item_space)))
            addRecyclerViewScrollListener()
        }
    }

    private fun addRecyclerViewScrollListener() {
        recyclerViewOnScrollListener = object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                val visibleItemCount = gridLayoutManager.childCount
                val totalItemCount = gridLayoutManager.itemCount
                val firstVisibleItemPosition = gridLayoutManager.findFirstVisibleItemPosition()

                if (!viewModel.isLoading && !viewModel.isLastPage
                      && visibleItemCount + firstVisibleItemPosition >= totalItemCount
                      && firstVisibleItemPosition >= 0
                      && totalItemCount >= (viewModel.data.size)) {
                    viewModel.isLoading = true
                    viewSlots()
                }
            }
        }
        book_slot_grid_view.addOnScrollListener(recyclerViewOnScrollListener)
    }

    private fun updateRecyclerViewLayoutManager(viewSlotData: ViewSlotResponseV3.Data) {
        viewSlotData.slot_days?.let { slotDays ->
            if (slotDays.isNotEmpty()) {
                slotDays[0].slots?.let { slots ->
                    viewModel.numOfRows = slots.size + 1
                }
            }
        }
        gridLayoutManager = GridLayoutManager(context, viewModel.numOfRows, GridLayoutManager.HORIZONTAL, false)
        book_slot_grid_view.layoutManager = gridLayoutManager
    }

    private fun getFormattedData(viewSlotData: ViewSlotResponseV3.Data, numOfRows: Int): ArrayList<Any> {
        val list = ArrayList<Any>()

        viewSlotData.slot_days?.let { slotDays ->
            fillRows(slotDays, list, numOfRows)

            // Save the latest date timestamp in currentLastDateTimeStamp in order to make the next api call.
            if (slotDays.isNotEmpty()) {
                viewModel.currentLastDateTimeStamp = slotDays[slotDays.size - 1].slot_date
                viewModel.bannerDate?.let {
                    viewModel.currentBannerLastDateTimestamp = slotDays[slotDays.size - 1].slot_date
                } ?: run {
                    viewModel.currentBannerLastDateTimestamp = null
                }
            }
        }
        return list
    }

    private fun fillRows(slotDays: List<SlotDay>, list: ArrayList<Any>, numOfRows: Int) {
        for (slotDayIndex in 0 until slotDays.size) {
            val slotDay = slotDays[slotDayIndex]
            slotDay.slots?.let { slots ->
                when {
                    slotDay.slot_date?.isToday(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z) == true -> list.add("Today")
                    slotDay.slot_date?.isTomorrow(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z) == true -> list.add("Tomorrow")
                    else -> list.add(slotDay.slot_date?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEE_DD_MMM).orEmpty())
                }
                // If we don't have exact number of slots for each slot day then return and don't populate rest of the data.
                if (slots.size < (numOfRows - 1)) {
                    return
                } else {
                    fillColumns(list, slots, slotDay, numOfRows)
                }
            }
        }
    }

    private fun fillColumns(list: ArrayList<Any>, slots: List<Slot>, slotDay: SlotDay, numOfRows: Int) {
        for (slotIndex in 0 until slots.size) {
            val slot = slots[slotIndex]
            if (book_slot_grid_column_header_container.childCount < numOfRows) {
                fillTimeSlots(slot)
            }
            slot.slot_info?.let { slotInfo ->
                slotInfo.slotDayAccesspointId = slotDay.accesspoint_id
                slotInfo.slotDayDispenseStoreId = slotDay.dispense_store_id
                slotInfo.slotDayFulfillStoreId = slotDay.fulfill_store_id
                slotInfo.slotDayRemoteSiteId = slotDay.remote_site_id
                list.add(slotInfo)
                context?.let {
                    val slotId = if (FeatureSettings.isCXOMigrationEnabled(it))
                        viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.cartSlotId
                    else viewModel.cartExtraInfo?.bookedSlotId

                    if (slotInfo.slot_id == slotId) {
                        viewModel.bookedSlotIdPositionInRecyclerViewGrid = viewModel.data.size + list.size - 1
                        viewModel.bookedSlotIdRowPositionInRecyclerViewGrid = slotIndex
                    }
                }
            }
        }
    }

    private fun fillTimeSlots(slot: Slot) {
        val textView = TextView(context)

        textView.text = resources.getString(R.string.slot_time_from_to,
              slot.slot_info?.start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_KK_MM_SS_Z, DateFormats.H_MM).orEmpty().toLowerCase(),
              slot.slot_info?.end_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_KK_MM_SS_Z, DateFormats.H_MM_AA).orEmpty().toLowerCase())

        val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, resources.getDimensionPixelSize(R.dimen.row_height), 1.0f)
        val itemMargin = resources.getDimensionPixelSize(R.dimen.grid_item_col_header_margin)
        val itemTopMargin = resources.getDimensionPixelSize(R.dimen.grid_item_space)
        params.setMargins(itemMargin, itemTopMargin, itemMargin, 0)
        params.gravity = Gravity.CENTER_VERTICAL or Gravity.END
        textView.layoutParams = params
        textView.gravity = Gravity.CENTER_VERTICAL or Gravity.END
        context?.let {
            textView.setTextSize(TypedValue.COMPLEX_UNIT_PX, it.resources.getDimension(R.dimen.text_size_14dp))
        }
        book_slot_grid_column_header_container.addView(textView)
    }

    /**
     * For Home Delivery fragment, if the slot is FULL, and user clicks on the item, then switch
     * the tab to ClickNCollect fragment.
     * If the slot is not full, handle the click in default way.
     */
    override fun onGridItemClick(itemData: Any, gridItemViewParent: View) {
        if (itemData is SlotInfo) {
            when {
                SlotStatus.UNAVAILABLE.type == itemData.status && itemData.hasRestrictedItem -> {
                    showPopUpWindowRestrictedItemCoachmark(gridItemViewParent)
                }
                SlotStatus.UNAVAILABLE.type == itemData.status -> {
                    slotEvents?.switchTabsTo(if (currentFragmentType == CLICK_AND_COLLECT) HOME_DELIVERY else CLICK_AND_COLLECT)
                }
                SlotStatus.AVAILABLE.type == itemData.status -> {
                    makeReserveCall(itemData, false) //deliveryImpactContinue = false, means delivery_impact = check
                    viewModel.slotInfoOfReserveCallRequest = itemData
                }
            }
        }
    }

    private fun showPopUpWindowRestrictedItemCoachmark(gridItemViewParent: View) {
        if (coachMark?.isShowing == true) {
            coachMark?.dismiss()
        }
        val view: View = ViewUtil.inflate(requireContext(), R.layout.popup_restricted_item)
        coachMark = BubbleCoachMark.BubbleCoachMarkBuilder(requireContext(), gridItemViewParent, view)
              .setTargetOffset(0.5f)
              .setDismissOnMove(true)
              .setShowBelowAnchor(true)
              .setTimeout(20000)
              .setPadding(0)
              .build()
        if (coachMark is BubbleCoachMark) {
            val bubbleCoachMart = coachMark as BubbleCoachMark
            bubbleCoachMart.setTopArrow(R.drawable.ic_arrow_up_24, requireContext().resources.getDimension(R.dimen.dp_12))
            bubbleCoachMart.setBottomArrow(R.drawable.ic_arrow_down_24, requireContext().resources.getDimension(R.dimen.dp_12))
        }
        coachMark?.let {
            it.show()
        }
    }

    private fun useBookSlotPreview(): Boolean {
        val prefs = context?.getSharedPreferences(SharedPreferencesUtil.GENERAL_PREFS, Activity.MODE_PRIVATE)
        prefs?.let {
            return it.getBoolean(SharedPreferencesUtil.PREF_TWO_FACTOR_BOOK_SLOT, true)
        }
        return true // default is true
    }

    private fun makeReserveCall(slotInfo: SlotInfo, deliveryImpactContinue: Boolean) {
        showLoader(true)
        viewModel.deliveryImpactContinue = deliveryImpactContinue
        var profileId = Authentication.getInstance().userViewResponse?.profileId
        context?.let {
            if (FeatureSettings.isProfileIdAcceptedInBookSlot(it) && !SingleProfile.getProfileResponse()?.profileId.isNullOrEmpty())
                profileId = SingleProfile.getProfileResponse()?.profileId

            if (profileId.isNullOrEmpty() || !Authentication.getInstance().isLoggedIn) {
                showLoginAlertDialog()
                return
            }
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                viewModel.makeCXOReserveCall(slotInfo, !deliveryImpactContinue, viewModel.viewSlotData,
                      getServiceAddressForTab(),
                      Authentication.getInstance().userViewResponse?.token,
                      profileId,
                      FeatureSettings.isSlotSecondaryStoreEnabled(it)
                )
            } else {
                viewModel.makeReserveCall(slotInfo, deliveryImpactContinue, viewModel.viewSlotData,
                      getServiceAddressForTab(),
                      Authentication.getInstance().userViewResponse?.token,
                      profileId,
                      FeatureSettings.isSlotSecondaryStoreEnabled(it)
                )
            }
        }
    }

    private fun handleDeliveryImpact(reserveSlotCXOResponse: ReserveSlotCXOResponse?) {
        showLoader(false)
        val deliveryImpact: DeliveryImpact? = reserveSlotCXOResponse?.data?.delivery_impact
        when {
            deliveryImpact?.unavailableRecipes?.isNotEmpty() == true -> {
                val intent = DeliveryImpactActivity.newIntent(activity, deliveryImpact, UnavailableRecipesFragment.TAG)
                startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
            }
            deliveryImpact?.ingredientsUnavailableRecipes?.isNotEmpty() == true -> { //TODO handle delivery impact for CXO in next story
                val intent = DeliveryImpactActivity.newIntent(activity, deliveryImpact, ImpactedRecipeListFragment.TAG)
                startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
            }
            deliveryImpact?.priceChangedItems?.isNotEmpty() == true || deliveryImpact?.promoChangedItems?.isNotEmpty() == true ||
                  deliveryImpact?.delistedItems?.isNotEmpty() == true -> {
                val intent = DeliveryImpactActivity.newIntent(activity, deliveryImpact, PreviewBookSlotPresenter.TAG)
                startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
            }
            reserveSlotCXOResponse?.errors?.isNotEmpty() == true -> {
                reserveSlotCXOResponse.errors?.let {
                    handleReserveSlotFailure(it[0])
                }
            }
            else -> reserveSlotCXOResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.let {
                onSlotBookSuccess()
            }
        }
    }

    private fun handleReserveSlotSuccess(reserveSlotResp3: ReserveSlotResponseV3?) {
        showLoader(false)

        if (!useBookSlotPreview()) {
            viewSlots()
        } else {
            when {
                reserveSlotResp3?.data?.unavailable_recipes?.isNotEmpty() == true -> {
                    val intent = DeliveryImpactActivity.newIntent(activity, reserveSlotResp3, UnavailableRecipesFragment.TAG)
                    startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
                }
                reserveSlotResp3?.data?.impacted_recipes?.isNotEmpty() == true -> {
                    val intent = DeliveryImpactActivity.newIntent(activity, reserveSlotResp3, ImpactedRecipeListFragment.TAG)
                    startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
                }
                PreviewBookSlotPresenter.getShelfItems(context, reserveSlotResp3?.data?.messages).isNotEmpty() -> {
                    val intent = DeliveryImpactActivity.newIntent(activity, reserveSlotResp3, PreviewBookSlotPresenter.TAG)
                    startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
                }
                reserveSlotResp3?.errors?.isNotEmpty() == true -> {
                    reserveSlotResp3.errors?.let {
                        handleReserveSlotFailure(it[0])
                    }
                }
                else -> reserveSlotResp3?.data?.slot_info?.let {
                    onSlotBookSuccess(reserveSlotResp3)
                }
            }
        }
    }

    private fun onSlotBookSuccess() {
        trackBookSlot()
        viewModel.shouldShowBookSlotConfirmPage = true
        setBookedSlotDetails()
        resetGrid()
        viewSlots()
    }

    private fun onSlotBookSuccess(reserveSlotResp3: ReserveSlotResponseV3?) {

        context?.let {
            val cartManager = CartManager.get(it)
            val slotType = if (FulfillmentTypes.DELIVERY.type == reserveSlotResp3?.data?.service_info?.fulfillment_type) {
                CartManager.SLOT_TYPE_DELIVERY
            } else {
                CartManager.SLOT_TYPE_CC
            }

            cartManager?.setSlotWasBooked(slotType)
            cartManager.refreshCart(it)
        }
        trackBookSlot()
        viewModel.shouldShowBookSlotConfirmPage = true
    }

    private fun trackBookSlot() {
        val fulfillmentType = if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType.orEmpty() else viewModel.cart?.fulfillmentType.orEmpty()
        val isHomeDelivery = FulfillmentTypes.DELIVERY.type == fulfillmentType
        val hasDeliveryPass = CartManager.get(context)?.activeDeliveryPass != null

        var slotCost: String? = if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.cartPriceSummary?.fulfillmentCost?.fulfillmentCostAfterDiscount?.toString() else viewModel.cart?.totalCost.orEmpty()
        if (slotCost != null) {
            slotCost = slotCost.replace("£", "")
        }
        //Track for book successful event
        Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BOOK_SLOT)
              .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata)
              .putString(Anivia.CUSTOMER_ID_KEY, Authentication.getInstance()?.userViewResponse?.singleProfileId
                    ?: " ")
              .putString(Anivia.NAME_KEY, BOOK_SLOT_CONFIRMATION)
              .putBoolean(Anivia.DP_USED_IN_BOOK_SLOT, hasDeliveryPass)
              .putString(Anivia.SLOT_COST_KEY, slotCost)
              .putString(Anivia.SLOT_DAY_KEY, RestUtils.getDayOfTheDate(if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM).toString() else viewModel.cart?.slotDate.orEmpty()))
              .putString(Anivia.SLOT_TYPE_KEY, if (isHomeDelivery) FULFILLMENT_TYPE_HD else FULFILLMENT_TYPE_CNC)
              .putString(Anivia.DAYS_TO_DELIVER_KEY, RestUtils.getDaysToDelivery(if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z) else viewModel.cart?.slotDate?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)))
              .putString(Anivia.DATE_TIME, System.currentTimeMillis().toString())
              .putString(Anivia.SLOT_TIME_KEY, if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM).toString() else viewModel.cart?.deliveryStartTime)
              .putString(Anivia.IS_EXPRESS_SLOT, (viewModel.slotInfoOfReserveCallRequest?.is_express_slot == true).toString())
              .putString(Anivia.IS_SAME_DAY, (viewModel.slotInfoOfReserveCallRequest?.isSameDayPremium == true).toString())
              .putString(Anivia.USER_GROUP, AnalyticsConfig.userGroup)
        )
    }

    private fun handleReserveSlotFailure(error: Error) {
        showLoader(false)
        trackBookSlotFailure(error, "v3/slots/reserve")
        Toast.makeText(context, "Could not book your slot. Trying again", Toast.LENGTH_SHORT).show()
        resetGrid()
        viewSlots()
    }

    private fun trackBookSlotFailure(error: Error, apiName: String) {
        val isHomeDelivery = FulfillmentTypes.DELIVERY.type == viewModel.cart?.fulfillmentType
        val cartManager = CartManager.get(context)
        val deliveryPass = cartManager?.activeDeliveryPass
        val hasDeliveryPass = deliveryPass != null

        var slotCost: String? = viewModel.cart?.totalCost.orEmpty()
        if (slotCost != null) {
            slotCost = slotCost.replace("£", "")
        }
        //Track for bookslot failure event
        Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.API_ERROR_EVENT)
              .putString(Anivia.API_NAME_KEY, apiName)
              .putString(Anivia.API_ERROR_CODE, error.code)
              .putString(Anivia.API_COPY, error.message)
              .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata)
              .putString(Anivia.CUSTOMER_ID_KEY, Authentication.getInstance()?.userViewResponse?.singleProfileId
                    ?: " ")
              .putString(Anivia.NAME_KEY, BOOK_SLOT_CONFIRMATION)
              .putBoolean(Anivia.DP_USED_IN_BOOK_SLOT, hasDeliveryPass)
              .putString(Anivia.SLOT_COST_KEY, slotCost)
              .putString(Anivia.SLOT_DAY_KEY, RestUtils.getDayOfTheDate(if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToDate(DateFormats.EEEE_DD_MMMM_YYYY).toString() else viewModel.cart?.slotDate.orEmpty()))
              .putString(Anivia.SLOT_TYPE_KEY, if (isHomeDelivery) FULFILLMENT_TYPE_HD else FULFILLMENT_TYPE_CNC)
              .putString(Anivia.DAYS_TO_DELIVER_KEY, RestUtils.getDaysToDelivery(if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z) else viewModel.cart?.slotDate?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)))
              .putString(Anivia.DATE_TIME, System.currentTimeMillis().toString())
              .putString(Anivia.SLOT_TIME_KEY, if (viewModel.isCXOMigrationEnabled) viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z).toString() else viewModel.cart?.deliveryStartTime)
              .putString(Anivia.IS_EXPRESS_SLOT, (viewModel.slotInfoOfReserveCallRequest?.is_express_slot == true).toString())
              .putString(Anivia.IS_SAME_DAY, (viewModel.slotInfoOfReserveCallRequest?.isSameDayPremium == true).toString())
        )
    }

    open fun viewSlots(shouldShowLoading: Boolean = false) {
        showLoader(shouldShowLoading)
        if (viewModel.isFirstPage && !isFullPageLoaderShown()) {
            viewModel.data.clear()
            showGridRetryView(shouldShow = true, isRetrying = true)
        }

        if (viewModel.fulfillmentType.isNotEmpty()) {
            val profileId = if (context != null && FeatureSettings.isProfileIdAcceptedInBookSlot(requireContext()))
                SingleProfile.getProfileResponse()?.profileId
            else {
                Authentication.getInstance().userViewResponse?.profileId
            }

            val xCatchmentEnabled = Authentication.getInstance().userViewResponse?.xoverCatchmentEnabled
                  ?: false

            if (!isViewSlotCallInProgress) {
                isViewSlotCallInProgress = true
                viewModel.makeViewSlotCall(profileId,
                      viewModel.fulfillmentType,
                      resources.getInteger(R.integer.view_slot_columns),
                      resources.getInteger(R.integer.view_slot_columns_first_load),
                      viewModel.serviceAddress,
                      xCatchmentEnabled,
                      if (this is BookSlotClickNCollectFragment) {
                          BookSlotViewModel.CLICK_COLLECT
                      } else {
                          BookSlotViewModel.HOME_DELIVERY
                      })
            }
        }
    }

    // Handling of showing the slots in a grid, vs the error scenario for grid will be handled here.
    private fun showGridRetryView(shouldShow: Boolean, isRetrying: Boolean = false, hideOptionalLinksOnErrorView: Boolean = false) {
        //This will show the "retry" view on top of grid with retry button.
        if (shouldShow) {
            book_slot_grid_holder.visibility = View.GONE
            book_slot_grid_error_holder.visibility = View.VISIBLE
            book_slot_retry_loading_grid_view.visibility = View.VISIBLE
            book_slot_retry_loading_grid_loader.visibility = View.GONE
        } else {
            // This block will hide the "retry" view, and show the normal grid.
            book_slot_grid_holder.visibility = View.VISIBLE
            book_slot_grid_error_holder.visibility = View.GONE
            book_slot_retry_loading_grid_view.visibility = View.GONE
            book_slot_retry_loading_grid_loader.visibility = View.GONE
        }
        // This will show the loader on grid retry view, this way user will know that the retry
        // button is pressed and loading is in progress.
        if (isRetrying) {
            book_slot_retry_loading_grid_loader.visibility = View.VISIBLE
            book_slot_grid_error_holder.visibility = View.GONE
        } else {
            book_slot_retry_loading_grid_loader.visibility = View.GONE
        }

        if (hideOptionalLinksOnErrorView) {
            add_or_update_address_textview.visibility = View.GONE
            add_or_update_address_textview_separator.visibility = View.GONE
            collect_your_shopping_from_us_textview.visibility = View.GONE
            collect_your_shopping_from_us_textview_separator.visibility = View.GONE
        } else {
            add_or_update_address_textview.visibility = View.VISIBLE
            add_or_update_address_textview_separator.visibility = View.VISIBLE
            collect_your_shopping_from_us_textview.visibility = View.VISIBLE
            collect_your_shopping_from_us_textview_separator.visibility = View.VISIBLE
        }
    }


    fun resetGrid() {
        isViewSlotCallInProgress = false
        viewModel.currentLastDateTimeStamp = null
        viewModel.currentBannerLastDateTimestamp = null
        viewModel.isFirstPage = true
        viewModel.isLastPage = false
        viewModel.data.clear()
        book_slot_grid_view?.recycledViewPool?.clear()
        bookSlotAdapter?.notifyDataSetChanged()
        if (book_slot_grid_column_header_container.childCount > 1) {
            book_slot_grid_column_header_container.removeViews(1, book_slot_grid_column_header_container.childCount - 1)
        }
        viewModel.bookedSlotIdRowPositionInRecyclerViewGrid = 0
        viewModel.bookedSlotIdPositionInRecyclerViewGrid = 0
        setBannerConfig(viewModel.bannerConfig)
    }

    private fun showLoginAlertDialog() {
        context?.let {
            val asdaAlertDialog = AsdaAlertDialog.newInstance(
                  leftButtonText = context?.getString(R.string.login_dialog_login_button).orEmpty(),
                  rightButtonText = context?.getString(R.string.cancel).orEmpty(),
                  message = context?.getString(R.string.sign_in_before_reserving_slot).orEmpty())
            asdaAlertDialog.onLeftButtonClick {
                val intent = Intent(requireContext(), LoginSpActivity::class.java)
                intent.putExtra(EXTRA_MESSAGE, getString(R.string.sign_in_before_reserving_slot))
                intent.putExtra(EXTRA_ORIGIN, "bookslot")
                startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_LOGIN)
                asdaAlertDialog.dismiss()
            }
            asdaAlertDialog.onRightButtonClick {
                showLoader(false)
                asdaAlertDialog.dismiss()
            }
            fragmentManager?.let {
                asdaAlertDialog.show(it, TAG)
            }
        }
    }

    private fun setupSecondaryStoreFooter() {
        if (FeatureSettings.isSlotSecondaryStoreEnabled(requireContext())) {
            secondary_slot_indicator_information.visible()
        } else {
            secondary_slot_indicator_information.gone()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            DeliveryImpactActivity.REQUEST_CODE -> {
                when (resultCode) {
                    RESULT_OK -> {
                        viewModel.slotInfoOfReserveCallRequest?.let { slotInfo ->
                            makeReserveCall(slotInfo, true) //deliveryImpactContinue = true, means delivery_impact = continue
                        }
                    }
                    ActivityRequestCodes.RESULT_CODE_DO_SHOPPING -> {
                        activity?.setResult(ActivityRequestCodes.RESULT_CODE_DO_SHOPPING)
                        activity?.finish()
                    }
                    ActivityRequestCodes.RESULT_CODE_DO_CHECKOUT -> {
                        activity?.setResult(ActivityRequestCodes.RESULT_CODE_DO_CHECKOUT)
                        activity?.finish()
                    }
                    ActivityRequestCodes.RESULT_CODE_FINISH_ACTIVITY -> {
                        activity?.finish()
                    }
                    ActivityRequestCodes.RESULT_REFRESH_BOOKED_SLOT -> {
                        (parentFragment as? BookSlotParentFragment)?.setBookedSlotChangedLiveData(true)
                    }
                }
            }
            ActivityRequestCodes.REQUEST_CODE_LOGIN -> {
                viewModel.slotInfoOfReserveCallRequest?.let {
                    makeReserveCall(it, false)
                }
            }
        }
    }

    fun setBannerConfig(bannerConfig: BannerConfig?) {
        bannerConfig?.let { _bannerConfig ->
            context?.let { _context ->
                val hasDeliveryPass = CartManager.get(context).activeDeliveryPass != null
                if (SharedPreferencesUtil.isSlotBannerForAll(_context) || (SharedPreferencesUtil.isSlotBannerForDP(_context) && hasDeliveryPass && !SlotUtils().isPriorityPassCustomer(Authentication.getInstance().userViewResponse))) {
                    banner_config_container?.visibility = View.VISIBLE

                    viewModel.bannerDate?.let {
                        if (_bannerConfig.backText?.isNotEmpty() == true) {
                            banner_text?.text = _bannerConfig.backText
                            banner_text?.setTextColor(ContextCompat.getColor(_context, R.color.asda_black))
                        }

                        banner_background_image.setBackgroundResource(R.drawable.button_background_gray)
                        banner_background_image.setImageDrawable(null)

                        banner_action?.text = Utils.fromHtml(_context.getString(R.string.back_to_regular_slots))
                        banner_action?.setTextColor(ContextCompat.getColor(_context, R.color.asda_blue))

                    } ?: run {
                        if (_bannerConfig.text?.isNotEmpty() == true) {
                            banner_text?.text = _bannerConfig.text
                            banner_text?.setTextColor(ContextCompat.getColor(_context, R.color.white))
                        }

                        if (_bannerConfig.backgroundImageUrl?.isNotEmpty() == true) {
                            banner_background_image.background = null
                            ImageLoader.load(context, banner_background_image, ImageLoaderRequest.Builder().path(_bannerConfig.backgroundImageUrl).build(), null)
                        }

                        if (_bannerConfig.buttonTitle?.isNotEmpty() == true) {
                            banner_action?.setTextColor(ContextCompat.getColor(_context, R.color.yellow_f7cc00))
                            banner_action?.text = Utils.fromHtml(_context.getString(R.string.underlined_text, _bannerConfig.buttonTitle))
                        }
                    }

                    banner_config_container?.setOnClickListener {
                        viewModel.bannerDate?.let {
                            viewModel.bannerDate = null
                        } ?: run {
                            //No String resource as its a date format.
                            val day = _bannerConfig.startDateComponent?.day
                            val month = _bannerConfig.startDateComponent?.month
                            val year = _bannerConfig.startDateComponent?.year
                            viewModel.bannerDate = "$year-$month-$day" + "T00:00:00Z"
                        }
                        resetGrid()
                        viewSlots(true)
                    }
                } else {
                    banner_config_container?.visibility = View.GONE
                }
            }
        } ?: run {
            banner_config_container?.visibility = View.GONE
        }
    }

    abstract fun setDeliveryHeaderText()

    abstract fun setAddress()

    abstract fun setExpressInfo()

    abstract fun getCurrentFragmentTitle(context: Context): String

    abstract fun inflateHeaderView(view: View)

    abstract fun onLocationTypeAddressClicked()

    abstract fun getServiceAddressForTab(): ServiceAddress?

    abstract fun setAdditionalCostText()

    abstract fun handleSlotData()

    abstract fun setExpressAvailable(expressAvailable: Boolean)
}

