package com.asda.android.app.bookslotv3.view.fragment

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.ImageSpan
import android.view.View
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.address.view.AddAddressActivity
import com.asda.android.app.account.address.view.SelectAddressActivity
import com.asda.android.app.bookslot.SlotBookedConfirmationPresenter
import com.asda.android.app.bookslotv3.BookSlotType.CLICK_AND_COLLECT
import com.asda.android.app.bookslotv3.BookSlotType.HOME_DELIVERY
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.dp.model.ShowDPCardValidatorModel
import com.asda.android.app.dp.validator.ShowDPOfferValidator
import com.asda.android.app.dp.view.DPOfferCardViewHandler
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.recurringslot.SlotSubscriptionManager
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityInfo
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.utils.ExpressUtils
import com.asda.android.utils.Utils
import kotlinx.android.synthetic.main.book_slot_delivery_pass.*
import kotlinx.android.synthetic.main.book_slot_header_dp_booked_slot.*
import kotlinx.android.synthetic.main.fragment_book_slot.*
import kotlinx.android.synthetic.main.home_card_express.*
import kotlinx.android.synthetic.main.order_summary_express_details.*

class BookSlotHomeDeliveryFragment : BookSlotFragment(HOME_DELIVERY) {
    private var ADDRESS_SELECTION_REQ_CODE: Int = 1001

    private var dpOfferCardViewHandler: DPOfferCardViewHandler? = null
    private val showDPOfferValidator: ShowDPOfferValidator by lazy { ShowDPOfferValidator() }
    private var mRecurringSlots: RecurringSlotsViewResponse? = null
    private var subscriptionEligibility: SubscriptionEligibilityInfo? = null
    private val mRecurringSlotsListener = object : RecurringSlotsViewResponse.Listener() {

        override fun onRecurringSlotsReceived(recurringSlotsViewResponse: RecurringSlotsViewResponse) {
            mRecurringSlots = recurringSlotsViewResponse
            updateRecurringSlotsView()
        }
    }

    companion object {
        const val TAG = "BookSlot-HomeDelivery"
        fun createInstance(slotEvents: SlotEvents): BookSlotFragment {
            val fragment = BookSlotHomeDeliveryFragment()
            fragment.slotEvents = slotEvents
            return fragment
        }
    }

    override fun onStart() {
        super.onStart()
        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_BOOK_SLOT_HD, Anivia.SECTION_BOOK_SLOT))
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it)) {
                if (!viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.cartSlotId.isNullOrBlank()
                      && !viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType.isNullOrBlank()
                      && FulfillmentTypes.DELIVERY.type != viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType) {
                    slotEvents?.switchTabsTo(CLICK_AND_COLLECT)
                }
            } else if (!viewModel.cart?.extraInfo?.bookedSlotId.isNullOrBlank()
                  && !viewModel.cart?.fulfillmentType.isNullOrBlank()
                  && FulfillmentTypes.DELIVERY.type != viewModel.cart?.fulfillmentType) {
                slotEvents?.switchTabsTo(CLICK_AND_COLLECT)
            }
        }
        setupRecurringSlotObservers()
    }

    override fun onDestroyView() {
        RecurringSlotsViewResponse.removeListener(mRecurringSlotsListener)
        super.onDestroyView()
    }

    override fun inflateHeaderView(view: View) {
        book_slot_delivery_pass_details_stub.visibility = View.VISIBLE
    }

    override fun getCurrentFragmentTitle(context: Context): String {
        return context.getString(R.string.home_delivery)
    }

    override fun setDeliveryHeaderText() {
        updateDeliveryPassView()
    }

    override fun setAddress() {

        // ServiceAddress might be populated from onActivityResult, if that is the case it takes
        // priority over the rest of the addresses found.

        if (viewModel.serviceAddress == null)
            if (FeatureSettings.isCXOMigrationEnabled(context!!))
                viewModel.serviceAddress = viewModel.getHomeDeliveryServiceAddress(
                      if (viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.slotExpired == false)
                          viewModel.cxoCart?.fulfillmentDetails?.fulfillmentAddress else null,
                      SingleProfile.getProfileResponse()?.addresses)
            else
                viewModel.serviceAddress = viewModel.getHomeDeliveryServiceAddress(viewModel.cartExtraInfo?.deliveryAddress,
                      SingleProfile.getProfileResponse()?.addresses)

        if (viewModel.serviceAddress != null) {
            setAddress(viewModel.serviceAddress)
        } else {
            showAddAddressScreen()
        }

        viewModel.fulfillmentType = FulfillmentTypes.DELIVERY.type
    }

    override fun setExpressInfo() {
        var showExpressInfo: Boolean = false
        if (context?.let { FeatureSettings.isCXOMigrationEnabled(it) } == true) {
            val cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
            showExpressInfo = cxoCart?.cartItemSummary?.lineItemCount ?: 0 <= 0 && cxoCart?.cartItemSummary?.totalQuantity ?: 0 <= 0 && Utils.isExpressEnabled(context)
                  && Utils.getExpressFlagHD()
        } else {
            showExpressInfo = context != null && CartManager.get(context).lineItemCount != null && CartManager.get(context).totalQuantity != null
                  && ExpressUtils.showExpressInfo(CartManager.get(context).lineItemCount, CartManager.get(context).totalQuantity) && Utils.isExpressEnabled(context)
                  && Utils.getExpressFlagHD()
        }
        if (showExpressInfo) {
            express_info.visibility = View.VISIBLE
            val spannableStringBuilder = SpannableStringBuilder()
                  .append(context?.getString(R.string.add_items_to_cart))
                  .append(context?.getString(R.string.double_spaces))
            spannableStringBuilder.setSpan(ImageSpan(requireContext(), R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            spannableStringBuilder.append(context?.getString(R.string.single_space)).append(context?.getString(R.string.slots_available))
            express_text.text = spannableStringBuilder
        } else {
            express_info.visibility = View.GONE
        }
    }

    private fun showAddAddressScreen() {
        AddAddressActivity.launchForResult(
              fragment = this,
              title = getString(R.string.select_address_add),
              onlyAddAddress = true,
              requestCode = ADDRESS_SELECTION_REQ_CODE
        )
    }

    private fun showSelectAddressScreen() {
        SelectAddressActivity.launchForResult(
              this,
              getString(R.string.choose_address),
              book_slot_location_type_address.text.toString(), // Currently selected address (formatted address)
              true,
              ADDRESS_SELECTION_REQ_CODE)
    }

    private fun updateDeliveryPassView() {
        val cartManager = CartManager.get(context)
        val cart = cartManager?.cart
        cart?.let {
            val activeDeliveryPass = cartManager.activeDeliveryPass
            if (activeDeliveryPass != null) {
                showDPView(activeDeliveryPass, cart)
            } else {
                showNoDpView()
            }
        }

        delivery_pass_holder.setOnClickListener { v ->
            DeliveryPassPurchaseActivity.start((activity as Activity), CartManager.get(context)?.activeDeliveryPass != null, v)
        }
    }

    private fun showDPView(activeDeliveryPass: DeliveryPass, cart: Cart) {
        var msg = context?.getString(R.string.dp_active_no_date_button)
        if (!TextUtils.isEmpty(activeDeliveryPass.dpMinSpendAmt) && CartManager.getAmountToReachDp(cart.totalBasketCost, activeDeliveryPass.dpMinSpendAmt) > 0) {
            msg += " " + context?.getString(R.string.minimum_order_value_msg, activeDeliveryPass.dpMinSpendAmt)
        }
        save_with_delivery_pass_text.text = msg
        learn_more_delivery_pass_text.visibility = View.GONE
        delivery_pass_holder?.let {
            it.visibility = View.GONE
            it.parent?.tryCast<View> {
                this.visibility = View.GONE
            }
        }
    }

    private fun showNoDpView() {
        val shouldShowDPOfferCard = showDPOfferValidator.validate(
              ShowDPCardValidatorModel(Authentication.getInstance() as Authentication, CartManager.get(context).activeDeliveryPass))
        if (shouldShowDPOfferCard) {
            dpOfferCardViewHandler?.load()
            book_slot_delivery_pass_holder?.visibility = View.GONE
        } else {
            save_with_delivery_pass_text.text = getString(R.string.save_with_ndelivery_pass) //Html.fromHtml(context?.getString(R.string.get_dp_button))
            learn_more_delivery_pass_text.visibility = View.VISIBLE
        }
        context?.let {
            if (FeatureSettings.isDeliveryPassBlocked(it)) {
                this.book_slot_delivery_pass_details_stub.visibility = View.GONE
            } else {
                this.book_slot_delivery_pass_details_stub.visibility = View.VISIBLE
            }
        }
    }

    override fun onLocationTypeAddressClicked() {
        showSelectAddressScreen()
    }

    override fun getServiceAddressForTab(): ServiceAddress? {
        return viewModel.serviceAddress
    }

    override fun openBookSlotConfirmationPage() {
        if (context != null && FeatureSettings.isSlotSubscriptionMigratedUser(context!!)) {
            val slotDates = viewModel.getBookedStartAndEndTime()
            val intent = DeliveryImpactActivity.newIntent(activity, CartManager.SLOT_TYPE_DELIVERY, slotDates.first, slotDates.second, SlotBookedConfirmationPresenter.TAG, subscriptionEligibility)
            startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
        } else
            super.openBookSlotConfirmationPage()
    }

    private fun updateRecurringSlotsView() {
        if (hasRecurringSlot()) {
            book_slot_recurring_slot_divider.visibility = View.VISIBLE
            book_slot_recurring_slot_text.visibility = View.VISIBLE
            setupRecurringSlotClickListener()
        } else {
            book_slot_recurring_slot_divider.visibility = View.GONE
            book_slot_recurring_slot_text.visibility = View.GONE
        }
    }

    private fun setupRecurringSlotClickListener() {
        book_slot_recurring_slot_text.setOnClickListener {
            val intent = DeliveryImpactActivity.newIntent(activity, RecurringSlotDetailsFragment.TAG)
            startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
        }
    }

    private fun hasRecurringSlot(): Boolean {
        context?.let { context ->
            if (FeatureSettings.isSlotSubscriptionMigratedUser(context) && FeatureSettings.isCXOMigrationEnabled(context)) {
                return subscriptionEligibility?.active_subscription == true
            }
            if (!FeatureSettings.isSlotSubscriptionMigratedUser(context) && FeatureSettings.isCXOMigrationEnabled(context)) {
                return Authentication.getInstance()?.userViewResponse?.hasActiveRecSlot == true
            }
        }
        return hasRecurringSlotInATG()
    }

    private fun hasRecurringSlotInATG(): Boolean {
        if (mRecurringSlots != null && mRecurringSlots?.recurringSlotDetails != null) {
            mRecurringSlots?.recurringSlotDetails?.let {
                for (slotDetails in it) {
                    if (slotDetails != null && !slotDetails.id.isNullOrBlank() && !slotDetails.isCancelled) {
                        return true
                    }
                }
            }
        }
        return false
    }

    private fun refreshRecurringSlots() {
        context?.let { context ->
            if (FeatureSettings.isSlotSubscriptionMigratedUser(context) && FeatureSettings.isCXOMigrationEnabled(context)) {
                SlotSubscriptionManager.refreshSubscriptionEligibility()
            } else if (!FeatureSettings.isSlotSubscriptionMigratedUser(context) && FeatureSettings.isCXOMigrationEnabled(context)) {
                SlotSubscriptionManager.recurringSlotEligibilityChanged.postValue(Authentication.getInstance()?.userViewResponse?.hasActiveRecSlot == true)
            } else
                AsdaServiceFactory.get().viewRecurringSlots(Authentication.getInstance().csrfToken, object : NetworkCallback<RecurringSlotsViewResponse>() {
                    override fun onFailure(error: Int?, data: RecurringSlotsViewResponse?) {
                        //Do nothing
                    }

                    override fun onSuccess(data: RecurringSlotsViewResponse) {
                        RecurringSlotsViewResponse.broadcast(data)
                    }
                })
        }
    }

    private fun setupSubscriptionChangedObserver() {
        SlotSubscriptionManager.subscriptionEligibilityChanged.observe(viewLifecycleOwner, Observer {
            subscriptionEligibility = it?.data?.eligibility_info
            updateRecurringSlotsView()
        })
    }

    private fun setupRecurringSlotEligibilityChangedObserver() {
        SlotSubscriptionManager.recurringSlotEligibilityChanged.observe(viewLifecycleOwner, Observer {
            updateRecurringSlotsView()
        })
    }

    private fun setupRecurringSlotObservers() {
        context?.let {
            when {
                FeatureSettings.isCXOMigrationEnabled(it) && FeatureSettings.isSlotSubscriptionMigratedUser(it) -> {
                    setupSubscriptionChangedObserver()
                }
                FeatureSettings.isCXOMigrationEnabled(it) && !FeatureSettings.isSlotSubscriptionMigratedUser(it) -> {
                    setupRecurringSlotEligibilityChangedObserver()
                }
                else -> {
                    RecurringSlotsViewResponse.addListener(mRecurringSlotsListener)
                }
            }
        }
        refreshRecurringSlots()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ADDRESS_SELECTION_REQ_CODE && resultCode == Activity.RESULT_OK) {
            data?.let {
                val selectedAddress: AddressBookResponse.Address = data.getParcelableExtra(SelectAddressActivity.SELECTED_ADDRESS)
                viewModel.serviceAddress = ServiceAddress(selectedAddress.postCode, selectedAddress.latitude, selectedAddress.longitude,
                      selectedAddress.city, selectedAddress.country, selectedAddress.line1, selectedAddress.line2, selectedAddress.addressId, selectedAddress.deliveryInstructions)
                setAddress()
                resetGrid()
                viewSlots()
            }
        } else if (requestCode == DeliveryImpactActivity.REQUEST_CODE && resultCode == ActivityRequestCodes.RESULT_CODE_RECURRING_SLOT_SUCCESS) {
            updateRecurringSlotsView()
        }
    }

    override fun setAdditionalCostText() {
        additional_cost_text.text = getString(R.string.the_asda_same_day_has_additional_cost, getString(R.string.delivery))
    }

    override fun handleSlotData() {
        context?.let { _context ->
            if (FeatureSettings.isCXOMigrationEnabled(_context)) {
                val slotInfo = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.fulfillmentDetails
                slotInfo?.let { _slotInfo ->
                    if (_slotInfo?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && _slotInfo?.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                          _slotInfo?.slotInfo?.cartSlotId?.isNotBlank() == true && _slotInfo?.slotInfo?.slotExpired != true) {
                        booked_slot_details_express?.visible()
                        booked_slot_details_stub?.gone()
                        val fulFillmentAddress = _slotInfo.fulfillmentAddress
                        fulFillmentAddress?.let { _fulFillmentAddress ->
                            express_store_name?.text = getString(R.string.express_delivery_address, _fulFillmentAddress?.address, _fulFillmentAddress?.city)
                            express_store_address?.text = _fulFillmentAddress?.postalCode

                            express_slot_date?.text = _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMMM_YYYY)
                            express_slot_time?.text = getString(R.string.express_slot_time, _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA), _slotInfo.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))
                            express_reserved_until?.text = getString(R.string.slot_reserved_until, _slotInfo.slotInfo?.bookedUntilTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))

                            if (_slotInfo.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                                context?.let { _context ->
                                    express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                                }
                                express_span?.visible()
                                express_span_bookslot?.gone()
                            } else {
                                express_span?.gone()
                            }
                        } ?: run {
                            booked_slot_details_express?.gone()
                        }
                    } else {
                        booked_slot_details_express?.gone()
                    }
                } ?: run {
                    booked_slot_details_express?.gone()
                }
            } else {
                val cart = CartManager.get(context).cart

                cart?.let { cartData ->
                    if (cartData.fulfillmentType == SlotResponse.FULFILLMENT_OPTION_HOME_DELIVERY &&
                          cartData.extraInfo?.bookedSlotId?.isNotBlank() == true && cartData.extraInfo?.bookedSlotId?.isNotEmpty() == true
                          && cartData.slotExpiredFlag?.equals("false") == true) {
                        booked_slot_details_express?.visible()
                        booked_slot_details_stub?.gone()
                        express_store_name?.text = getString(R.string.express_delivery_address, cartData.extraInfo?.deliveryAddress?.address, cartData.extraInfo?.deliveryAddress?.city)
                        express_store_address?.text = cartData.extraInfo?.deliveryAddress?.postalCode

                        express_slot_date?.text = cartData.slotDate
                        express_slot_time?.text = getString(R.string.express_slot_time, cartData?.deliveryStartTime, cartData?.deliveryEndTime)
                        express_reserved_until?.text = getString(R.string.slot_reserved_until, cartData.bookedUntilTime)

                        if (cart.isExpress == "true") {
                            context?.let {_context ->
                                express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                            }
                            express_span?.visible()
                            express_span_bookslot?.gone()
                        } else {
                            express_span?.gone()
                        }
                    } else {
                        booked_slot_details_express?.gone()
                    }
                } ?: run {
                    booked_slot_details_express?.gone()
                }
            }
        }
    }

    override fun setExpressAvailable(expressAvailable: Boolean) {
        if (Utils.isExpressEnabled(context) && expressAvailable) {
            setExpressInfo()

            context?.let { _context ->
                if (FeatureSettings.isCXOMigrationEnabled(_context)) {
                    val cartItemList = com.asda.android.cxo.CartManager.instance?.getCartItems()
                    val cart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
                    cart?.let { _cart ->
                        if (_cart.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                              _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotBlank() == true) {
                            express_info.gone()
                            express_span_bookslot?.gone()
                        } else {
                            if (cartItemList?.isEmpty() == true) {
                                express_info.visibility = View.VISIBLE
                                express_span_bookslot?.gone()
                            } else {
                                express_info.visibility = View.GONE
                                express_span_bookslot?.visible()
                            }
                        }
                    }
                } else {
                    val cart = CartManager.get(context).cart
                    cart?.let { _cart ->
                        if (_cart.fulfillmentType == SlotResponse.FULFILLMENT_OPTION_HOME_DELIVERY &&
                              _cart.extraInfo?.bookedSlotId?.isNotBlank() == true && _cart.extraInfo?.bookedSlotId?.isNotEmpty() == true) {
                            express_info.gone()
                            express_span_bookslot?.gone()
                        } else {
                            if (_cart.numItemsInBasket != "0") {
                                express_info.visibility = View.GONE
                                express_span_bookslot?.visible()
                            } else {
                                express_info.visibility = View.VISIBLE
                                express_span_bookslot?.gone()
                            }
                        }
                    }
                }
            }
        } else {
            express_span_bookslot?.gone()
        }
    }
}

