package com.asda.android.app.bookslotv3.view.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.app.bookslotv3.adapter.BookSlotParentAdapter
import com.asda.android.app.bookslotv3.viewmodel.BookSlotParentViewModel
import com.asda.android.app.bookslotv3.BookSlotType
import com.asda.android.app.bookslotv3.view.CustomViewPager
import com.asda.android.base.core.view.fragments.BaseFragment
import kotlinx.android.synthetic.main.fragment_book_slot_parent.*


class BookSlotParentFragment : BaseFragment(), BookSlotFragment.SlotEvents {

    companion object {
        const val TAG = "BookSlotParentFragment"
        const val IS_EXPRESS_BANNER_CLICKED = "express_banner_clicked"
        const val SWITCH_TO_CNC_TAB = "switch_to_cnc_tab"
        fun getInstance(expressBannerClicked: Boolean = false, maxItemReachedExpress: Boolean = false): BookSlotParentFragment {
            val fragment = BookSlotParentFragment()
            val bundle = Bundle()
            bundle.putBoolean(IS_EXPRESS_BANNER_CLICKED, expressBannerClicked)
            bundle.putBoolean(SWITCH_TO_CNC_TAB, maxItemReachedExpress)
            fragment.arguments = bundle
            return fragment
        }
    }

    lateinit var viewModel: BookSlotParentViewModel

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        viewModel = ViewModelProviders.of(this).get(BookSlotParentViewModel::class.java)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_book_slot_parent, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupViewPager(view)
    }

    override fun onResume() {
        super.onResume()
        setupActionBar()
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun isParentFragment(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return getString(R.string.book_slot)
    }

    private fun setupViewPager(view: View) {
        val viewPager = view.findViewById<CustomViewPager>(R.id.book_slot_view_pager)
        var isExpressBannerClicked = false
        var switchToCNCTab = false
        arguments?.let {

            if (it.containsKey(IS_EXPRESS_BANNER_CLICKED))
                isExpressBannerClicked = it.getBoolean(IS_EXPRESS_BANNER_CLICKED)
            if (it.containsKey(SWITCH_TO_CNC_TAB))
                switchToCNCTab = it.getBoolean(SWITCH_TO_CNC_TAB)
        }


        context?.let {
            viewPager.adapter = BookSlotParentAdapter(
                  childFragmentManager,
                  BookSlotHomeDeliveryFragment.createInstance(this),
                  BookSlotClickNCollectFragment.createInstance(this, isExpressBannerClicked, switchToCNCTab),
                  it)
        }

        viewPager.setPagingEnabled(false)
        tab_layout.setupWithViewPager(viewPager)
    }

    override fun switchTabsTo(destinationSlotFragmentType: String) {
        if (destinationSlotFragmentType == BookSlotType.CLICK_AND_COLLECT) {
            tab_layout.getTabAt(1)?.select() // goto cnc
        } else if (destinationSlotFragmentType == BookSlotType.HOME_DELIVERY) {
            tab_layout.getTabAt(0)?.select() // goto hd
        }
    }

    fun getBookedSlotChangedLiveData(): MediatorLiveData<Boolean> {
        return viewModel.hasBookedSlotChanged
    }

    fun setBookedSlotChangedLiveData(hasChanged: Boolean) {
        viewModel.hasBookedSlotChanged.postValue(hasChanged)
    }
}
