package com.asda.android.app.bookslotv3.viewmodel

import android.app.Application
import android.util.Log
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.app.bookslot.BannerConfig
import com.asda.android.app.bookslotv3.BookSlotType.CLICK_AND_COLLECT
import com.asda.android.app.bookslotv3.service.BookSlotRepository
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.bookslotv3.validator.BookSlotValidator
import com.asda.android.app.bookslotv3.view.fragment.BookSlotFragment
import com.asda.android.base.core.*
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.SlotResponse
import com.asda.android.restapi.service.data.bookslotv3.*
import com.asda.android.restapi.service.data.bookslotv3.ErrorCodes
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.service.base.CartManager
import com.asda.android.service.repository.CartRepository
import com.asda.android.utils.Utils
import com.fasterxml.jackson.databind.ObjectMapper
import io.reactivex.observers.ResourceSingleObserver
import java.util.*

class BookSlotViewModel(application: Application) : AndroidViewModel(application) {

    companion object {
        const val HOME_DELIVERY = "Slots-HomeDelivery"
        const val CLICK_COLLECT = "Slots-CNC"
    }

    var cart: Cart? = null
    var cartManager: CartManager? = null
    var cartExtraInfo: Cart.CartExtraInfo? = null
    var cxoCart: com.asda.android.restapi.service.data.cart.Cart? = null
    var cxoCartManager: com.asda.android.cxo.CartManager? = null

    var currentLastDateTimeStamp: String? = null //format would be DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z
    var currentBannerLastDateTimestamp: String? = null
    var bannerDate: String? = null
    var viewSlotData: ViewSlotResponseV3.Data? = null
    var data = ArrayList<Any>()
    var isLoading: Boolean = false
    var isFirstPage: Boolean = true
    var isLastPage: Boolean = false
    var shouldShowBookSlotConfirmPage = false
    var numOfRows = 1
    var serviceAddress: ServiceAddress? = null
    var isServiceAddressFromCart: Boolean? = null
    var fulfillmentType = ""
    var bookedSlotIdPositionInRecyclerViewGrid = 0
    var bookedSlotIdRowPositionInRecyclerViewGrid = 0
    var deliveryImpactContinue = false
    var slotInfoOfReserveCallRequest: SlotInfo? = null
    var isExpressEnabled: Boolean = false
    var isSlotEligibilityMinPurchaseEnabled: Boolean = false
    var isCXOMigrationEnabled: Boolean = false
    var bannerConfig: BannerConfig? = null
    var enableExpress = false

    var total = MutableLiveData<Int>()
    private val cxoReserveSlotResponseLiveData = MediatorLiveData<ReserveSlotCXOResponse>()
    private val serviceEligibilityLiveData = MediatorLiveData<ServiceEligibilityData>()
    private val reserveSlotResponseLiveData = MediatorLiveData<ReserveSlotResponseV3>()
    private val viewSlotResponseLiveData = MediatorLiveData<ViewSlotResponseV3>()


    private val bookSlotRepository = BookSlotRepository()
    private val cartRepository = CartRepository()


    fun getReserveSlotResponseLiveData(): MediatorLiveData<ReserveSlotResponseV3> {
        return reserveSlotResponseLiveData
    }

    fun getCXOReserveSlotResponseLiveData(): MediatorLiveData<ReserveSlotCXOResponse> {
        return cxoReserveSlotResponseLiveData
    }

    fun geViewSlotLiveData(): MediatorLiveData<ViewSlotResponseV3> {
        return viewSlotResponseLiveData
    }

    fun getServiceEligibilityLiveData(): MediatorLiveData<ServiceEligibilityData> {
        return serviceEligibilityLiveData
    }

    fun makeCXOReserveCall(slotInfo: SlotInfo, deliveryImpactContinue: Boolean,
                           viewSlotData: ViewSlotResponseV3.Data?,
                           serviceAddress: ServiceAddress?,
                           token: String?,
                           profileId: String?,
                           pickAccesspointIdFromSlotLevel: Boolean) {
        slotInfoOfReserveCallRequest = slotInfo
        cartRepository.reserveSlot(cxoCart?.cartMetadata?.cartId, deliveryImpactContinue, getCXOReserveSlotRequest(slotInfo, viewSlotData, serviceAddress,
              token, profileId, isExpressEnabled, pickAccesspointIdFromSlotLevel))?.subscribe(object : ResourceSingleObserver<ReserveSlotCXOResponse>() {
            override fun onSuccess(reserveSlotCXOResponse: ReserveSlotCXOResponse) {
                cxoReserveSlotResponseLiveData.postValue(reserveSlotCXOResponse)
                cxoCart = reserveSlotCXOResponse.data?.cart
                com.asda.android.cxo.CartManager.instance?.updatedCart?.postValue(reserveSlotCXOResponse.data?.cart)
                com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart = reserveSlotCXOResponse.data?.cart
                if (SLOT_TYPE.EXPRESS.type == reserveSlotCXOResponse.data?.cart?.fulfillmentDetails?.slotInfo?.slotType) {
                    com.asda.android.cxo.CartManager.instance?.setExpressSection?.postValue(true)
                }
                updateCartDataWithServiceEligibility(reserveSlotCXOResponse.data?.cart?.fulfillmentDetails?.fulfillmentAddress?.postalCode)
            }

            override fun onError(e: Throwable) {
                cxoReserveSlotResponseLiveData.postValue(ReserveSlotCXOResponse(errors = arrayListOf(Error(code = ErrorCodes.HTTP_ERROR))))
            }

        })
    }

    private fun updateCartDataWithServiceEligibility(postcode: String?) {
        postcode?.let {
            bookSlotRepository.makeServiceEligibilityCall(it, false)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
                override fun onSuccess(t: ServiceEligibilityData) {
                    cxoCartManager?.let {
                        SlotUtils().updateMinOrderValueAndDeliverySurcharge(t, it)
                    }
                }

                override fun onError(e: Throwable) { //Do nothing
                }

            })
        }
    }

    fun makeReserveCall(slotInfo: SlotInfo, deliveryImpactContinue: Boolean,
                        viewSlotData: ViewSlotResponseV3.Data?,
                        serviceAddress: ServiceAddress?,
                        token: String?,
                        profileId: String?,
                        pickAccesspointIdFromSlotLevel: Boolean) {
        slotInfoOfReserveCallRequest = slotInfo
        bookSlotRepository.makeReserveCall(AsdaServiceSettings.getApiKey(),
              getReserveSlotRequestV3(
                    slotInfo = slotInfo,
                    deliveryImpactContinue = deliveryImpactContinue,
                    viewSlotData = viewSlotData,
                    serviceAddress = serviceAddress,
                    token = token,
                    profileId = profileId,
                    pickAccesspointIdFromSlotLevel = pickAccesspointIdFromSlotLevel))?.subscribe(object : ResourceSingleObserver<ReserveSlotResponseV3>() {
            override fun onSuccess(reserveSlotResponseV3: ReserveSlotResponseV3) {
                reserveSlotResponseLiveData.postValue(reserveSlotResponseV3)
                cartManager?.maxItemsForSlot = reserveSlotResponseV3.data?.order_info?.max_items_for_given_sla ?: 0
                Utils.setExpressFlag(enableExpress)
                if (isSlotEligibilityMinPurchaseEnabled) getMinOrderValueFromServiceEligibility()
            }

            override fun onError(e: Throwable) {
                reserveSlotResponseLiveData.postValue(ReserveSlotResponseV3(errors = arrayListOf(Error(code = ErrorCodes.HTTP_ERROR))))
            }
        })
    }

    private fun getMinOrderValueFromServiceEligibility() {
        bookSlotRepository.makeServiceEligibilityCall(SlotUtils().getPostCodeForServiceEligibility(), false)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
            override fun onSuccess(t: ServiceEligibilityData) {
                cartManager?.let {
                    SlotUtils().updateMinOrderValueAndDeliverySurcharge(t, it)
                }
            }

            override fun onError(e: Throwable) { //Do nothing
            }
        })
    }

    private fun getCXOReserveSlotRequest(slotInfo: SlotInfo,
                                         viewSlotData: ViewSlotResponseV3.Data?,
                                         serviceAddress: ServiceAddress?,
                                         token: String?,
                                         profileId: String?,
                                         isExpressEnabled: Boolean,
                                         pickAccesspointIdFromSlotLevel: Boolean
    ): ReserveSlotCXORequest {
        val reserveSlotCXORequest = ReserveSlotCXORequest()
        val reserveSlotRequestDataV3 = ReserveSlotRequestDataV3()
        reserveSlotRequestDataV3.token = token
        reserveSlotRequestDataV3.request_origin = "android"

        // accesspoint_info
        reserveSlotRequestDataV3.accesspoint_info = AccessPoint()
        if (pickAccesspointIdFromSlotLevel && slotInfo.slotAccesspointId != null) {
            reserveSlotRequestDataV3.accesspoint_info?.accesspoint_id = slotInfo.slotAccesspointId
            reserveSlotRequestDataV3.accesspoint_info?.dispense_store_id = slotInfo.slotDispenseStoreId
            reserveSlotRequestDataV3.accesspoint_info?.fulfill_store_id = slotInfo.slotFulfillStoreId
            reserveSlotRequestDataV3.accesspoint_info?.remote_site_id = slotInfo.slotRemoteSiteId
        } else {
            reserveSlotRequestDataV3.accesspoint_info?.accesspoint_id = slotInfo.slotDayAccesspointId
            reserveSlotRequestDataV3.accesspoint_info?.dispense_store_id = slotInfo.slotDayDispenseStoreId
            reserveSlotRequestDataV3.accesspoint_info?.fulfill_store_id = slotInfo.slotDayFulfillStoreId
            reserveSlotRequestDataV3.accesspoint_info?.remote_site_id = slotInfo.slotDayRemoteSiteId
        }

        // slot_info
        reserveSlotRequestDataV3.slot_info = SlotInfo(
              slot_id = slotInfo.slot_id,
              start_time = slotInfo.start_time,
              end_time = slotInfo.end_time,
              window = (if (slotInfo.window != null) slotInfo.window else SlotWindow.TWO_HOUR_SLOT.toString()),
              isPrimary = slotInfo.isPrimary)

        if (isExpressEnabled) {
            reserveSlotRequestDataV3.slot_info?.is_express_slot = slotInfo.is_express_slot
            reserveSlotRequestDataV3.slot_info?.sla_in_mins = slotInfo.sla_in_mins
        }

        // service_info
        reserveSlotRequestDataV3.service_info = ServiceInfo(fulfillment_type = viewSlotData?.service_info?.fulfillment_type)

        // service_address
        serviceAddress?.let {
            reserveSlotRequestDataV3.service_address = ServiceAddress(it.postcode,
                  it.latitude, it.longitude,
                  address_id = it.address_id,
                  address_line_one = it.address_line_one,
                  city = it.city,
                  country = it.country,
                  delivery_instructions = it.delivery_instructions
            )
        }

        // customer_info
        reserveSlotRequestDataV3.customer_info = CustomerInfo(profileId)

        // order_info
        reserveSlotRequestDataV3.order_info = OrderInfo(
              order_id = cxoCart?.cartMetadata?.cartId,
              parent_order_id = cxoCart?.cartMetadata?.amendedOrderId,
              restricted_item_types = mutableListOf())
        reserveSlotCXORequest.data.slot = reserveSlotRequestDataV3
        return reserveSlotCXORequest
    }

    fun getReserveSlotRequestV3(slotInfo: SlotInfo,
                                deliveryImpactContinue: Boolean,
                                viewSlotData: ViewSlotResponseV3.Data?,
                                serviceAddress: ServiceAddress?,
                                token: String?,
                                profileId: String?,
                                pickAccesspointIdFromSlotLevel: Boolean
    ): ReserveSlotRequestV3 {
        val reserveSlotRequestV3 = ReserveSlotRequestV3()

        reserveSlotRequestV3.data.token = token
        reserveSlotRequestV3.data.request_origin = "android"
        reserveSlotRequestV3.data.delivery_impact = (if (deliveryImpactContinue) ReserveSlotRequestDataV3.DeliveryImpact_Continue else ReserveSlotRequestDataV3.DeliveryImpact_Check)

        // accesspoint_info
        reserveSlotRequestV3.data.accesspoint_info = AccessPoint()
        if (pickAccesspointIdFromSlotLevel && slotInfo.slotAccesspointId != null) {
            reserveSlotRequestV3.data.accesspoint_info?.accesspoint_id = slotInfo.slotAccesspointId
            reserveSlotRequestV3.data.accesspoint_info?.dispense_store_id = slotInfo.slotDispenseStoreId
            reserveSlotRequestV3.data.accesspoint_info?.fulfill_store_id = slotInfo.slotFulfillStoreId
            reserveSlotRequestV3.data.accesspoint_info?.remote_site_id = slotInfo.slotRemoteSiteId
        } else {
            reserveSlotRequestV3.data.accesspoint_info?.accesspoint_id = slotInfo.slotDayAccesspointId
            reserveSlotRequestV3.data.accesspoint_info?.dispense_store_id = slotInfo.slotDayDispenseStoreId
            reserveSlotRequestV3.data.accesspoint_info?.fulfill_store_id = slotInfo.slotDayFulfillStoreId
            reserveSlotRequestV3.data.accesspoint_info?.remote_site_id = slotInfo.slotDayRemoteSiteId
        }


        // slot_info
        reserveSlotRequestV3.data.slot_info = SlotInfo(
              slot_id = slotInfo.slot_id,
              start_time = slotInfo.start_time,
              end_time = slotInfo.end_time,
              window = (if (slotInfo.window != null) slotInfo.window else SlotWindow.TWO_HOUR_SLOT.toString()),
              isPrimary = slotInfo.isPrimary)

        if (isExpressEnabled) {
            reserveSlotRequestV3.data.slot_info?.is_express_slot = slotInfo.is_express_slot
            reserveSlotRequestV3.data.slot_info?.sla_in_mins = slotInfo.sla_in_mins
        }

        // service_info
        reserveSlotRequestV3.data.service_info = ServiceInfo(fulfillment_type = viewSlotData?.service_info?.fulfillment_type)

        // service_address
        serviceAddress?.let {
            reserveSlotRequestV3.data.service_address = ServiceAddress(it.postcode, it.latitude, it.longitude, address_id = it.address_id)
        }

        // customer_info
        reserveSlotRequestV3.data.customer_info = CustomerInfo(profileId)

        // order_info
        reserveSlotRequestV3.data.order_info = OrderInfo(
              order_id = cart?.basketId,
              parent_order_id = cart?.amendedOrderId,
              restricted_item_types = mutableListOf())

        return reserveSlotRequestV3
    }


    fun makeViewSlotCall(profileId: String?,
                         fulfillmentType: String?,
                         viewSlotColumns: Int,
                         viewSlotColumnsFirstLoad: Int,
                         serviceAddress: ServiceAddress?,
                         xCatchmentEnabled: Boolean,
                         currentView: String) {
        if (isFirstPage) {
            bookSlotRepository.makeServiceEligibilityCall(serviceAddress?.postcode.orEmpty(), false)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
                override fun onSuccess(serviceEligibilityData: ServiceEligibilityData) {
                    Utils.setExpressFlag(serviceEligibilityData)
                    if (currentView == CLICK_COLLECT) {
                        enableExpress = Utils.getExpressFlagCNC()
                    } else {
                        enableExpress = Utils.getExpressFlagHD()
                    }
                    makeViewSlotCallWithSEData(profileId, fulfillmentType, viewSlotColumns, viewSlotColumnsFirstLoad, serviceAddress, enableExpress, xCatchmentEnabled)
                }

                override fun onError(e: Throwable) {
                    //Do nothing
                }
            })
        } else {
            makeViewSlotCallWithSEData(profileId, fulfillmentType, viewSlotColumns, viewSlotColumnsFirstLoad, serviceAddress, enableExpress, xCatchmentEnabled)
        }
    }

    private fun makeViewSlotCallWithSEData(profileId: String?,
                                           fulfillmentType: String?,
                                           viewSlotColumns: Int,
                                           viewSlotColumnsFirstLoad: Int,
                                           serviceAddress: ServiceAddress?,
                                           enableExpress: Boolean,
                                           xCatchmentEnabled: Boolean) {
        bookSlotRepository.makeViewSlotCall(getViewSlotRequest(
              profileId = profileId,
              fulfillmentType = fulfillmentType,
              viewSlotColumns = viewSlotColumns,
              viewSlotColumnsFirstLoad = viewSlotColumnsFirstLoad,
              serviceAddress = serviceAddress,
              enableExpress = enableExpress), xCatchmentEnabled)?.subscribe(object : ResourceSingleObserver<ViewSlotResponseV3>() {
            override fun onSuccess(viewSlotResponseV3: ViewSlotResponseV3) {
                viewSlotResponseLiveData.postValue(viewSlotResponseV3)
            }

            override fun onError(e: Throwable) {
                viewSlotResponseLiveData.postValue(ViewSlotResponseV3(errors = arrayListOf(Error(code = ErrorCodes.HTTP_ERROR))))
            }
        })
    }

    fun getViewSlotRequest(profileId: String?,
                           fulfillmentType: String?,
                           viewSlotColumns: Int,
                           viewSlotColumnsFirstLoad: Int,
                           serviceAddress: ServiceAddress?,
                           enableExpress: Boolean
    ): ViewSlotRequestV3 {
        val serviceInfo = ServiceInfo(fulfillment_type = fulfillmentType, enable_express = enableExpress)
        // Populate the start and end dates.
        // If there is a previous date saved, use that else use the current timestamp.
        lateinit var startDateString: String
        var endDateString: String? = null

        bannerDate?.let {_bannerDate ->
            currentBannerLastDateTimestamp?.let{
                startDateString = (it).getDateAfter(1)
                endDateString = it.getDateAfter(viewSlotColumns + 1)
            } ?: run {
                //IF we have current bannerdate user had clicked view slot selected from banners.
                startDateString = _bannerDate.getDateAfter(0)
                endDateString = _bannerDate.getDateAfter(viewSlotColumns + 1)
            }
        }?: run {
            //For 1st time the slots should be loaded for 4days (5days for tablet) and in the subsequent
            //calls the call should be made for next 3days (4days for tablet).
            currentLastDateTimeStamp?.let {
                startDateString = (it).getDateAfter(1)
                endDateString = it.getDateAfter(viewSlotColumns + 1)
            } ?: run {
                startDateString = Date().convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)
                if (isCXOMigrationEnabled) {
                    if (!cxoCart?.fulfillmentDetails?.slotInfo?.casprSlotId.isNullOrEmpty()) {
                        cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime?.let {
                            endDateString = it
                        }
                    }
                } else {
                    if (!cartExtraInfo?.bookedSlotId.isNullOrEmpty()) {
                        cart?.slotDate?.let {
                            endDateString = it.convertToFormat(DateFormats.EEE_DD_MMM_YYYY, DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)
                        }
                    }
                }

                //This logic is added to support auto-scroll to the booked slot date. Whichever date is
                //later one is picked. Or if the endDateString is not populated yet, it is filled in here.
                val defaultEndDate = Date().getDateAfter(viewSlotColumnsFirstLoad)
                if (endDateString.isNullOrBlank() || defaultEndDate.after(endDateString?.convertToDate(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)))
                    endDateString = defaultEndDate.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)
            }
        }


        val viewSlotRequestData = ViewSlotRequestData(
              customer_info = CustomerInfo(profileId),
              order_info = getOrderInfo(),
              service_address = ServiceAddress(serviceAddress?.postcode, serviceAddress?.latitude, serviceAddress?.longitude),
              service_info = serviceInfo,
              start_date = startDateString,
              end_date = endDateString,
              reserved_slot_id = getReserveSlotId())
        return ViewSlotRequestV3(viewSlotRequestData)
    }

    @VisibleForTesting
    fun getReserveSlotId(): String? {
        return if (isCXOMigrationEnabled) {
            if (cxoCartManager?.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.slotExpired == true) {
                ""
            } else {
                cxoCart?.fulfillmentDetails?.slotInfo?.casprSlotId
            }
        } else {
            cartExtraInfo?.bookedSlotId
        }
    }

    private fun getOrderInfo(): OrderInfo {
        val orderInfo = OrderInfo()
        try {
            if (isCXOMigrationEnabled) {
                orderInfo.volume = cxoCart?.cartItemSummary?.volume
                orderInfo.weight = cxoCart?.cartItemSummary?.weight
                orderInfo.sub_total_amount = cxoCart?.cartPriceSummary?.total ?: 0.0
            } else {
                orderInfo.volume = cart?.volume?.toDouble()
                orderInfo.weight = cart?.weight?.toDouble()
                orderInfo.sub_total_amount = Utils.poundStringToDouble(cart?.trolleyDetails?.subTotal)
            }
        } catch (e: Exception) {
            Log.e(BookSlotFragment.TAG, "Error in converting cart?.volume/cart?.weight to double", e)
        }

        if (isCXOMigrationEnabled) {
            orderInfo.line_item_count = cxoCart?.cartItemSummary?.lineItemCount ?: 0
            orderInfo.total_quantity = cxoCart?.cartItemSummary?.totalQuantity ?: 0
            orderInfo.restricted_item_types = cxoCart?.fulfillmentDetails?.slotInfo?.slotRestrictions
            orderInfo.order_id = cxoCart?.cartMetadata?.cartId
            orderInfo.parent_order_id = cxoCart?.cartMetadata?.amendedOrderId
            orderInfo.max_items_for_given_sla = cxoCart?.fulfillmentDetails?.slotInfo?.expressItemLimit ?: 0
        } else {
            orderInfo.line_item_count = (cartManager?.lineItemCount ?: "0").toInt()
            orderInfo.total_quantity = (cartManager?.totalQuantity ?: "0").toInt()
            orderInfo.restricted_item_types = cart?.restrictedItemTypes.orEmpty()
            orderInfo.order_id = cart?.basketId
            orderInfo.parent_order_id = cart?.amendedOrderId
            orderInfo.max_items_for_given_sla = cartManager?.maxItemsForSlot ?: 0
        }
        return orderInfo
    }

    /**
     * If cart contains delivery address, return the address.
     * Else check if there is a default address present in singleProfile response.
     * Else return null.
     */
    fun getHomeDeliveryServiceAddress(deliveryAddress: SlotResponse.DeliveryAddress?, singleProfileAddresses: Array<AddressBookResponse.Address>?): ServiceAddress? {

        if (deliveryAddress != null && !deliveryAddress.postalCode.isNullOrBlank()
              && !deliveryAddress.address.isNullOrBlank()) {

            return ServiceAddress(deliveryAddress.postalCode,
                  deliveryAddress.latitude, deliveryAddress.longitude,
                  deliveryAddress.city,
                  deliveryAddress.country,
                  deliveryAddress.address,
                  deliveryAddress.address2)
        }

        return getServiceAddress(singleProfileAddresses)
    }

    fun getHomeDeliveryServiceAddress(cartDeliveryAddress: com.asda.android.restapi.service.data.cart.Cart.FulfillmentAddress?, singleProfileAddresses: Array<AddressBookResponse.Address>?): ServiceAddress? {

        if (cartDeliveryAddress != null && !cartDeliveryAddress.postalCode.isNullOrBlank()
              && !cartDeliveryAddress.address.isNullOrBlank()) {

            return ServiceAddress(cartDeliveryAddress.postalCode,
                  cartDeliveryAddress.latitude, cartDeliveryAddress.longitude,
                  cartDeliveryAddress.city,
                  cartDeliveryAddress.country,
                  cartDeliveryAddress.address,
                  cartDeliveryAddress.address2,
                  cartDeliveryAddress.deliveryInstructions)
        }

        return getServiceAddress(singleProfileAddresses)
    }

    private fun getServiceAddress(singleProfileAddresses: Array<AddressBookResponse.Address>?): ServiceAddress? {
        val defaultDeliveryAddress = SlotUtils().getSingleProfileDefaultDeliveryAddress(singleProfileAddresses)
        if (defaultDeliveryAddress != null) {
            return ServiceAddress(defaultDeliveryAddress.postalCode,
                  defaultDeliveryAddress.latitude, defaultDeliveryAddress.longitude,
                  defaultDeliveryAddress.city, defaultDeliveryAddress.country,
                  defaultDeliveryAddress.line1,
                  defaultDeliveryAddress.line2,
                  defaultDeliveryAddress.addressId,
                  defaultDeliveryAddress.deliveryInstruction)
        }

        return null
    }


    fun makeServiceEligibilityCall(postcode: String) {

        bookSlotRepository.makeServiceEligibilityCall(postcode)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
            override fun onSuccess(t: ServiceEligibilityData) {
                serviceEligibilityLiveData.postValue(t)
            }

            override fun onError(e: Throwable) {
                serviceEligibilityLiveData.postValue(ServiceEligibilityData(errors = arrayListOf(Error(code = ErrorCodes.HTTP_ERROR))))
            }
        })
    }

    fun isFragmentTypeSameAsReservedSlotType(currentFragmentType: String, cartFulfillmentType: String?): Boolean {
        return if (currentFragmentType == CLICK_AND_COLLECT) {
            (FulfillmentTypes.CNC.type == cartFulfillmentType || FulfillmentTypes.INSTOREPICKUP.type == cartFulfillmentType || FulfillmentTypes.POPUP.type == cartFulfillmentType)
        } else {
            (FulfillmentTypes.DELIVERY.type == cartFulfillmentType || FulfillmentTypes.SHIPTO.type == cartFulfillmentType)
        }
    }

    fun checkIfItsLastPage(data: ViewSlotResponseV3.Data?) {
        data?.let {
            if (it.grid?.is_horizon_extended == true && it.end_date == it.grid?.end_date_for_subscribers) {
                isLastPage = true
                return
            }
            if (it.grid?.is_horizon_extended == false && it.end_date == it.grid?.end_date) {
                isLastPage = true
                return
            }
        }
    }

    fun viewingSlotsForBookedSlotAddress(): Boolean {

        return if (isCXOMigrationEnabled) {
            if (cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type) {
                BookSlotValidator().isAddressMatch(serviceAddress, cxoCart?.fulfillmentDetails?.fulfillmentAddress)
            } else {
                (serviceAddress?.dispenseStoreId == cxoCart?.fulfillmentDetails?.slotInfo?.storeId ||
                      serviceAddress?.remoteSiteId == cxoCart?.fulfillmentDetails?.slotInfo?.storeId ||
                      (isServiceAddressFromCart == true && serviceAddress?.storeId == cxoCart?.fulfillmentDetails?.slotInfo?.storeId))
            }
        } else {
            if (cart?.fulfillmentType == FulfillmentTypes.DELIVERY.type) {
                BookSlotValidator().isAddressMatch(serviceAddress, cartExtraInfo?.deliveryAddress)
            } else {
                (serviceAddress?.dispenseStoreId == cartExtraInfo?.cncStoreAddress?.storeid ||
                      serviceAddress?.remoteSiteId == cartExtraInfo?.cncStoreAddress?.storeid ||
                      (isServiceAddressFromCart == true && serviceAddress?.storeId == cartExtraInfo?.cncStoreAddress?.storeid))
            }
        }
    }

    fun getBookedStartAndEndTime(): Pair<String, String> {
        if (isCXOMigrationEnabled) {
            return Pair(cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime.orEmpty(), cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime.orEmpty())
        } else {
            val splitString = cart?.slotDate?.split(",")
            var startTimeFormatted = ""
            var endTimeFormatted = ""
            if (splitString?.size ?: 0 > 1) {
                val date = splitString?.get(1)?.trim()
                val slotStartDateTime = "$date ${cart?.deliveryStartTime}"
                startTimeFormatted = slotStartDateTime?.convertToFormat(
                      inputDateFormat = DateFormats.DD_MMM_YYYY_HH_MM_A,
                      outputDateFormat = DateFormats.YYYY_MM_DD_T_KK_MM_SS_Z,
                      inputTimeZone = TimeZoneFormat.UK,
                      outputTimezone = TimeZoneFormat.UTC).orEmpty()

                val endStartDateTime = "$date ${cart?.deliveryEndTime}"
                endTimeFormatted = endStartDateTime.convertToFormat(
                      inputDateFormat = DateFormats.DD_MMM_YYYY_HH_MM_A,
                      outputDateFormat = DateFormats.YYYY_MM_DD_T_KK_MM_SS_Z,
                      inputTimeZone = TimeZoneFormat.UK,
                      outputTimezone = TimeZoneFormat.UTC)
            }
            return Pair(startTimeFormatted, endTimeFormatted)
        }
    }

    fun setBannerConfig(bannerSlotConfig: String?) {
        bannerSlotConfig?.let {
            try {
                val mapper = ObjectMapper()
                val banner = mapper.readValue<BannerConfig>(it, BannerConfig::class.java)
                banner?.let {
                    bannerConfig = banner
                }
            } catch (ex: Exception) {
                Log.e(BookSlotFragment.TAG, "Unable to parse banner information.")
            }
        }
    }
}
