package com.asda.android.app.cart.forgottenitems.viewmodel

import android.content.Context
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.ViewModel
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.app.main.ApplicationComponent
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.restapi.utils.SharedPreferencesUtil

class ForgottenItemsViewModel : ViewModel() {
    private val userSegmentSeparator = "|"
    private val p13NRecipeSegment = "P13N_Recipes"

    fun getUserSegment(applicationComponent: ApplicationComponent): String {
        val userSegmentArr: List<String?> = listOf(getUserSegmentForBYG(applicationComponent), getUserSegmentForRecipe(applicationComponent))
        var userSegment = userSegmentArr.filterNotNull().filter { segment -> segment.isNotEmpty() }.joinToString(userSegmentSeparator)
        if (userSegment.isNotEmpty()) {
            userSegment = userSegmentSeparator + userSegment
        }
        return userSegment
    }

    @VisibleForTesting
    fun getUserSegmentForBYG(applicationComponent: ApplicationComponent): String? {
        var userSegment = SharedPreferencesUtil.getP13nBYGUserSegment(applicationComponent.applicationContext)
        if (userSegment.isNullOrEmpty()) {
            userSegment = applicationComponent.sitespectAbTestEngine.getDataFromLiveVariable(SitespectP13NMetrics.P13N_BYG_CAMPAIGN_ID, SitespectP13NMetrics.P13N_BYG_USER_SEGMENT_KEY) as String?
        }
        return userSegment
    }

    @VisibleForTesting
    fun getUserSegmentForRecipe(applicationComponent: ApplicationComponent): String? {
        return if (isP13NBYGRecipeEnabled(applicationComponent.applicationContext)) {
            p13NRecipeSegment
        } else {
            null
        }
    }

    @VisibleForTesting
    fun isP13NBYGRecipeEnabled(context : Context): Boolean {
        return FeatureSettings.isP13NBYGRecipeEnabled(context)
    }
}
