package com.asda.android.app.cart.repository

import android.content.Context
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.MutableLiveData
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.beforeyougo.formatter.CartItemIdsFormatter
import com.asda.android.app.beforeyougo.formatter.CartRecipeIdsFormatter
import com.asda.android.app.cart.datasource.ICartDataSource
import com.asda.android.app.cart.model.APIErrorEventModel
import com.asda.android.app.cart.model.IEvent
import com.asda.android.app.cart.model.P13NInfoModel

import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.scheduler.ISchedulerProvider
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable

class CartRepository(private val schedulerProvider: ISchedulerProvider): ICartDataSource {

    private val disposable: CompositeDisposable = CompositeDisposable()
    val personalizationLiveData: MutableLiveData<BaseStateResponse<PersonalizationResponse>> = MutableLiveData()
    private val includeSponsoredProducts = false

    val p13NImpressionTrackerRepository = P13NImpressionTrackerRepository()
    val apiErrorTrackerRepository = ApiErrorTrackerRepository()

    val offersLiveData: MutableLiveData<BaseStateResponse<PersonalizationResponse>> = MutableLiveData()

    fun invokeP13NApi(itemIdsFormatter: CartItemIdsFormatter, context: Context, recipeIdsFormatter: CartRecipeIdsFormatter, placementId: String, limit: String) {
        disposable.add(getPersonalizationResponse(itemIdsFormatter, context, recipeIdsFormatter, placementId, limit)
              .subscribeOn(schedulerProvider.io())
              .observeOn(schedulerProvider.mainThread())
              .subscribe(
                    { personalizationLiveData.postValue(BaseStateResponse(it)) },
                    { personalizationLiveData.postValue(BaseStateResponse(it)) },
                    {
                        //OnComplete is empty as there will be one personalization response result, that will be taken care in onNext
                    },
                    {
                        //OnSubscribe is empty as there is no operation to be performed on onSubscribe()
                    }
              )
        )

    }

    @VisibleForTesting
    fun getPersonalizationResponse(itemIdsFormatter: CartItemIdsFormatter, context: Context,
                                           recipeIdsFormatter: CartRecipeIdsFormatter, placementId: String, limit: String): Observable<PersonalizationResponse> {
        val cartManager = CartManager.get(context)
        return AsdaServiceFactory.get().getPersonalizationPlacements(placementId,
              null,
              limit,
              itemIdsFormatter.format(cartManager),
              null,
              null,
              itemIdsFormatter.format(cartManager),
              null,
              includeSponsoredProducts,
              Authentication.getInstance().loginResponse,
              cartManager.isAmending,
              Anivia.SECTION_TROLLEY,
              recipeIdsFormatter.format(cartManager))
    }

    override fun execute(eventInfo: IEvent) {
        when (eventInfo) {
            is P13NInfoModel -> p13NImpressionTrackerRepository.execute(eventInfo)
            is APIErrorEventModel -> apiErrorTrackerRepository.execute(eventInfo)
        }
    }

    fun invokeP13NOffersApi(itemIdsFormatter: CartItemIdsFormatter, context: Context, recipeIdsFormatter: CartRecipeIdsFormatter, placementId: String, limit: String) {
        disposable.add(getPersonalizationResponse(itemIdsFormatter, context, recipeIdsFormatter, placementId, limit)
              .subscribeOn(schedulerProvider.io())
              .observeOn(schedulerProvider.mainThread())
              .subscribe(
                    { offersLiveData.postValue(BaseStateResponse(it)) },
                    { offersLiveData.postValue(BaseStateResponse(it)) },
                    {
                        //OnComplete is empty as there will be one personalization response result, that will be taken care in onNext
                    },
                    {
                        //OnSubscribe is empty as there is no operation to be performed on onSubscribe()
                    }
              )
        )

    }


}


