package com.asda.android.app.cart.utils

import android.content.Context
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils.isDevBuild


fun getP13NTrolleyPlacementId(context: Context): String {
    return SharedPreferencesUtil.getPrefP13nTrolleyPlacementId(context)
}

fun getP13NTrolleyLimit(context: Context): String {
    return SharedPreferencesUtil.getPrefP13nTrolleyLimit(context)
}

fun isP13NForTrolleyEnabled(context: Context): Boolean {
    var enabled = false
    if (!isDevBuild()) {
        // In production build, do not consider the force on/off setting.
        enabled = SharedPreferencesUtil.getP13nTrolleyEnabledFlag(context)
    } else {
        // This is dev build, look for P13N For Trolley Force On/Off setting or return
        val p13nTrolleySetting = SharedPreferencesUtil.getP13nTrolleySetting(context, isDevBuild())
        when (p13nTrolleySetting) {
            SharedPreferencesUtil.PREF_P13N_TROLLEY_SETTING_HONOR_CONFIG_FLAG -> enabled = SharedPreferencesUtil.getP13nTrolleyEnabledFlag(context)
            SharedPreferencesUtil.PREF_P13N_TROLLEY_SETTING_FORCE_ON -> enabled = true
            SharedPreferencesUtil.PREF_P13N_TROLLEY_SETTING_FORCE_OFF -> {
                //No operation to be performed if setting is off
            }
            else -> {
                //No operation to be performed for default case
            }
        }
    }
    return enabled
}

fun getP13NTCampaignId(context: Context): String {
    return SharedPreferencesUtil.getPrefP13nTrolleyCampaignId(context)
}

fun getP13NTrolleyVariationId(context: Context): String {
    return SharedPreferencesUtil.getPrefP13nTrolleyVariationId(context)
}

fun getOffersTrolleyPlacementId(context: Context): String {
    return SharedPreferencesUtil.getPrefOffersTrolleyPlacementId(context)
}

fun getOffersTrolleyLimit(context: Context): String {
    return SharedPreferencesUtil.getPrefOffersTrolleyLimit(context)
}

