package com.asda.android.app.cart.viewmodel

import android.app.Application
import android.content.Context
import androidx.lifecycle.*
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.beforeyougo.formatter.CartItemIdsFormatter
import com.asda.android.app.beforeyougo.formatter.CartRecipeIdsFormatter
import com.asda.android.app.cart.model.IEvent
import com.asda.android.app.cart.repository.CartRepository
import com.asda.android.app.cart.utils.*
import com.asda.android.app.cart.validator.CartCampaignValidator
import com.asda.android.base.core.formatter.BygPageViewInfoFormatter
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.scheduler.SchedulerProvider
import com.asda.android.restapi.service.data.cms.PersonalizationViewInfo
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.utils.Utils

class CartViewModel(application: Application) : AndroidViewModel(application) {

    val itemIdsFormatter = CartItemIdsFormatter()
    val recipeIdsFormatter = CartRecipeIdsFormatter()
    var context: Context

    var repository: CartRepository

    val p13nInfoFormatter = BygPageViewInfoFormatter()

    val cartCampaignValidator = CartCampaignValidator()

    init {
        repository = setCartRepository()
        context = getApplication() as Context
    }

    fun invokePersonalizationApi() {
        val placementId = getPlacementId()
        val limit = getLimit()
        repository.invokeP13NApi(itemIdsFormatter, context, recipeIdsFormatter, placementId, limit)
    }

    fun getPersonalizationResponseLiveData(): MutableLiveData<BaseStateResponse<PersonalizationResponse>> {
        return repository.personalizationLiveData
    }

    fun getPlacementId(): String {
        return getP13NTrolleyPlacementId(context)
    }

    fun setCartRepository(): CartRepository {
        return CartRepository(SchedulerProvider())
    }

    fun getLimit(): String {
        return getP13NTrolleyLimit(context)
    }

    fun execute(eventInfo: IEvent?) {
        eventInfo?.let {
            repository.execute(it)
        }
    }

    fun invokePersonalizationOffersApi() {
        val placementId = getOffersPlacementId()
        val limit = getOffersLimit()
        repository.invokeP13NOffersApi(itemIdsFormatter, context, recipeIdsFormatter, placementId, limit)
    }

    fun getOffersResponseLiveData(): MutableLiveData<BaseStateResponse<PersonalizationResponse>> {
        return repository.offersLiveData
    }

    fun getOffersLimit(): String {
        return getOffersTrolleyLimit(context)
    }

    fun getOffersPlacementId(): String {
        return getOffersTrolleyPlacementId(context)
    }

    fun getPersonalizationInfo(personalizationResponse: PersonalizationResponse) : PersonalizationViewInfo {
        val dynamicTitle = getStrategyTitle(personalizationResponse)
        val viewInfo = PersonalizationViewInfo()
        viewInfo.trackingCode = Anivia.SECTION_TROLLEY
        viewInfo.title = dynamicTitle
        viewInfo.isHorizontal = true
        viewInfo.isShowTitle = true
        return viewInfo
    }

    fun getStrategyTitle(personalizationResponse: PersonalizationResponse): String {
        return Utils.removeHtmlTagsAndLineBreaks(personalizationResponse.results?.get(0)?.strategyMessage)
    }

}
