package com.asda.android.app.dp.view

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.CompoundButton
import androidx.appcompat.app.AlertDialog
import androidx.core.text.HtmlCompat
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.account.view.AccountFragment
import com.asda.android.app.account.view.BaseAccountActivity
import com.asda.android.app.dp.viewmodel.DPViewModel
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.recurringslot.SlotSubscriptionRepository
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.databinding.DpDetailsBinding
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.features.account.view.isValidCardAdded
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.three_ds.ThreeDsDataSource
import com.asda.android.utils.Utils
import io.reactivex.observers.ResourceSingleObserver

class DPDetailsFragment : FeaturedFragment() {

    private lateinit var mDPModel: DPViewModel
    private lateinit var mBinding: DpDetailsBinding
    var purchaseType: String? = null
        private set
    private var mOpenWarningDialog: Dialog? = null
    private var mOpenWarningListener: MutableMap<Int, View.OnClickListener>? = null
    private lateinit var threeDsDataSource: ThreeDsDataSource
    private val mHandler = Handler()

    private val mCloseListener = View.OnClickListener {
        try {
            mOpenWarningDialog?.dismiss()
            mBinding.toggleAutoRenew.isChecked = false
        } catch (e: Exception) {
            Log.w(TAG, Log.getStackTraceString(e))
        }
    }

    private val mAddCardListener = View.OnClickListener {
        try {
            mOpenWarningDialog?.dismiss()
        } catch (e: Exception) {
            Log.w(TAG, Log.getStackTraceString(e))
        }

        val addEditPaymentCardFragment =
            SPAddEditPaymentCardFragment.newInstance(null, true, false, purchaseType)
        addEditPaymentCardFragment.setOnCardAddedListener(object :
            SPAddEditPaymentCardFragment.OnCardAddedListener {
            override fun onCardAdded(
                creditCardResponse: AsdaResponse,
                isSingleUser: Boolean,
                cardNumber: String?
            ) {
                if (isValidCC(creditCardResponse)) {
                    mDPModel.dataCollection(
                        (creditCardResponse as PaymentDetailsResponse).cards!![0],
                        cardNumber,
                        ""
                    )
                }
            }
        })

        addEditPaymentCardFragment.setOnSaveStateChange(object :
            SPAddEditPaymentCardFragment.OnSaveStateChange {
            override fun onChange(codeChange: Int) {
                if (CODE_NO_CHANGE == codeChange) {
                    mBinding.toggleAutoRenew.isChecked = false
                }
            }
        })
        push(addEditPaymentCardFragment)
    }


    override fun onAttach(context: Context) {
        super.onAttach(context)
        setUpListeners()
        setUpViewModels()
        setUpObservers()
    }

    private fun setUpListeners() {
        mOpenWarningListener = mutableMapOf()
        mOpenWarningListener!![R.id.close] = mCloseListener
        mOpenWarningListener!![R.id.add_card] = mAddCardListener
    }

    private fun setUpViewModels() {
        threeDsDataSource = ThreeDsDataSource();
        mDPModel = ViewModelProviders.of(this).get(DPViewModel::class.java)
    }

    private fun setUpObservers() {
        mDPModel.reloadDPDetailsLiveData.observe(this, Observer {

        })
        mDPModel.reloadDPDetailsLiveData.observe(this, Observer { isReloadingRequired ->
                val content = isReloadingRequired.getContentIfNotHandled()
                content?.let {
                    if (it) {
                        loadDataFromActiveDP()
                    } else {
                        DialogFactory.createAlertDialog(
                              context?.getString(R.string.error_card_auth),
                              context,
                              SCREEN_NAME
                        ).show()
                    }
                }
        })

        mDPModel.performSecureAuthEditDPLiveData.observe(this,
            Observer { performSecureAuthCheckoutDPOneTimeLiveDataEvent ->
                val content =
                    performSecureAuthCheckoutDPOneTimeLiveDataEvent.getContentIfNotHandled()
                content?.let {
                    performSecureAuthenticationAndEditDP(
                        it.editDpResponse,
                        it.paymentCards,
                        it.cardNumber,
                        it.cvv
                    )
                }
            }
        )
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = DpDetailsBinding.inflate(inflater, container, false)
        mBinding.model = mDPModel
        mBinding.handler = DPDetailsViewHandler()
        mBinding.fragment = this
        return mBinding.root
    }

    override fun onResume() {
        super.onResume()
        loadDataFromActiveDP()
    }

    private fun bindViewFromModel(
        context: Context?,
        dpViewModel: DPViewModel,
        binding: DpDetailsBinding,
        token: String
    ) {
        binding.model = mDPModel
        binding.toggleAutoRenew.setOnCheckedChangeListener(object :
            CompoundButton.OnCheckedChangeListener {
            override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {
                if (mDPModel.mIsAutoRenewActive.get() != isChecked) {
                    enableAutoRenew(isChecked)
                }
            }

            private fun enableAutoRenew(isChecked: Boolean) {
                showProgress(R.string.please_wait)

                AsdaServiceFactory.get()
                    .editDP(
                        dpViewModel.orderId,
                        isChecked,
                        token,
                        "",
                        object : NetworkCallback<EditDpResponse>() {
                            override fun onFailure(error: Int?, data: EditDpResponse) {
                                dismissProgress()
                                if (!isAdded) return
                                AsdaDialogHelper.displayErrorDialog(
                                    data,
                                    error,
                                    context,
                                    Anivia.PV_DP_ORDER_SUMMARY
                                )
                                binding.toggleAutoRenew.isChecked = false
                                loadDataFromActiveDP()
                            }

                            override fun onSuccess(data: EditDpResponse) {
                                dismissProgress()
                                if (!isAdded) return
                                loadDataFromActiveDP()
                                val loginResponse =
                                    (Authentication.getInstance() as Authentication).loginResponse
                                if (loginResponse != null) {
                                    safeAdd(
                                        AsdaServiceFactory.get().initSessionSingleProfile(true).subscribe(
                                            { response ->
                                                Authentication.getInstance().userViewResponse =
                                                    response
                                            },
                                            { throwable -> })
                                    )
                                }
                                validateDP(isChecked)
                            }
                        })
            }
        })
    }

    private fun openWarningDialog() {
        context?.let {
            mOpenWarningDialog =
                AsdaDialogHelper.displayCustomDialog(
                    R.layout.dp_renew_no_card,
                    mOpenWarningListener,
                    null,
                    false,
                    it
                )
        }
    }

    private fun loadDataFromActiveDP() {
        mDPModel.mIsLoading.set(true)
        AsdaServiceFactory.get().viewUserDP(object : NetworkCallback<ViewDPResponse>() {

            override fun onFailure(error: Int?, data: ViewDPResponse) {
                if (isAdded) {
                    mDPModel.mIsLoading.set(false)
                }
            }

            override fun onSuccess(data: ViewDPResponse?) {
                if (!isAdded) return
                if (!data?.activeDpPromotions.isNullOrEmpty()) {
                    data?.activeDpPromotions?.get(0)?.let {
                        mDPModel.init(it)
                        purchaseType = getPurchaseType(it)
                        bindViewFromModel(
                            context,
                            mDPModel,
                            mBinding,
                            Authentication.getInstance().csrfToken
                        )
                        loadRecurringSlots()
                        mDPModel.loadCardData()
                    }

                } else {
                    mDPModel.mIsLoading.set(false)
                    context?.let {
                        mDPModel.mIsDeliveryPassBlocked.set(FeatureSettings.isDeliveryPassBlocked(it))
                        val dpUnavailableText =
                            SharedPreferencesUtil.getDeliveryPassUnavailableText(it)
                        val slotUnavailableText =
                            SharedPreferencesUtil.getSlotUnavailableTextWithoutDp(it)
                        if (mDPModel.mIsDeliveryPassBlocked.get()) {
                            mDPModel.mActiveDPTitletext.set(
                                HtmlCompat.fromHtml(
                                    dpUnavailableText, HtmlCompat.FROM_HTML_MODE_LEGACY
                                ).toString() + System.lineSeparator() + System.lineSeparator()
                                      + HtmlCompat.fromHtml(
                                    slotUnavailableText,
                                    HtmlCompat.FROM_HTML_MODE_LEGACY
                                )
                            )
                        } else {
                            mDPModel.mActiveDPTitletext.set(it.getString(R.string.active_delivery_pass))
                        }
                    }
                }
            }
        })

        val activeDp = (Authentication.getInstance() as Authentication).activeDeliveryPass
        if (activeDp != null) {
            mBinding.title.text = DPViewModel.stripPriceFromName(activeDp.name)
        }
    }

    private fun getPurchaseType(it: ViewDPResponse.DeliveryPassItem): String? {
        return if (STATUS_RENEW_PENDING == it.status || STATUS_TRIAL_RENEW_PENDING == it.status)
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP_RENEW
        else
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP
    }

    private fun validateDP(isChecked: Boolean) {
        mDPModel.mIsLoading.set(true)
        AsdaServiceFactory.get().viewUserDP(object : NetworkCallback<ViewDPResponse>() {

            override fun onFailure(error: Int?, data: ViewDPResponse) {
                if (isAdded) {
                    mDPModel.mIsLoading.set(false)
                }
            }

            override fun onSuccess(data: ViewDPResponse) {
                if (isAdded) {
                    mDPModel.mIsLoading.set(false)
                    if (isChecked) {
                        if (isValidCardAdded() || isCardDetailsPresent(data)) {
                            showDialog(DIALOG_ENABLE_AUTO_RENEW)
                        } else {
                            openWarningDialog()
                        }
                    }
                }
            }
        })
    }

    private fun isCardDetailsPresent(data: ViewDPResponse?): Boolean {
        return (data != null && data.activeDpPromotions != null && data.activeDpPromotions!!.size > 0 && !TextUtils.isEmpty(
            data.activeDpPromotions!![0].dpCardType
        )
              && !TextUtils.isEmpty(data.activeDpPromotions!![0].dpCardNumber))
    }

    private fun loadRecurringSlots() {
        if (FeatureSettings.isSlotSubscriptionMigratedUser(context!!)) {
            SlotSubscriptionRepository().getSubscriptionEligibility(SingleProfile.getProfileResponse()!!.profileId!!)!!.subscribe(
                object : ResourceSingleObserver<SubscriptionEligibilityResponse>() {
                    override fun onSuccess(response: SubscriptionEligibilityResponse) {
                        if (isAdded) {
                            dismissProgress()
                            mDPModel.mHasRecurringSlot.set(response.data?.eligibility_info?.active_subscription == true)
                        }
                    }

                    override fun onError(e: Throwable) {
                        mDPModel.mIsLoading.set(false)
                        dispose()
                    }
                })
        } else {
            AsdaServiceFactory.get().viewRecurringSlots(
                Authentication.getInstance().csrfToken,
                object : NetworkCallback<RecurringSlotsViewResponse>() {
                    override fun onFailure(error: Int?, data: RecurringSlotsViewResponse) {
                        mDPModel.mIsLoading.set(false)
                    }

                    override fun onSuccess(data: RecurringSlotsViewResponse?) {
                        if (isAdded) {
                            mDPModel.mIsLoading.set(false)
                            mDPModel.mHasRecurringSlot.set(
                                data?.recurringSlotDetails?.get(0)?.isCancelled == false && !data.recurringSlotDetails?.get(
                                    0
                                )?.id.isNullOrEmpty()
                            )
                        }
                    }
                })
        }
    }

    override fun hasActionBar(): Boolean {
        val activity = activity
        return activity is AsdaActivity || activity is DeliveryPassPurchaseActivity || activity is BaseAccountActivity || parentFragment is AccountFragment
    }

    override fun getActionBarTitle(): String? {
        return if (activity != null) getString(R.string.delivery_pass) else ""
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun onCreateDialog(dialog: Int): Dialog? {
        when (dialog) {
            DIALOG_CANCEL_TRIAL -> {
                val builder = AlertDialog.Builder(activity!!)
                return builder.setMessage(
                    getString(
                        R.string.dp_cancel_trial_confirmation,
                        mDPModel.formatFullISO(mDPModel.freeTrialEndDateIso)
                    )
                )
                    .setPositiveButton(R.string.keep_pass, null)
                    .setNegativeButton(R.string.cancel_pass) { dialog1, whichButton -> cancelDP() }
                    .create()
            }

            DIALOG_CANCEL_DP -> {
                val builder = AlertDialog.Builder(activity!!)
                return builder.setMessage(R.string.dp_autorenew_cancel_confirmation)
                    .setPositiveButton(R.string.no, null)
                    .setNegativeButton(R.string.yes) { dialog1, whichButton -> cancelDP() }
                    .create()
            }

            DIALOG_VOUCHER_INFO -> {
                val builder = AlertDialog.Builder(activity!!)
                return builder.setMessage(
                    getString(
                        R.string.dp_voucher_info,
                        Utils.addPoundIfNecessary(mDPModel.rewardValue)
                    )
                )
                    .setPositiveButton(R.string.ok, null)
                    .create()
            }

            DIALOG_AUTO_RENEW_INFO -> {
                val builder = AlertDialog.Builder(activity!!)
                return builder.setMessage(R.string.dp_autorenew_info)
                    .setPositiveButton(R.string.ok, null)
                    .create()
            }

            DIALOG_ENABLE_AUTO_RENEW -> {
                val builder = AlertDialog.Builder(activity!!)
                return builder.setMessage(R.string.dp_autorenew_on)
                    .setPositiveButton(R.string.ok, null)
                    .create()
            }
        }
        return super.onCreateDialog(dialog)
    }

    private fun cancelDP() {
        showProgress(R.string.please_wait)
        safeAdd(
            AsdaServiceFactory.get().cancelDP(Authentication.getInstance().csrfToken)
                .subscribe({
                    dismissProgress()
                    if (isAdded) {
                        loadDataFromActiveDP()
                    }
                }, { throwable ->
                    dismissProgress()
                    if (isAdded) {
                        showDialog(throwable)
                    }
                })
        )
    }

    private fun isValidCC(creditCardResponse: AsdaResponse?): Boolean {
        return if (creditCardResponse is PaymentDetailsResponse) {
            creditCardResponse.cards?.let {
                !it.isNullOrEmpty() && !it[0]?.cardId.isNullOrEmpty()
            } ?: run {
                false
            }
        } else false
    }

    private fun performSecureAuthenticationAndEditDP(
          editDpResponse: EditDpResponse,
          paymentCard: PaymentDetailsResponse.PaymentCards?,
          cardNumber: String?,
          cvv: String?
    ) {

        context?.let {
            threeDsDataSource.doThreeDsSecureCheckout(it, editDpResponse, "", FeatureSettings.isThreedsEnabledForDp(it) && FeatureSettings.isThreedsEnabledForEditDp(it))
            threeDsDataSource.getThreeDsLiveData().observe(this, Observer { threeDsResponse ->
                if (AsdaServiceSettings.DEBUG) {
                    Log.i(TAG, "editdp()" + threeDsResponse.status)
                }
                handleThreeDsResponse(threeDsResponse, paymentCard, cardNumber, cvv)
            })
        }
    }

    private fun handleThreeDsResponse(
        threeDsResponse: ThreeDsDataSource.ThreeDsResult?,
        paymentCard: PaymentDetailsResponse.PaymentCards?,
        cardNumber: String?, cvv: String?
    ) {
        threeDsResponse?.let {
            if (it.status == context?.getString(R.string.threeDsSuccess)) {
                handleThreeDsSuccess(it, paymentCard, cardNumber, cvv)
            } else if (it.status == context?.getString(R.string.threeDsFailed) && context != null) {
                handleThreeDsFailed(it)
            }
        }
    }

    private fun handleThreeDsSuccess(
        it: ThreeDsDataSource.ThreeDsResult,
        paymentCard: PaymentDetailsResponse.PaymentCards?, cardNumber: String?, cvv: String?
    ) {
        if (!it.jwt.isNullOrEmpty()) {
            mDPModel.newUpdateDP(paymentCard, cardNumber, cvv, it.jwt)
        }
        if (!it.paRes.isNullOrEmpty()) {
            mDPModel.newUpdateDP(paymentCard, cardNumber, cvv, it.paRes)
        }
    }

    private fun handleThreeDsFailed(it: ThreeDsDataSource.ThreeDsResult) {
        if (it.errorNumber != null && it.errorDescription != null) {
            mHandler.postDelayed({
                DialogFactory.createAlertDialog(
                    context?.getString(R.string.error_card_auth),
                    context,
                    SCREEN_NAME
                ).show()
            }, DELAY_SECONDS.toLong())
        }
        if (it.errorResponse?.firstErrorMessage != null) {
            AsdaDialogHelper.displayErrorDialog(it.errorResponse, 1000, context, SCREEN_NAME)
        } else {
            DialogFactory.createAlertDialog(
                context?.getString(R.string.error_card_auth),
                context,
                SCREEN_NAME
            ).show()
        }
    }

    companion object {
        const val SCREEN_NAME = "Delivery Pass Details"
        const val TAG = "DPDetailsFragment"

        private const val STATUS_RENEW_PENDING = "RENEW_PENDING"
        private const val STATUS_TRIAL_RENEW_PENDING = "TRIAL_RENEW_PENDING"
        private const val CODE_NO_CHANGE = -1

        const val DIALOG_CANCEL_TRIAL = 1
        const val DIALOG_VOUCHER_INFO = 2
        const val DIALOG_AUTO_RENEW_INFO = 3
        const val DIALOG_CANCEL_DP = 4
        private const val DIALOG_ENABLE_AUTO_RENEW = 5
        private const val DELAY_SECONDS = 50
    }
}
