package com.asda.android.app.dp.view

import android.view.View
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.checkout.SelectCardFragment
import com.asda.android.app.dp.viewmodel.DPViewModel
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.recurringslot.RecurringSlotInfoFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.view.ViewUtils

class DPDetailsViewHandler {

    companion object {
        const val TAG = "DPViewModel"
    }

    fun onClickVoucherInfo(v: View, fragment: DPDetailsFragment?) {
        fragment?.let {
            it.showDialog(DPDetailsFragment.DIALOG_VOUCHER_INFO)
        }
    }

    fun onClickViewDPs(v: View, fragment: DPDetailsFragment?) {
        fragment?.let {
            DeliveryPassPurchaseActivity.startDPList(it.activity, v)
        }
    }

    fun onClickRecurringSlotInfo(v: View, fragment: DPDetailsFragment?) {
        fragment?.push(RecurringSlotInfoFragment.newInstance())
    }


    fun onClickAutoRenewInfo(v: View, mDPDetailsFragment: DPDetailsFragment?) {
        mDPDetailsFragment?.showDialog(DPDetailsFragment.DIALOG_AUTO_RENEW_INFO)
    }


    fun onClickLearnMore(v: View, mDPDetailsFragment: DPDetailsFragment?) {
        if (mDPDetailsFragment != null) {
            DeliveryPassIntroActivity.launch(mDPDetailsFragment.activity!!)
        }
    }

    fun onClickViewRecurringSlot(v: View, mDPDetailsFragment: DPDetailsFragment?) {
        mDPDetailsFragment?.push(RecurringSlotDetailsFragment.newInstance())
    }

    fun onClickFAQ(v: View) {
        if (v.context != null) {
            ViewUtils.openWebPage(SharedPreferencesUtil.getDPFAQUrl(v.context), v.context, TAG)
            Tracker.get().trackEvent(
                AsdaAnalyticsEvent(Anivia.DP_FAQ_EVENT)
                    .putString(Anivia.ENGAGEMENT_REPORT_KEY, "Delivery Pass : FAQ Views")
            )
        }
    }

    fun onClickTC(v: View) {
        if (v.context != null) {
            ViewUtils.openWebPage(
                SharedPreferencesUtil.getDPTermsAndConditionsUrl(v.context),
                v.context,
                TAG
            )
        }
    }


    fun onClickChangeCard(
        v: View,
        mDPDetailsFragment: DPDetailsFragment?,
        dpViewModel: DPViewModel
    ) {
        if (mDPDetailsFragment != null) {
            val selectCardFragment =
                SelectCardFragment.newInstance(true, mDPDetailsFragment.purchaseType, true)
            selectCardFragment.setOnSPCardSelectedListener { creditCard, cardNumber, cvv ->
                if (creditCard != null && FeatureSettings.isThreedsEnabledForDp(v.context) && FeatureSettings.isThreedsEnabledForEditDp(v.context)) {
                    dpViewModel.dataCollection(creditCard, cardNumber, cvv)
                } else {
                    dpViewModel.updateDP(creditCard)
                }
            }
            mDPDetailsFragment.push(selectCardFragment)
        }
    }

    fun onClickCancelDP(
        v: View,
        mDPDetailsFragment: DPDetailsFragment?,
        mIsFreeTrialActive: Boolean
    ) {
        if (mDPDetailsFragment != null) {
            if (mIsFreeTrialActive) {
                mDPDetailsFragment.showDialog(DPDetailsFragment.DIALOG_CANCEL_TRIAL)
            } else {
                mDPDetailsFragment.showDialog(DPDetailsFragment.DIALOG_CANCEL_DP)
            }
        }
    }

}
