package com.asda.android.app.dp.view

import android.app.Activity
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.style.StrikethroughSpan
import android.view.View
import androidx.core.text.HtmlCompat
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import com.asda.android.R
import com.asda.android.app.dp.formatter.DPPromoEndDateFormatter
import com.asda.android.app.dp.model.ShowDPCardValidatorModel
import com.asda.android.app.dp.validator.ShowDPOfferValidator
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity.Companion.startDPList
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils

class DPOfferCardViewHandler {
    val mIsVisible = ObservableBoolean()
    val mDPOfferText = ObservableField<CharSequence>("")
    val mDPOfferExpire = ObservableField<CharSequence>("")

    private val dpPromoEndDateFormatter = DPPromoEndDateFormatter()
    private val showDPOfferValidator = ShowDPOfferValidator()
    private var mActivity: Activity? = null

    fun attach(activity: Activity?) {
        mActivity = activity
    }

    fun load() {
        if (showDPOfferValidator.validate(
                ShowDPCardValidatorModel(
                    (Authentication.getInstance() as Authentication),
                    CartManager.get(mActivity).activeDeliveryPass
                )
            )
        ) {
            val offers = (Authentication.getInstance() as Authentication).deliveryPassOffers

            mActivity?.let { activity ->
                if (offers?.isNotEmpty() == true && !offers[0]?.name.isNullOrEmpty()) {
                    offers[0]?.let {
                        mIsVisible.set(true)
                        val text = HtmlCompat.fromHtml(
                            activity.getString(
                                R.string.dp_home_offer,
                                it.name,
                                Utils.addPoundIfNecessary(it.promoPrice),
                                Utils.addPoundIfNecessary(it.originalPrice)
                            ),
                            HtmlCompat.FROM_HTML_MODE_LEGACY
                        )
                        val wasPriceStyled: SpannableString = SpannableString(text)
                        wasPriceStyled.setSpan(
                            StrikethroughSpan(),
                            text.toString().lastIndexOf('£') + 1,
                            text.length,
                            SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE
                        )
                        mDPOfferText.set(wasPriceStyled)
                        mDPOfferExpire.set(
                            activity.getString(
                                R.string.dp_exclusive_offer_expire,
                                dpPromoEndDateFormatter.format(it.promoEndDate)
                            )
                        )
                    }
                } else {
                    hide()
                }
            } ?: kotlin.run {
                hide()
            }
        } else {
            hide()
        }
    }

    fun onClickBuyDP(v: View?) {
        mActivity?.let {
            startDPList(it, v!!)
        }
    }

    fun hide() {
        mIsVisible.set(false)
    }
}
