package com.asda.android.app.dp.view

import android.app.Activity
import android.content.Intent
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView

import androidx.viewpager.widget.PagerAdapter
import androidx.viewpager.widget.ViewPager

import com.asda.android.R
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.ui.BaseAppCompatActivity
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.view.ViewUtils
import kotlinx.android.synthetic.main.dp_intro.*


class DeliveryPassIntroActivity : BaseAppCompatActivity(), ViewPager.OnPageChangeListener,
    View.OnClickListener {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setOrientation()
        setContentView(R.layout.dp_intro)
        setUpViews()
        SharedPreferencesUtil.setShownDPHowto(
            this,
            com.asda.android.utils.Utils.getApplicationVersion(applicationContext)
        )
    }

    private fun setUpViews() {
        pager.adapter = Adapter()
        pager.addOnPageChangeListener(this)
        button1.setOnClickListener(this@DeliveryPassIntroActivity)
        button2.setOnClickListener(this@DeliveryPassIntroActivity)
    }

    private fun setOrientation() {
        requestedOrientation =
            if (RestUtils.isTablet(this)) ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED else ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
    }

    override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
        //Not required
    }

    override fun onPageSelected(position: Int) {
        for (i in 0 until indicators.childCount) {
            (indicators.getChildAt(i) as ImageView).setImageResource(if (i == position) R.drawable.dot_on_green else R.drawable.dot_off_gray)
        }

        if (position == 0) {
            button1.setText(R.string.skip)
        } else {
            button1.setText(R.string.previous)
        }
        button2.visibility =
            if (position == indicators.childCount - 1) View.INVISIBLE else View.VISIBLE
    }

    override fun onPageScrollStateChanged(state: Int) {
        //Not required
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.button1 -> if (getString(R.string.skip) == (v as TextView).text) {
                if (!FeatureSettings.isDeliveryPassBlocked(applicationContext)) {
                    val intent = Intent(this, DeliveryPassPurchaseActivity::class.java)
                    ViewUtils.startActivity(this, intent, v)
                }
                finish()
            } else if (pager.currentItem > 0) {
                pager.currentItem = pager.currentItem - 1
            }
            R.id.button2 -> if (pager.currentItem < pager.childCount) {
                pager.currentItem = pager.currentItem + 1
            }
        }
    }

    private inner class Adapter : PagerAdapter() {
        internal val introPages = intArrayOf(
            R.layout.dp_intro_page_1,
            R.layout.dp_intro_page_2,
            R.layout.dp_intro_page_3,
            R.layout.dp_intro_page_4
        )
        private val mLayoutInflater: LayoutInflater =
            LayoutInflater.from(this@DeliveryPassIntroActivity)

        override fun getCount(): Int {
            return introPages.size
        }

        override fun isViewFromObject(view: View, obj: Any): Boolean {
            return view === obj
        }

        override fun instantiateItem(container: ViewGroup, position: Int): Any {
            val view = mLayoutInflater.inflate(introPages[position], container, false)
            container.addView(view)
            if (position == introPages.size - 1) {
                view.findViewById<View>(R.id.view_dps).setOnClickListener { v ->
                    if (!FeatureSettings.isDeliveryPassBlocked(applicationContext)) {
                        val intent = Intent(
                            this@DeliveryPassIntroActivity,
                            DeliveryPassPurchaseActivity::class.java
                        )
                        ViewUtils.startActivity(this@DeliveryPassIntroActivity, intent, v)
                    }
                    finish()
                }
            } else if (position == 3) {
                (view.findViewById<View>(R.id.details) as TextView).text =
                    SharedPreferencesUtil.getDPMinSpendCopy(this@DeliveryPassIntroActivity)
            }
            return view
        }

        override fun destroyItem(container: ViewGroup, position: Int, obj: Any) {
            container.removeView(obj as View)
        }
    }

    companion object {

        fun launch(activity: Activity) {
            val intent = Intent(activity, DeliveryPassIntroActivity::class.java)
            activity.startActivity(intent)
        }
    }

}

