package com.asda.android.app.dp.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View

import com.asda.android.R
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.base.core.view.actitivies.AsdaAppCompatActivity
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils

class DeliveryPassPurchaseActivity : AsdaAppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        Utils.addNewRefToLeakDetector(TAG, this)
        super.onCreate(savedInstanceState)
        supportActionBar?.let { it.setDisplayShowHomeEnabled(false) }
        setContentView(R.layout.container_fragment)
        handleSavedState(savedInstanceState)
    }

    private fun handleSavedState(savedInstanceState: Bundle?) {
        savedInstanceState ?: redirect()
    }

    private fun redirect() {
        intent?.let {
            when {
                it.hasExtra(START_DETAILS) && it.getBooleanExtra(START_DETAILS, false) -> push(
                    DPDetailsFragment()
                )
                it.hasExtra(START_DP_LIST) && it.getBooleanExtra(
                    START_DP_LIST,
                    false
                ) -> push(DeliveryPassListFragment())
                else -> handleDefaultLaunch()
            }
        } ?: run {
            handleDefaultLaunch()
        }
    }

    private fun handleDefaultLaunch() {
        if (!SharedPreferencesUtil.hasShownDPHowto(this)) {
            DeliveryPassIntroActivity.launch(this)
            finish()
        } else {
            push(DeliveryPassListFragment())
        }
    }

    override fun getFragmentContainer(): Int {
        return R.id.fragment_container
    }

    companion object {
        private const val TAG = "DeliveryPassPurchaseActivity"
        private const val START_DETAILS = "start_details"
        private const val START_DP_LIST = "start_dp_list"

        fun start(activity: Activity?, startDPDetails: Boolean, v: View?) {
            if(activity == null){
                return
            }
            val intent = Intent(activity, DeliveryPassPurchaseActivity::class.java)
            intent.putExtra(START_DETAILS, startDPDetails)
            ViewUtils.startActivityForResult(
                activity,
                intent,
                ActivityRequestCodes.REQUEST_CODE_DELIVERY_PASS_CHECKOUT,
                v
            )
        }

        fun startDPList(activity: Activity?, v: View?) {
            if(activity == null){
                return
            }
            val intent = Intent(activity, DeliveryPassPurchaseActivity::class.java)
            intent.putExtra(START_DP_LIST, true)
            ViewUtils.startActivityForResult(
                activity,
                intent,
                ActivityRequestCodes.REQUEST_CODE_DELIVERY_PASS_CHECKOUT,
                v
            )
        }
    }
}
