package com.asda.android.app.dp.viewmodel

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.app.account.UserAccountDetails
import com.asda.android.app.dp.OneTimeLiveDataEvent
import com.asda.android.app.dp.actionstates.CreditCardExistsActionState
import com.asda.android.app.dp.model.DeliveryPassItemModel
import com.asda.android.restapi.service.base.IAuthentication
import com.asda.android.service.base.Authentication

class DPListViewModel(application: Application) : BaseDPViewModel(application) {

    private val creditCardExists: MutableLiveData<OneTimeLiveDataEvent<CreditCardExistsActionState>> =
        MutableLiveData()

    fun getCreditCardExistsLiveData(): LiveData<OneTimeLiveDataEvent<CreditCardExistsActionState>> {
        return creditCardExists
    }

    fun getAuthentication(): IAuthentication? {
        return Authentication.getInstance()
    }

    fun checkCreditCardExists(deliveryPassPromoId: String?, dpModel: DeliveryPassItemModel?) {
        creditCardExists.postValue(OneTimeLiveDataEvent(CreditCardExistsActionState.Loading))
        val isFreeTrial = isFreeTrial(dpModel)
        safeAdd(
            UserAccountDetails
                .getInstance()
                .getUserCreditCardsDP(deliveryPassPromoId, isFreeTrial)
                .subscribe({ cardCheckoutModel ->
                    creditCardExists.postValue(
                        OneTimeLiveDataEvent(
                            CreditCardExistsActionState.Loaded(
                                cardCheckoutModel, deliveryPassPromoId, dpModel, isFreeTrial
                            )
                        )
                    )
                }, { throwable ->
                    creditCardExists.postValue(
                        OneTimeLiveDataEvent(
                            CreditCardExistsActionState.Error(
                                throwable
                            )
                        )
                    )
                })
        )
    }


    private fun isFreeTrial(dpModel: DeliveryPassItemModel?): Boolean {
        return getAuthentication()?.isDPFreeTrialEligible == true
              && (dpModel?.isMonthlyFreeTrial == true || dpModel?.isOneTimeFreeTrial == true)
    }


}

