package com.asda.android.app.dp.viewmodel

import android.app.Application
import android.content.Context
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.style.StrikethroughSpan
import android.util.Log
import androidx.core.text.HtmlCompat
import androidx.databinding.Bindable
import androidx.databinding.Observable
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.R
import com.asda.android.app.dp.OneTimeLiveDataEvent
import com.asda.android.app.dp.callbacks.CardinalInitCallbackImpl
import com.asda.android.app.dp.model.PerformSecureAuthEditDP
import com.asda.android.app.dp.view.DPDetailsFragment
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.AsdaServiceImpl
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.viewmodel.PaymentCardsViewModel
import com.asda.android.three_ds.ThreeDsUtils
import com.asda.android.three_ds.helper.ThreeDS
import com.asda.android.utils.ISO8601Duration
import com.asda.android.utils.Utils
import io.reactivex.ObservableSource
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import pangea.getEncryptCreditCardService
import java.text.DateFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

class DPViewModel(application: Application) : BaseDPViewModel(application), Observable {

    private val reloadDPDetails = MutableLiveData<OneTimeLiveDataEvent<Boolean>>()
    val reloadDPDetailsLiveData: LiveData<OneTimeLiveDataEvent<Boolean>>
        get() = reloadDPDetails

    private val performSecureAuthEditDP =
        MutableLiveData<OneTimeLiveDataEvent<PerformSecureAuthEditDP>>()
    val performSecureAuthEditDPLiveData: LiveData<OneTimeLiveDataEvent<PerformSecureAuthEditDP>>
        get() = performSecureAuthEditDP
    private val mInputFormatter: DateFormat
    private val mFullDateFormat: DateFormat
    private var mIsInstallmentDP: Boolean = false
    private var durationInMonths: Int = 0

    private var mDPName: String? = null
    private var mDPStartDate: String? = null
    var mDPEndDate: String? = null
    var mCreditCardId: String? = null
    var mCreditCardType: String? = null
    var mCreditCardNumber: String? = null
    var mCreditCardNickname: String? = null
    var orderId: String? = null
        private set
    var freeTrialEndDateIso: String? = null
    @get:Bindable
    var rewardValue: String? = null
    val asdaService: AsdaServiceImpl by lazy {
        AsdaServiceFactory.get()
    }
    private var deviceChannel = ""

    // Data binding fields
    val mIsAutoRenewActive = ObservableBoolean()
    val mAutoRenewDate = ObservableField("")
    val mIsFreeTrial = ObservableBoolean()
    val mIsFreeTrialActive = ObservableBoolean()
    val mFreeTrialEndDate = ObservableField("")
    val mFreeTrialPaymentStart = ObservableField("")
    val mCardDetails = ObservableField("")
    val mNumberOfShoppings = ObservableField("")
    val mAmountSaved = ObservableField("")
    val mShopsToVoucher = ObservableField("")
    val mHasSavings = ObservableBoolean()
    private var mDPCost: String? = null
    val mRenewalPrice = ObservableField<String>()
    val mHasActiveDP = ObservableBoolean()
    val mShowRenewal = ObservableBoolean()
    val mRecurringSlotText = ObservableField<CharSequence>("")
    val mSpecialOfferName = ObservableField<CharSequence>("")
    val mSpecialOfferMinSpend = ObservableField("")
    val mSpecialOfferEnd = ObservableField("")
    val mIsLoading = ObservableBoolean(true)
    val mIsRecurringSlotsAvailable = ObservableBoolean()
    val mHasRecurringSlot = ObservableBoolean()
    val mCardExpiring = ObservableBoolean()
    val mCardExpired = ObservableBoolean()
    val mNextPayment = ObservableField("")
    val mIsInstallmentActive = ObservableBoolean()
    val mUsesInstallmentsForDuration = ObservableBoolean()
    val mShowCardData = ObservableBoolean()
    @get:Bindable
    var status: String? = null
        private set
    val mCostPresentTerm = ObservableField("")
    val mHasSave = ObservableBoolean()
    val mHasDPCost = ObservableBoolean()
    val mDurationString = ObservableField<String>()
    val mIsDeliveryPassBlocked = ObservableBoolean()
    val mActiveDPTitletext = ObservableField<CharSequence>("")

    val isDPPlus = ObservableBoolean()
    val remainingTopupShopsDPPlus = ObservableField("")
    val usedTopupShopsDPPlus = ObservableField("")

    val context: Context
        get() = getApplication<Application>().applicationContext

    val title: String
        @Bindable
        get() = stripPriceFromName(mDPName)

    val validityText: String
        @Bindable
        get() = context.getString(R.string.dp_validity, mDPStartDate, mDPEndDate)

    val subTitle: String
        @Bindable
        get() {
            return if (mIsInstallmentDP && mUsesInstallmentsForDuration.get()) {
                context.getString(
                    if (mIsFreeTrialActive.get()) R.string.dp_price_monthly_freetrial else R.string.dp_price_monthly,
                    mDPCost
                )
            } else {
                context.resources.getQuantityString(
                    if (mIsFreeTrialActive.get()) R.plurals.dp_price_one_payment_freetrial else R.plurals.dp_price_one_payment,
                    durationInMonths,
                    mDPCost,
                    durationInMonths
                )
            }
        }

    val freeTrialWarning: String
        @Bindable
        get() {
            val price: String = if (mIsInstallmentDP && mUsesInstallmentsForDuration.get()) {
                context.getString(R.string.dp_price_monthly, mDPCost)
            } else {
                context.resources.getQuantityString(
                    R.plurals.dp_price_one_payment,
                    durationInMonths,
                    mDPCost,
                    durationInMonths
                )
            }
            return context.getString(R.string.dp_free_trial_ends, mFreeTrialEndDate.get(), price)
        }

    init {

        mInputFormatter = SimpleDateFormat("dd-MMM-yyyy", Locale.UK)
        mInputFormatter.timeZone = Utils.UK.TIME_ZONE
        mFullDateFormat = SimpleDateFormat("EEE dd MMM yyyy", Locale.UK)
        mFullDateFormat.timeZone = Utils.UK.TIME_ZONE

        // Special offer section
        val offers = authentication?.deliveryPassOffers

        if (offers != null && offers.isNotEmpty() && offers[0] != null &&
            !offers[0].sku.isNullOrEmpty()
        ) {
            val text = HtmlCompat.fromHtml(
                AsdaApplication.getComponent().applicationContext.getString(
                    R.string.dp_special_offer_promo,
                    offers[0].name,
                    Utils.addPoundIfNecessary(offers[0].promoPrice),
                    Utils.addPoundIfNecessary(offers[0].originalPrice)
                ), HtmlCompat.FROM_HTML_MODE_LEGACY
            )

            val wasPriceStyled = SpannableString(text)
            wasPriceStyled.setSpan(
                StrikethroughSpan(),
                text.toString().lastIndexOf('£') + 1,
                text.length,
                SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE
            )

            mSpecialOfferName.set(wasPriceStyled)
            mSpecialOfferMinSpend.set("£40")
            mSpecialOfferEnd.set(formatISO(offers[0].promoEndDate))
        }
    }

    fun init(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        mDPName = activeDpPromotion.name
        mDPStartDate = formatISO(activeDpPromotion.dpStartDate)
        mDPEndDate = formatISO(activeDpPromotion.termEndDate)
        mCreditCardType = activeDpPromotion.dpCardType
        mCreditCardNumber = activeDpPromotion.dpCardNumber
        mIsDeliveryPassBlocked.set(FeatureSettings.isDeliveryPassBlocked(context))
        if (!activeDpPromotion.dpCardType.isNullOrEmpty() &&
            !activeDpPromotion.dpCardNumber.isNullOrEmpty()
        ) {
            //activeDpPromotion.dpCardNumber only returns last 4 digits
            if (activeDpPromotion.dpCardNumber != null && activeDpPromotion.dpCardNumber!!.length <= 5) {
                activeDpPromotion.dpCardNumber =
                    HIDDEN_DIGITS_STARS + activeDpPromotion.dpCardNumber!!
            }
            mCreditCardNickname = context.getString(
                R.string.formatted_card_nickname,
                activeDpPromotion.dpCardType,
                activeDpPromotion.dpCardNumber
            )
        } else {
            mCreditCardNickname = context.getString(R.string.dp_no_card)
        }

        orderId = activeDpPromotion.dpAutorenewOrderId

        val installments =
            if (activeDpPromotion.totalInstallments.isNullOrEmpty()) 0 else Integer.parseInt(
                activeDpPromotion.totalInstallments!!
            )
        mIsInstallmentDP = installments > 0
        try {
            val passDuration = ISO8601Duration(activeDpPromotion.passDuration).months.toInt()
            if (passDuration == 1 && mIsInstallmentDP) {
                durationInMonths = installments
                mUsesInstallmentsForDuration.set(true)
            } else {
                durationInMonths = passDuration
                mUsesInstallmentsForDuration.set(false)
            }
            mDurationString.set(
                context.resources.getQuantityString(
                    R.plurals.dp_duration,
                    durationInMonths,
                    durationInMonths
                )
            )
        } catch (e: ParseException) {
            Log.w(TAG, "Error calculating pass duration", e)
        }

        mDPCost =
            Utils.addPoundIfNecessary(
                if (mIsInstallmentDP && !activeDpPromotion.nextInstallmentPrice.isNullOrEmpty()) activeDpPromotion.nextInstallmentPrice
                else activeDpPromotion.renewalPrice
            )
        mRenewalPrice.set(activeDpPromotion.renewalPrice)
        mIsRecurringSlotsAvailable.set(
            "true".equals(
                activeDpPromotion.isRecurringSlotsAvailable!!,
                ignoreCase = true
            )
        )

        mHasActiveDP.set(true)

        setAutoRenewDetails(activeDpPromotion)

        mIsInstallmentActive.set(activeDpPromotion.isInstallmentActive)


        // Card Details
        updateCardDetails(context)

        setRecurringSlotText()
        mActiveDPTitletext.set(context.getString(R.string.active_delivery_pass))
        // Init free trial
        setFreeTrialDetails(activeDpPromotion)

        mShowCardData.set(mHasActiveDP.get() || mIsFreeTrialActive.get() || mIsInstallmentActive.get() || mIsAutoRenewActive.get())

        // Next Payment
        setNextPayement(activeDpPromotion)

        // Init savings

        setSavingsDetails(activeDpPromotion)

        //DP Plus details
        setDPPlusDetails(activeDpPromotion)
    }

    private fun setAutoRenewDetails(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        mShowRenewal.set(activeDpPromotion.status != null && !activeDpPromotion.status!!.contains("CANCELLED"))
        mIsAutoRenewActive.set(
            "true".equals(
                activeDpPromotion.renewableAfterTermExpiry!!,
                ignoreCase = true
            )
        )
        if (!activeDpPromotion.renewalDate.isNullOrEmpty()) {
            mAutoRenewDate.set(formatISO(activeDpPromotion.renewalDate))
        }
    }

    private fun setFreeTrialDetails(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        mIsFreeTrial.set(ViewDPResponse.TRIAL_STATUS == activeDpPromotion.status || ViewDPResponse.TRIAL_CANCELLED_STATUS == activeDpPromotion.status)
        if (mIsFreeTrial.get()) {

            activeDpPromotion.freeTrialDuration?.let {
                freeTrialEndDateIso = it.endDate
                mFreeTrialEndDate.set(formatISO(it.endDate))
            } ?: run {
                mFreeTrialEndDate.set(formatISO(activeDpPromotion.dpEndDateISO))
            }
            mIsFreeTrialActive.set(ViewDPResponse.TRIAL_STATUS == activeDpPromotion.status)
            mFreeTrialPaymentStart.set(formatFullISO(activeDpPromotion.nextPaymentDate))
        }
    }

    private fun setSavingsDetails(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        mHasSavings.set(!activeDpPromotion.rewards?.noOfShops.isNullOrEmpty())
        /*
        Might need re-enabling it later
        try {

            activeDpPromotion.rewards?.let {
                val dpSaveRequired =
                    java.lang.Float.parseFloat(Utils.convertToParseableString(it.fullSavings)) > 0
                mHasSave.set(dpSaveRequired)
                val dpCostrequired =
                    java.lang.Float.parseFloat(Utils.convertToParseableString(it.costPresentTerm)) > 0
                mHasDPCost.set(dpCostrequired)
            }
        } catch (ex: Exception) {
            Log.w(TAG, "Error in parse float", ex)
        }*/

        if (mHasSavings.get()) {
            activeDpPromotion.rewards?.let {
                mNumberOfShoppings.set(it.noOfShops)
                mShopsToVoucher.set(it.noOfShopsForReward)
                mAmountSaved.set(Utils.addPoundIfNecessary(it.fullSavings))
                rewardValue = it.rewardValue
                mCostPresentTerm.set(it.costPresentTerm)
            }
        }
        status = activeDpPromotion.status
    }

    private fun setDPPlusDetails(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        isDPPlus.set(activeDpPromotion.isDPPlus)
        if (isDPPlus.get()) {
            remainingTopupShopsDPPlus.set(activeDpPromotion.remainingTopupShops.toString())
            usedTopupShopsDPPlus.set(activeDpPromotion.totalTopupShopsUsed.toString())
        }
    }

    private fun setRecurringSlotText() {
        if (mIsDeliveryPassBlocked.get()) {
            mRecurringSlotText.set(
                HtmlCompat.fromHtml(
                    SharedPreferencesUtil.getSlotUnavailabletext(context),
                    HtmlCompat.FROM_HTML_MODE_LEGACY
                ).toString()
                      + System.lineSeparator() + System.lineSeparator() +
                      HtmlCompat.fromHtml(
                          SharedPreferencesUtil.getSlotUnavailableAsterisksText(context),
                          HtmlCompat.FROM_HTML_MODE_LEGACY
                      )
            )
        } else {
            mRecurringSlotText.set(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string
                            .dp_recurring_slot
                    ), HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )
        }
    }

    private fun setNextPayement(activeDpPromotion: ViewDPResponse.DeliveryPassItem) {
        if (mIsFreeTrial.get() && mIsFreeTrialActive.get() || mIsInstallmentDP && mIsInstallmentActive.get()) {
            mNextPayment.set(formatISO(activeDpPromotion.nextPaymentDate))
        } else {
            mNextPayment.set(formatISO(activeDpPromotion.renewalDate))
        }
    }

    private fun updateCardDetails(context: Context) {
        var cardDetails: String? = context.getString(R.string.unavailable)
        if (!mCreditCardNickname.isNullOrEmpty()) {
            cardDetails = mCreditCardNickname
        }
        mCardDetails.set(cardDetails)
    }

    private fun formatISO(dateISO: String?): String {
        return mInputFormatter.format(Date(Utils.getTimeInMillisFromIso(dateISO)))
    }

    fun formatFullISO(dateISO: String?): String {
        return mFullDateFormat.format(Date(Utils.getTimeInMillisFromIso(dateISO)))
    }

    override fun addOnPropertyChangedCallback(onPropertyChangedCallback: Observable.OnPropertyChangedCallback) {
        //Implementation Not required
    }

    override fun removeOnPropertyChangedCallback(onPropertyChangedCallback: Observable.OnPropertyChangedCallback) {
        //Implementation Not required
    }

    fun updateDP(paymentCard: PaymentDetailsResponse.PaymentCards?) {
        paymentCard?.cardId?.let {
            AsdaServiceFactory.get().editDP(
                orderId,
                mIsAutoRenewActive.get(),
                Authentication.getInstance().csrfToken,
                it,
                object : NetworkCallback<EditDpResponse>() {
                    override fun onFailure(error: Int?, data: EditDpResponse) {
                        reloadDPDetails.postValue(OneTimeLiveDataEvent(java.lang.Boolean.FALSE))
                    }

                    override fun onSuccess(data: EditDpResponse) {
                        reloadDPDetails.postValue(OneTimeLiveDataEvent(java.lang.Boolean.TRUE))
                    }
                })
        }
    }

    fun dataCollection(
        paymentCard: PaymentDetailsResponse.PaymentCards,
        cardNumber: String?, cvv: String?
    ) {
        mIsLoading.set(true)
        safeAdd(
            createJwt(paymentCard.cardId, paymentCard.paymentToken, "", EDIT_DP)
                .subscribe(
                    { createJwtResponse ->
                        doCollectData(
                            createJwtResponse,
                            paymentCard,
                            cardNumber,
                            cvv
                        )
                    },
                    { throwable ->
                        if (throwable is RxFailure && throwable.integer != -2) {
                            Log.i(TAG, "CreateJwt API failed. " + throwable.integer)
                        }
                        newUpdateDP(paymentCard, cardNumber, cvv, null)
                    })
        )
    }

    fun doCollectData(
        createJwtResponse: CreateJwtResponse,
        paymentCard: PaymentDetailsResponse.PaymentCards,
        cardNumber: String?,
        cvv: String?
    ) {
        if (isValidForCollectData(createJwtResponse, paymentCard, context)) {
            val mThreeDS = ThreeDS()
            val cardinalInitCallback = CardinalInitCallbackImpl()
            mThreeDS.collectData(createJwtResponse.jwt!!, Utils.isDebug(), cardinalInitCallback)
            newUpdateDP(paymentCard, cardNumber, cvv, null)
            if (Utils.isDebug()) {
                Log.d(TAG, "JWT: " + createJwtResponse.jwt!!)
            }
        } else {
            newUpdateDP(paymentCard, cardNumber, cvv, null)
        }
    }

    fun isValidForCollectData(
        createJwtResponse: CreateJwtResponse?,
        paymentCard: PaymentDetailsResponse.PaymentCards,
        context: Context?
    ): Boolean {
        return (!createJwtResponse?.jwt.isNullOrEmpty()
              && Utils.isThreeDs2XEnabled(context, paymentCard.cardBrand) && isThreedsEnabledForDp(context) && isThreedsEnabledForEditDp(context))
    }

    fun isThreedsEnabledForDp(context: Context?): Boolean {
        context?.let {
            return FeatureSettings.isThreedsEnabledForDp(it)
        }
        return false
    }

    fun isThreedsEnabledForEditDp(context: Context?): Boolean {
        context?.let {
            return FeatureSettings.isThreedsEnabledForEditDp(it)
        }
        return false
    }

    fun newUpdateDP(
        paymentCard: PaymentDetailsResponse.PaymentCards?,
        cardNumber: String?,
        cvv: String?,
        paRes: String?
    ) {
        mIsLoading.set(true)

        paymentCard?.cardId?.let {

            val disposable = getEncryptCreditCardService(context)!!
                .flatMap {
                    if (!cvv.isNullOrEmpty()) {
                        return@flatMap it.rxEncryptCvv(cvv)
                    } else {
                        return@flatMap it.rxEncryptCc(cardNumber!!)
                    }
                }
                .flatMap { encryptedCc ->
                    if (ThreeDsUtils.isCardinalInitSuccess) {
                        deviceChannel = ThreeDsUtils.DEVICE_CHANNEL
                    }
                    return@flatMap AsdaServiceFactory.get().rxEditDeliveryPass(
                        orderId,
                        mIsAutoRenewActive.get(),
                        Authentication.getInstance().csrfToken,
                        paymentCard.cardId,
                        encryptedCc.encryptedCc,
                        encryptedCc.encryptedCvv,
                        encryptedCc.keyId,
                        encryptedCc.phase,
                        encryptedCc.integrityCheck,
                        deviceChannel,
                        paRes,
                        paymentCard.paymentToken,
                        paymentCard.nameOnCard,
                        paymentCard.cardBrand,
                        paymentCard.expiryDate,
                        paymentCard.address?.line1.orEmpty(),
                        paymentCard.address?.line2.orEmpty(),
                        paymentCard.address?.postalCode.orEmpty(),
                        paymentCard.address?.city.orEmpty()
                    )
                }
                .onErrorResumeNext { throwable: Throwable ->
                    return@onErrorResumeNext handleNewUpdateDPErrorResumeNext(
                        throwable,
                        paymentCard, cardNumber, cvv
                    )
                }
                .subscribe({ _ ->
                    mIsLoading.set(false)
                    reloadDPDetails.postValue(OneTimeLiveDataEvent(true))
                }, { throwable ->
                    mIsLoading.set(false)
                    if (throwable is RxFailure && throwable.integer != API_ERROR_VALUE) {
                        AsdaDialogHelper.displayErrorDialog(
                            throwable.asdaResponse,
                            throwable.integer,
                            context,
                            DPDetailsFragment.SCREEN_NAME
                        )
                    }
                })
            safeAdd(disposable)
        }
    }

    private fun handleNewUpdateDPErrorResumeNext(
        throwable: Throwable,
        paymentCard: PaymentDetailsResponse.PaymentCards,
        cardNumber: String?,
        cvv: String?
    ): ObservableSource<out EditDpResponse>? {
        if (throwable is RxFailure) {
            val editDpResponse = throwable.asdaResponse as EditDpResponse
            if (editDpResponse.threeDsData?.threeDsAuthRequired == true) {
                if (AsdaServiceSettings.DEBUG) {
                    Log.i(TAG, "editdp() 3DS auth needed");
                }

                CartManager.get(context)?.let {
                    ThreeDsUtils.trackThreeDSEvent(
                        editDpResponse,
                        context.getString(R.string.editdp_page_name),
                        "",
                        Authentication.getInstance().userViewResponse.profileId!!,
                        CartManager.get(context).cart!!
                    )
                }

                performSecureAuthEditDP.postValue(
                    OneTimeLiveDataEvent(
                        PerformSecureAuthEditDP(
                            editDpResponse,
                            paymentCard,
                            cardNumber,
                            cvv
                        )
                    )
                )

                return io.reactivex.Observable.error(RxFailure(-2, null))
            }
        }
        return io.reactivex.Observable.error(throwable)
    }

    fun createJwt(cardNickName: String?, paymentToken: String?, orderId: String, editDp: String)
          : Single<CreateJwtResponse> {

        val payment = CreateJwtRequest.Payment()
        payment.cardNickName = cardNickName
        payment.paymentToken = paymentToken
        val payload = CreateJwtRequest.Payload()
        payload.payment = payment
        payload.orderId = orderId
        payload.modifyType = editDp
        payload.token = Authentication.getInstance().csrfToken
        val request = CreateJwtRequest(payload)
        return asdaService.rxCreateJwt(request).toObservable().singleOrError()
    }

    fun loadCardData() {
        if (mHasActiveDP.get()) {
            val profileResponse = SingleProfile.getProfileResponse()
            profileResponse?.profileId?.let {
                val model = PaymentCardsViewModel(SingleProfile.getService())
                val d = model.getPaymentCardsBasic(it)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({
                        updateDPModel(it)
                    }, { _ ->
                        Log.w(TAG, "Error getting credit card data")
                    })
                safeAdd(d)
            }
        }
    }

    private fun updateDPModel(paymentDetailsResponse: PaymentDetailsResponse?) {
        paymentDetailsResponse?.let {
            if (!it.cards.isNullOrEmpty()) {
                val firstCard = it.cards!![0]
                mCardExpired.set(firstCard.expired)
                mCardExpiring.set(firstCard.aboutToExpire)
                mCreditCardId = firstCard.cardId
            }
        }
    }

    companion object {

        private const val TAG = "DPViewModel"

        private var authentication: Authentication? = null
            get() {
                if (field == null) {
                    authentication = Authentication.getInstance() as Authentication
                }
                return field
            }
        private const val API_ERROR_VALUE = -2
        private const val EDIT_DP = "editdp"

        fun stripPriceFromName(name: String?): String {
            return name?.replace("\\(£.*\\)".toRegex(), "")?.trim { it <= ' ' } ?: ""
        }

        private const val HIDDEN_DIGITS_STARS = "**** **** **** "
    }

}
