package com.asda.android.app.dynamic

import android.content.Context
import android.content.res.Configuration
import android.util.DisplayMetrics
import android.view.WindowManager
import android.widget.ImageView
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Merchandising
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.Size
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils

@MountSpec
object BannerComponentSpec {

    @OnCreateMountContent
    internal fun onCreateMountContent(c: ComponentContext): ImageView {
        return ImageView(c)
    }

    @OnMount
    internal fun onMount(c: ComponentContext, v: ImageView, @Prop handler: DynamicPageFragment.OnActionHandler, @Prop banner: Merchandising.Banner) {
        ImageLoader.load(c, v, ImageLoaderRequest.Builder().path(banner.bannerimage).build(), null)
        wireListeners(handler, banner, v)
    }

    /**
     * @param totalItems Required for BannerRows, we must know how many items are going to be displayed in a single row and use it only
     * when is a tablet or landscape.
     */
    @OnMeasure
    internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size, @Prop newAspectRatio: Float, @Prop(optional = true) totalItems: Int) {
        val metrics = DisplayMetrics()
        val windowManager = c.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        windowManager.defaultDisplay.getMetrics(metrics)
        val isTablet = RestUtils.isTablet(c)
        val config = c.resources.configuration
        val isLandscape = config.orientation == Configuration.ORIENTATION_LANDSCAPE
        var height = metrics.heightPixels

        if (isTablet || isLandscape) {
            height = height / 4
            if (totalItems > 0 && isTablet && !isLandscape) {
                height = height / totalItems
            }
        }

        var width = (height * newAspectRatio).toInt()
        if (width > metrics.widthPixels) {
            width = metrics.widthPixels
            height = (width / newAspectRatio).toInt()
        }
        MeasureUtils.measureWithDesiredPx(
                widthSpec,
                heightSpec,
                width,
                height,
                size)
    }

    private fun wireListeners(handler: DynamicPageFragment.OnActionHandler, banner: Merchandising.Banner, view: ImageView) {
        view.setOnClickListener { v -> handler.onMerchandisingClicked(banner, view) }
    }
}
