package com.asda.android.app.dynamic


import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import com.asda.android.R
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.AdTrackingServiceQuery
import com.asda.android.admonetization.criteo.AdUtils
import com.asda.android.app.shop.view.BannerViewHolder
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.restapi.service.data.Merchandising
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.Size
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils
import com.asda.android.utils.view.ViewUtil

@MountSpec
object BannerNonXMComponentSpec {

    @OnPrepare
    internal fun onPrepare(c : ComponentContext) {
    }

    @OnCreateMountContent
    internal fun onCreateMountContent(c: ComponentContext): View {
        return LayoutInflater.from(c).inflate(R.layout.admonetization_hp_merchandising_banner_card, null, false)
    }

    @OnMount
    internal fun onMount(c: ComponentContext, v: View,
                         @Prop handler: DynamicPageFragment.OnActionHandler,
                         @Prop banner: Merchandising.Banner) {
        updateViews(c, v, handler, banner)
    }

    /**
     * @param totalItems Required for BannerRows, we must know how many items are going to be displayed in a single row and use it only
     * when is a tablet or landscape.
     */
    @OnMeasure
    internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size, @Prop(optional = true) totalItems: Int) {
        val columnCount = Math.max(2, ViewUtil.getScreenWidth(c) / ViewUtil.dpToPixels(c.resources.getDimension(R.dimen.hp_banner_view_width).toInt(), c))
        val cardHeight = c.resources.getDimension(R.dimen.hp_banner_view_height).toInt()
        var height = totalItems / columnCount * cardHeight
        if (totalItems % columnCount != 0) {
            height += cardHeight
        }
        MeasureUtils.measureWithDesiredPx(widthSpec, heightSpec, c.resources.getDimension(R.dimen.hp_banner_view_width).toInt(), height, size)
    }

    private fun wireListeners(handler: DynamicPageFragment.OnActionHandler, banner: Merchandising.Banner, view: ImageView) {
        view.setOnClickListener { v -> handler.onMerchandisingClicked(banner, view) }
    }

    private fun updateViews(c: ComponentContext, v: View, handler: DynamicPageFragment.OnActionHandler, banner: Merchandising.Banner) {
        var imageView = v.findViewById(R.id.banner_image_full_merchandising_view) as ImageView
        if (banner.bannerDisplayType != null && banner.bannerDisplayType.contains(AdConstants.STATIC)) {
            v.findViewById<View>(R.id.banner_image_full_merchandising_view).visibility = View.VISIBLE
            v.findViewById<View>(R.id.banner_image_full_merchandising_view).contentDescription = AdUtils().getDescriptionFromMerchandisingBanner(banner)
            v.findViewById<View>(R.id.banner_item_view_group).visibility = View.INVISIBLE
            v.findViewById<View>(R.id.banner_merchandising_view).visibility = View.INVISIBLE
            wireListeners(handler, banner, imageView)
        } else {
            imageView = v.findViewById(R.id.banner_merchandising_view) as ImageView
            val mBannerViewHolder = BannerViewHolder(v, 1, handler)
            if(banner.bannerDisplayType != null && banner.bannerDisplayType.contains(AdConstants.DYNAMIC) && banner.bannerItem != null && banner.bannerItem?.imageURL != null)
                banner.merchandisingimage = banner.bannerItem?.imageURL?.let { it?.trim() }
            if(!banner.merchandisingimage.isNullOrEmpty())
                mBannerViewHolder.updateViews(banner, c.baseContext)
        }
        if(!banner.merchandisingimage.isNullOrEmpty()) {
            ImageLoader.load(c, imageView, ImageLoaderRequest.Builder().path(banner.merchandisingimage).build(), null)
            AdTrackingServiceQuery().sendBannerViewEvent(banner.trackingInformation)
        }
    }
}
