package com.asda.android.app.dynamic

import android.os.Bundle
import android.util.Log
import android.widget.ImageView
import com.asda.android.app.shop.MerchandisingHandler
import com.asda.android.app.shop.ShelfDynamicSectionLoader
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.RecipeAPI
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.restapi.service.data.recipes.RecipeFacetsListResponse
import com.asda.android.recipes.R
import com.asda.android.recipes.RecipeUtils
import com.asda.recipes.adapters.controllers.NonCuratedRecipePagedListController
import com.asda.android.recipes.controllers.SearchRecipePagedListController
import com.asda.android.recipes.view.fragments.EXTRA_IS_SEARCH
import com.asda.android.recipes.view.fragments.EXTRA_SEARCH_TERM
import com.asda.android.recipes.view.fragments.SearchRecipeListFragment
import com.asda.android.recipes.viewmodel.SearchRecipeViewModel
import com.asda.android.base.core.view.DialogFactory
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers

const val EXTRA_PAGE_ID = "extra_page_id"
const val EXTRA_QUERY_STRING = "extra_query_string"

class NonCuratedRecipeListFragment: SearchRecipeListFragment(), ShelfDynamicSectionLoader.LoadListener {

    private val actionHandler = object: DynamicPageFragment.OnActionHandler {
        override fun onFragmentPushed(fragment: BaseFragment, breadCrumb: String) {
            push(fragment, breadCrumb)
        }

        override fun onSimilarClicked(item: ShelfListItem) {

        }

        override fun onMerchandisingClicked(banner: Merchandising.Banner, view: ImageView) {
            try {
                MerchandisingHandler.handleMerchandisingEvent(activity,
                        getCurrentFragmentManager(),
                        banner,
                        view,
                        banner.getTrackingCode())
            } catch (ex: IllegalArgumentException) {

            }

        }

        override fun onDynamicPageFragmentPushed(fragment: DynamicPageFragment) {
            push(fragment)
        }
    }

    lateinit private var dynamicLoader: ShelfDynamicSectionLoader

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        dynamicLoader = ShelfDynamicSectionLoader(context,
                arguments?.getString(EXTRA_PAGE_ID) ?: "/pages/mobileapp/recipes/noncuratedlisting",
                arguments?.getString(EXTRA_QUERY_STRING), null, actionHandler,null, null).apply {
            setLoadListener(this@NonCuratedRecipeListFragment)
        }
        dynamicLoader.load(null)
    }

    override fun getEpoxyViewController(): NonCuratedRecipePagedListController {
        activity?.let {
            return NonCuratedRecipePagedListController(it)
                  .apply {
                      onClearAllClicked = fun() {
                          activeFilters = emptyArray()
                          constructList(androidViewModel, epoxyController, true)
                          updateRefineButtonState()
                      }
                  }
        } ?: run {
            throw Throwable("activity no present")
        }
    }

    override fun constructList(viewModel: SearchRecipeViewModel, controller: SearchRecipePagedListController, forceReload: Boolean) {
        val dialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR_NOT_CANCELABLE, getString(R.string.loading), activity)
        dialog.show()
        // We need to translate XM dimensionID to facets before calling /recipe/list API
        ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", RecipeAPI::class.java)
                ?.listFacets()
                ?.subscribeOn(Schedulers.io())
                ?.observeOn(AndroidSchedulers.mainThread())
                ?.subscribe(object: ResourceObserver<RecipeFacetsListResponse>() {
                    override fun onComplete() {
                        dispose()
                    }

                    override fun onNext(t: RecipeFacetsListResponse) {
                        dismissDialog()
                        super@NonCuratedRecipeListFragment.setActiveFacets(RecipeUtils.getActiveFacets(t, arguments?.getString(EXTRA_QUERY_STRING)))
                        super@NonCuratedRecipeListFragment.constructList(viewModel, controller, forceReload)
                    }

                    override fun onError(e: Throwable) {
                        dismissDialog()
                        super@NonCuratedRecipeListFragment.setActiveFacets(null)
                        super@NonCuratedRecipeListFragment.constructList(viewModel, controller, forceReload)
                        dispose()
                    }

                    private fun dismissDialog() {
                        try {
                            dialog.dismiss()
                        } catch (ignore: Throwable) {
                            Log.w(TAG, ignore)
                        }
                    }
                })
    }

    override fun onLoadFinish(success: Boolean) {
        if (success) {
            (epoxyController as? NonCuratedRecipePagedListController)?.addDynamicSection(dynamicLoader)
        }
    }

    companion object {
        const val TAG = "NonCuratedRecipeList"
        @JvmStatic
        fun newInstance(searchTerm: String, pageId: String?, queryString: String?) = NonCuratedRecipeListFragment().apply {
            arguments = Bundle().apply {
                putString(EXTRA_SEARCH_TERM, searchTerm)
                putString(EXTRA_PAGE_ID, pageId)
                putString(EXTRA_QUERY_STRING, queryString)
                putBoolean(EXTRA_IS_SEARCH, false)
            }
        }
    }
}
