package com.asda.android.app.dynamic

import android.graphics.Typeface
import android.text.Layout
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import androidx.core.content.ContextCompat
import androidx.core.graphics.drawable.RoundedBitmapDrawable
import com.asda.android.R
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.shop.AsdaFilter
import com.asda.android.app.shop.FilterActivity
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.cms.Link
import com.facebook.litho.*
import com.facebook.litho.annotations.*
import com.facebook.litho.widget.Image
import com.facebook.litho.widget.Text
import com.facebook.litho.widget.VerticalGravity
import com.facebook.yoga.YogaEdge


@LayoutSpec
class RecipeCollectionComponentSpec {
    companion object {

        @JvmStatic
        @OnCreateLayout
        @Suppress("UNUSED_PARAMETER")
        fun onCreateLayout(c: ComponentContext, @Prop handler: DynamicPageFragment.OnActionHandler,
                           @Prop drawable: RoundedBitmapDrawable, @Prop(optional = true) title: String?,
                           @Prop(optional = true) description: String?, @State isPressed:Boolean): ComponentLayout =

                Column.create(c)
                        .child(Image.create(c)
                                .drawable(drawable))
                        .child(Text.create(c)
                                .text(title)
                                .textSizeSp(22f)
                                .verticalGravity(VerticalGravity.CENTER)
                                .textAlignment(Layout.Alignment.ALIGN_CENTER)
                                .textStyle(Typeface.BOLD))
                        .child(Text.create(c)
                                .text(description)
                                .textSizeSp(16f)
                                .verticalGravity(VerticalGravity.CENTER)
                                .textAlignment(android.text.Layout.Alignment.ALIGN_CENTER)
                                .textStyle(Typeface.ITALIC))
                        .clickHandler(RecipeCollectionComponent.onClick(c))
                        .touchHandler(RecipeCollectionComponent.onTouch(c))
                        .backgroundColor(ContextCompat.getColor(c, if (isPressed) R.color.very_light_grey else R.color.white))
                        .border(Border.create(c)
                                .widthDip(YogaEdge.ALL, 16)
                                .color(YogaEdge.ALL, ContextCompat.getColor(c, R.color.white))
                                .build())
                        .paddingDip(YogaEdge.ALL, 8f)
                        .contentDescription(title)
                        .build()

        @JvmStatic
        @OnEvent(ClickEvent::class)
        @Suppress("UNUSED_PARAMETER")
        fun onClick(c: ComponentContext, @FromEvent view: View,
                    @Prop handler: DynamicPageFragment.OnActionHandler,
                    @Prop(optional = true) title: String?,
                    @Prop(optional = true) link: Link?,
                    @Prop(optional = true) isCurated: String?,
                    @Prop(optional = true) searchTerm: String?) {
            if (isCurated?.equals("yes", true) == true) {
                link?.let {
                    if (!TextUtils.isEmpty(it.path)) {
                        handler.onDynamicPageFragmentPushed(DynamicPageFragment.newInstance(it.path, "&Ntt=$searchTerm", if (RestUtils.isTablet(c)) 2 else 1))
                    }
                }
            } else {
                handler.onFragmentPushed(NonCuratedRecipeListFragment.newInstance(title ?: "Recipe Listing", link?.path, link?.queryString).apply {
                    REQUEST_CODE_FILTER = ActivityRequestCodes.REQUEST_CODE_FILTER

                    if (RestUtils.isTablet(context)) {
                        refineButtonId = R.id.filter_button_breadcrumbs
                    } else {
                        refineButtonId = R.id.filter_button
                    }

                    onRefineClicked = { view ->
                        FilterActivity.showFilterForRecipesSearch(activity, this, ActivityRequestCodes.REQUEST_CODE_FILTER,
                                sortMode, recipeSortOrder, "",
                                activeFilters?.map { AsdaFilter(it) }?.toTypedArray(), view)
                    }
                }, title)
            }
        }

        @JvmStatic
        @OnUpdateState
        fun updatePressed(isPressed: StateValue<Boolean>) {
            isPressed.set(!isPressed.get())
        }

        @JvmStatic
        @OnEvent(TouchEvent::class)
        @Suppress("UNUSED_PARAMETER")
        fun onTouch(c: ComponentContext, @FromEvent view: View,
                    @FromEvent motionEvent: MotionEvent): Boolean {

            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> RecipeCollectionComponent.updatePressedAsync(c)
                MotionEvent.ACTION_UP -> RecipeCollectionComponent.updatePressedAsync(c)
                MotionEvent.ACTION_CANCEL -> RecipeCollectionComponent.updatePressedAsync(c)
            }
            return false
        }
    }
}
