package com.asda.android.app.dynamic

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.widget.CheckBox
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.restapi.service.data.recipes.RecipeResponse
import com.asda.android.recipes.RecipeUtils
import com.asda.android.recipes.RecipesManager
import com.asda.android.recipes.view.activities.RecipeDetailsActivity
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import com.asda.android.restapi.service.data.SiteConfigResponse
import com.facebook.litho.*
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils
import com.asda.android.utils.view.ViewUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceObserver
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers

@MountSpec
class RecipeListingComponentSpec {
    companion object {
        @JvmStatic
        @OnCreateMountContent
        internal fun onCreateMountContent(c: ComponentContext): View {
            return (c.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater).inflate(com.asda.android.recipes.R.layout.recipe_list_item_layout, null, false)
        }

        @JvmStatic
        @OnBind
        @Suppress("UNUSED_PARAMETER")
        internal fun onBind(c: ComponentContext, view: View, @Prop(optional = true) imageUrl: String?, @Prop(optional = true) title: String?,
                            @Prop(optional = true) description: String?, @Prop(optional = true) readyTime: String?,
                            @Prop(optional = true) ingredientsNumber: String?, @Prop(optional = true) costPerServing: String?,
                            @Prop(optional = true) recipeId: String?,  @Prop(optional = true) servingSize: String? ,@State isPressed:Boolean) {
            SiteConfigDataSource.getSiteConfigResponse()
                    ?.subscribeOn(Schedulers.io())
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribe(object: ResourceSingleObserver<SiteConfigResponse>() {
                        override fun onSuccess(configResponse: SiteConfigResponse) {
                            val url = RecipeUtils.composeScene7ImageUrl(configResponse.scene7Host, imageUrl)
                            ImageLoader.load(c, view.findViewById(com.asda.android.recipes.R.id.recipe_image_view), ImageLoaderRequest.Builder().path(url).placeHolder(R.drawable.recipe_placeholder).errorRes(R.drawable.recipe_placeholder).build()
                                  , null)
                            dispose()
                        }

                        override fun onError(e: Throwable) {
                            ImageLoader.load(c, view.findViewById(com.asda.android.recipes.R.id.recipe_image_view), ImageLoaderRequest.Builder().path(R.drawable.recipe_placeholder).build())
                            dispose()
                        }
                    })
            view.findViewById<TextView>(com.asda.android.recipes.R.id.recipe_title_text_view)?.text = title
            view.findViewById<TextView>(com.asda.android.recipes.R.id.recipe_desc_text_view)?.text = description
            view.findViewById<TextView>(com.asda.android.recipes.R.id.cook_time)?.text = RecipeUtils.formatPrepTime(readyTime?.toInt())
            view.findViewById<TextView>(com.asda.android.recipes.R.id.ingredients_count)?.text = if (TextUtils.isEmpty(ingredientsNumber) || !TextUtils.isDigitsOnly(ingredientsNumber)) c.getString(com.asda.android.recipes.R.string.ingredients_fallback) else
                ingredientsNumber?.let { Integer.valueOf(it) }?.let { c.resources.getQuantityString(com.asda.android.recipes.R.plurals.ingredients, it, ingredientsNumber) }
            //TODO Enable cost per serving once pricing issues are resolved

            // view.findViewById<TextView>(com.asda.android.recipes.R.id.price_per_serving)?.text =  c.getString(com.asda.android.recipes.R.string.listing_cost_serving, com.asda.android.service.RestUtils.addPoundIfNecessary(costPerServing))
            view.setBackgroundColor(ContextCompat.getColor(c, if (isPressed) R.color.very_light_grey else R.color.white))
            view.setOnClickListener {
                onClick(c, recipeId, servingSize)
            }
            view.setOnTouchListener { _, event ->
                onTouch(c, event)
            }
            val checkBox = view.findViewById<CheckBox>(com.asda.android.recipes.R.id.favorite)
            checkBox?.isChecked = RecipesManager.instance?.isFavorite(recipeId) ?: false
            checkBox?.setOnClickListener { _ ->
                recipeId?.let {
                    RecipesManager.instance?.let { manager ->
                        if (manager.isFavorite(recipeId)) {
                            manager.removeRecipeFromFavourites(manager.authentication.profileId ?: "", it)
                                    ?.subscribe(object: ResourceObserver<RecipeResponse>() {
                                        override fun onComplete() {
                                            dispose()
                                        }

                                        override fun onNext(t: RecipeResponse) {

                                        }

                                        override fun onError(e: Throwable) {
                                            dispose()
                                        }
                                    })
                        } else {
                            manager.addRecipeToFavourites(manager.authentication.profileId ?: "", it)
                                    ?.subscribe(object: ResourceObserver<RecipeResponse>() {
                                        override fun onComplete() {
                                            dispose()
                                        }

                                        override fun onNext(t: RecipeResponse) {

                                        }

                                        override fun onError(e: Throwable) {
                                            dispose()
                                        }
                                    })
                        }
                    }
                }
            }
        }

        @JvmStatic
        @OnMeasure
        @Suppress("UNUSED_PARAMETER")
        internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size) {
            MeasureUtils.measureWithDesiredPx(widthSpec, heightSpec, ViewUtil.dpToPixels(300, c), ViewUtil.dpToPixels(385, c), size)
        }

        @JvmStatic
        @OnEvent(ClickEvent::class)
        fun onClick(c: ComponentContext, @Prop(optional = true) recipeId: String?, @Prop(optional = true) servingSize: String?) {
            if (FeatureSettings.isCXOMigrationEnabled(c)) {
                if (com.asda.android.cxo.CartManager.instance?.hasRecipe(recipeId) == true && Authentication.getInstance().isLoggedIn) {
                    c.startActivity(Intent(c, RecipeDetailsActivity::class.java)
                          .putExtra(RecipeDetailsActivity.EXTRA_CUSTOMIZE, true)
                          .putExtra(RecipeDetailsActivity.EXTRA_SERVING_SIZE, RecipesManager.instance?.getServingSize(recipeId))
                          .putExtra(RecipeDetailsActivity.EXTRA_RECIPE_ID, recipeId))
                } else {
                    c.startActivity(Intent(c, RecipeDetailsActivity::class.java).putExtra(RecipeDetailsActivity.EXTRA_RECIPE_ID, recipeId))
                }
            } else {
                if (CartManager.get(c).hasRecipe(recipeId) && Authentication.getInstance().isLoggedIn) {
                    c.startActivity(Intent(c, RecipeDetailsActivity::class.java)
                          .putExtra(RecipeDetailsActivity.EXTRA_CUSTOMIZE, true)
                          .putExtra(RecipeDetailsActivity.EXTRA_SERVING_SIZE, RecipesManager.instance?.getServingSize(recipeId))
                          .putExtra(RecipeDetailsActivity.EXTRA_RECIPE_ID, recipeId))
                } else {
                    c.startActivity(Intent(c, RecipeDetailsActivity::class.java).putExtra(RecipeDetailsActivity.EXTRA_RECIPE_ID, recipeId))
                }
            }
        }

        @JvmStatic
        @OnUpdateState
        fun updatePressed(isPressed: StateValue<Boolean>) {
            isPressed.set(!isPressed.get())
        }

        @JvmStatic
        @OnEvent(TouchEvent::class)
        fun onTouch(c: ComponentContext, @FromEvent motionEvent: MotionEvent): Boolean {
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> RecipeListingComponent.updatePressedAsync(c)
                MotionEvent.ACTION_UP -> RecipeListingComponent.updatePressedAsync(c)
                MotionEvent.ACTION_CANCEL -> RecipeListingComponent.updatePressedAsync(c)
            }
            return false
        }
    }
}
