package com.asda.android.app.dynamic

import android.content.Context
import android.content.res.Configuration
import android.util.DisplayMetrics
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import com.asda.android.R
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.AdTrackingServiceQuery
import com.asda.android.admonetization.criteo.AdUtils
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Merchandising
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.Size
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils

@MountSpec
object SingleCriteoSecondaryNavBannerSpec {
    @OnCreateMountContent
    internal fun onCreateMountContent(c: ComponentContext): View {
        return LayoutInflater.from(c).inflate(R.layout.admonetization_secondary_nav_banner, null, false)
    }

    @OnMount
    internal fun onMount(c: ComponentContext, v: View, @Prop handler: DynamicPageFragment.OnActionHandler, @Prop banner: Merchandising.Banner) {
        updateViews(c, v, handler, banner)
    }

    /**
     * @param totalItems Required for BannerRows, we must know how many items are going to be displayed in a single row and use it only
     * when is a tablet or landscape.
     */
    @OnMeasure
    internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size, @Prop newAspectRatio: Float) {
        val metrics = DisplayMetrics()
        val windowManager = c.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        windowManager.defaultDisplay.getMetrics(metrics)
        val isTablet = RestUtils.isTablet(c)
        val config = c.resources.configuration
        val isLandscape = config.orientation == Configuration.ORIENTATION_LANDSCAPE
        var height = metrics.heightPixels

        if (isTablet || isLandscape) {
            height /= 4
        }

        var width = (height * newAspectRatio).toInt()
        if (width > metrics.widthPixels) {
            width = metrics.widthPixels
            height = (width / newAspectRatio).toInt()
        }
        MeasureUtils.measureWithDesiredPx(
              widthSpec,
              heightSpec,
              width,
              height,
              size)
    }

    private fun wireListeners(handler: DynamicPageFragment.OnActionHandler, banner: Merchandising.Banner, view: ImageView) {
        view.setOnClickListener { v -> handler.onMerchandisingClicked(banner, view) }
    }

    private fun updateViews(c: ComponentContext, v: View, handler: DynamicPageFragment.OnActionHandler, banner: Merchandising.Banner) {
        var imageView = v.findViewById(R.id.banner_image_full_merchandising_view) as ImageView
        if (banner.bannerDisplayType != null && banner.bannerDisplayType.contains(AdConstants.STATIC)) {
            v.findViewById<View>(R.id.banner_image_full_merchandising_view).visibility = View.VISIBLE
            v.findViewById<View>(R.id.banner_image_full_merchandising_view).contentDescription = AdUtils().getDescriptionFromMerchandisingBanner(banner)
            wireListeners(handler, banner, imageView)
        }
        if(!banner.merchandisingimage.isNullOrEmpty()) {
            ImageLoader.load(c, imageView, ImageLoaderRequest.Builder().path(banner.merchandisingimage).build(), null)
            AdTrackingServiceQuery().sendBannerViewEvent(banner.trackingInformation)
        }
    }
}
