package com.asda.android.app.dynamic.manager

import android.content.Context
import android.net.Uri
import android.text.TextUtils
import android.util.Log
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.constants.AdConstants.CCD_ACCOUNT
import com.asda.android.admonetization.criteo.constants.AdConstants.CUSTOMER_ID
import com.asda.android.admonetization.criteo.constants.AdConstants.ENDCA_USER_SEGMENTS
import com.asda.android.admonetization.criteo.constants.AdConstants.NO_PARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.NRPP
import com.asda.android.admonetization.criteo.constants.AdConstants.RETAILIER_VISITOR_ID
import com.asda.android.admonetization.criteo.constants.AdConstants.SHIP_ON_DATE
import com.asda.android.admonetization.criteo.AdUtils
import com.asda.android.analytics.anivia.AniviaTracker.getVisitorIdFromDisk
import com.asda.android.app.dynamic.ComponentMap
import com.asda.android.base.core.*
import com.asda.android.products.ui.utils.Constants
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.cms.*
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.utils.Utils.isQAServer
import java.util.*


object CMSUtils {

    private const val TAG = "CMSUtils"
    private const val BANNER_STATIC_PLACEMENT_KEY = "IM"
    private const val BANNER_CAROUSEL_PLACEMENT_KEY = "SC"
    private const val BG_IMAGE_KEY = "bg_img"
    private const val URL_REDIRECTION_KEY = "url_redirection"
    const val CRITEO_TAXONOMY_PAGE_ID = "viewCategoryAppAndroid"
    const val HOME_PAGE_ID = "viewHomeAppAndroid"

    fun getDimensionParameter(dimensionId: String?, param: String?): String? {
        try {
            if (dimensionId != null && param != null) {
                /**
                 * Added host to parse Uri
                 */
                val urlDimension = "http://localhost/" + if (dimensionId.contains("?")) dimensionId else "?$dimensionId"
                val uriDimension = Uri.parse(urlDimension)
                val parametersDimension = RestUtils.getUriParamValues(uriDimension)

                if (parametersDimension.containsKey(param)) {
                    return Uri.encode(parametersDimension[param])
                }
            }
        } catch (ex: Exception) {
            Log.e(TAG, "Error parsing uri: ", ex)
        }

        return null
    }

    fun getDimensionParameterRegex(dimensionId: String?, param: String?, regex: String?): Array<String>? {
        try {
            if (dimensionId != null && param != null && regex != null) {
                /**
                 * Added host to parse Uri
                 */

                val urlDimension = "http://localhost/" + if (dimensionId.contains("?")) dimensionId else "?$dimensionId"
                val uriDimension = Uri.parse(urlDimension)
                val parametersDimension = RestUtils.getUriParamValues(uriDimension)
                if (parametersDimension.containsKey(param)) {
                    return parametersDimension[param]!!.split(regex.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                }
            }
        } catch (ex: Exception) {
            Log.e(TAG, "Error parsing uri: ", ex)
        }

        return null
    }

    fun getDimensionParamRegexByPos(dimensionId: String?, param: String?, regex: String?, pos: Int): String? {
        try {
            if (dimensionId != null && param != null && regex != null) {
                /**
                 * Added host to parse Uri
                 */
                val urlDimension = "http://localhost/" + if (dimensionId.contains("?")) dimensionId else "?$dimensionId"
                val uriDimension = Uri.parse(urlDimension)
                val parametersDimension = RestUtils.getUriParamValues(uriDimension)

                if (parametersDimension.containsKey(param)) {
                    return parametersDimension[param]!!.split(regex.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()[pos]
                }
            }
        } catch (ex: Exception) {
            Log.e(TAG, "Error parsing uri: ", ex)
        }

        return null
    }

    fun getDimensionIdRegexFromRange(dimensionId: String?, param: String?, regex: String?, start: Int, end: Int): String? {
        try {
            if (dimensionId != null && param != null && regex != null && start <= end) {
                /**
                 * Added host to parse Uri
                 */
                val urlDimension = "http://localhost/" + if (dimensionId.contains("?")) dimensionId else "?$dimensionId"
                val uriDimension = Uri.parse(urlDimension)
                val parametersDimension = RestUtils.getUriParamValues(uriDimension)

                if (parametersDimension.containsKey(param)) {
                    val ids = parametersDimension[param]!!.split(regex.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                    val newDimensionBuilder = StringBuilder()
                    for (index in 0..ids.size - 1) {
                        if (index >= start && index < end) {
                            newDimensionBuilder.append(ids[index]).append(" ")
                        } else if (index == end) {
                            newDimensionBuilder.append(ids[index])
                        } else if (index > end) {
                            break
                        }
                    }
                    return newDimensionBuilder.toString()
                }
            }
        } catch (ex: Exception) {
            Log.e(TAG, "Error parsing uri: ", ex)
        }

        return null
    }

    fun hasProductList(contentList: List<Content>?): Boolean {
        if (contentList != null) {
            for (content in contentList) {
                if (content != null && (content is ProductPage || content is SpecialOffersPlaceholder) && content !is ProductSpotlightOffers) {
                    return true
                }
            }
        }
        return false
    }

    /**
     * Map pageId to corresponding personalization service query parameter.
     * [ContentManager.mDimensionId] will be the value of this param.
     * @param pageId
     * @return query parameter for p13n service
     */
    fun getId(pageId: String?): String {
        if (pageId == null) {
            return PageId.NONE
        }
        when (pageId) {
            "/pages/mobileapp/browse/cat" -> return PageId.CATEGORY_ID
            "/pages/mobileapp/browse/dept" -> return PageId.DEPARTMENT_ID
            "/pages/mobileapp/browse/aisle" -> return PageId.AISLE_ID
            "/pages/mobileapp/browse/shelf" -> return PageId.SHELF_ID
            "/pages/mobileapp/browse/product" -> return PageId.SKU_ID
            else -> return PageId.NONE
        }
    }

    /**
     * retrieves value for specified page
     * @param pageId
     * @param dimensionId
     * @param categoryId
     * @return
     */
    fun getPageValue(pageId: String, dimensionId: String?, categoryId: String?): String? {
        return if (PageId.SKU_ID == CMSUtils.getId(pageId) && !TextUtils.isEmpty(dimensionId)) dimensionId else categoryId
    }

    fun getP13NImpressionMetric(pageId: String): Int {
        when (pageId) {
            "/pages/mobileapp/browse/cat" -> return SitespectP13NMetrics.CATEGORY_IMPRESSION
            "/pages/mobileapp/browse/dept" -> return SitespectP13NMetrics.DEPARTMENT_IMPRESSION
            "/pages/mobileapp/browse/aisle" -> return SitespectP13NMetrics.AISLE_IMPRESSION
            "/pages/mobileapp/browse/product" -> return SitespectP13NMetrics.PDP_IMPRESSION
            else -> return 0
        }
    }

    fun getP13NCartAddMetric(pageId: String): Int {
        when (pageId) {
            "/pages/mobileapp/browse/cat" -> return SitespectP13NMetrics.CATEGORY_CART_ADD
            "/pages/mobileapp/browse/dept" -> return SitespectP13NMetrics.DEPARTMENT_CART_ADD
            "/pages/mobileapp/browse/aisle" -> return SitespectP13NMetrics.AISLE_CART_ADD
            "/pages/mobileapp/browse/product" -> return SitespectP13NMetrics.PDP_CART_ADD
            else -> return 0
        }
    }

    fun getStaticBanners(bannerRow: BannerRow?): List<Content>? {
        val banners = ArrayList<Content>()
        if (bannerRow?.content() != null) {
            bannerRow.content()?.forEach { content ->
                if (content.contents != null) {
                    content?.contents?.forEach { c ->
                        banners.add(c)
                    }
                }
            }
        }
        return banners
    }

    private fun getBannerCarousel(bannerContent: Array<Content>?): BannerCarousel? {
        var bannerCarousel: BannerCarousel? = null
        if (bannerContent != null && bannerContent.isNotEmpty() && bannerContent[0] is BannerCarousel && (bannerContent[0] as BannerCarousel).isCriteoBanner) {
            bannerCarousel = bannerContent[0] as BannerCarousel
            bannerCarousel.mediaItems?.indices.let {
                val mediaItems = bannerCarousel.mediaItems
                if (mediaItems != null) {
                    for (i in mediaItems.indices) {
                        mediaItems[i].contentDescription = AdUtils().getDescriptionFromCarousalBanner(mediaItems[i])
                        mediaItems[i].placementKey = bannerCarousel.placementKey
                    }
                }
            }
        }
        return bannerCarousel
    }

    fun updateHomePageHookLogic(data: HookLogic) {
        if (data.pageId.isNullOrEmpty())
            data.pageId = Constants.HOME_PAGE_APP
        if (data.dimensionId.isNullOrEmpty())
            data.dimensionId = Constants.PLACEHOLDER_DIMENTION_ID
    }

    fun getEligibleCarouselBanners(componentManager: ContentManager, data: Content, honorCriteoFalg : Boolean): BannerCarousel? {
        val contents = data?.contents
        val cartridges = contents?.get(0)?.mainContent
        var homePageIntroPlaceHolderIndex = -1

        if (cartridges != null) {
            for (i in cartridges.indices) {
                if (cartridges[i] is HomePageIntroPlaceHolder) {
                    homePageIntroPlaceHolderIndex = i
                    break
                }
            }
        }
        var bannerContent = (cartridges?.get(homePageIntroPlaceHolderIndex) as HomePageIntroPlaceHolder).introBanners?.get(0)?.contents
        if(bannerContent == null)
            return null
        else if(honorCriteoFalg) {
            return getBannerCarousel(bannerContent)
        } else
            return bannerContent[0] as BannerCarousel

    }

    fun getEligibleBanners(componentManager: ContentManager, componentMap: ComponentMap, honorCriteoFalg : Boolean): List<Content>? {
        val cartridges = componentManager.getTreeCartridges(componentMap)
        var bannerRowIndex = -1

        for (i in cartridges.indices) {
            if (cartridges[i] is BannerRow) {
                bannerRowIndex = i
                break
            }
        }

        val staticBanners = getStaticBanners(cartridges[bannerRowIndex] as BannerRow?)
        if(honorCriteoFalg)
            return staticBanners?.filter { it.isCriteoBanner == true }
        else
            return staticBanners
    }

    fun getLinkPath(urlRedirection: String): String {
        return urlRedirection.substringBefore("/N")
    }

    fun getQueryString(urlRedirection: String): String {
        return urlRedirection.substringAfterLast("/")
    }

    fun addCriteoApiQueryParameters(context: Context): HashMap<String, String?> {
        val data = HashMap<String, String?>()
        data[RETAILIER_VISITOR_ID] = getVisitorIdFromDisk(context)
        data["retail-shop"] = AsdaServiceFactory.get().storeId
        data["regionId"] = AsdaServiceFactory.get().storeId
        if ((Authentication.getInstance() as Authentication).isLoggedIn) {
            data[CUSTOMER_ID] = SingleProfile.getProfileResponse()?.profileId
        } else {
            data[CUSTOMER_ID] = " "
        }
        if (isQAServer(context))
            data[CCD_ACCOUNT] = AdConstants.TRACKING_ASDA_DEV_APP
        else
            data[CCD_ACCOUNT] = AdConstants.TRACKING_ASDA_APP

        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_CATEGORY"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_DEPARTMENT"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_AISLE"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_SHELF"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_HOME"] = HOME_PAGE_ID

        return data
    }

    fun monetizationCmsQueryParameters(): HashMap<String, String?> {
        val data = HashMap<String, String?>()
        data[ENDCA_USER_SEGMENTS] = AsdaServiceFactory.get().userSegments
        data[SHIP_ON_DATE] = AsdaServiceFactory.get().shipDate
        data[NRPP] = "60"
        data[NO_PARAM] = "0"
        data["store_id"] = AsdaServiceFactory.get().storeId
        return data
    }
}
