package com.asda.android.app.home.view

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.base.FragmentConfig
import com.asda.android.app.bookslotv3.view.activity.BookSlotActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.shop.MerchandisingHandler
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.base.core.HOME_PAGE
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.interfaces.IHomeManager
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.CartManager.SLOT_TYPE_CC
import com.asda.android.service.base.CartManager.SLOT_TYPE_DELIVERY
import com.asda.android.utils.view.ViewUtils

object HomeManagerImpl : IHomeManager {

    const val STORE_LOCATION_TAG = "StoreLocationViewModel"

    override fun onClickStartShopping(asdaActivity: Activity) {
        switchSection(asdaActivity, FragmentConfig.TAG_SHOP, null)
    }

    override fun launchShoppingList(sourceFragment: Fragment, context: Context) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), AdConstants.TYPE_SHOPPING_LISTS, null, null)
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, FragmentConfig.TAG_LIST))
        }
    }

    override fun launchRecipe(sourceFragment: Fragment, context: Context) {
        context.tryCast<Activity> {
            MerchandisingHandler.handleMerchandisingEvent(this, null, AdConstants.TYPE_RECIPES, null, null)
        }
    }

    override fun launchMessages(sourceFragment: Fragment, context: Context) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), AdConstants.TYPE_MESSAGES, null, null)
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, FragmentConfig.TAG_MSG_CENTER))
        }
    }

    override fun launchStoreFinder(sourceFragment: Fragment, context: Context) {

        var mOpenPetrolPopup : Dialog? = null
        val mOpenPetrolPopupListener: Map<Int, View.OnClickListener>? = mutableMapOf(
              R.id.btn_ok_webpage to View.OnClickListener {
                  mOpenPetrolPopup?.dismiss()
                  ViewUtils.openWebPage(SharedPreferencesUtil.getStoreLocatorUrl(context), context, STORE_LOCATION_TAG)
              },
              R.id.btn_cancel_webpage to View.OnClickListener {
                  mOpenPetrolPopup?.dismiss()
              }
        )
        mOpenPetrolPopup = AsdaDialogHelper.displayCustomDialog(R.layout.alert_petrol_store_locator, mOpenPetrolPopupListener, null, false, context)
    }

    override fun launchBookSlotCC(context: Context, fromView: View?) {
        launchBookSlot(context, fromView, SLOT_TYPE_CC)
    }

    override fun launchBookSlotDelivery(context: Context, fromView: View?) {
        launchBookSlot(context, fromView, SLOT_TYPE_DELIVERY)
    }

    private fun launchBookSlot(context: Context, fromView: View?, slotType: Int) {
        if (context !is AsdaActivity) {
            Log.d(javaClass.name, "context is not of type AsdaActivity, returning without launching book slot with tab $slotType")
            return
        }

        val intent = Intent(context, BookSlotActivity::class.java)
        intent.putExtra(BookSlotActivity.EXTRA_TARGET_TAB, slotType)
        ViewUtils.startActivityForResult(context, intent, ActivityRequestCodes.REQUEST_CODE_BOOK_SLOT, fromView)
    }

    private fun switchSection(context: Context, tag: String, bundle: Bundle?) {

        if (context !is AsdaActivity) {
            Log.d(javaClass.name, "context is not of type AsdaActivity, returning without launching $tag")
            return
        }

        context.fragmentSwitcher?.let {
            it.switchToFragment(tag, null)
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, tag))
        } ?: run {
            Log.d(javaClass.name, "fragmentSwitcher is null, returning without launching $tag")
        }
    }
}
