package com.asda.android.app.location

import android.content.Context
import android.content.Intent
import android.util.Log
import android.widget.Toast
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.SECTION_LOCATION_PERMISSION
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.view.BasePermissionActivity
import com.asda.android.app.view.PERMISSION_FRAGMENT
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.utils.Utils
import com.asda.android.utils.view.Permissions

private const val TAG = "CheckInLocationActivity"
const val CHECK_IN_LOCATION_ALLOW = "checkin_location_allow"
const val CHECK_IN_LOCATION_TURN_ON = "checkin_location_turnon"
const val CHECK_IN_LOCATION_NO_LOCATION = "checkin_no_location"
const val CHECK_IN_ERROR = "checkin_error"

const val CHECK_IN_ORDER_ID = "check_in_order_id"
const val CHECK_IN_STORE_PHONE_NO = "check_in_store_phone_no"
const val CHECK_IN_CUSTOMER_ID = "check_in_customer_id"

fun startCheckinLocationsActivity(context: Context, orderId: String?, customerId: String?, phoneNo: String? = null, type: String) {
    val intent = Intent(context, CheckInLocationActivity::class.java)
    intent.putExtra(CHECK_IN_ORDER_ID, orderId)
    phoneNo?.let {
        intent.putExtra(CHECK_IN_STORE_PHONE_NO, phoneNo)
    }
    customerId?.let {
        intent.putExtra(CHECK_IN_CUSTOMER_ID, customerId)
    }
    intent.putExtra(PERMISSION_FRAGMENT, type)

    context.startActivity(intent)
}

class CheckInLocationActivity : BasePermissionActivity() {
    private var orderId: String? = null
    private var customerId: String? = null
    var permissionListener: LocationPermissionListener? = null

    override fun pushFragment(permissionFragment: String?, intent: Intent?) {
        permissionFragment?.let {
            when (it) {
                CHECK_IN_LOCATION_ALLOW, CHECK_IN_LOCATION_TURN_ON, CHECK_IN_LOCATION_NO_LOCATION -> {
                    intent?.getStringExtra(CHECK_IN_ORDER_ID)?.let {
                        orderId = it
                        customerId = intent.getStringExtra(CHECK_IN_CUSTOMER_ID)
                        push(newInstanceCheckInLocationFragment(permissionFragment, it, customerId))
                    } ?: run {
                        finish()
                    }
                }
                CHECK_IN_ERROR -> {
                    intent?.getStringExtra(CHECK_IN_ORDER_ID)?.let {
                        orderId = it
                        customerId = intent.getStringExtra(CHECK_IN_CUSTOMER_ID)
                        push(newInstanceCheckInLocationFragment(permissionFragment, it, customerId,
                                intent.getStringExtra(CHECK_IN_STORE_PHONE_NO)))
                    }
                }
                else -> {
                    if (Utils.isDebug() && Utils.isDevBuild()) {
                        Log.d(TAG, "Not valid fragment")
                    } else {

                    }
                }
            }
        } ?: run {
            if (Utils.isDebug() && Utils.isDevBuild()) {
                Log.d(TAG, "Not valid fragment")
            }
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        fragmentManager.popBackStack()
    }

    public override fun onResume() {
        super.onResume()
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        when (requestCode) {
            Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION -> {
                Permissions.onLocationPermissionRequestReceived(this)
                //in android M preview grantResults always returns DENY even if permission was granted.
                //and there is no good way to find out that permission is permanently denied
                if (Permissions.isLocationPermissionGranted(this)) {
                    //ASDAA-10526 allow button tap
                    Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAP_EVENT)
                            .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata)
                            .putString(Anivia.BUTTON_NAME_KEY, getString(R.string.allow))
                            .putString(Anivia.NAME_KEY, SECTION_LOCATION_PERMISSION)
                            .putString(Anivia.OVERLAY_COPY_KEY, getString(R.string.checkin_location_permission_key)))
                    if (Utils.isDebug()) {
                        Toast.makeText(this, "Location permission granted on System Dialog", Toast.LENGTH_SHORT).show()
                    }
                    androidx.localbroadcastmanager.content.LocalBroadcastManager.getInstance(this).sendBroadcast(Intent(Permissions.LocationPermissionGrantedEvent.ACTION))
                } else {
                    Toast.makeText(this, R.string.no_permission_for_location, Toast.LENGTH_SHORT).show()
                    permissionListener?.onLocationPermissionReceived(requestCode, permissions, grantResults)
                }
            }
        }
    }

    interface LocationPermissionListener {
        fun onLocationPermissionReceived(requestCode: Int, permissions: Array<String>, grantResults: IntArray)
    }
}
