package com.asda.android.app.location

import android.Manifest
import android.app.Activity
import android.app.Application
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.content.IntentSender
import android.content.pm.PackageManager
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.model.createCheckinNotification
import com.asda.android.app.model.logAnalyticsForCheckinSuccess
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.OrderManager
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import com.asda.android.lastmile.*
import com.asda.android.lastmile.model.OrderStatus
import com.asda.lastmile.details.view.callStore
import com.asda.lastmile.details.view.getOrder
import com.google.android.gms.common.api.GoogleApiClient
import com.google.android.gms.common.api.Status
import com.google.android.gms.location.*
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.Permissions
import com.asda.android.utils.view.Permissions.requestLocationPermission
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable

private const val TAG = "CheckInLocationFragment"

private const val SCREEN_NAME = "CheckIn Location Fragment"

private const val CHECKIN_FRAGMENT_TYPE = "checkin_fragment_type"

private const val GLOBAL_PERM_REQUEST = 100

fun newInstanceCheckInLocationFragment(type: String, orderId: String?, customerId: String?, phoneNo: String? = null): CheckInLocationFragment {
    val fragment = CheckInLocationFragment()
    val bundle = Bundle()
    bundle.putString(CHECKIN_FRAGMENT_TYPE, type)
    bundle.putString(CHECK_IN_ORDER_ID, orderId)
    bundle.putString(CHECK_IN_STORE_PHONE_NO, phoneNo)
    bundle.putString(CHECK_IN_CUSTOMER_ID, customerId)
    fragment.arguments = bundle
    return fragment
}

class CheckInLocationFragment : FeaturedFragment(), CheckInLocationActivity.LocationPermissionListener {

    private lateinit var fragmentActivity: androidx.fragment.app.FragmentActivity
    private lateinit var fragmentDisplayType: String
    private lateinit var orderId: String
    private lateinit var customerId: String

    private lateinit var locationAllowedImage: ImageView
    private lateinit var locationNotAllowedImage: ImageView
    private lateinit var title: TextView
    private lateinit var description: TextView
    private lateinit var privacyLink: TextView
    private lateinit var tandc: TextView
    private lateinit var primaryAction: Button
    private lateinit var secondaryAction: Button
    private var phoneNumber: String? = null
    private var checkinManager: LastMileCheckinManager? = null
    private var progressDialog: Dialog? = null
    private val compositeDisposable = CompositeDisposable()
    private var isCheckedIn = false
    private var isFragmentResuming = false
    private var isGlobalLocationSvcOn: Boolean? = null
    var status: Status? = null

    override fun loadBundleData(bundle: Bundle?) {
        bundle?.let {
            fragmentDisplayType = it.getString(CHECKIN_FRAGMENT_TYPE).toString()
            orderId = it.getString(CHECK_IN_ORDER_ID).toString()
            phoneNumber = it.getString(CHECK_IN_STORE_PHONE_NO)
            customerId = it.getString(CHECK_IN_CUSTOMER_ID).toString()
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val view: View = ViewUtil.inflate(fragmentActivity, R.layout.checkin_location_fragment, null)
        locationAllowedImage = ViewUtil.findViewById(view, R.id.location_allowed)
        locationNotAllowedImage = ViewUtil.findViewById(view, R.id.location_not_allowed)
        title = ViewUtil.findViewById(view, R.id.title_message)
        description = ViewUtil.findViewById(view, R.id.desc_message)
        privacyLink = ViewUtil.findViewById(view, R.id.privacy_link)
        tandc = ViewUtil.findViewById(view, R.id.privacy_link2)
        primaryAction = ViewUtil.findViewById(view, R.id.primary_action)
        secondaryAction = ViewUtil.findViewById(view, R.id.secondary_action)
        initializeView()
        return view
    }

    override fun onLocationPermissionReceived(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        primaryAction.setText(R.string.settings_title)
        Toast.makeText(activity, R.string.app_location_message, Toast.LENGTH_LONG).show()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        if (SharedPreferencesUtil.isUsgrCheckinEnabled(context)) {
            checkinManager = USGRLastMileCheckinManager.getInstance(AsdaApplication.getInstance() as Application)
        } else {
            if (SharedPreferencesUtil.isLocalMockCheckinEnabled(context)) {
                checkinManager = MockLastMileCheckinManager.getInstance(OrderManager.get().orders)
            } else {
                checkinManager = ASDALastMileCheckinManager.getInstance(activity?.application
                        ?: throw Throwable("activity/applicatiom is null"),
                        getString(R.string.checkin_sdk_asda_consumer_id),
                        getString(R.string.checkin_sdk_asda_env),
                        getString(R.string.checkin_sdk_cine_version_asda),
                        getString(R.string.checkin_sdk_pegasus_version_asda),
                        getString(R.string.checkin_sdk_maps_api_key),
                        getString(R.string.checkin_sdk_asda_tenant_id),
                        getString(R.string.checkin_sdk_asda_vertical_id),
                        Authentication.getInstance(),
                        AsdaActivity::class.java.name)
            }
        }
        progressDialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR,
                getString(R.string.checkin_progress), context)
        val checkinLocationActivity = activity as CheckInLocationActivity
        checkinLocationActivity.permissionListener = this
    }

    override fun onStart() {
        super.onStart()

        val builder = LocationSettingsRequest.Builder()
                .addLocationRequest(LocationRequest.create().apply {
                    priority = LocationRequest.PRIORITY_HIGH_ACCURACY
                    interval = 10 * 1000
                    fastestInterval = 5 * 1000
                })
                .addLocationRequest(LocationRequest.create().apply {
                    priority = LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY
                })
                .addLocationRequest(LocationRequest.create().apply {
                    priority = LocationRequest.PRIORITY_LOW_POWER
                })

        val googleApiClient = GoogleApiClient.Builder(this.fragmentActivity.applicationContext)
                .addApi(LocationServices.API)
                .addConnectionCallbacks(object : GoogleApiClient.ConnectionCallbacks {
                    override fun onConnected(bundle: Bundle?) {
                        if (BuildConfig.DEBUG) {
                            Log.e(TAG, "connected with bundle ${bundle}")
                        }
                    }

                    override fun onConnectionSuspended(result: Int) {
                        if (BuildConfig.DEBUG) {
                            Log.e(TAG, "suspended with result ${result}")
                        }
                    }
                })
                .addOnConnectionFailedListener {
                    if (BuildConfig.DEBUG) {
                        Log.e(TAG, "connection Failed while retriving the location setting permission! with  ${it}")
                    }
                }
                .build()
        val pendingResult = LocationServices.SettingsApi.checkLocationSettings(googleApiClient,
                builder.build())

        pendingResult.setResultCallback {
            status = it.getStatus()
            if (BuildConfig.DEBUG) {
                Log.d(TAG, "Connection Status -> ${status}")
            }

            when (status?.statusCode) {
                LocationSettingsStatusCodes.SUCCESS -> {
                    if (Permissions.isLocationPermissionGranted(context) && !isCheckedIn) {
                        checkin(orderId)
                        isCheckedIn = true
                    }
                    isGlobalLocationSvcOn = true
                    primaryAction.text = getString(R.string.settings_title)
                }
                LocationSettingsStatusCodes.RESOLUTION_REQUIRED -> {
                    if (android.os.Build.VERSION.SDK_INT <= android.os.Build.VERSION_CODES.LOLLIPOP_MR1) {
                        try {
                            if (this.isAdded) {
                                // Show the dialog by calling startResolutionForResult(),
                                // and check the result in onActivityResult().
                                this.startIntentSenderForResult(status?.resolution?.intentSender, REQUEST_CHECK_SETTINGS_PRE_LOLLIPOP, null, 0, 0, 0, null)
                            }
                        } catch (e: IntentSender.SendIntentException) {
                            // Ignore the error.
                            if (BuildConfig.DEBUG) {
                                Log.e(TAG, "error: ${e}")
                            }
                        }
                    } else {
                        isGlobalLocationSvcOn = false
                        primaryAction.text = getString(R.string.allow)
                    }
                }
                LocationSettingsStatusCodes.SETTINGS_CHANGE_UNAVAILABLE -> {
                    if (BuildConfig.DEBUG) {
                        Log.e(TAG, "LocationSettingsStatusCodes.SETTINGS_CHANGE_UNAVAILABLE")
                        Toast.makeText(context, "GPS service not available on this device.", Toast.LENGTH_LONG).show()
                    }
                }
            }
        }
        googleApiClient.connect()
    }

    override fun onPause() {
        super.onPause()
        isFragmentResuming = true
    }

    override fun onStop() {
        super.onStop()
        progressDialog?.let {
            if (it.isShowing) {
                try {
                    it.dismiss()
                } catch (e: Exception) {
                    if (Utils.isDebug()) {
                        Log.e(TAG, "Error closing the dialog: ${e}")
                    }
                }
            }
        }
        progressDialog = null
    }

    override fun onDestroy() {
        super.onDestroy()
        compositeDisposable.clear()
        val checkInLocationActivity = activity as CheckInLocationActivity
        checkInLocationActivity.permissionListener = null
    }

    private fun checkin(orderId: String?) {
        orderId?.let {
            getOrder(it)?.let {
                isAdded
                val disposable = checkinManager?.checkin(order = it, customerId = customerId, notification = createCheckinNotification(context
                        ?: throw Throwable("context is null")))
                        ?.startWith(OrderStatus(isLoading = true))
                        ?.subscribeOn(AndroidSchedulers.mainThread())
                        ?.subscribe({ status ->
                            when (status) {
                                is OrderStatus -> {
                                    if (status.error) {
                                        if (Utils.isDebug()) {
                                            Toast.makeText(context, "Error Checking in Please try again!!", Toast.LENGTH_SHORT).show()
                                            Log.e(TAG, "error: ${status.error}")
                                        }
                                        activity?.finish()

                                    } else if (status.isLoading) {
                                        //show the progress
                                        if (progressDialog == null) {
                                            progressDialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR,
                                                    getString(R.string.checkin_progress), context)
                                        }
                                        progressDialog?.show()
                                    } else {
                                        //todo:vmk ASDAA-10524
                                        logAnalyticsForCheckinSuccess(Authentication.getInstance().profileId
                                                ?: "unknown",
                                                it.orderId ?: "unknown",
                                                AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                                                        ?: "unknown",
                                                status.etaMillis ?: "unknown",
                                                "")//vmk TBD how to get distance here?
                                        //dismiss the progress and finish the activity
                                        progressDialog?.let {
                                            if (it.isShowing) {
                                                try {
                                                    progressDialog?.dismiss()
                                                } catch (e: Exception) {
                                                    if (BuildConfig.DEBUG) {
                                                        Log.e(TAG, "Dialog dismiss failed with ${e.localizedMessage}")
                                                    }
                                                }
                                            }
                                        }
                                        activity?.finish()
                                    }
                                }
                            }
                        },
                                { error ->
                                    if (Utils.isDebug()) {
                                        //todo:vmk add popup saying checkin error and call store
                                        Toast.makeText(context, "Error Checking in Pleas try again!!", Toast.LENGTH_SHORT).show()
                                        Log.e(TAG, "error: ${error.message}")
                                    }
                                    activity?.finish()
                                },
                                {
                                    activity?.finish()
                                    if (Utils.isDebug()) {
                                        Log.i(TAG, "manual checkin completed")
                                    }
                                })
                disposable?.let {
                    compositeDisposable.add(it)
                }
            }
        } ?: run {
            if (Utils.isDebug()) {
                Log.e(TAG, "Could not perform checkin as orderId is null")
            }
        }
    }

    private fun initializeView() {
        if (!TextUtils.isEmpty(fragmentDisplayType)) {
            when (fragmentDisplayType) {
                CHECK_IN_LOCATION_ALLOW -> {
                    setClockImage(false)
                    title.setText(R.string.checkin_location_allow_title)
                    description.setText(R.string.checkin_location_base_desc)
                    primaryAction.setText(R.string.allow)
                    primaryAction.setOnClickListener {
                        if (isGlobalLocationSvcOn == false) {
                            //launch the Global permission request
                            requestGlobalLocationPermission()
                        } else {
                            if (primaryAction.text.contains(getString(R.string.allow))) {
                                if (!Permissions.isLocationPermissionGranted(fragmentActivity)) {
                                    requestLocationPermission(fragmentActivity)
                                    SharedPreferencesUtil.setCheckinLocationFlag(fragmentActivity, false)
                                }
                            } else {
                                //launch settings
                                launchSettings()
                            }
                        }

                        //ASDAA-10526 overlay
                        Tracker.get().trackEvent(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                                .putString(Anivia.OVERLAY_COPY_KEY, this.getResources().getString(R.string.checkin_location_permission_key))
                                .putString(Anivia.NAME_KEY, this.getResources().getString(R.string.checkin_enable_location_permission))
                                .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                                        ?: "unknown"))
                    }
                }
                CHECK_IN_LOCATION_TURN_ON -> {
                    setClockImage(false)
                    title.setText(R.string.checkin_location_turnon_title)
                    description.setText(R.string.checkin_location_base_desc)
                    primaryAction.setText(R.string.settings_title)
                    secondaryAction.visibility = View.VISIBLE
                    primaryAction.setOnClickListener { launchSettings() }
                    secondaryAction.setText(R.string.exit)
                    secondaryAction.setOnClickListener {
                        popBack()
                        push(newInstanceCheckInLocationFragment(CHECK_IN_LOCATION_NO_LOCATION, orderId, customerId))
                    }
                }
                CHECK_IN_LOCATION_NO_LOCATION -> {
                    setClockImage(true)
                    title.text = getString(R.string.checkin_no_location_title)
                    description.setText(R.string.checkin_no_location_desc)
                    primaryAction.setText(R.string.settings_title)
                    secondaryAction.visibility = View.INVISIBLE
                    primaryAction.setOnClickListener { launchSettings() }
                    secondaryAction.setText(R.string.exit)
                    secondaryAction.setOnClickListener {
                        val intent = Intent()
                        intent.putExtra(CHECK_IN_ORDER_ID, orderId)
                        fragmentActivity.setResult(ActivityRequestCodes.REQUEST_CODE_CHECKIN_CONTINUE, intent)
                        fragmentActivity.finish()
                    }
                }
                CHECK_IN_ERROR -> {
                    setClockImage(true)
                    title.setText(R.string.checkin_error_title)
                    description.setText(R.string.checkin_failure_subtitle)
                    description.visibility = View.INVISIBLE
                    primaryAction.setText(R.string.checkin_failure_callstore)
                    secondaryAction.visibility = View.VISIBLE
                    phoneNumber?.let {
                        primaryAction.visibility = View.VISIBLE
                    } ?: run {
                        primaryAction.visibility = View.INVISIBLE
                    }
                    primaryAction.setOnClickListener {
                        //Check for Phone call permission else show error
                        this@CheckInLocationFragment.context?.let {
                            if (ContextCompat.checkSelfPermission(it, Manifest.permission.CALL_PHONE)
                                    == PackageManager.PERMISSION_GRANTED) callStore(it, phoneNumber) else {
                                Toast.makeText(it, getString(R.string.checkin_no_permission_to_call), Toast.LENGTH_SHORT).show()
                            }
                        } ?: run {
                            if (Utils.isDebug()) {
                                Log.e(TAG, "Not able to make call as context is null!!")
                            }
                        }
                    }
                    secondaryAction.setText(R.string.exit)
                    secondaryAction.setOnClickListener {
                        fragmentActivity.finish()
                    }
                }
                else -> if (Utils.isDebug() && Utils.isDevBuild()) {
                    Log.d(TAG, "Not valid checkin fragmentActivity type")
                }
            }

            privacyLink.text = Utils.fromHtml(getString(R.string.checkin_location_bottom_link))
            tandc.text = Utils.fromHtml(getString(R.string.checkin_location_bottom_link2))
            privacyLink.setOnClickListener { ViewUtils.openWebPage(SharedPreferencesUtil.getPrivacyPolicyUrl(fragmentActivity), context, SCREEN_NAME) }
            tandc.setOnClickListener { ViewUtils.openWebPage(SharedPreferencesUtil.getTermsAndConditionsUrl(fragmentActivity), context, SCREEN_NAME) }
        }
    }

    private fun launchSettings() {
        isGlobalLocationSvcOn?.let {
            if (!it) {
                requestGlobalLocationPermission()
            } else {
                Permissions.openPermissionsWindow(fragmentActivity)
            }
        } ?: run {
            Permissions.openPermissionsWindow(fragmentActivity)
        }
    }

    private fun requestGlobalLocationPermission() {
        // Location settings are not satisfied. But could be fixed by showing the user
        // a dialog.
        try {
            // Show the dialog by calling startResolutionForResult(),
            // and check the result in onActivityResult().
            status?.startResolutionForResult(activity, GLOBAL_PERM_REQUEST);

        } catch (e: IntentSender.SendIntentException) {
            // Ignore the error.
            if (BuildConfig.DEBUG) {
                Log.e(TAG, "status.startResolutionForResult failed with  ${e.localizedMessage}")
            }
        }
    }

    private fun popBack() {
        val currentFragmentManager = getCurrentFragmentManager()
        currentFragmentManager?.popBackStack()
    }

    fun setClockImage(isNoLocation: Boolean) {
        locationAllowedImage.visibility = if (isNoLocation) {
            View.GONE
        } else {
            View.VISIBLE
        }
        locationNotAllowedImage.visibility = if (isNoLocation) {
            View.VISIBLE
        } else {
            View.GONE
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        fragmentActivity = context as androidx.fragment.app.FragmentActivity
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String = ""

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        val states = LocationSettingsStates.fromIntent(data)
        when (requestCode) {
            REQUEST_CHECK_SETTINGS_PRE_LOLLIPOP -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        if (BuildConfig.DEBUG) {
                            Log.e(TAG, "required changes made")
                        }
                        if (Permissions.isLocationPermissionGranted(context) && !isCheckedIn) {
                            checkin(orderId)
                            isCheckedIn = true
                        }
                        isGlobalLocationSvcOn = true
                        primaryAction.text = getString(R.string.settings_title)
                    }

                    Activity.RESULT_CANCELED -> {
                        // The user was asked to change settings, but chose not to
                        if (!states.isGpsUsable()) {
                            // Degrade gracefully depending on what is available
                            if (BuildConfig.DEBUG) {
                                Log.e(TAG, "user did not allow the permission")
                            }
                            Toast.makeText(activity, R.string.app_location_message, Toast.LENGTH_LONG).show()
                            activity?.finish()
                        }
                    }
                }
            }
        }
    }
}
