package com.asda.android.app.main

import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.exception.CaughtExceptionEvent
import com.asda.android.app.main.view.UpgradeActivity
import com.asda.android.firebase.RemoteConfig
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.utils.Utils
import org.json.JSONArray

object RemoteConfigController  {

        private val TAG = "RemoteConfigController"
        fun validateStore(activity: Activity?) {
            val storeId = AsdaServiceFactory.get().lastStoreIdFromSessionMetadata

            if (activity != null && storeId != null) {
                var remoteConfigCallback: RemoteConfig.RemoteConfigCallback = object : RemoteConfig.RemoteConfigCallback {
                    override fun onDownloadSuccess() {
                        SharedPreferencesUtil.setRemoteConfigDetails(activity)
                        val myVersion = Utils.getApplicationVersion(activity)
                        RemoteConfig.let {
                            var appMinimumVersion: Int = getAppVersion(it, "appMinimumVersionForStores")

                            var requireUpdate = false
                            var minimumVersionStoreIds = it?.getValue("appMinimumVersionStoreIds")
                            if (!minimumVersionStoreIds.isNullOrEmpty()) {
                                var storeIds = JSONArray(minimumVersionStoreIds)
                                for (i in 0 until storeIds.length()) {
                                    if (storeId.equals(storeIds.getString(i))) {
                                        requireUpdate = true
                                        break
                                    }
                                }
                            }

                            if (myVersion != -1 && appMinimumVersion > 0 && myVersion < appMinimumVersion && requireUpdate) {
                                val intent = Intent(activity, UpgradeActivity::class.java)
                                if (!it?.getValue("appForceUpgradeMessage").toString().isNullOrEmpty()) {
                                    intent.putExtra(UpgradeActivity.EXTRA_MESSAGE, it?.getValue("appForceUpgradeMessage").toString())
                                }
                                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                activity.startActivity(intent)
                                activity.finish()
                            }
                        }
                    }

                    override fun onFailure() {
                        Log.e(TAG, "Failed to retrieve APG config.")
                    }
                }
                setFirebaseDefaultsAndFetch(activity, remoteConfigCallback)
            }
        }

    /**
     * This function is written to avoid duplicate code in this class.
     * This function is supposed to be invoked from this particular class, by either of the stated keys.
     * @param remoteConfig = remote config instance.
     * @param key =  the key to be searched in remote config.
     * @return app version corresponding to the key from remote config
     */
    private fun getAppVersion(remoteConfig: RemoteConfig, key: String): Int {
        return if (key.equals("appMinimumVersion") || key.equals("appMinimumVersionForStores") || key.equals("appCurrentVersion")) {
            var appVersion: Int
            try {
                appVersion = Integer.parseInt(remoteConfig.getValue(key))
            } catch (e: Exception) {
                appVersion = -1
                Log.e(TAG, e.message.toString())
            }
            return appVersion
        } else {
            -1
        }
    }

        fun getConfigForAppUpdate(activity: Activity?) {
            val storeId = AsdaServiceFactory.get().lastStoreIdFromSessionMetadata

            if (activity != null && storeId != null) {
                var remoteConfigCallback: RemoteConfig.RemoteConfigCallback = object : RemoteConfig.RemoteConfigCallback {

                    override fun onDownloadSuccess() {
                        SharedPreferencesUtil.setRemoteConfigDetails(activity)
                        val myVersion = Utils.getApplicationVersion(activity)
                        RemoteConfig.let {
                            var appMinimumVersion: Int = getAppVersion(it, "appMinimumVersion")

                            var appMinimumStoreVersion: Int = getAppVersion(it, "appMinimumVersionForStores")

                            var appCurrentVersion: Int = getAppVersion(it, "appCurrentVersion")

                            var requireUpdate = false
                            var minimumVersionStoreIds = it?.getValue("appMinimumVersionStoreIds")
                            if (!minimumVersionStoreIds.isNullOrEmpty()) {
                                var storeIds = JSONArray(minimumVersionStoreIds)
                                for (i in 0 until storeIds.length()) {
                                    if (storeId.equals(storeIds.getString(i))) {
                                        requireUpdate = true
                                        break
                                    }
                                }
                            }

                            if (myVersion != -1) {
                                if (appMinimumVersion > 0 && (myVersion < appMinimumVersion || myVersion < appMinimumStoreVersion && requireUpdate)) {
                                    val intent = Intent(activity, UpgradeActivity::class.java)
                                    if (!it?.getValue("appForceUpgradeMessage").toString().isNullOrEmpty()) {
                                        intent.putExtra(UpgradeActivity.EXTRA_MESSAGE, it?.getValue("appForceUpgradeMessage").toString())
                                    }
                                    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                    activity.startActivity(intent)
                                    activity.finish()
                                } else if (appCurrentVersion > 0 && myVersion < appCurrentVersion &&
                                      it?.getValue("appCurrentVersion") != null && !activity.isFinishing()) {
                                    AlertDialog.Builder(activity)
                                          .setMessage(R.string.newer_version_available)
                                          .setPositiveButton(activity.getText(R.string.upgrade_now_nag_button)) { dialog, which ->
                                              try {
                                                  activity.startActivity(Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=com.asda.android")))
                                              } catch (e: ActivityNotFoundException) {
                                                  Log.e(TAG, "unable to open market")
                                                  Tracker.get().trackException(CaughtExceptionEvent(e))
                                                  Toast.makeText(activity, activity.getText(R.string.oops_error), Toast.LENGTH_SHORT).show()
                                              }
                                          }
                                          .setNegativeButton(activity.getText(R.string.upgrade_later_nag_button), null)
                                          .setCancelable(true)
                                          .create().show()
                                }

                            }
                        }
                    }

                    override fun onFailure() {
                        Log.e(TAG, "Failed to retrieve APG config.")
                    }
                }

                setFirebaseDefaultsAndFetch(activity, remoteConfigCallback)
            }
        }

        fun setFirebaseDefaultsAndFetch(activity: Context, callBack: RemoteConfig.RemoteConfigCallback) {
            RemoteConfig.setdefaultConfigValues(activity, Utils.isDevBuild())
            SharedPreferencesUtil.setRemoteConfigDetails(activity)
            RemoteConfig.getRemoteConfigValues(activity, callBack)
        }

    }

