package com.asda.android.app.model

import android.text.TextUtils
import android.util.Log
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.restapi.service.data.CheckoutDeliveryPassResponse
import com.asda.android.restapi.service.data.CheckoutResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.three_ds.ThreeDsUtils
import com.asda.android.three_ds.helper.ThreeDS
import com.asda.android.three_ds.three_ds_2x.ThreeDS2XImpl
import com.asda.android.utils.Utils


const val TAG = "CardCheckout"

data class CardCheckoutModel(val creditCardResponse: PaymentDetailsResponse?, val checkoutResponse: CheckoutResponse?, val checkoutDeliveryPassResponse: CheckoutDeliveryPassResponse?) {

    val mContext = AsdaApplication.getInstance()?.applicationContext

    val paymentCardsList: List<PaymentDetailsResponse.PaymentCards>
        get() {
            var cards: List<PaymentDetailsResponse.PaymentCards> = ArrayList()
            checkoutResponse?.let {
                it.paymentDetails?.let {
                    //We just need to check if SP as legacy already takes care of it.
                    it.orderCardNickName?.let {
                        if (it.toLowerCase().contains("temp")) {
                            checkoutResponse.paymentDetails?.card = it
                        }
                    }
                    cards = cards.plus(it)
                }
            }

            checkoutDeliveryPassResponse?.let { deliveryPassResponse ->
                deliveryPassResponse.paymentDetails?.let {cardResponse ->
                    cardResponse.orderCardNickName?.let {
                        if (it.toLowerCase().contains("temp")) {
                            checkoutDeliveryPassResponse.paymentDetails?.card = it
                        }
                    }
                    cardResponse.jwt?.let {jwt ->
                        if (!TextUtils.isEmpty(jwt) && Utils.isThreeDs2XEnabled(mContext, cardResponse.cardBrand) && FeatureSettings.isThreedsEnabledForDp(mContext!!)) {
                            val threeDS = ThreeDS()
                            threeDS.collectData(jwt, Utils.isDebug(), object : ThreeDS2XImpl.CardinalInitCallback {
                                override fun onSuccess(consumerSessionId: String?) {
                                    if (!TextUtils.isEmpty(consumerSessionId)) {
                                        ThreeDsUtils.isCardinalInitSuccess = true
                                    }
                                }

                                override fun onFailed(errorNumber: Int?, errorDesc: String?) {
                                    if (!TextUtils.isEmpty(errorNumber.toString()) || !TextUtils.isEmpty(errorDesc)) {
                                        ThreeDsUtils.isCardinalInitSuccess = false
                                    }
                                }
                            })
                            if (Utils.isDebug()) {
                                Log.d(TAG, "JWT: $jwt")
                            }
                        }
                    }
                    cards = cards.plus(cardResponse)
                }
            }

            creditCardResponse?.let {
                it.cards?.let {
                    cards = cards.plus(it)
                }
            }
            cards = removeDuplicates(cards)
            return cards
        }

    private fun removeDuplicates(paymentCards: List<PaymentDetailsResponse.PaymentCards>): List<PaymentDetailsResponse.PaymentCards> {
        for (creditCard in paymentCards) {
            creditCard.card = creditCard.card?.replace("x", "*")
        }

        return paymentCards.distinctBy {
            it.card
        }
    }
}
