package com.asda.android.app.model

import android.app.Activity
import android.app.Dialog
import android.app.Notification
import android.app.PendingIntent
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.os.Looper
import android.util.Log
import android.view.View
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.databinding.BaseObservable
import androidx.databinding.Bindable
import com.asda.android.BR
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.base.FragmentConfig
import com.asda.android.app.data.RecoveryAppHelper
import com.asda.android.app.location.CHECK_IN_ERROR
import com.asda.android.app.location.CHECK_IN_LOCATION_ALLOW
import com.asda.android.app.location.CHECK_IN_LOCATION_TURN_ON
import com.asda.android.app.location.startCheckinLocationsActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.orders.OrderFragment
import com.asda.android.app.view.WismoProgressView
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.lastmile.LastMileCheckinManager
import com.asda.android.lastmile.model.*
import com.asda.android.lastmile.view.LastMileNotificationFactory
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.ViewUtils
import com.asda.lastmile.details.view.EXTRA_MESSAGE_CUSTOMER_ID
import com.asda.lastmile.details.view.EXTRA_MESSAGE_ORDERID
import com.asda.lastmile.details.view.LMDetailsActivity
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable

private const val LOCATION_NAME: String = "Home"

fun createCheckinNotification(context: Context): Notification {
    val intent = Intent(context, AsdaActivity::class.java)
    val pendingIntent = PendingIntent.getActivity(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT)
    return LastMileNotificationFactory.createCheckInNotification(context,
          context.getString(R.string.checkin_order_title), pendingIntent)
}

@Suppress("UNUSED_PARAMETER")
fun logAnalyticsForCheckinSuccess(customerId: String, orderId: String, storeId: String, initialTimeInSec: String, initialDistance: String) {
    val event = AsdaAnalyticsEvent(Anivia.CHECKIN_SUCCESS_EVENT)
    event.updateTimestamp()
    event.putString(Anivia.COLLECTION_STORE_ID_KEY, storeId)
          .putString(Anivia.CUSTOMER_ID_KEY, customerId)
          .putString(Anivia.ORDER_ID_KEY, orderId)
          .putString(Anivia.CHECKIN_STATUS, "yes")
    Tracker.get().trackEvent(event)
}

fun createLMActivity(context: Context, orderId: String, customerId: String) {
    val intent = Intent(context, LMDetailsActivity::class.java)
    intent.putExtra(EXTRA_MESSAGE_ORDERID, orderId)
    intent.putExtra(EXTRA_MESSAGE_CUSTOMER_ID, customerId)
    context.startActivity(intent)
}

class LastmileCardModel(private val mContext: Context, private val mInFlightOrder: ViewOrderResponse.Order?,
                        val lastMileCheckinManager: LastMileCheckinManager?, private val loadListener: LoadingListener?) : BaseObservable() {

    val TAG = "LastmileCardModel"
    val DETAILS = "DETAILS"
    @get:Bindable
    var slotClickMessage: String? = DETAILS
        private set
    @get:Bindable
    var orderStatus: String? = null
        private set
    @get:Bindable
    var progress: Int = 0
        private set
    @get:Bindable
    var error: Boolean = false
        private set
    private var mButtonState: Int = 0
    @get:Bindable
    var isDetailEnabled: Boolean = false
        private set
    private val compositeDisposable = CompositeDisposable()
    @get:Bindable
    var statusTextColor: Int = 0
        private set
    @get:Bindable
    var statusBgColor: Int = 0
        private set
    @get:Bindable
    var titleTextColor: Int = 0
        private set
    @get:Bindable
    var orderDetailsTextVisibility = false
        private set
    @get:Bindable
    var orderTitleSecondary = "*Estimated times may change due to traffic, etc. Please check back regularly."
    @get:Bindable
    var startDotText = "Placed"
        private set
    @get:Bindable
    var endDotText = "Collected"
        private set
    @get:Bindable
    var hintText = "Ready for Collection"
        private set
    @get:Bindable
    var hintTextVisibility = true
        private set
    @get:Bindable
    var orderTitle = "Order is being processed"
        private set
    @get:Bindable
    var progressError = false
        private set
    @get:Bindable
    var orderTitleVisibility = false
        private set
    @get:Bindable
    var secondaryTextVisibility = false
        private set
    @get:Bindable
    var checkinBtnVisibility: Boolean? = true
    @get:Bindable
    var checkinBtnLabel: String? = "CHECK IN"
    private var mAmendsDialog: AlertDialog? = null
    @get:Bindable
    var btnClickable: Boolean? = true

    var orderNumber: String? = null
        @Bindable
        get() = mInFlightOrder?.orderId ?: ""


    val total: String?
        @Bindable
        get() = mInFlightOrder?.totalOrderCost ?: ""

    @get:Bindable
    var isLoading: Boolean

    var dialog: Dialog? = null
    var checkinDisposable: Disposable? = null

    private val alreadyAmendingDialog: View
        get() {
            val alreadyAmendingDialog = ViewUtil.inflate<View>(mContext, R.layout.order_detail_already_amending_dialog)
            alreadyAmendingDialog.findViewById<View>(R.id.already_amending_cancel_amends).setOnClickListener { _ ->
                try {
                    mAmendsDialog?.dismiss()
                } catch (e: Exception) {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "Error while dismissing dialog " + e)
                    }
                }
            }
            alreadyAmendingDialog.findViewById<View>(R.id.already_amending_confirm_amends).setOnClickListener { _ ->
                try {
                    mAmendsDialog?.dismiss()
                } catch (e: Exception) {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "Error while dismissing dialog " + e)
                    }
                }

                gotoCart()
            }
            return alreadyAmendingDialog
        }

    init {
        setDefaultColors(mContext)
        isLoading = true
        isDetailEnabled = true
        notifyPropertyChanged(BR.isLoading)
        setCheckinContent(mInFlightOrder)
    }

    private fun setDefaultColors(context: Context) {
        statusTextColor = ContextCompat.getColor(context, R.color.white)
        statusBgColor = ContextCompat.getColor(context, R.color.asda_blue)
        titleTextColor = ContextCompat.getColor(context, R.color.order_details_line_gray)
    }

    fun onClickMessage(view: View) {
        if (view is TextView && view.text.toString().contentEquals(DETAILS)) {
            mInFlightOrder?.let {
                val args = Bundle()
                args.putString(OrderFragment.ORDER_ID_KEY, mInFlightOrder.orderId)
                args.putBoolean(OrderFragment.IS_LEGACY_CHECKIN_ORDER, false)
                if (mContext is AsdaActivity) {
                    val fragmentSwitcher = mContext.fragmentSwitcher
                    fragmentSwitcher?.switchToFragment(FragmentConfig.TAG_ORDERS, args)
                }
            }
        }
    }

    private fun setCheckinContent(order: ViewOrderResponse.Order?) {
        order?.orderId?.let {
            lastMileCheckinManager?.let {
                val disposable = it.getOrderState(SharedPreferencesUtil.USGR_CUSTOMER_ID, mInFlightOrder!!)
//                        ?.timeout(20, TimeUnit.SECONDS, Schedulers.newThread())
                      .subscribe({ response ->
                          updateUi(response)
                      }, { error ->
                          if (Utils.isDebug()) {
                              Log.e(TAG, "error with: " + error)
                          }
                          updateUi(OrderStatus(null, null, null, false, true))
                      }, {
                          if (Utils.isDebug()) {
                              Log.d(TAG, "completed!!")
                          }
                      })
                disposable?.let {
                    compositeDisposable.add(it)
                }

            }
        }
    }

    private fun updateUi(response: CheckinEligibleOrderStatus?) {
        android.os.Handler(Looper.getMainLooper()).post {
            if (Utils.isDebug()) {
                Log.i(TAG, "ORDER STATE:: ************ ${response.toString()} *************** ${this}")
            }

            when (response) {
                is OrderStatus -> {
                    val os = response
                    if (os.isLoading) {
                        isLoading = true
                        notifyChange()
                    } else {
                        isLoading = false
                        if (os.error) {
                            orderStatus = mContext.getString(R.string.error_status)
                            statusBgColor = ContextCompat.getColor(mContext, R.color.asda_dark_red)
                            statusTextColor = ContextCompat.getColor(mContext, R.color.white)
                            btnClickable = false
                            notifyChange()
                        } else {
                            orderStatus = os.orderStatus
                            createBtnLabel(os.orderStatus)
                            mapStatusToProgress(os)
                            notifyChange()
                        }
                    }
                }
                is CheckinStsPushPermissionDenied -> Log.d(TAG, "CheckinStsPushPermissionDenied!")
                is CheckinNotPermittedOnTablet -> {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "Checkin Not allowed on Tablet!")
                    }
                    if (mContext is Activity) {
                        if (dialog == null) {
                            dialog = DialogFactory.createAlertDialog(mContext.getString(R.string.tablet_checkin_title), mContext.getString(R.string.tablet_checkin_content), object : DialogInterface.OnClickListener {
                                override fun onClick(p0: DialogInterface?, p1: Int) {
                                    //ASDAA-10529
                                    Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                                          .putString(Anivia.BUTTON_NAME_KEY, mContext.getString(R.string.ok))
                                          .putString(Anivia.NAME_KEY, LOCATION_NAME))
                                }
                            }, "", mContext, null)
                        }
                        //ASDAA-10529
                        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                              .putString(Anivia.OVERLAY_TITLE_KEY, mContext.getString(R.string.tablet_checkin_title))
                              .putString(Anivia.NAME_KEY, LOCATION_NAME)
                              .putString(Anivia.OVERLAY_COPY_KEY, mContext.getString(R.string.tablet_checkin_content)))
                        dialog?.show()
                    } else {
                        //only show the toast
                        Toast.makeText(mContext, mContext.getString(R.string.tablet_checkin_content), Toast.LENGTH_SHORT).show()
                    }
                }
                is CheckinStsLocationPermissionDenied -> {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "CheckinStsLocationPermissionDenied!")
                    }
                    val isFirstTime = SharedPreferencesUtil.getCheckinLocatonFlag(mContext)
                    var fragmentType = CHECK_IN_LOCATION_ALLOW
                    if (!isFirstTime) {
                        fragmentType = CHECK_IN_LOCATION_TURN_ON
                    }
                    var customerId = Authentication.getInstance().profileId
                    startCheckinLocationsActivity(mContext, mInFlightOrder?.orderId, customerId
                          ?: "empty", type = fragmentType)
                }
            }
        }
    }

    private fun mapStatusToProgress(status: OrderStatus?) {
        startDotText = WismoProgressView.PLACED_TEXT
        progress = 3
        status?.let {
            when (it.orderStatus) {
                ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> {
                    startDotText = ""
                    endDotText = ""
                    hintText = ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY
                    hintTextVisibility = true
                }
                ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> {
                    startDotText = ""
                    endDotText = ""
                    hintText = ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE
                    hintTextVisibility = true
                }
                ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> {
                    startDotText = ""
                    endDotText = WismoProgressView.COLLECTED_TEXT
                    progress = 4
                    hintTextVisibility = false
                }
                else -> {
                    startDotText = ""
                    endDotText = ""
                    hintText = mContext.getString(R.string.orderReadyForCollection)
                    hintTextVisibility = true
                }
            }
        }
    }

    private fun createBtnLabel(orderStatus: String?) {
        btnClickable = true
        orderStatus?.let {
            when (it) {
                ViewOrderResponse.Order.ORDER_STATUS_LOC_PERM_DENIED -> checkinBtnLabel = mContext.getString(R.string.checkin)

                ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        checkinBtnLabel = mContext.getString(R.string.checkin)
                    } else {
                        checkinBtnLabel = mContext.getString(R.string.labelOnYourWay)
                    }
                }
                ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        checkinBtnLabel = mContext.getString(R.string.checkin)
                    } else {
                        checkinBtnLabel = mContext.getString(R.string.labelYouAreHere)
                    }
                }
                ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        checkinBtnLabel = mContext.getString(R.string.checkin)
                    } else {
                        checkinBtnLabel = mContext.getString(R.string.labelOrderCollected)
                        statusBgColor = ContextCompat.getColor(mContext, R.color.asda_green)
                        statusTextColor = ContextCompat.getColor(mContext, R.color.white)
                    }
                }
                else -> checkinBtnLabel = mContext.getString(R.string.checkin)
            }
        }
    }

    fun cleanup() {
        compositeDisposable.clear()
        dialog = null
    }

    private fun updateTextVisiblity() {
        orderTitleVisibility = secondaryTextVisibility
    }

    fun onButtonClick(view: View) {
        var customerId = Authentication.getInstance().profileId
        if (SharedPreferencesUtil.isUsgrCheckinEnabled(mContext)) {
            customerId = SharedPreferencesUtil.USGR_CUSTOMER_ID
        }
        val notification = createCheckinNotification(mContext)
        if ((view as Button).text.contains(mContext.getString(R.string.collection_details), true)) {
            if (Utils.isDebug()) {
                Toast.makeText(mContext, "COLLECTION DETAILS button Clicked!!", Toast.LENGTH_SHORT).show()
            }
            createLMActivity(mContext, orderNumber ?: "empty", customerId ?: "empty")
        } else if (view.text.contains(mContext.getString(R.string.checkin), true)) {
            mInFlightOrder?.let {
                customerId?.let {
                    checkinDisposable = lastMileCheckinManager?.checkin(mInFlightOrder, customerId, notification)
                          ?.flatMap { response ->
                              when (response) {
                                  is CheckinStsPushPermissionDenied -> {
                                      Log.i(TAG, "CheckinStsPushPermissionDenied!")
                                      Observable.just(response)
                                  }
                                  is CheckinNotPermittedOnTablet -> {
                                      view.isClickable = true
                                      Log.i(TAG, "Checkin Not allowed on Tablet!")
                                      Observable.just(response)
                                  }
                                  is CheckinStsLocationPermissionDenied -> {
                                      Log.i(TAG, "CheckinStsLocationPermissionDenied!")
                                      Observable.just(response)
                                  }
                                  is OrderStatus -> {
                                      lastMileCheckinManager.getOrderState(customerId, mInFlightOrder)
                                  }
                              }
                          }
                          ?.doOnNext { response ->
                              when (response) {
                                  is OrderStatus -> {
                                      if (response.error) {
                                          view.isClickable = false
                                          startCheckinLocationsActivity(mContext,
                                                orderNumber,
                                                customerId,
                                                mInFlightOrder.accessPointInfos?.get(0)?.contactInformation?.contactPhoneNo,
                                                CHECK_IN_ERROR)
                                      }
                                  }
                              }
                          }
                          ?.startWith(OrderStatus(isLoading = true))
                          ?.subscribe({ response ->
                              updateUi(response)
                              if (response is OrderStatus && !response.isLoading) {
                                  logAnalyticsForCheckinSuccess(Authentication.getInstance().profileId
                                        ?: "unknown",
                                        mInFlightOrder.orderId ?: "unknown",
                                        AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                                              ?: "unknown",
                                        response.etaMillis.toString(),
                                        "unknown"
                                  )
                              }
                          }, { error ->
                              view.isClickable = true
                              if (Utils.isDebug()) {
                                  Log.e(TAG, "error with: " + error)
                              }
                              updateUi(OrderStatus(null, null, null, false, true))
                              checkinDisposable?.dispose()

                          }, {
                              view.isClickable = true
                              if (Utils.isDebug()) {
                                  Log.d(TAG, "completed!!")
                              }
                              checkinDisposable?.dispose()
                          })
                }
            } ?: run {
                if (Utils.isDebug()) {
                    Log.e(TAG, "mInFlightOrder is null")
                }
            }
        } else if (view.text.contains(mContext.getString(R.string.labelOrderCollected), true)) {
            val recoveryAppData = RecoveryAppHelper.RecoveryAppData(mContext, mInFlightOrder?.orderId, mInFlightOrder?.fulfillmentType);
            recoveryAppData.let {
                ViewUtils.openRecoverySurvey(mContext, it, LOCATION_NAME)
            }
            //todo: vmk ASDA-10525 needs clarfication as this is button tap instead of overlay event
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                  .putString(Anivia.BUTTON_NAME_KEY, view.text.toString())
                  .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                        ?: "unknown"))
        }
    }

    private fun gotoCart() {
        if (mContext is AsdaActivity) {
            val fragmentSwitcher = mContext.fragmentSwitcher
            fragmentSwitcher?.switchToFragment(FragmentConfig.TAG_CART, null)
        } else {
            (mContext as AsdaActivity).setResult(ActivityRequestCodes.RESULT_CODE_SHOW_CART)
            mContext.finish()
        }
    }

    override fun notifyChange() {
        super.notifyChange()
        if (!isLoading) {
            loadListener?.onComplete()
        }
    }

    interface LoadingListener {
        fun onComplete()
    }
}
