package com.asda.android.app.orders

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.os.Looper
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.data.RecoveryAppHelper
import com.asda.android.app.location.CHECK_IN_ERROR
import com.asda.android.app.location.CHECK_IN_LOCATION_ALLOW
import com.asda.android.app.location.CHECK_IN_LOCATION_TURN_ON
import com.asda.android.app.location.startCheckinLocationsActivity
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.model.createCheckinNotification
import com.asda.android.app.model.createLMActivity
import com.asda.android.app.model.logAnalyticsForCheckinSuccess
import com.asda.android.app.orders.ods.getOrderService
import com.asda.android.app.orders.ods.mapBFFODStoWismoOrder
import com.asda.android.app.orders.ods.mapOdsOrderToWismoOrder
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.ShelfListAdapter
import com.asda.android.app.view.WismoProgressView
import com.asda.android.lastmile.*
import com.asda.android.orders.OdsManager
import com.asda.android.recipes.RecipeUtils
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.restapi.service.data.recipes.RecipeCartAddResponse
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.OrderManager
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.lastmile.model.*
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import com.asda.android.utils.view.ViewUtil
import io.reactivex.Observable
import io.reactivex.SingleTransformer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers

private const val LOCATION_NAME: String = "Order Details"

class CheckinEligibleOrderDetailPresenter : OrderDetailsPresenter() {

    private var checkinManager: LastMileCheckinManager? = null
    private var dialog: Dialog? = null
    var amendOrderButton: Button? = null
    var editPaymentButton: Button? = null
    var cancelOrderButton: Button? = null
    var continueShoppingButton: Button? = null
    var stopAmendingButton: Button? = null
    var feedackButton: Button? = null
    var feedackText: TextView? = null
    var lastMileCheckinButton: Button? = null
    var orderStatusTextView: TextView? = null
    var loadingView: View? = null
    var title: TextView? = null
    var deliveryDetailsContainer: View? = null


    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (SharedPreferencesUtil.isUsgrCheckinEnabled(mContext)) {
            checkinManager = USGRLastMileCheckinManager.getInstance(AsdaApplication.getInstance()!!)
        } else {
            if (SharedPreferencesUtil.isLocalMockCheckinEnabled(mContext)) {
                checkinManager = MockLastMileCheckinManager.getInstance(OrderManager.get().orders)
            } else {
                checkinManager = ASDALastMileCheckinManager.getInstance(AsdaApplication.getInstance()!!)
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        val rootView = super.onCreateView(inflater, fragmentContainer, savedInstanceState)
        amendOrderButton = mHeaderView.findViewById(R.id.amend_order)
        editPaymentButton = mHeaderView.findViewById(R.id.edit_payment_button)
        cancelOrderButton = mHeaderView.findViewById(R.id.cancel_order)
        continueShoppingButton = mHeaderView.findViewById(R.id.continue_shopping_button)
        stopAmendingButton = mHeaderView.findViewById(R.id.stop_amending)
        feedackButton = mHeaderView.findViewById(R.id.feedback_button)
        feedackText = mHeaderView.findViewById(R.id.feedback_text)
        lastMileCheckinButton = mHeaderView.findViewById(R.id.last_mile_checkin_button)
        setCheckinListener()
        orderStatusTextView = mHeaderView.findViewById(R.id.order_status)

        loadingView = mHeaderView.findViewById(R.id.taxonomy_loading_view)
        title = mHeaderView.findViewById(R.id.title)
        deliveryDetailsContainer = ViewUtil.findViewById(mHeaderView, R.id.delivery_details_container)
        return rootView
    }

    override fun loadData() {
        val customerId = Authentication.getInstance().profileId
        val order = OrderManager.get().getViewOrder(mOrderId)
        if (customerId != null && order != null) {
            mContainerView.findViewById<View>(R.id.loading_page).visibility = View.VISIBLE
            if(FeatureSettings.isBFFforODSEnabled(mContext)){
                OdsManager.Companion.instance?.getBffOrderDetails(mOrderId)
                        ?.map{ mapBFFODStoWismoOrder(it) }
                        ?.compose(applyUrlTransform())
                        ?.toObservable()
                        ?.subscribeOn(Schedulers.io())
                        ?.observeOn(AndroidSchedulers.mainThread())
                        ?.doOnNext{
                            if (mListView.adapter != null && mListView.adapter is ShelfListAdapter) {
                                (mListView.adapter as ShelfListAdapter).unsetFooterView()
                            }
                            handleResponse(it)
                        }
                        ?.flatMap {
                            checkinManager?.getOrderState(customerId, order)
                        }
                        ?.startWith(OrderStatus(isLoading = true))
                        ?.subscribe(getWismoObserver())
            }else {
                getOrderService()?.getOrderDetails(mOrderId)
                        ?.map {
                            mapOdsOrderToWismoOrder(it)
                        }?.compose(applyUrlTransform())
                        ?.toObservable()
                        ?.subscribeOn(Schedulers.io())
                        ?.observeOn(AndroidSchedulers.mainThread())
                        ?.doOnNext {
                            if (mListView.adapter != null && mListView.adapter is ShelfListAdapter) {
                                (mListView.adapter as ShelfListAdapter).unsetFooterView()
                            }
                            handleResponse(it)
                        }
                        ?.flatMap {
                            checkinManager?.getOrderState(customerId, order)
                        }
                        ?.startWith(OrderStatus(isLoading = true))
                        ?.subscribe(getWismoObserver())
            }
        }
    }

    private fun getWismoObserver() =
            object : ResourceObserver<CheckinEligibleOrderStatus>() {
                override fun onComplete() {
                    dispose()
                    if (Utils.isDebug()) {
                        Log.d(TAG, "completed!!")
                    }
                }

                override fun onNext(response: CheckinEligibleOrderStatus) {
                    when (response) {
                        is OrderStatus -> {
                            val orderStatus = OrderStatus(response.orderId,
                                    response.total,
                                    response.orderStatus,
                                    response.isLoading,
                                    response.error,
                                    OrderManager.get().getViewOrder(mOrderId)?.accessPointInfos?.get(0),
                                    response.store,
                                    response.etaHumanReadble,
                                    response.etaMillis)
                            Log.d(TAG, "CheckinOrder Status: $orderStatus")
                            updateUi(orderStatus)
                        }
                    }
                }

                override fun onError(error: Throwable) {
                    if (Utils.isDebug()) {
                        Log.e(TAG, "error with: " + error)
                    }
                    if (error is RxFailure) {
                        AsdaDialogHelper.displayErrorDialog(null, error.integer,
                                mContext, false,
                                { pop() },
                                "CheckinEligibleOrderDetailPresenter")
                    }
                }

            }

    private fun applyUrlTransform(): SingleTransformer<WismoOrderResponse, WismoOrderResponse> {
        return SingleTransformer { single ->
            single.flatMap { wismoOrderResponse ->
                SiteConfigDataSource.getSiteConfigResponse()
                        ?.map {
                            wismoOrderResponse.payload?.recipes?.forEach { recipe: RecipeCartAddResponse.Recipe ->
                                recipe.imageUrl = RecipeUtils.composeScene7ImageUrl(it.scene7Host, recipe.scene7AssetId)
                            }
                            wismoOrderResponse
                        }?.onErrorReturn { wismoOrderResponse }
            }
        }
    }

    override fun toggleHeaderLoading(loading: Boolean) {
        if (loading) {
            loadingView?.visibility = View.VISIBLE
        } else {
            loadingView?.visibility = View.GONE
        }
        title?.visibility = View.GONE

        deliveryDetailsContainer?.visibility = View.GONE

        toggleButtonVisibilities()
    }

    private fun toggleButtonVisibilities() {
        amendOrderButton?.visibility = View.GONE
        editPaymentButton?.visibility = View.GONE
        cancelOrderButton?.visibility = View.GONE
        continueShoppingButton?.visibility = View.GONE
        stopAmendingButton?.visibility = View.GONE
        feedackButton?.visibility = View.GONE
        feedackText?.visibility = View.GONE
        lastMileCheckinButton?.visibility = View.VISIBLE
    }

    fun setCheckinListener() {
        lastMileCheckinButton?.setOnClickListener { view ->
            val order = OrderManager.get().getViewOrder(mOrderId)
            var customerId = Authentication.getInstance().profileId
            if (SharedPreferencesUtil.isUsgrCheckinEnabled(mContext)) {
                customerId = SharedPreferencesUtil.USGR_CUSTOMER_ID
            }
            val notification = createCheckinNotification(mContext)
            if ((view as Button).text.contains(mContext.getString(R.string.collection_details), true)) {
                if (Utils.isDebug()) {
                    Toast.makeText(mContext, "COLLECTION DETAILS button Clicked!!", Toast.LENGTH_SHORT).show()
                }
                createLMActivity(mContext, mOrderId ?: "empty", customerId ?: "empty")
            } else if (view.text.contains(mContext.getString(R.string.checkin), true)) {
                order?.let {
                    customerId?.let {
                        val disposable = checkinManager?.checkin(order, customerId, notification)
                                ?.flatMap { response ->
                                    when (response) {
                                        is CheckinStsPushPermissionDenied -> {
                                            Log.i(TAG, "CheckinStsPushPermissionDenied!")
                                            Observable.just(response)
                                        }
                                        is CheckinNotPermittedOnTablet -> {
                                            view.isClickable = true
                                            Log.i(TAG, "Checkin Not allowed on Tablet!")
                                            Observable.just(response)
                                        }
                                        is CheckinStsLocationPermissionDenied -> {
                                            Log.i(TAG, "CheckinStsLocationPermissionDenied!")
                                            Observable.just(response)
                                        }
                                        is OrderStatus -> {
                                            customerId.let {
                                                checkinManager?.getOrderState(customerId, order)
                                            }
                                        }
                                    }
                                }
                                ?.doOnNext { response ->
                                    when (response) {
                                        is OrderStatus -> {
                                            if (response.error) {
                                                view.isClickable = false
                                                var contactPhoneNo: String? = null
                                                order.accessPointInfos?.let {
                                                    if (it.isNotEmpty()) {
                                                        contactPhoneNo = it[0].contactInformation?.contactPhoneNo
                                                    }
                                                }
                                                startCheckinLocationsActivity(mContext,
                                                      mOrderId,
                                                      customerId,
                                                      contactPhoneNo,
                                                      CHECK_IN_ERROR)
                                            } else {
                                                //ASDAA-10524
                                                logAnalyticsForCheckinSuccess(Authentication.getInstance().profileId
                                                      ?: "unknown",
                                                      mOrderId ?: "unknown",
                                                      AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                                                            ?: "unknown",
                                                      response.etaMillis.toString(),
                                                      "unknown"
                                                )
                                            }
                                        }
                                    }
                                }
                              ?.startWith(OrderStatus(isLoading = true))
                                ?.subscribe({ response ->
                                    updateUi(response)
                                }, { error ->
                                    view.isClickable = true
                                    if (Utils.isDebug()) {
                                        Log.e(TAG, "error with: " + error)
                                    }
                                    updateUi(OrderStatus(null, null, null, false, true))
                                    mCompositeDisposable.clear()

                                }, {
                                    view.isClickable = true
                                    if (Utils.isDebug()) {
                                        Log.d(TAG, "completed!!")
                                    }
                                    mCompositeDisposable.clear()
                                })
                        disposable?.let {
                            mCompositeDisposable.add(it)
                        }
                    }
                } ?: run {
                    if (Utils.isDebug()) {
                        Log.e(TAG, "mInFlightOrder is null")
                    }
                }
            } else if (view.text.contains(mContext.getString(R.string.labelOrderCollected), true)) {
                val recoveryAppData = RecoveryAppHelper.RecoveryAppData(mContext, order?.orderId, order?.fulfillmentType);
                recoveryAppData.let {
                    ViewUtils.openRecoverySurvey(mContext, it, LOCATION_NAME)
                }
                //todo: vmk ASDA-10525 needs clarfication as this is button tap instead of overlay event
                Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                        .putString(Anivia.BUTTON_NAME_KEY, view.text.toString())
                        .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                                ?: "unknown"))
            }
        }
    }

    private fun updateUi(response: CheckinEligibleOrderStatus?) {
        android.os.Handler(Looper.getMainLooper()).post {
            if (Utils.isDebug()) {
                Log.i(TAG, "ORDER STATE:: ************ ${response.toString()} *************** ${this}")
            }

            when (response) {
                is OrderStatus -> {
                    val os = response
                    if (os.isLoading) {
                        mContainerView.findViewById<View>(R.id.loading_page).visibility = View.VISIBLE
                    } else {
                        mContainerView.findViewById<View>(R.id.loading_page).visibility = View.GONE
                        if (os.error) {
                            orderStatusTextView?.text = mContext.getString(R.string.error_status)
                            orderStatusTextView?.setBackgroundColor(ContextCompat.getColor(mContext, R.color.asda_dark_red))
                            orderStatusTextView?.setTextColor(ContextCompat.getColor(mContext, R.color.white))
                            lastMileCheckinButton?.isClickable = false
                        } else {
                            orderStatusTextView?.text = os.orderStatus
                            createBtnLabel(os.orderStatus)
                            mapStatusToProgress(os)
                        }
                    }
                }
                is CheckinStsPushPermissionDenied -> Log.d(TAG, "CheckinStsPushPermissionDenied!")
                is CheckinNotPermittedOnTablet -> {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "Checkin Not allowed on Tablet!")
                    }
                    if (mContext is Activity) {
                        if (dialog == null) {
                            dialog = DialogFactory.createAlertDialog(mContext.getString(R.string.tablet_checkin_title), mContext.getString(R.string.tablet_checkin_content), object : DialogInterface.OnClickListener {
                                override fun onClick(p0: DialogInterface?, p1: Int) {
                                    //ASDAA-10529
                                    Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                                            .putString(Anivia.BUTTON_NAME_KEY, mContext.getString(R.string.ok))
                                            .putString(Anivia.NAME_KEY, LOCATION_NAME))
                                }
                            }, "", mContext, null)
                        }
                        //ASDAA-10529
                        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                                .putString(Anivia.OVERLAY_TITLE_KEY, mContext.getString(R.string.tablet_checkin_title))
                                .putString(Anivia.NAME_KEY, LOCATION_NAME)
                                .putString(Anivia.OVERLAY_COPY_KEY, mContext.getString(R.string.tablet_checkin_content)))
                        dialog?.show()
                    } else {
                        //only show the toast
                        Toast.makeText(mContext, mContext.getString(R.string.tablet_checkin_content), Toast.LENGTH_SHORT).show()
                    }
                }
                is CheckinStsLocationPermissionDenied -> {
                    if (Utils.isDebug()) {
                        Log.d(TAG, "CheckinStsLocationPermissionDenied!")
                    }
                    val isFirstTime = SharedPreferencesUtil.getCheckinLocatonFlag(mContext)
                    var fragmentType = CHECK_IN_LOCATION_ALLOW
                    if (!isFirstTime) {
                        fragmentType = CHECK_IN_LOCATION_TURN_ON
                    }
                    var customerId = Authentication.getInstance().profileId
                    startCheckinLocationsActivity(mContext, mOrderId, customerId
                            ?: "empty", type = fragmentType)
                }
            }
        }
    }

    private fun createBtnLabel(orderStatus: String?) {
        lastMileCheckinButton?.isClickable = true
        orderStatus?.let {
            when (it) {
                ViewOrderResponse.Order.ORDER_STATUS_LOC_PERM_DENIED -> mHeaderView.findViewById<Button>(R.id.last_mile_checkin_button).text = mContext.getString(R.string.checkin)

                ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        lastMileCheckinButton?.text = mContext.getString(R.string.checkin)
                    } else {
                        lastMileCheckinButton?.text = mContext.getString(R.string.labelOnYourWay)
                    }
                }
                ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        lastMileCheckinButton?.text = mContext.getString(R.string.checkin)
                    } else {
                        lastMileCheckinButton?.text = mContext.getString(R.string.labelYouAreHere)
                    }
                }
                ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> {
                    if (mContext.resources.getBoolean(R.bool.isTablet)) {
                        lastMileCheckinButton?.text = mContext.getString(R.string.checkin)
                    } else {
                        lastMileCheckinButton?.text = mContext.getString(R.string.labelOrderCollected)
                        orderStatusTextView?.setBackgroundColor(ContextCompat.getColor(mContext, R.color.asda_green))
                        orderStatusTextView?.setTextColor(ContextCompat.getColor(mContext, R.color.white))
                    }
                }
                else -> lastMileCheckinButton?.text = mContext.getString(R.string.checkin)
            }
        }
    }

    private fun mapStatusToProgress(status: OrderStatus?) {
        val progressView: WismoProgressView = mHeaderView.findViewById(R.id.order_detail_progress)
        progressView.setStartDotText(WismoProgressView.PLACED_TEXT)
        progressView.progress = 3
        status?.let {
            when (it.orderStatus) {
                ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> {
                    progressView.setStartDotText("")
                    progressView.setEndDotText("")
                    progressView.setHintText(ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY)
                    progressView.setHintTextVisibility(true)
                }
                ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> {
                    progressView.setStartDotText("")
                    progressView.setEndDotText("")
                    progressView.setHintText(ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE)
                    progressView.setHintTextVisibility(true)
                }
                ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> {
                    progressView.progress = 4
                    progressView.setStartDotText("")
                    progressView.setEndDotText("")
                    progressView.setHintText(ViewOrderResponse.Order.ORDER_STATUS_COLLECTED)
                    progressView.setHintTextVisibility(false)
                }
                else -> {
                    progressView.setStartDotText("")
                    progressView.setEndDotText("")
                    progressView.setHintText(mContext.getString(R.string.orderReadyForCollection))
                    progressView.setHintTextVisibility(true)
                }
            }
        }
    }
}
