package com.asda.android.app.orders

import android.view.View
import com.asda.android.R
import com.asda.android.app.orders.ods.getOrderService
import com.asda.android.app.orders.ods.mapBFFODStoWismoOrder
import com.asda.android.app.orders.ods.mapOdsOrderToWismoOrder
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.ShelfListAdapter
import com.asda.android.base.core.ErrorCodes
import com.asda.android.orders.OdsManager
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.restapi.service.data.recipes.RecipeCartAddResponse
import com.asda.android.recipes.RecipeUtils
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import io.reactivex.SingleTransformer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.DisposableSingleObserver
import io.reactivex.schedulers.Schedulers

class LegacyOrderDetailPresenter : OrderDetailsPresenter() {

    private val BFF_TIMEOUT = "ETIMEDOUT"
    private val TAG = "LegacyOrderDetailPresenter"

    override fun loadData() {
        mContainerView.findViewById<View>(R.id.loading_page).visibility = View.VISIBLE
        if (FeatureSettings.isBFFforODSEnabled(mContext)) {
            OdsManager.instance?.getBffOrderDetails(mOrderId)
                    ?.map { mapBFFODStoWismoOrder(it) }
                    ?.compose(applyUrlTransform())
                    ?.subscribeOn(Schedulers.io())
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribeWith(getDisposableObserver())
                    ?.let { mCompositeDisposable.add(it) }
        } else {
            getOrderService()?.getOrderDetails(mOrderId)
                    ?.map { mapOdsOrderToWismoOrder(it) }
                    ?.compose(applyUrlTransform())
                    ?.subscribeOn(Schedulers.io())
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribeWith(getDisposableObserver())
                    ?.let { mCompositeDisposable.add(it) }
        }
    }

    private fun getDisposableObserver() = object : DisposableSingleObserver<WismoOrderResponse>() {
        override fun onSuccess(wismoOrderResponse: WismoOrderResponse) {
            if (wismoOrderResponse.errorCode.equals(BFF_TIMEOUT)) {
                AsdaDialogHelper.displayErrorDialog(null, ErrorCodes.ERROR_CODE_JSON_ERROR, mContext, false, { pop() }, TAG)
                //TODO when we remove the loading screen, the order details screen appears with empty fields. This needs a fix
                //mContainerView.findViewById<View>(R.id.loading_page).visibility = View.GONE
            } else {
                if (mListView.adapter != null && mListView.adapter is ShelfListAdapter) {
                    (mListView.adapter as ShelfListAdapter).unsetFooterView()
                }
                handleResponse(wismoOrderResponse)
            }
        }

        override fun onError(e: Throwable) {
            if (e is RxFailure) {
                AsdaDialogHelper.displayErrorDialog(null, e.asdaResponse?.statusCodeNumber
                        ?: e.integer, mContext, false, { pop() }, TAG)
                //TODO when we remove the loading screen, the order details screen appears with empty fields. This needs a fix
                //mContainerView.findViewById<View>(R.id.loading_page).visibility = View.GONE
            } else {
                //TODO handle a mapping error by providing an appropriate response to the screen.
            }
        }
    }

    private fun applyUrlTransform(): SingleTransformer<WismoOrderResponse, WismoOrderResponse> {
        return SingleTransformer { single ->
            single.flatMap { wismoOrderResponse ->
                SiteConfigDataSource.getSiteConfigResponse()
                        ?.map {
                            wismoOrderResponse.payload?.recipes?.forEach { recipe: RecipeCartAddResponse.Recipe ->
                                recipe.imageUrl = RecipeUtils.composeScene7ImageUrl(it.scene7Host,
                                        recipe.scene7AssetId)
                            }
                            wismoOrderResponse
                        }?.onErrorReturn { wismoOrderResponse }
            }
        }
    }
}

