package com.asda.android.app.recurringslot

import android.app.Application
import android.view.View
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import androidx.lifecycle.AndroidViewModel
import com.asda.android.R
import com.asda.android.app.bookslotv3.subscription.SubscriptionAdapter
import com.asda.android.app.bookslotv3.subscription.SubscriptionAdapterListener
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.restapi.service.data.recurringslots.SlotSubscription
import com.asda.android.restapi.service.data.recurringslots.SubscriptionInfo
import com.asda.android.restapi.service.data.recurringslots.SubscriptionsResponse
import com.asda.android.utils.Utils
class RecurringSlotAccountViewModel(application: Application) : AndroidViewModel(application) {
    private val tag = "RecurSlotAccVM"
    val mFrequency = ObservableField("")
    val showPeakSeasonMessage = ObservableBoolean(false)
    val showPeakSeasonLink = ObservableBoolean(false)
    val peakSeasonMessage = ObservableField("")
    var peakSeasonFaqUrl = ""

    val mIsLoading = ObservableBoolean(true)
    private var mFragment: RecurringSlotDetailsFragment? = null
    var id: String? = null
    var subscriptionInfo: SubscriptionInfo? = null
    var subscriptionAdapter: SubscriptionAdapter? = null

    fun attach(fragment: RecurringSlotDetailsFragment) {
        mFragment = fragment
    }

    fun setData(subscriptionsResponse: SubscriptionsResponse?): Boolean {
        subscriptionsResponse?.data?.subscriptions?.let {
            if (it.isEmpty()) {
                return false
            }
            subscriptionInfo = it[0].subscription_info
            val context = AsdaApplication.getInstance()
            id = subscriptionInfo?.subscription_id

            setRecyclerAdapter(it)

            mFrequency.set(context?.getString(R.string.rs_frequency_header_label, Utils.getFrequencyLabel(context, subscriptionInfo?.frequency)))


            context?.let { _context ->
                SlotUtils().getPeakSeasonConfig(_context)?.let { _config ->
                    _config.message?.let { _message ->
                        if (_message.isNotEmpty() && _message.isNotBlank()) {
                            showPeakSeasonMessage.set(true)
                            peakSeasonMessage.set(_message)

                            _config.faqUrl?.let { _faqUrl ->
                                if (_faqUrl.isNotEmpty() && _faqUrl.isNotBlank()) {
                                    showPeakSeasonLink.set(true)
                                    peakSeasonFaqUrl = _faqUrl
                                }
                            }
                        }
                    }
                }
            }
        }

        mIsLoading.set(false)
        return true
    }

    private fun setRecyclerAdapter(subscriptions: List<SlotSubscription>?) {
        mFragment?.context?.let { _context ->
            subscriptions?.let { _subscriptions ->
                if (_subscriptions.isNotEmpty() && _subscriptions[0].subscription_info?.subscription_id?.isNotEmpty() == true
                      && _subscriptions[0].subscription_info?.subscription_bookings?.isNotEmpty() == true) {
                    subscriptionAdapter = SubscriptionAdapter(_context, _subscriptions[0].subscription_info, _subscriptions[0].subscription_info?.subscription_bookings,
                          object : SubscriptionAdapterListener {
                              override fun onSubscriptionSkip(subscriptionId: String, skipId: String, bookingStartDate: String, bookingEndDate: String) {
                                  mFragment?.onClickSkip(subscriptionId, skipId, _context.getString(R.string.rs_item_delete_date, bookingStartDate, bookingEndDate))
                              }
                          })
                    mFragment?.setAdapter(subscriptionAdapter)
                }
            }

        }
    }

    fun onClickCancel(view: View) {
        mFragment?.onClickCancel()
    }

    fun onClickFAQ(view: View) {
        mFragment?.onClickFAQ()
    }

    fun onClickLearnMore(view: View){
        mFragment?.onClickLearnMore(peakSeasonFaqUrl)
    }
}
