package com.asda.android.app.recurringslot

import com.asda.android.restapi.utils.SimpleCalendar
import com.asda.android.utils.ISO8601TimeStampDecoder

class RecurringSlotParsingOutput(private val interval: String = "") {
    var startTs: Long = 0
        private set
    var endTs: Long = 0
        private set

    @Throws(Exception::class)
    fun parse(): RecurringSlotParsingOutput {
        val startStr: String
        val endStr: String
        val intervalDelimiterIndex = interval.indexOf('/')
        if (intervalDelimiterIndex < 0) {
            startStr = ""
            endStr = interval
        } else {
            startStr = interval.substring(0, intervalDelimiterIndex)
            endStr = interval.substring(intervalDelimiterIndex + 1, interval.length)
        }
        var startCal: SimpleCalendar? = null
        if (startStr.isNotEmpty()) {
            startCal = ISO8601TimeStampDecoder.decode(startStr)
            startCal?.let {
                startTs = it.time
            }
        }
        var endCal: SimpleCalendar?
        var endCalNew: SimpleCalendar? = null
        if (endStr.isNotEmpty()) {
            endCal = ISO8601TimeStampDecoder.decode(endStr)
            if (startCal != null) {
                endCal?.let { endCal ->
                    endCalNew = SimpleCalendar(
                          endCal.year ?: startCal.year,
                          endCal.month ?: startCal.month,
                          endCal.day ?: startCal.day,
                          endCal.hours ?: startCal.hours,
                          endCal.minutes ?: startCal.minutes,
                          endCal.seconds ?: startCal.seconds,
                          endCal.tzHours ?: startCal.tzHours,
                          endCal.tzMinutes ?: startCal.tzMinutes,
                          endCal.zoneOffsetPositive ?: startCal.zoneOffsetPositive)
                }
            }
        }
        startCal?.let {
            startTs = it.time
        }
        endCalNew?.let {
            endTs = it.time
        }
        return this
    }
}
