package com.asda.android.app.recurringslot

import com.asda.android.restapi.service.data.recurringslots.SubscriptionCreation
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityResponse
import com.asda.android.restapi.service.data.recurringslots.SubscriptionFreqOptionsResponse
import com.asda.android.restapi.service.data.recurringslots.SubscriptionsResponse
import io.reactivex.Single
import retrofit2.http.*

interface SlotSubscriptionAPI {

    @GET("slot/subscription")
    fun getSubscriptions(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                         @Query("include_all") includeAll: Boolean = true): Single<SubscriptionsResponse>

    @GET("slot/subscription-eligibility")
    fun getSubscriptionEligibility(@Header("WM_CONSUMER.USER_ID") singleProfileId: String): Single<SubscriptionEligibilityResponse>

    @GET("slot/subscription/options")
    fun getSubscriptionFrequencyOptions(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                                        @Query("slot_date") slotDate: String): Single<SubscriptionFreqOptionsResponse>

    @POST("slot/subscription")
    fun createSubscription(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                           @Body subscriptionRequest: SubscriptionCreation): Single<SubscriptionCreation>

    @POST("recslot/subscriptions")
    fun createSubscriptionForATGUsers(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                                      @Body subscriptionRequest: SubscriptionCreation): Single<SubscriptionCreation>

    @GET("recslot/subscriptions")
    fun getSubscriptionForATGUsers(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                                   @Query("include_all") includeAll: Boolean = true): Single<SubscriptionsResponse>

    @PUT("slot/subscription/booking/{booking_id}")
    fun skipSubscription(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                         @Path("booking_id") booking_id: String): Single<SubscriptionCreation>

    @PUT("slot/subscription/{booking_id}")
    fun cancelSubscription(@Header("WM_CONSUMER.USER_ID") singleProfileId: String,
                           @Path("booking_id") booking_id: String,
                           @Body subscriptionInfo: SubscriptionCreation): Single<SubscriptionCreation>
}
