package com.asda.android.app.recurringslot

import androidx.lifecycle.MediatorLiveData
import com.asda.android.restapi.service.data.UserViewResponse
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityResponse
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers

object SlotSubscriptionManager {

    val subscriptionEligibilityChanged = MediatorLiveData<SubscriptionEligibilityResponse?>()
    val recurringSlotEligibilityChanged = MediatorLiveData<Boolean?>()

    fun refreshSubscriptionEligibility() {
        SingleProfile.getProfileResponse()?.profileId?.let {
            SlotSubscriptionRepository().getSubscriptionEligibility(it)?.subscribe { subscriptionEligibilityResponse: SubscriptionEligibilityResponse?, _: Throwable? ->
                subscriptionEligibilityChanged.postValue(subscriptionEligibilityResponse)
            }
        }
    }

    fun refreshSubscriptionEligibilityForATGUsers() {
        AsdaServiceFactory.get().initSessionSingleProfile(true)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
              ?.subscribe(object : ResourceObserver<UserViewResponse>() {
                  override fun onComplete() {
                      dispose()
                  }

                  override fun onNext(response: UserViewResponse) {
                      Authentication.getInstance()?.userViewResponse = response
                      recurringSlotEligibilityChanged.postValue(response.hasActiveRecSlot)
                      dispose()
                  }

                  override fun onError(e: Throwable) {
                      dispose()
                  }
              })
    }
}

