package com.asda.android.app.recurringslot

import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.recurringslots.*
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers

class SlotSubscriptionRepository {

    fun getSubscriptionEligibility(singleProfileId: String): Single<SubscriptionEligibilityResponse>? {
        return getService()?.getSubscriptionEligibility(singleProfileId)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun getSubscriptions(singleProfileId: String, includeAll: Boolean): Single<SubscriptionsResponse>? {
        return getService()?.getSubscriptions(singleProfileId, includeAll)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun getSubscriptionFrequencyOption(singleProfileId: String, slotDate: String): Single<SubscriptionFreqOptionsResponse>? {
        return getService()?.getSubscriptionFrequencyOptions(singleProfileId, slotDate)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun createSubscription(singleProfileId: String, subscriptionRequest: SubscriptionCreation): Single<SubscriptionCreation>? {
        return getService()?.createSubscription(singleProfileId, subscriptionRequest)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun createSubscriptionForATG(singleProfileId: String, subscriptionRequest: SubscriptionCreation): Single<SubscriptionCreation>? {
        return getService()?.createSubscriptionForATGUsers(singleProfileId, subscriptionRequest)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun getSubscriptionForATGUsers(singleProfileId: String, includeAll: Boolean): Single<SubscriptionsResponse>? {
        return getService()?.getSubscriptionForATGUsers(singleProfileId, includeAll)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun skipSubscription(singleProfileId: String, bookingId: String): Single<SubscriptionCreation>? {
        return getService()?.skipSubscription(singleProfileId, bookingId)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun cancelSubscription(singleProfileId: String, bookingId: String, subscription: SubscriptionCreation): Single<SubscriptionCreation>? {
        return getService()?.cancelSubscription(singleProfileId, bookingId, subscription)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun getService(): SlotSubscriptionAPI? {
        return ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "",
              SlotSubscriptionAPI::class.java)
    }

}
