package com.asda.android.app.recurringslot.module.view

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.AdapterView.OnItemSelectedListener
import android.widget.Button
import android.widget.Spinner
import android.widget.TextView
import androidx.annotation.VisibleForTesting
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.recurringslot.module.view.adapters.DateSpinnerAdapter
import com.asda.android.app.recurringslot.module.view.adapters.DaySpinnerAdapter
import com.asda.android.app.recurringslot.module.view.adapters.SuggestedSlotAdapter
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import com.asda.android.restapi.service.data.SiteConfigResponse
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.service.base.CartManager
import io.reactivex.observers.ResourceSingleObserver


class SlotSuggestionFragment : FeaturedFragment() {

    private var activity: Activity? = null
    private var slotSuggestedInfo: SlotSuggestedInfo? = null
    private var spanId = ""
    private var startDate = ""
    private var endDate = ""
    private var label = ""
    private var currentView: View? = null
    var suggestionSelectListener: SuggestionSelectListener? = null

    companion object {
        const val TAG = "SlotSuggestionFragment"
        const val ERROR_SLOT_RECURRING_SLOTS_UNAVAILABLE = "SLOT_RECURRING_SLOTS_UNAVAILABLE"

        private const val EXTRA_SLOT_SUGGESTED_INFO = "extra_slot_suggested_info"
        private const val EXTRA_SLOT_SPAN_ID = "extra_slot_span_id"
        private const val EXTRA_LABEL = "extra_label"
        private const val EXTRA_START_DATE = "extra_start_date"
        private const val EXTRA_END_DATE = "extra_end_date"

        fun newInstance(startDate: String?, endDate: String?, spanId: String?, label: String?, slotSuggestedInfo: SlotSuggestedInfo?): SlotSuggestionFragment {
            val fragment = SlotSuggestionFragment()
            val args = Bundle()
            args.putParcelable(EXTRA_SLOT_SUGGESTED_INFO, slotSuggestedInfo)
            args.putString(EXTRA_SLOT_SPAN_ID, spanId)
            args.putString(EXTRA_LABEL, label)
            args.putString(EXTRA_START_DATE, startDate)
            args.putString(EXTRA_END_DATE, endDate)
            fragment.arguments = args
            return fragment
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        activity?.let { _context ->
            currentView?.let { currentView ->

                SiteConfigDataSource.getSiteConfigResponse()?.subscribe(object : ResourceSingleObserver<SiteConfigResponse>() {
                    override fun onSuccess(configResponse: SiteConfigResponse) {
                        dispose()
                        val containerOptionOne = currentView.findViewById<View>(R.id.container_option_1)
                        val containerOptionTwo = currentView.findViewById<View>(R.id.container_option_2)
                        if (configResponse.slotsSuggestionOptions?.isRecurringListFlowEnabled == true) {
                            containerOptionTwo?.visibility = View.VISIBLE
                            containerOptionOne?.visibility = View.GONE
                            createListSuggestedSlots(_context, currentView, slotSuggestedInfo, spanId)
                        } else {
                            containerOptionOne?.visibility = View.VISIBLE
                            containerOptionTwo?.visibility = View.GONE
                            createDateSelectSpinner(_context, currentView, slotSuggestedInfo, spanId)
                        }

                        val btnCancel = currentView.findViewById<Button>(R.id.btn_cancel)
                        btnCancel?.setOnClickListener {
                            pop()
                        }

                        val unavailableText = currentView.findViewById<TextView>(R.id.recurring_slot_unavailable)
                        val dayAndStartHour = startDate?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_H_MM_AA)
                        val endHour = endDate?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA)
                        unavailableText?.text = _context.getString(R.string.rs_details_unavailable, dayAndStartHour, endHour)
                    }

                    override fun onError(e: Throwable) {
                        dispose()
                        pop()
                    }
                }) ?: run {
                    pop()
                }
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        currentView = inflater.inflate(R.layout.fragment_slot_suggestion, fragmentContainer, false)
        return currentView!!
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    fun createListSuggestedSlots(context: Context, currentView: View, data: SlotSuggestedInfo?, spanId: String) {
        val recyclerView = currentView?.findViewById<RecyclerView>(R.id.slots_recycler_view)
        val btnBookSlot = currentView?.findViewById<Button>(R.id.btn_book_slot)

        val slotList = ArrayList<SubscriptionCreation>()

        data?.let {
            it.slotDays?.forEach { _suggestedDay ->
                _suggestedDay.slotDays?.forEach {
                    slotList.add(getSubscriptionCreationRequest(spanId, _suggestedDay.slot_date, it))
                }
            }
        }
        val suggestedAdapter = SuggestedSlotAdapter(context, slotList)

        recyclerView?.apply {
            adapter = suggestedAdapter
            layoutManager = LinearLayoutManager(context)
        }

        btnBookSlot?.setOnClickListener {
            suggestedAdapter.getSelectedItem()?.let {
                suggestionSelectListener?.onSuggestionSelected(it, label)
                pop()
            }
        }
    }

    fun createDateSelectSpinner(context: Context, currentView: View, data: SlotSuggestedInfo?, spanId: String) {
        data?.let { slotSuggestedInfo ->
            slotSuggestedInfo.slotDays?.let {
                val dateSpinner = currentView.findViewById<Spinner>(R.id.start_date_dropdown)
                val dateSpinnerAdapter = DateSpinnerAdapter(context, R.layout.address_simple_spinner_item, it)
                dateSpinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
                dateSpinner.setSelection(0)
                dateSpinner?.adapter = dateSpinnerAdapter
                dateSpinner?.setOnItemSelectedListener(object : OnItemSelectedListener {

                    override fun onItemSelected(adapterView: AdapterView<*>, view: View,
                                                position: Int, id: Long) {

                        val dateSlotInfo = dateSpinnerAdapter.getItem(position)
                        createDaySelectSpinner(context, currentView, dateSlotInfo, spanId)
                    }

                    override fun onNothingSelected(adapter: AdapterView<*>) {}
                })
            }
        }
    }

    fun createDaySelectSpinner(context: Context, currentView: View, data: SlotSuggestedDay?, spanId: String) {
        data?.let { suggestedDateData ->
            val startTimeHint = currentView.findViewById<TextView>(R.id.start_time_hint)
            val startTimeSpinner = currentView.findViewById<Spinner>(R.id.start_time_dropdown)
            val missingDateView = currentView.findViewById<TextView>(R.id.missing_date_hint)
            val btnBookSlot = currentView.findViewById<Button>(R.id.btn_book_slot)
            var timeSelected: SlotsSuggested? = null

            btnBookSlot?.setOnClickListener {
                timeSelected?.let { _time ->
                    val request = getSubscriptionCreationRequest(spanId, data.slot_date, _time)
                    suggestionSelectListener?.onSuggestionSelected(request, label)
                    pop()
                }
            }

            startTimeHint?.visibility = View.VISIBLE
            startTimeSpinner?.visibility = View.VISIBLE
            missingDateView?.visibility = View.GONE


            suggestedDateData.slotDays?.let {
                val daySpinnerAdapter = DaySpinnerAdapter(context, android.R.layout.simple_spinner_item, it)
                daySpinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
                startTimeSpinner?.setSelection(0)
                startTimeSpinner?.adapter = daySpinnerAdapter

                startTimeSpinner?.setOnItemSelectedListener(object : OnItemSelectedListener {

                    override fun onItemSelected(adapterView: AdapterView<*>, view: View,
                                                position: Int, id: Long) {
                        timeSelected = daySpinnerAdapter.getItem(position)
                    }

                    override fun onNothingSelected(adapter: AdapterView<*>) {}
                })
            }
        }
    }

    //TODO Move this to formatters.
    @VisibleForTesting
    fun getSubscriptionCreationRequest(spanId: String, startDate: String?, slotsSuggested: SlotsSuggested): SubscriptionCreation {
        val subscriptionCreation = SubscriptionCreation()
        subscriptionCreation.data = SubscriptionCreationData()

        var serviceAddress: ServiceAddress? = null

        activity?.let { _context ->
            if (FeatureSettings.isCXOMigrationEnabled(_context)) {
                val cxoCart = com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart
                serviceAddress = ServiceAddress(
                      postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
                      latitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.latitude,
                      longitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.longitude,
                      address_1 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address,
                      address_2 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address2,
                      city = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.city,
                      country = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.country
                )

                subscriptionCreation.data?.subscription_info = SubscriptionInfo(
                      frequency = spanId,
                      group = "PT2H",
                      start_date = startDate,
                      end_date = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
                      start_time = cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
                      end_time = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
                      postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
                      fulfillment_type = cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType,
                      reservation_id = cxoCart?.fulfillmentDetails?.slotInfo?.reservationId,
                      service_address = serviceAddress,
                      suggested_start_time = slotsSuggested.slotInfo?.start_time,
                      suggested_end_time = slotsSuggested.slotInfo?.end_time,
                      cart_id = cxoCart?.cartMetadata?.cartId
                )

            } else {
                val cart = CartManager.get(_context)?.cart

                subscriptionCreation.data?.subscription_info = SubscriptionInfo(
                      frequency = spanId,
                      group = "PT2H",
                      start_time = cart?.deliveryStartTime,
                      end_time = cart?.deliveryEndTime,
                      suggested_start_time = slotsSuggested.slotInfo?.start_time,
                      suggested_end_time = slotsSuggested.slotInfo?.end_time
                )
            }
        }

        return subscriptionCreation
    }


    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return activity?.getString(R.string.rs_title)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        activity = context as Activity
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        bundle?.let {
            slotSuggestedInfo = it.getParcelable(EXTRA_SLOT_SUGGESTED_INFO)
            spanId = it.getString(EXTRA_SLOT_SPAN_ID, "")
            label = it.getString(EXTRA_LABEL, "")
            startDate = it.getString(EXTRA_START_DATE, "")
            endDate = it.getString(EXTRA_END_DATE, "")
        }
    }

    interface SuggestionSelectListener {
        fun onSuggestionSelected(subscriptionCreation: SubscriptionCreation, label: String)
    }

}
