package com.asda.android.app.recurringslot.module.view.adapters

import android.content.Context
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.restapi.service.data.recurringslots.SubscriptionCreation


class SuggestedSlotAdapter(private val context: Context, private val slotList: ArrayList<SubscriptionCreation>) : RecyclerView.Adapter<SuggestedSlotAdapter.ViewHolder>() {
    private var lastCheckedPosition = -1

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = View.inflate(parent.context, R.layout.single_radio_item, null)
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.radioButton?.isChecked = position == lastCheckedPosition
        holder.radioButton?.text = context.getString(R.string.rs_suggested_date_time,
              slotList[position].data?.subscription_info?.suggested_start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM),
              slotList[position].data?.subscription_info?.suggested_start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
              slotList[position].data?.subscription_info?.suggested_end_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))
        holder.bind()
    }

    override fun getItemCount(): Int {
        return slotList.size
    }

    fun getSelectedItem(): SubscriptionCreation? {
        if (lastCheckedPosition >= 0 && lastCheckedPosition <= slotList.size) {
            return slotList[lastCheckedPosition]
        }
        return null
    }

    inner class ViewHolder : RecyclerView.ViewHolder {
        internal var radioButton: RadioButton? = null

        constructor(view: View) : super(view) {
            radioButton = view.findViewById(R.id.single_radio)
        }

        fun bind() {
            radioButton?.setOnClickListener {
                val copyOfLastCheckedPosition = lastCheckedPosition
                lastCheckedPosition = adapterPosition
                notifyItemChanged(copyOfLastCheckedPosition)
                notifyItemChanged(lastCheckedPosition)
            }
        }
    }
}
