package com.asda.android.app.settings.featureflag.view

import android.content.Context
import android.content.res.TypedArray
import android.os.Build
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.LinearLayout
import androidx.annotation.RequiresApi
import androidx.annotation.VisibleForTesting
import com.asda.android.R
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils
import kotlinx.android.synthetic.main.settings_feature_flag.view.*


class FeatureFlagView : LinearLayout {
    private val TAG = "FeatureFlagView"
    private val KEY = "key"
    private val VALUE = "value"

    private val DELIMITER = ";"
    private val FEATURE_NAME_DEFAULT = "Feature"
    private val FEATURE_FLAGS_DEFAULT = "Force On"+ DELIMITER + "Force Off" + DELIMITER + "Honor Config"
    private val FEATURE_SELECTION_DEFAULT = 0

    var featureFlagsAdapter:ArrayAdapter<String> = ArrayAdapter(context, android.R.layout.simple_spinner_item)

    var featureName:String = FEATURE_NAME_DEFAULT
    var featureFlags:String = FEATURE_FLAGS_DEFAULT
    var selectedIndex:Int = FEATURE_SELECTION_DEFAULT

    constructor(context: Context) : super(context) {
        initialize(context, null)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        initialize(context, attrs)
    }

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        initialize(context, attrs)
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int, defStyleRes: Int) : super(context, attrs, defStyleAttr, defStyleRes) {
        initialize(context, attrs)
    }

    /**
     * This method is called from all constructors. The attribute values are extracted in this
     * method and further used to either inflate the layout file and set the data/image accordingly.
     */
    private fun initialize(context: Context, attrs: AttributeSet?) {
        if (attrs != null) {
            inflateFeatureFlagView(getInflater(context))
            feature_flags.onItemSelectedListener = onItemClickListener
            extractAttributes(context, attrs)
            populateFeature()
            selectSavedFeaturePreference()
        }
    }

    private val onItemClickListener:AdapterView.OnItemSelectedListener = object: AdapterView.OnItemSelectedListener {
        override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
            //Toast.makeText(context, featureFlagsAdapter.getItem(position).getKey(), Toast.LENGTH_SHORT).show()
            saveFeaturePreference()
        }

        override fun onNothingSelected(p0: AdapterView<*>?) {
            // do nothing
        }
    }

    private fun getInflater(context: Context): LayoutInflater {
        return context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
    }

    private fun inflateFeatureFlagView(inflater: LayoutInflater) {
        inflater.inflate(R.layout.settings_feature_flag, this, true)

        // Initialize the spinner
        featureFlagsAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        feature_flags.setAdapter(featureFlagsAdapter)
    }

    private fun extractAttributes(context: Context, attrs: AttributeSet?) {
        var typedArray:TypedArray? = null
        try {
            typedArray = context.obtainStyledAttributes(attrs, R.styleable.FeatureFlagView)
            typedArray?.let {
                featureName = it.getString(R.styleable.FeatureFlagView_featureName)
                      ?: FEATURE_NAME_DEFAULT
                featureFlags = it.getString(R.styleable.FeatureFlagView_featureFlags)
                      ?: FEATURE_FLAGS_DEFAULT
                selectedIndex = it.getInt(R.styleable.FeatureFlagView_selectedIndex, FEATURE_SELECTION_DEFAULT)
            }
        } catch (ex:Exception) {
            if (Utils.isDebug()) {
                Log.e(TAG, ex.stackTrace.toString())
            }
        } finally {
            typedArray?.recycle()
        }
    }

    @VisibleForTesting
    fun populateFeature() {
        feature_name.text = featureName
        val flags = getFlagList(featureFlags)
        featureFlagsAdapter.clear()
        featureFlagsAdapter.addAll(flags)
        featureFlagsAdapter.notifyDataSetChanged()
    }

    @VisibleForTesting
    fun getFlagList(flagsString:String?):List<String> {
        if (!flagsString.isNullOrEmpty()) {
            return flagsString.split(DELIMITER)
        }
        return mutableListOf()
    }

    @VisibleForTesting
    fun getPositionToSelect(flagsCount:Int):Int {
        // Make sure the adapter is populated, if not just return -1
        if (flagsCount < 1) {
            return -1
        }

        // Get Saved position
        var currentPos = SharedPreferencesUtil.getSetting(context, featureName, selectedIndex)

        // Make sure the currentPos is selectable index, if not return zeroeth
        if (currentPos < 0 || currentPos >= flagsCount) {
            currentPos = 0
        }
        return currentPos
    }

    private fun selectSavedFeaturePreference() {
        val currentPos = getPositionToSelect(featureFlagsAdapter.count)

        // The method `getPositionToSelect()` will make sure we get the selectable index, if not it'll be -1
        if (currentPos != -1) {
            feature_flags.setSelection(currentPos)
        }
    }

    private fun saveFeaturePreference() {
        SharedPreferencesUtil.setSetting(context, featureName, feature_flags.selectedItemPosition)
    }
}
