package com.asda.android.app.shop


import android.content.Context
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.ImageView
import android.widget.RadioButton
import android.widget.TextView
import androidx.annotation.VisibleForTesting
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.app.shop.CustomFilterAdapter.ViewHolder
import com.asda.android.app.shop.DynamicFilterAdapter.*
import com.asda.android.base.core.view.adapter.HeaderRecyclerViewAdapter
import com.asda.android.utils.view.ViewUtil
import java.util.*

class CustomFilterAdapter(private val context: Context, private val items: ArrayList<AsdaFilter>) : androidx.recyclerview.widget.RecyclerView.Adapter<ViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view: View
        val viewHolder: ViewHolder

        when (viewType) {
            VIEW_TYPE_FILTER_ITEM -> {
                view = inflateView(context, R.layout.shelf_filter_list_dynamic, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.checkBox?.setOnClickListener { _ ->

                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val filter = items[position]
                        val isChecked: Boolean? = viewHolder.checkBox?.isChecked
                        isChecked?.let {
                            if (it) {
                                ShopFilters.instance.addSelectedNutritionFilter(filter)
                            }
                        }
                    }
                }
            }

            VIEW_TYPE_FILTER_ITEM_NUTRITION -> {
                view = inflateView(context, R.layout.shelf_filter_list_dynamic, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.checkBox?.setOnClickListener { _ ->
                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val selectedFilter = items[position]
                        if (getSelectedNutritionFilters().contains(selectedFilter)) {
                            getSelectedNutritionFilters().remove(selectedFilter)
                        } else {
                            getSelectedNutritionFilters().add(selectedFilter)
                        }
                    }
                }
            }

            VIEW_TYPE_NUTRITION_HEADER -> {
                view = inflateView(context, R.layout.shelf_filter_nutrition_header, parent, false)
                viewHolder = ViewHolder(view, viewType)
            }

            VIEW_TYPE_UNAVAILABLE_FILTER_NOTE -> {
                view = inflateView(context, R.layout.shelf_filter_custom_view, parent, false)
                viewHolder = ViewHolder(view, viewType)
            }

            VIEW_TYPE_USER_FILTER -> {
                view = inflateView(context, R.layout.custom_user_filter, parent, false)
                viewHolder = ViewHolder(view, viewType)
            }
            else -> {
                view = inflateView(context, R.layout.shelf_filter_list_dynamic, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.checkBox?.setOnClickListener { _ ->
                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val filter = items[position]
                        val isChecked: Boolean? = viewHolder.checkBox?.isChecked
                        isChecked?.let {
                            if (it) {
                                ShopFilters.instance.addSelectedNutritionFilter(filter)
                            }
                        }
                    }
                }
            }
        }

        return viewHolder
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val filter = items[position]
        if (filter.type == VIEW_TYPE_FILTER_ITEM_NUTRITION) {
            holder.checkBox?.tag = filter
            if (!filter.isActive) {
                holder.checkBox?.isChecked = false
                holder.checkBox?.isEnabled = false
                holder.textView?.setTextColor(ContextCompat.getColor(context, R.color.grey_53))
                holder.textView?.alpha = 0.4F
                holder.textView?.text = ShopFilters.instance.nutritionMap[items[position].name]?.second
            } else {
                holder.checkBox?.isEnabled = true
                holder.checkBox?.isChecked = ShopFilters.instance.selectedNutritionFilters.contains(items[position])
                holder.textView?.setBackgroundColor(ContextCompat.getColor(context, R.color.white))
                holder.textView?.alpha = 1.0F
                holder.textView?.text = ShopFilters.instance.nutritionMap[items[position].name]?.second
            }
        } else if (filter.type == VIEW_TYPE_USER_FILTER) {
            holder.textView?.text = filter.name
        }
    }

    fun clearAll() {
        ShopFilters.instance.selectedNutritionFilters.clear()
        notifyDataSetChanged()
    }

    fun getSelectedNutritionFilters(): ArrayList<AsdaFilter> {
        return ShopFilters.instance.selectedNutritionFilters
    }

    override fun getItemCount(): Int {
        return items.size
    }

    override fun getItemViewType(position: Int): Int {
        return items[position].type
    }

    @VisibleForTesting
    fun inflateView(context:Context, resId:Int, parent:ViewGroup, attachToRoot:Boolean):View{
        return ViewUtil.inflate(context, resId, parent, attachToRoot)
    }

    inner class ViewHolder(itemView: View, viewType: Int) : HeaderRecyclerViewAdapter.BaseViewHolder(itemView, viewType) {
        var textView: TextView? = null
        var checkBox: CheckBox? = null
        var radioButton: RadioButton? = null
        var imageView: ImageView? = null
        var cardView: androidx.cardview.widget.CardView? = null

        init {
            when (viewType) {
                VIEW_TYPE_FILTER_ITEM_NUTRITION -> {
                    textView = ViewUtil.findViewById(itemView, R.id.text)
                    checkBox = ViewUtil.findViewById(itemView, R.id.filter_option_checkbox)
                }
                VIEW_TYPE_NUTRITION_HEADER -> textView = ViewUtil.findViewById(itemView, R.id.selected_filters)
                VIEW_TYPE_UNAVAILABLE_FILTER_NOTE -> textView = ViewUtil.findViewById(itemView, R.id.text_container)

                VIEW_TYPE_USER_FILTER -> {
                    textView = ViewUtil.findViewById(itemView, R.id.filter_name)
                    radioButton = ViewUtil.findViewById(itemView, R.id.filter_option_radio)
                    imageView = ViewUtil.findViewById(itemView, R.id.filter_options)
                    cardView = ViewUtil.findViewById(itemView, R.id.edit_container)
                }
            }
        }
    }
}
