package com.asda.android.app.shop

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Typeface
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.res.ResourcesCompat
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.dynamic.ProductPageUtils
import com.asda.android.app.shop.DynamicFilterAdapter.VIEW_TYPE_FILTER_ITEM
import com.asda.android.app.view.AsdaFullScreenActivity
import com.asda.android.base.core.view.ui.SafeRecyclerView
import com.asda.android.coachmark.view.BubbleCoachMark
import com.asda.android.coachmark.view.CoachMark
import com.asda.android.restapi.service.data.CustomNutritionFilter
import com.asda.android.restapi.service.data.CustomNutritionFilterResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import com.asda.android.utils.view.ViewUtil
import io.reactivex.observers.DisposableObserver


class CustomFiltersActivity : AsdaFullScreenActivity() {

    private var infoText: TextView? = null
    private var containerView: LinearLayout? = null
    private var dummyView: View? = null
    private var filterNameLayout: TextInputLayout? = null
    private var filterName: TextInputEditText? = null
    private var clearAllButton: Button? = null
    private var applyFiltersButton: Button? = null
    private var viewModel: CustomFiltersViewModel? = null
    private var customFilters: ArrayList<AsdaFilter> = ArrayList()
    private var customFilter: CustomNutritionFilterResponse.Filter? = null
    private var filtersList: SafeRecyclerView? = null
    private var adapter: CustomFilterAdapter? = null
    private var progressDialog: ProgressDialog? = null
    private var contentPath: String? = null
    private var bubbleCoachMark: CoachMark? = null
    private var oldFilterName: String? = null

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Utils.addNewRefToLeakDetector(TAG, this)
        setContentView(R.layout.shop_custom_filter)
        customFilter = ShopFilters.instance.customNutritionFilter
        contentPath = intent?.getStringExtra(DynamicFilterAdapter.EXTRA_CONTENT_PATH)
        setupView()
        if (customFilter == null) {
            setHeaderText(getString(R.string.create_custom_filter))
        } else {
            setHeaderText(getString(R.string.refine_title))
        }
        viewModel = ViewModelProviders.of(this).get(CustomFiltersViewModel::class.java)
        filterName?.let { oldFilterName = it.text.toString() }
        filterNameLayout?.clearFocus()
    }

    private fun setupView() {
        containerView = findViewById(R.id.container)
        dummyView = findViewById(R.id.dummy_view)
        infoText = findViewById(R.id.filter_note)
        filterNameLayout = findViewById(R.id.filter_name_layout)
        filterName = findViewById(R.id.filter_name)
        filtersList = findViewById(R.id.filter_list)
        clearAllButton = findViewById(R.id.clear_all_filters)
        applyFiltersButton = findViewById(R.id.apply_filters)

        for (filter: AsdaFilter in ShopFilters.instance.nutritionFilters) {
            filter.isActive = true
            customFilters.add(filter)
        }
        ShopFilters.instance.selectedNutritionFilters.clear()
        customFilter?.filters?.let {
            if (it.isNotEmpty()) {
                filterName?.setText(customFilter?.filterName)
                for (i in 0 until it.size) {
                    for (filter in it) {
                        if (filter.name == it[i].name) {
                            val categoryName = DynamicFilterAdapter.NUTRITION_FACET
                            val refinement = ProductPageUtils.createCustomRefinement(it[i].name, contentPath, it[i].dimensionId, categoryName, false)
                            val dynamicFilter = ProductPageUtils.createDynamicFilter(refinement, VIEW_TYPE_FILTER_ITEM)
                            ShopFilters.instance.addSelectedNutritionFilter(dynamicFilter)
                            adapter?.notifyDataSetChanged()
                        }

                    }
                }
            }
        }

        filtersList?.layoutManager = androidx.recyclerview.widget.LinearLayoutManager(this)
        adapter = CustomFilterAdapter(this, customFilters)
        filtersList?.adapter = adapter

        applyFiltersButton?.visibility = View.VISIBLE
        clearAllButton?.visibility = View.VISIBLE
        clearAllButton?.setOnClickListener { _ -> adapter?.clearAll() }
        applyFiltersButton?.setOnClickListener { _ ->
            saveFilters()
        }

    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        showCoachMark()
    }

    private fun showCoachMark() {
        if (!SharedPreferencesUtil.nutritionCoachMarksShown(this@CustomFiltersActivity)) {
            dummyView?.let {
                createCoachMark(getString(R.string.coachmark_header_1), getString(R.string.coachmark_body_1),
                        getString(R.string.skip), getString(R.string.coachmark_footer_1), it)
            }
        }
    }

    private fun saveFilters() {
        if (!TextUtils.isEmpty(filterName?.text)) {
            try {
                progressDialog = ProgressDialog(this@CustomFiltersActivity)
                progressDialog?.setMessage(baseContext.getString(R.string.loading))
                progressDialog?.show()
            } catch (ex: Exception) {
                Log.w(TAG, ex)
            }

            val items = ArrayList<CustomNutritionFilter.FilterItem>()
            for (filter: AsdaFilter in ShopFilters.instance.selectedNutritionFilters) {
                val item = CustomNutritionFilter.FilterItem()
                item.name = filter.name
                item.dimensionId = ShopFilters.instance.nutritionMap.get(filter.name)?.first
                item.desc = ""
                items.add(item)
            }

            val itemsArray = arrayOfNulls<CustomNutritionFilter.FilterItem>(items.size)
            for (filterItem: CustomNutritionFilter.FilterItem in items) {
                if (filterItem.name != null && filterItem.dimensionId == null) {
                    filterItem.name?.let { filterItem.dimensionId = ShopFilters.instance.nutritionMap[it]?.first }
                }
            }
            items.toArray(itemsArray)
            customFilter?.let {
                val filter = CustomNutritionFilter()
                filter.filterName = filterName?.text.toString()
                @SuppressWarnings("unchecked")
                filter.filterItems = itemsArray as Array<CustomNutritionFilter.FilterItem>
                safeAdd(viewModel?.editFilters(filterName?.text.toString(), oldFilterName
                        ?: "", itemsArray)?.subscribeWith<DisposableObserver<CustomNutritionFilterResponse>>(object : DisposableObserver<CustomNutritionFilterResponse>() {
                    override fun onNext(customNutritionFilterResponse: CustomNutritionFilterResponse) {
                        if (Utils.isDebug()) {
                            Log.d(TAG, "Edit Custom Filter onNext")
                        }
                        customNutritionFilterResponse.filterList?.let {
                            if (it.isNotEmpty()) {
                                ShopFilters.instance.customNutritionFilters.clear()
                                ShopFilters.instance.customNutritionFilters.addAll(it)
                                Authentication.getInstance().userViewResponse?.filterList = it
                            }
                        }
                        val f: CustomNutritionFilterResponse.Filter = CustomNutritionFilterResponse.Filter()
                        f.filterName = filter.filterName
                        ShopFilters.instance.selectedCustomNutritionFilter = f
                        gotoDynamicFilterActivity()
                    }

                    override fun onError(e: Throwable) {
                        if (Utils.isDebug()) {
                            Log.e(TAG, "Edit Custom Filter onError", e)
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }

                    }

                    override fun onComplete() {
                        if (Utils.isDebug()) {
                            Log.d(TAG, "Edit Custom Filter onComplete")
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }

                    }
                }))
            } ?: run {
                val filter = CustomNutritionFilter()
                filter.filterName = filterName?.text.toString()
                filter.filterItems = itemsArray as Array<CustomNutritionFilter.FilterItem>
                safeAdd(viewModel?.createCustomFilter(filterName?.text.toString(), itemsArray)?.subscribeWith<DisposableObserver<CustomNutritionFilterResponse>>(object : DisposableObserver<CustomNutritionFilterResponse>() {
                    override fun onNext(customNutritionFilterResponse: CustomNutritionFilterResponse) {
                        if (Utils.isDebug()) {
                            Log.d(TAG, "Create Custom Filter onNext")
                        }
                        customNutritionFilterResponse.filterList?.let {
                            if (it.isNotEmpty()) {
                                ShopFilters.instance.customNutritionFilters.clear()
                                ShopFilters.instance.customNutritionFilters.addAll(it)
                                Authentication.getInstance().userViewResponse?.filterList = it
                            }
                        }
                        val f: CustomNutritionFilterResponse.Filter = CustomNutritionFilterResponse.Filter()
                        f.filterName = filter.filterName
                        ShopFilters.instance.selectedCustomNutritionFilter = f
                        gotoDynamicFilterActivity()
                    }

                    override fun onError(e: Throwable) {
                        if (Utils.isDebug()) {
                            Log.e(TAG, "Create Custom Filter onError", e)
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }

                    }

                    override fun onComplete() {
                        if (Utils.isDebug()) {
                            Log.d(TAG, "Create Custom Filter onComplete")
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }

                    }
                }))

                val event = AsdaAnalyticsEvent(Anivia.CUSTOM_FILTER)
                        .putString(Anivia.NAME_KEY, getScreenName())
                        .putString(Anivia.CUSTOM_FILTER_KEY, filterName?.text.toString())
                val filters = Utils.formatAnalyticsFilters(ShopFilters.instance.selectedNutritionFilters)
                if (ShopFilters.instance.selectedCustomNutritionFilter != null && ShopFilters.instance.selectedCustomNutritionFilter?.filterName != null) {
                    event.putString(Anivia.CUSTOM_FILTER, ShopFilters.instance.selectedCustomNutritionFilter?.filterName)
                }
                if (filters.size > 0) {
                    event.putArray(Anivia.FILTER_ARRAY, filters)
                    Tracker.get().trackEvent(event)
                }
            }
        }
    }

    private fun getScreenName(): String {
        return if (intent != null && !TextUtils.isEmpty(intent.getStringExtra(EXTRA_SCREEN_NAME))) {
            intent.getStringExtra(EXTRA_SCREEN_NAME).trim { it <= ' ' }
        } else {
            SCREEN_NAME
        }
    }

    fun createCoachMark(title: String, body: String, buttonText: String, footer: String, anchor: View) {
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_NOSENSOR
        val view: View = ViewUtil.inflate(this@CustomFiltersActivity, R.layout.coach_mark_content)
        ViewUtil.setText(R.id.coach_mark_header, view, title)
        ViewUtil.setText(R.id.coach_mark_body, view, body)
        ViewUtil.setText(R.id.coach_mark_button, view, buttonText)
        ViewUtil.setText(R.id.coach_mark_footer, view, footer)

        when (anchor) {
            dummyView -> bubbleCoachMark = BubbleCoachMark.BubbleCoachMarkBuilder(this@CustomFiltersActivity, anchor, view)
                    .setTargetOffset(1.0f)
                    .setShowBelowAnchor(true)
                    .setPadding(10)
                    .setOnDismissListener(object : CoachMark.OnDismissListener {
                        override fun onDismiss() {
                            createCoachMark(getString(R.string.coachmark_header_2), getString(R.string.coachmark_body_2),
                                    getString(R.string.skip), "", filterName!!)
                        }
                    })
                    .build()
            filterName -> {
                ViewUtil.findViewById<TextView>(view, R.id.coach_mark_body).setTextColor(ResourcesCompat.getColor(resources,R.color.avocado_green, null))
                ViewUtil.findViewById<TextView>(view, R.id.coach_mark_body).typeface = Typeface.DEFAULT_BOLD
                bubbleCoachMark = BubbleCoachMark.BubbleCoachMarkBuilder(this@CustomFiltersActivity, anchor, view)
                        .setTargetOffset(0.50f)
                        .setShowBelowAnchor(true)
                        .setPadding(10)
                        .setOnDismissListener(object : CoachMark.OnDismissListener {
                            override fun onDismiss() {
                                createCoachMark(getString(R.string.coachmark_header_3), getString(R.string.coachmark_body_3),
                                        getString(R.string.done), getString(R.string.coachmark_footer_3), applyFiltersButton!!)
                            }
                        })
                        .build()
            }

            applyFiltersButton -> {
                ViewUtil.findViewById<TextView>(view, R.id.coach_mark_body).setTextColor(ResourcesCompat.getColor(resources, R.color.avocado_green, null))
                ViewUtil.findViewById<TextView>(view, R.id.coach_mark_footer).setTextColor(ResourcesCompat.getColor(resources, R.color.text, null))
                ViewUtil.findViewById<TextView>(view, R.id.coach_mark_footer).typeface = Typeface.DEFAULT_BOLD
                bubbleCoachMark = BubbleCoachMark.BubbleCoachMarkBuilder(this@CustomFiltersActivity, anchor, view)
                        .setTargetOffset(0.50f)
                        .setShowBelowAnchor(false)
                        .setPadding(10)
                        .setOnDismissListener(object: CoachMark.OnDismissListener {
                            override fun onDismiss() {
                                requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED
                            }
                        })
                        .build()
            }
        }

        val bubble = bubbleCoachMark
        window.decorView.rootView.post(bubble!!::show)
        SharedPreferencesUtil.setNutritionCoachMarksShown(this@CustomFiltersActivity)
    }

    private fun gotoDynamicFilterActivity() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        containerView?.rootView?.let { ViewUtil.hideKeyboard(it) }
        finish()
    }

    public override fun getMenuResourceId(): Int {
        return R.menu.nutrition_save_filter
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.action_save_filter -> {
                saveFilters()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    companion object {
        private val TAG = "CustomFiltersActivity"
        val EXTRA_FILTERS = "data_filters"
        private val EXTRA_SCREEN_NAME = "extra_screen_name"
        private val SCREEN_NAME = "Shelf"
    }
}
