package com.asda.android.app.shop

import android.annotation.SuppressLint
import android.content.Context
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentPagerAdapter
import androidx.viewpager.widget.ViewPager
import com.asda.android.R
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.base.FragmentConfig
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.restapi.search.model.AsdaSearchData
import com.asda.android.app.view.SearchableFragment
import com.asda.android.base.interfaces.IBaseFragment
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.recipes.view.fragments.FavRecipeListFragment
import com.google.android.material.tabs.TabLayout

const val FREQUENTLY_BOUGHT = "frequently_bought"

class FavoritesFragment: SearchableFragment() {
    private val TAG = "FavoritesFragment"
    private var pager: ViewPager? = null
    private var favPagerAdapter: FavPagerAdapter? = null
    private var tabLayout: TabLayout? = null
    private var progress: View? = null

    @SuppressLint("MissingSuperCall")
    override// Some code is still shared with presenters like BreadcrumbsFragment
    fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.fav_container_fragment, fragmentContainer, false)
        pager = view.findViewById(R.id.pager)
        tabLayout = view.findViewById(R.id.tab_layout)
        progress = view.findViewById(R.id.loading_view)
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        var isFrequentlyBought = false
        dynamicArguments?.let {
            isFrequentlyBought = it.getBoolean(FREQUENTLY_BOUGHT, false)
            dynamicArguments = null
        }

        tabLayout?.setupWithViewPager(pager)
        tabLayout?.visibility = View.VISIBLE
        favPagerAdapter = FavPagerAdapter(childFragmentManager, isFrequentlyBought, view.context)
        pager?.adapter = favPagerAdapter
        pager?.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
                //implementation not required
            }

            override fun onPageSelected(position: Int) {
                Log.e(TAG, "page selected $position")
                if (position == 1) {
                    progress?.visibility = View.GONE
                }
            }

            override fun onPageScrollStateChanged(state: Int) {
                //implementation not required
            }
        })
    }

    override fun onSearchRequested(searchData: AsdaSearchData) {
        super.onSearchRequested(searchData)
        popToRoot()
        val bundle = Bundle().apply {
            putParcelable(ShopFragment.ARG_SEARCH_DATA, searchData)
        }
        (activity as? AsdaActivity)?.fragmentSwitcher?.switchToFragment(FragmentConfig.TAG_SHOP, bundle)
    }

    override fun isParentFragment() = true

    private inner class FavPagerAdapter(fm: FragmentManager, private val isFrequentlyBought: Boolean, private val ctx: Context) : FragmentPagerAdapter(fm) {
        private val MAX_TAB_COUNT = 2

        override fun getItem(position: Int): Fragment = when (position) {
            0 -> FavoritesShelfFragment.newInstance(isFrequentlyBought).apply {
                setTaxonomy("Favourites")
                transitionListener = object: TransitionListener {
                    override fun onPushed(fragment: IBaseFragment, addToBackStack: Boolean) = false

                    override fun onPopped(tag: String?) =
                        (activity as? AsdaActivity)?.fragmentSwitcher?.onBackPressed() ?: false
                }
            }
            1 -> FavRecipeListFragment.newInstance().apply {
                REQUEST_CODE_FILTER = ActivityRequestCodes.REQUEST_CODE_FILTER

                onRefineClicked = { view ->
                    FilterActivity.showFilterForRecipesFavorites(activity, this, ActivityRequestCodes.REQUEST_CODE_FILTER,
                            sortMode, recipeSortOrder, sortIndex, activeFilters?.map { AsdaFilter(it) }?.toTypedArray(), view)
                }
            }
            else -> FavoritesShelfFragment.newInstance(isFrequentlyBought).apply {
                setTaxonomy("Favourites")
            }
        }

        override fun getPageTitle(position: Int): String = when (position) {
            0 -> ctx.getString(R.string.label_products)
            1 -> ctx.getString(R.string.label_recipes)
            else -> ctx.getString(R.string.label_products)
        }

        override fun getCount(): Int {
               return MAX_TAB_COUNT
        }
    }
}
