package com.asda.android.app.shop

import androidx.annotation.StringDef
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.ViewModel
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.RelatedSearchClickEvent
import com.asda.android.analytics.events.RelatedSearchImpressionEvent
import com.asda.android.search.model.SearchTermDTO
import com.asda.android.service.base.Authentication


class ShelfListFragmentViewModel : ViewModel() {

    companion object {
        @Retention(AnnotationRetention.SOURCE)
        @StringDef(REFINED, RELATED)
        annotation class SearchType

        const val REFINED = "Refined"
        const val RELATED = "Related"

        var searchType = ""
        var previousSearchTerm: String = ""
    }

    fun sendImpressionAnalytics(searchTerm: String, itemList: List<SearchTermDTO>, @SearchType searchType: String) {
        itemList.take(8).forEachIndexed { i, searchTermDTO ->
            val event = RelatedSearchImpressionEvent(searchTerm, searchType,getCustomerId(),searchTermDTO.searchTerm,(i+1).toString())
            getTracker().trackEvent(event)
        }
    }

    fun sendClickAnalytics(searchTerm: String, clickedTerm: String, position: Int, searchType: Int) {
        val event = RelatedSearchClickEvent(searchTerm, getSearchType(searchType), getCustomerId(), clickedTerm, position.toString())
        getTracker().trackEvent(event)
    }

    fun getSearchType(searchType: Int): String {
        return if(searchType == 0) REFINED else RELATED
    }

    @VisibleForTesting
    fun getCustomerId(): String {
        return Authentication.getInstance().profileId.orEmpty()
    }

    @VisibleForTesting
    fun getTracker(): Tracker {
        return Tracker.get()
    }

}
