package com.asda.android.app.shop

import com.asda.android.restapi.service.data.CustomNutritionFilterResponse
import java.util.*


private const val TAG = "Shop Filters"

class ShopFilters {

    var nutritionMap = mutableMapOf<String, Pair<String, String>>()
    var sortFilter = mutableListOf<AsdaFilter>()

    var selectedFilters = ArrayList<AsdaFilter>()
    var nutritionFilters = mutableListOf<AsdaFilter>()
    var selectedNutritionFilters = ArrayList<AsdaFilter>()
    var customNutritionFilters = mutableListOf<CustomNutritionFilterResponse.Filter>()
    var selectedCustomNutritionFilter: CustomNutritionFilterResponse.Filter? = null
    var customNutritionFilter: CustomNutritionFilterResponse.Filter? = null

    var favoritesAvailableFilters = mutableListOf<AsdaFilter>()
    var favoritesUnavailableFilters = mutableListOf<AsdaFilter>()

    fun addSelectedNutritionFilter(filter: AsdaFilter) {
        if (!selectedNutritionFilters.contains(filter)) {
            selectedNutritionFilters.add(filter)
        }
    }

    fun addFavoritesAvailableFilters(filter: AsdaFilter) {
        if (!favoritesAvailableFilters.contains(filter)) {
            favoritesAvailableFilters.add(filter)
        }
    }

    fun addFavoritesUnavailableFilters(filter: AsdaFilter) {
        if (!favoritesUnavailableFilters.contains(filter)) {
            favoritesUnavailableFilters.add(filter)
        }
    }

    fun clearNutritionFilters() {
        nutritionFilters.clear()
        customNutritionFilters.clear()
        sortFilter.clear()
    }

    fun addAllNutritionFilters(filters: ArrayList<AsdaFilter>) {
        for (filter in filters) {
            if (!nutritionFilters.contains(filter)) {
                nutritionFilters.add(filter)
            }
        }
    }

    fun getSelectedNutritionIds(): String? {
        val nutritionFilters = selectedNutritionFilters
        val nutritionFacets = StringBuilder()
        for (filter in nutritionFilters) {
            filter.singleDimensionId?.let {
                val index = nutritionFilters.indexOf(filter)
                nutritionFacets.append(it)
                if (index >= 0 && index < (nutritionFilters.size - 1)) {
                    nutritionFacets.append(",")
                }
            } ?: run {
                filter.refinement?.dimensionId?.let {
                    val index = nutritionFilters.indexOf(filter)
                    nutritionFacets.append(it)
                    if (index >= 0 && index < (nutritionFilters.size - 1)) {
                        nutritionFacets.append(",")
                    }
                }
            }
        }
        return nutritionFacets.toString()
    }

    fun clearAllFilters() {
        selectedNutritionFilters.clear()
        selectedFilters.clear()
    }

    companion object {
        private var mShopFiltersInstance: ShopFilters? = null

        val instance: ShopFilters
            get() {
                if (mShopFiltersInstance == null) {
                    mShopFiltersInstance = ShopFilters()
                }
                return mShopFiltersInstance as ShopFilters
            }
    }
}
