package com.asda.android.app.shop.dynamicshelf.viewmodel

import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.dynamic.RefinementComponent
import com.asda.android.app.dynamic.manager.CMSUtils
import com.asda.android.app.shop.AsdaFilter
import com.asda.android.app.shop.specialoffers.viewmodel.SpecialOffersViewModel
import com.asda.android.restapi.service.data.cms.Content
import com.asda.android.restapi.service.data.cms.Personalization
import com.asda.android.restapi.service.data.cms.Refinement
import com.facebook.litho.widget.RecyclerBinder
import java.util.*

class DynamicShelfViewModel : SpecialOffersViewModel() {
    private val SEPARATOR_SPACE = " "
    private val TAXONOMY_ALL_OFFERS = "All Offers"
    var taxonomyType: String? = null
    var dimensionId: String? = null
    var refinement: Refinement? = null

    fun updatePersonalizationContents(refinement: Refinement?, pageId: String?, sortBy: String?, filters: ArrayList<AsdaFilter>?, nutritionFilters: ArrayList<AsdaFilter>?, contents: MutableList<out Content>?, pageNum: Int): Boolean {
        var isUpdated = false
        contents?.let { contentList ->
            for (content in contentList) {
                if (content is Personalization) {
                    isUpdated = true
                    content.pageId = pageId
                    content.viewInfo.isHorizontal = false
                    content.viewInfo.isShowTitle = false
                    content.viewInfo.isRemovePadding = true
                    refinement?.let { refinement1 ->
                        refinement1.properties?.let { property ->
                            content.trackingCode = Anivia.CATEGORY_MERCH_SPECIAL_ALL_OFFERS
                            content.taxonomyType = property.categoryType
                            content.repositoryId = property.categoryRepositoryId
                            content.pagination.pageNum = pageNum
                            dimensionId = getDimensionId(refinement1)
                            this.refinement = refinement1
                            filters?.let { filterList ->
                                if (filterList.size > 0) {
                                    filterList.last().refinement?.let {
                                        dimensionId = getDimensionId(it)
                                        this.refinement = it
                                        it.properties?.let { property ->
                                            property.categoryRepositoryId?.let {
                                                content.repositoryId = it
                                            }
                                            property.categoryType?.let {
                                                content.taxonomyType = it
                                            }
                                        }
                                    }

                                }
                            }
                            nutritionFilters?.let {
                                content.pagination.filterBy = (it.map { asdaFilter -> asdaFilter.singleDimensionId }).joinToString(",")
                            }
                            sortBy?.let {
                                content.pagination.sortBy = it.replace(":store_id", "")
                            }
                        }

                    }
                }
            }
        }
        return isUpdated
    }

    fun getDimensionIds(defaultDimensionId: String?, refinement: Refinement?, excludeRefinement: Refinement?, filters: ArrayList<AsdaFilter>?, nutritionFilters: ArrayList<AsdaFilter>?): String? {
        var ids: String? = null
        var excludeFilter: AsdaFilter? = null
        var activeFilters: List<AsdaFilter>? = filters
        excludeRefinement?.let {
            excludeFilter = AsdaFilter(it)
        }
        filters?.let {
            if (excludeFilter != null) {
                if (it.size == 0) {
                    activeFilters = listOf(excludeFilter!!)
                } else if (it.size > 1) {
                    activeFilters = (it.filter { asdaFilter -> asdaFilter.singleDimensionId != excludeFilter!!.singleDimensionId })
                }
            }
            activeFilters?.let {
                ids = (it.map { asdaFilter -> asdaFilter.singleDimensionId }).joinToString(SEPARATOR_SPACE)
            }
        }
        nutritionFilters?.let {
            if (!ids.isNullOrEmpty()) {
                ids += SEPARATOR_SPACE
            }
            val nutritionFilterIds = (it.map { asdaFilter -> asdaFilter.singleDimensionId }).joinToString(SEPARATOR_SPACE)
            if (ids.isNullOrEmpty()) {
                ids = nutritionFilterIds
            } else {
                ids += nutritionFilterIds
            }
        }

        if (ids.isNullOrEmpty()) {
            refinement?.let {
                val refinementId = getDimensionParameter(refinement.dimensionId, "N")
                if (ids.isNullOrEmpty()) {
                    ids = refinementId
                } else {
                    ids += refinementId
                }
            }
        }
        defaultDimensionId?.let {
            if (ids.isNullOrEmpty()) {
                ids = getDimensionParameter(defaultDimensionId, "N")
            }
        }
        return ids
    }

    fun isRefinementViewAdded(recyclerBinder: RecyclerBinder): Boolean {
        for (i in 0 until recyclerBinder.itemCount) {
            if (recyclerBinder.getRenderInfoAt(i).component is RefinementComponent) {
                return true
            }
        }
        return false
    }

    fun getTaxonomyName(isSpecialOffers: Boolean, refinement: Refinement?):String{
        refinement?.let {
            return it.label.toString()
        }
        return if(isSpecialOffers)
            TAXONOMY_ALL_OFFERS
        else
            ""
    }

    @VisibleForTesting
    fun getDimensionParameter(dimensionId: String?, param: String?): String? {
        return CMSUtils.getDimensionParameter(dimensionId, param)
    }

}
