package com.asda.android.app.shop.productdetail.view

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import com.asda.android.R
import com.asda.android.base.core.view.actitivies.BaseAppCompatActivity
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.data.cart.Cart

class ProductDetailsActivity : BaseAppCompatActivity() {

    private var itemId: String? = null
    private var item: ShelfListItem? = null
    private var cxoItem: Cart.CartItems? = null
    private var specialOffers: Boolean = false
    private var isCXOTrolley: Boolean = false
    private var specialOffersTagIsClickable: Boolean = false
    private var favoriteMode: Boolean = false

    override fun getFragmentContainer() = R.id.fragment_container

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.fragment_container)
        getArguments()

        supportActionBar?.title = getString(R.string.product_details)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)


        val fragment = newProductFragment()
        /**
         * TODO: In case if item or itemId is not sent, then it will show a blank page.
         * This should not happen, but just to be safe, attach an error view.
         */
        fragment?.let {
            push(it)
        }
    }

    private fun getArguments() {
        intent?.let {
            isCXOTrolley = it.getBooleanExtra(EXTRA_IS_CXO_TROLLEY, false)
            itemId = it.getStringExtra(EXTRA_ITEM_ID)
            // itemId can come from internal Activity launch or an implicit intent. So we check both.
            if (TextUtils.isEmpty(itemId)) {
                itemId = intent.data?.getQueryParameter(QUERY_ITEM_ID) ?: ""
            }

            if (isCXOTrolley) {
                cxoItem = it.getParcelableExtra(EXTRA_ITEM)
            } else {
                item = it.getParcelableExtra(EXTRA_ITEM)
            }
            specialOffers = it.getBooleanExtra(EXTRA_SPECIAL_OFFERS, false)
            specialOffersTagIsClickable = it.getBooleanExtra(EXTRA_SPECIAL_OFFERS_TAG_CLICKABLE, false)
            favoriteMode = it.getBooleanExtra(EXTRA_FAVORITES_MODE, false)

            if (!specialOffersTagIsClickable) {
                specialOffersTagIsClickable = intent.data?.getBooleanQueryParameter(EXTRA_SPECIAL_OFFERS_TAG_CLICKABLE, false)
                        ?: false
            }
        }
    }

    private fun newProductFragment(): ProductDetailsFragment? {
        item?.let {
            return ProductDetailsFragment.newInstance(it, specialOffers, specialOffersTagIsClickable)
        }

        cxoItem?.let {
            return ProductDetailsFragment.newInstance(it, specialOffers, specialOffersTagIsClickable)
        }

        TextUtils.isEmpty(itemId)?.let {
            return ProductDetailsFragment.newInstance(itemId, specialOffersTagIsClickable)
        }
    }


    companion object {
        private const val QUERY_ITEM_ID = "item_id"
        private const val EXTRA_ITEM_ID = "extra_item_id"
        private const val EXTRA_ITEM = "extra_item"
        private const val EXTRA_IS_CXO_TROLLEY = "extra_is_cxo_trolley"
        private const val EXTRA_SPECIAL_OFFERS = "extra_special_offers"
        private const val EXTRA_SPECIAL_OFFERS_TAG_CLICKABLE = "extra_special_offers_tag_clickable"
        private const val EXTRA_FAVORITES_MODE = "extra_favorites_mode"

        fun launch(
              context: Context,
              item: ShelfListItem,
              specialOffers: Boolean,
              specialOffersTagIsClickable: Boolean,
              favoriteMode: Boolean) {
            val intent = Intent(context, ProductDetailsActivity::class.java)
            intent.putExtra(EXTRA_ITEM, item)
            intent.putExtra(EXTRA_SPECIAL_OFFERS, specialOffers)
            intent.putExtra(EXTRA_SPECIAL_OFFERS_TAG_CLICKABLE, specialOffersTagIsClickable)
            intent.putExtra(EXTRA_FAVORITES_MODE, favoriteMode)
            context.startActivity(intent)
        }

        fun launch(context: Context, cxoItem: Cart.CartItems, itemId: String, cxoTrolley: Boolean) {
            val intent = Intent(context, ProductDetailsActivity::class.java)
            intent.putExtra(EXTRA_ITEM, cxoItem)
            intent.putExtra(EXTRA_IS_CXO_TROLLEY, cxoTrolley)
            intent.putExtra(EXTRA_ITEM_ID, itemId)
            context.startActivity(intent)
        }
    }
}
