package com.asda.android.app.shop.productdetail.view

import android.app.Activity
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.util.Log
import android.view.View
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.asda.android.R
import com.asda.android.analytics.info.CartItemAnalyticsInfo
import com.asda.android.app.direct.DirectActivity
import com.asda.android.app.dynamic.LinkSaveDynamicPageFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.CXOHelper
import com.asda.android.app.shop.MerchandisingHandler
import com.asda.android.app.shop.WriteReviewActivity
import com.asda.android.app.shop.productdetail.helper.ProductHelper
import com.asda.android.app.shoppinglists.ShoppingListActivity
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.FragmentUtils
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.base.interfaces.IProductManager
import com.asda.android.base.interfaces.IQuantityController
import com.asda.android.base.interfaces.ShopContext
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.products.ui.detail.constants.PdpConstants.SOURCE_SCREEN_NAME
import com.asda.android.products.ui.detail.model.ProductDetailsExtraInfo
import com.asda.android.products.ui.detail.view.ProductDetailsActivityNew
import com.asda.android.products.ui.filters.view.FiltersActivity
import com.asda.android.restapi.app.product.model.FilterGroup
import com.asda.android.restapi.app.product.model.QuantityControllerViewInfoModel
import com.asda.android.restapi.app.product.model.SearchInfoModel
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.Callback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.IAsdaService
import com.asda.android.restapi.service.base.Iconography
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.IconographyResponse
import com.asda.android.restapi.service.data.ProductDetails
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPdp
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.restapi.service.data.direct.ReviewsResponse
import com.asda.android.search.view.SearchResultsFragment
import com.asda.android.service.base.CartManager
import com.asda.android.taxonomy.features.linksave.view.LinkSaveFragment
import com.asda.android.taxonomy.features.shelf.view.ShelfFragment
import com.asda.android.utils.getMissingCount
import com.asda.android.utils.view.startOnTrolleyModify
import java.math.BigDecimal

const val TAG = "ProductManager"

var filterGroups: ArrayList<FilterGroup> = arrayListOf()

object ProductManager : IProductManager {

    override fun getLinkSaveFragment(context: Context, promotionId: String?, promotionText: String?): BaseFragment {
        val fragment: BaseFragment
        if (FeatureSettings.isCmsLinkSaveEnabled(context)) {
            fragment = LinkSaveFragment.newInstance(promotionId, promotionText)
        } else {
            fragment = LinkSaveDynamicPageFragment.newInstance(promotionId)
            fragment.campaignId = promotionText
        }

        return fragment
    }

    override fun launchLinkSave(context: Context, currentFragmentManager: FragmentManager?, transitionListener: TransitionListener?, promotionId: String?, promotionText: String?) {
        val fragment = getLinkSaveFragment(context, promotionId, promotionText)
        FragmentUtils.push(currentFragmentManager, transitionListener, fragment, addToBackStack = true, animate = true, breadcrumb = promotionText, replace = true)
    }

    override fun onTrolleyModify(context: Context, listingName: String, itemId: String?, itemQtyForCartAdd: String?, sourcePageId: String?) {
        if (context is Activity)
            startOnTrolleyModify(context, listingName, itemId, itemQtyForCartAdd, sourcePageId)
    }

    private val cxoHelper = CXOHelper()
    private val productHelper = ProductHelper()
    override fun getCartIcon(view: View?): View? {
        return view?.findViewById(R.id.cart_total_price)
    }

    override fun onItemUpdated(context: Context, statusCode: Int, title: String?, screenName: String?) {
        if (statusCode == CartManager.STATUS_CODE_ERROR) {
            if (FeatureSettings.isCXOMigrationEnabled(context)) {
                com.asda.android.cxo.CartManager.instance?.cartResponse?.errors?.toMutableList()?.let {
                    cxoHelper.showErrorDialog(context, it)
                }
            }
            return
        }

        if (statusCode == CartManager.STATUS_CODE_MAX_QTY_EXCEEDED) {
            DialogHelper.displayErrorDialog(context.getString(R.string.max_qty_error, title), context, screenName)
        }

        if (com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.expressItemLimit ?: 0 > 0
              && com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.cartItemSummary?.totalQuantity ?: 0 > 0
              && com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.expressItemLimit ==
              com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.cartItemSummary?.totalQuantity) {
            handleAlert(context, CartManager.PRODUCT_SLOT_LIMIT_REACHED)
        }
    }

    override fun onItemRemoveFinished(context: Context) {
        if (!FeatureSettings.isCXOMigrationEnabled(context)) {
            val updatedCart = CartManager.get(context)?.cart
            updatedCart?.removedRecipes?.let {
                if (it.isNotEmpty()) {
                    val clickListener: DialogInterface.OnClickListener = DialogInterface.OnClickListener { dialog, which -> dialog.dismiss() }
                    val resources = context.resources
                    val title = resources.getString(R.string.recipe_removed_title)
                    val foregroundColorSpan = ForegroundColorSpan(ContextCompat.getColor(context, R.color.notification_red))
                    val ssBuilder = SpannableStringBuilder(title)
                    ssBuilder.setSpan(
                          foregroundColorSpan,
                          0,
                          title.length,
                          Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                    )
                    val messageBuilder = StringBuilder()
                    for ((_, recipeName) in it) {
                        messageBuilder.append(recipeName)
                        messageBuilder.append(", ")
                    }
                    val positiveText = context.resources.getString(R.string.ok)
                    var removalMessage = resources.getString(R.string.recipe_removed_message, messageBuilder.toString())
                    removalMessage = removalMessage.substring(0, removalMessage.length - 2)
                    DialogFactory.createAlertDialog(ssBuilder, removalMessage, positiveText, clickListener, null, context, TAG).show()
                }
            }
        }
    }

    override fun getIconographyResponse(context: Context, iconId: String?, callback: Callback<Iconography>, notifyCallbackIfInitialized: Boolean, asdaService: IAsdaService): IconographyResponse.Icon? {
        iconId?.let {
            return Iconography.getInstance(context, callback, notifyCallbackIfInitialized, asdaService)?.getIcon(context, iconId)
        }
        return null
    }

    override fun getPromoMissingCount(context: Context, promotionId: String?): Int {
        return getMissingCount(context, promotionId)
    }

    override fun launchProductDetail(sourceFragment: Fragment?, context: Context, item: IroProductDetailsPlp.Items, isBigIcon: Boolean, specialOffers: Boolean, specialOffersTagIsClickable: Boolean, favoriteMode: Boolean, bannerCampaignId: String?) {
        if (FeatureSettings.isPdpTempEnabled(context)) {
            item.itemId?.let { itemId ->
                sourceFragment?.let {
                    it.tryCast<BaseFragment> {
                        val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                              .newInstance(
                                    productDetailsExtraInfo = ProductDetailsExtraInfo(
                                          itemId = itemId, specialOffers = specialOffers, specialOffersTagIsClickable = specialOffersTagIsClickable, bannerCampaignId = bannerCampaignId
                                    )
                              )
                        this.push(fragment, getString(R.string.product_details), isReplace = shouldReplacePDPFragment(sourceFragment))
                    }
                } ?: kotlin.run {
                    ProductDetailsActivityNew.launch(context,
                          ProductDetailsExtraInfo(
                                itemId = itemId, specialOffers = specialOffers, specialOffersTagIsClickable = specialOffersTagIsClickable, bannerCampaignId = bannerCampaignId
                          )
                    )
                }
            }
        } else {
            ProductDetailsActivity.launch(context, ShelfListItem.buildShelfObject(item, specialOffersTagIsClickable, isBigIcon), specialOffers, specialOffersTagIsClickable, favoriteMode)
        }
    }

    override fun launchProductDetail(sourceFragment: Fragment?, context: Context, itemId: String, isBigIcon: Boolean) {
        if (FeatureSettings.isPdpTempEnabled(context)) {
            sourceFragment?.let {
                it.tryCast<BaseFragment> {
                    val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                          .newInstance(
                                productDetailsExtraInfo = ProductDetailsExtraInfo(
                                      itemId = itemId, specialOffers = false, specialOffersTagIsClickable = false, bannerCampaignId = null
                                )
                          )
                    if (this is ShelfFragment)
                        this.push(fragment, getString(R.string.product_details), isReplace = false)
                    else
                        this.push(fragment, getString(R.string.product_details))
                }
            }
        }
    }

    override fun launchProductDetail(sourceFragment: Fragment?, context: Context, item: ShelfListItem, isBigIcon: Boolean, specialOffers: Boolean, specialOffersTagIsClickable: Boolean, favoriteMode: Boolean, bannerCampaignId: String?) {
        if (FeatureSettings.isPdpTempEnabled(context)) {
            item.id?.let { itemId ->
                sourceFragment?.let {
                    it.tryCast<BaseFragment> {
                        val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                              .newInstance(productDetailsExtraInfo = ProductDetailsExtraInfo(
                                    itemId = itemId, specialOffers = specialOffers, specialOffersTagIsClickable = specialOffersTagIsClickable, bannerCampaignId = bannerCampaignId)
                              )

                        this.push(fragment, getString(R.string.product_details), isReplace = shouldReplacePDPFragment(sourceFragment))
                    }
                } ?: kotlin.run {
                    ProductDetailsActivityNew.launch(context,
                          ProductDetailsExtraInfo(itemId = itemId, specialOffers = specialOffers, specialOffersTagIsClickable = specialOffersTagIsClickable, bannerCampaignId = bannerCampaignId)
                    )
                }
            }
        } else {
            ProductDetailsActivity.launch(context, item, false, true, false)
        }
    }

    override fun launchProductReviews(context: Context, item: IroProductDetailsPdp.Items, imageUrl: String, isShowWriteReview: Boolean, reviewStatistics: ReviewsResponse.Includes.Product.ReviewStatistics?, view: View) {
        context.tryCast<com.asda.android.base.core.view.ui.BaseAppCompatActivity> {
            DirectActivity.launchProductReviews(
                  this,
                  true,
                  item.itemId,
                  item.item?.itemName,
                  item.price?.priceInfo?.price,
                  imageUrl,
                  reviewStatistics,
                  item.item?.cin,
                  isShowWriteReview,
                  view
            )
        }
    }

    override fun launchProductDetail(sourceFragment: Fragment?, context: Context, cxoItem: com.asda.android.restapi.service.data.cart.Cart.CartItems, itemId: String, isCXOTrolley: Boolean) {
        if (FeatureSettings.isPdpTempEnabled(context)) {
            ProductDetailsActivityNew.launch(context, ProductDetailsExtraInfo(itemId = itemId))
        } else {
            ProductDetailsActivity.launch(context, cxoItem, itemId, isCXOTrolley)
        }
    }

    override fun launchProductDetailFromBanner(context: Context, fragmentManager: FragmentManager?, itemId: String, specialOffers: Boolean, specialOffersTagIsClickable: Boolean, bannerCampaignId: String?) {
        if (FeatureSettings.isPdpTempEnabled(context)) {
            ProductDetailsActivityNew.launch(context,
                  ProductDetailsExtraInfo(
                        itemId = itemId,
                        specialOffers = specialOffers,
                        specialOffersTagIsClickable = specialOffersTagIsClickable,
                        bannerCampaignId = bannerCampaignId
                  )
            )
        } else {
            val productDetailsFragment = ProductDetailsFragment.newInstance(itemId, specialOffers, specialOffersTagIsClickable);
            productDetailsFragment.campaignId = bannerCampaignId;
            val breadcrumb = context.getString(R.string.banner_promotion)
            fragmentManager?.let { fragManager ->
                productDetailsFragment?.let { productDetailsFrag ->
                    MerchandisingHandler.pushFragment(fragManager, productDetailsFrag, false, breadcrumb)
                }
            }

        }
    }

    private fun getItemQuantityController(context: Context, shelfListItem: ShelfListItem, viewInfoModel: QuantityControllerViewInfoModel, view: View): ItemQuantityController {
        var cartItem: Cart.CartItem? = null
        var cxoCartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems? = null
        val itemQuantityController: ItemQuantityController
        if (FeatureSettings.isCXOMigrationEnabled(context)) {
            cxoCartItem = cxoHelper.getItemInCartCXO(shelfListItem, viewInfoModel.isShoppingList)
            itemQuantityController = cxoHelper.initCXOItemQuantityController(cxoCartItem, shelfListItem, viewInfoModel.alwaysShowPopup, viewInfoModel.isShoppingList)
        } else {
            cartItem = getCartItemFrom(context, shelfListItem, viewInfoModel.mode, viewInfoModel.isShoppingList)
            itemQuantityController = cxoHelper.initItemQuantityController(cartItem, shelfListItem, viewInfoModel.alwaysShowPopup)
        }

        viewInfoModel.isValidForQuantityController = cartItem != null || cxoCartItem != null || viewInfoModel.alwaysShowPopup
        when {
            viewInfoModel.isValidForQuantityController -> {
                when {
                    FeatureSettings.isCXOMigrationEnabled(context) && cxoCartItem != null -> itemQuantityController.update(cxoCartItem)
                    cartItem != null -> itemQuantityController.update(cartItem)
                }
                when {
                    viewInfoModel.listType == ShoppingListActivity.ListType.USER_GENERATED -> itemQuantityController.showQtyPopup(context, view, shelfListItem.allowPartialQty)
                    viewInfoModel.listType == ShoppingListActivity.ListType.SYSTEM_GENERATED -> {
                        itemQuantityController.setAllowZeroQuantity(false)
                        itemQuantityController.showQtyPopup(context, view, shelfListItem.allowPartialQty, false)
                    }
                }
            }
        }
        return itemQuantityController
    }

    override fun launchWriteProductReview(context: Context, productId: String?, name: String?, price: String?, imageUrl: String?) {
        WriteReviewActivity.start(context, productId, name, price, imageUrl)
    }

    override fun getItemQuantityController(context: Context, item: IroProductDetailsPlp.Items, viewInfoModel: QuantityControllerViewInfoModel, view: View): ItemQuantityController {
        val shelfListItem = ShelfListItem.buildShelfObject(item, viewInfoModel.specialOffersTagIsClickable, viewInfoModel.isBigIcon)
        return getItemQuantityController(context, shelfListItem, viewInfoModel, view)
    }

    override fun getItemQuantityController(context: Context, item: IroProductDetailsPdp.Items, viewInfoModel: QuantityControllerViewInfoModel, view: View): ItemQuantityController {
        val shelfListItem = ShelfListItem.buildShelfObjectPdp(item, viewInfoModel.specialOffersTagIsClickable)
        return getItemQuantityController(context, shelfListItem, viewInfoModel, view)
    }

    override fun getCartItemFrom(context: Context, item: ShelfListItem, mode: Int, isShoppingList: Boolean): Cart.CartItem? {
        return productHelper.getCartItemFrom(context, item, mode, isShoppingList)
    }

    override fun validateQuantity(context: Context, item: IroProductDetailsPlp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean, mode: Int, shopContext: ShopContext, quantityController: IQuantityController): Pair<BigDecimal, BigDecimal> {
        return productHelper.validateQuantity(context, item, specialOffersTagIsClickable, isBigIcon, mode, shopContext, quantityController)
    }

    override fun validateQuantity(context: Context, item: IroProductDetailsPdp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean, mode: Int, shopContext: ShopContext, quantityController: IQuantityController): Pair<BigDecimal, BigDecimal> {
        return productHelper.validateQuantity(context, item, specialOffersTagIsClickable, isBigIcon, mode, shopContext, quantityController)
    }

    override fun getCartAnalytics(context: Context, position: Int, activeSort: Int, searchInfoModel: SearchInfoModel, firstCartAdd: Boolean, shelfListItem: ShelfListItem, pageName: String?): CartItemAnalyticsInfo {
        return productHelper.getCartAnalytics(context, position, activeSort, searchInfoModel, firstCartAdd, shelfListItem, pageName)
    }

    override fun getQuantityText(context: Context, item: IroProductDetailsPlp.Items, viewInfoModel: QuantityControllerViewInfoModel, isAvailable: Boolean): String? {
        val shelfListItem = ShelfListItem.buildShelfObject(item, viewInfoModel.specialOffersTagIsClickable, viewInfoModel.isBigIcon)
        return getQuantityText(context, shelfListItem, viewInfoModel, isAvailable)
    }

    override fun getQuantityText(context: Context, item: IroProductDetailsPdp.Items, viewInfoModel: QuantityControllerViewInfoModel, isAvailable: Boolean): String? {
        val shelfListItem = ShelfListItem.buildShelfObjectPdp(item, viewInfoModel.specialOffersTagIsClickable)
        return getQuantityText(context, shelfListItem, viewInfoModel, isAvailable)
    }

    private fun getQuantityText(context: Context, shelfListItem: ShelfListItem, viewInfoModel: QuantityControllerViewInfoModel, isAvailable: Boolean): String? {
        var cartItem: Cart.CartItem? = null
        var cxoCartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems? = null
        if (FeatureSettings.isCXOMigrationEnabled(context)) {
            cxoCartItem = cxoHelper.getItemInCartCXO(shelfListItem, viewInfoModel.isShoppingList)
        } else {
            cartItem = getCartItemFrom(context, shelfListItem, viewInfoModel.mode, viewInfoModel.isShoppingList)
        }
        var quantityText = ""
        if (isAvailable || viewInfoModel.isShoppingList) {
            if (FeatureSettings.isCXOMigrationEnabled(context) && cxoCartItem != null && cxoCartItem.quantity != null) {
                quantityText = RestUtils.getCXOItemQuantity(cxoCartItem, viewInfoModel.isShoppingList, false)
            } else {
                cartItem?.let {
                    quantityText = cartItem.quantityString
                }
            }
            if (ProductDetails.PRICE_PER_WEIGHT_WEIGHTED.equals(shelfListItem.pricePerWt, ignoreCase = true)) {
                quantityText += " kg"
            }
        } else {
            quantityText = context.getString(R.string.add)
        }
        if (quantityText.isEmpty()) {
            quantityText = context.getString(R.string.add)
        }
        return quantityText
    }

    override fun getFilterGroups(): ArrayList<FilterGroup> {
        return filterGroups
    }

    override fun setFilterGroups(filters: MutableList<FilterGroup>) {
        getFilterGroups().clear()
        getFilterGroups().addAll(filters)
    }

    override fun launchFilter(activity: Activity, requestCode: Int, sourceScreenName: String?) {
        if (filterGroups.isNotEmpty()) {
            val intent = Intent(activity, FiltersActivity::class.java)
            intent.putExtra(SOURCE_SCREEN_NAME, sourceScreenName)
            activity.startActivityForResult(intent, requestCode)
        } else {
            Log.e(TAG, "Please set filterGroups before launching FiltersActivity")
        }
    }

    override fun handleAlert(context: Context, code: Int) {
        context?.let { _context ->
            if (code == CartManager.PRODUCT_SLOT_LIMIT_REACHED) {
                cxoHelper.showQuantityCheckDialog(context)
            }
        }
    }

    override fun getItemInCartCXOFromItemId(itemId: String): com.asda.android.restapi.service.data.cart.Cart.CartItems? {
        return cxoHelper.getItemInCartCXOFromItemId(itemId)
    }

    private fun shouldReplacePDPFragment(sourceFragment: Fragment) = when (sourceFragment) {
        is ShelfFragment -> false
        is SearchResultsFragment -> false
        else -> true
    }
}
