package com.asda.android.app.shop.specialoffers.viewmodel

import android.content.Context
import androidx.annotation.VisibleForTesting
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.app.main.ApplicationComponent
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyPageEventModel
import com.asda.android.app.shop.taxonomy.parser.DimensionIdParser
import com.asda.android.app.shop.taxonomy.viewmodel.TaxonomyViewModel
import com.asda.android.restapi.service.data.cms.Content
import com.asda.android.restapi.service.data.cms.Personalization
import com.asda.android.restapi.service.data.cms.Refinement

open class SpecialOffersViewModel : TaxonomyViewModel() {
    private val USER_SERGMENT = "|mobileAllOffersP13N"
    private val USER_SERGMENT_OFFERS_ORCH = "|mobileAllOffersP13NOrchestration"
    private val category = Refinement.TYPE_SPECIAL_OFFERS
    val prefix = "special offers"
    val pageEventModel = TaxonomyPageEventModel()

    fun getCategory(refinement: Refinement? = null): String {
        var result = category
        refinement?.let {
            it.properties?.let { properties ->
                properties.categoryType?.let { type ->
                    result = type
                }
            }
        }
        return result
    }

    fun getDimensionId(refinement: Refinement?): String? {
        return refinement?.let { DimensionIdParser().parse(it.dimensionId) }
    }

    fun getUserSegment(isSpecialOffers: Boolean, appComponent: ApplicationComponent): String? {
        return if (isSpecialOffers && isAllOffersP13NEnabled(appComponent.applicationContext) && isLoggedIn()) {
            if (appComponent.sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_ALL_OFFERS_VARIATION_ID)) {
                USER_SERGMENT
            } else {
                USER_SERGMENT_OFFERS_ORCH
            }
        } else {
            null
        }
    }

    @VisibleForTesting
    fun isAllOffersP13NEnabled(context: Context): Boolean {
        return FeatureSettings.isAllOffersP13NEnabled(context)
    }

    fun updatePersonalizationContents(contents: Array<Content>?): Boolean {
        var isUpdated = false
        contents?.let {
            for (content in contents) {
                if (content is Personalization) {
                    isUpdated = true
                    content.allowNetworkCall = false
                    content.viewInfo.isHorizontal = false
                    content.viewInfo.isShowTitle = false
                }
            }
        }
        return isUpdated
    }

}
