package com.asda.android.app.shop.taxonomy.formatter

import androidx.annotation.VisibleForTesting
import com.asda.android.app.shop.taxonomy.model.TaxonomyIdFormatterModel
import com.asda.android.app.shop.taxonomy.parser.AncestorDimensionIdParser
import com.asda.android.app.shop.topoffers.viewmodel.TopOffersViewModel
import com.asda.android.base.core.formatter.IFormatter
import com.asda.android.restapi.service.data.cms.Refinement

class TaxonomyIdFormatter : IFormatter<TaxonomyIdFormatterModel, String> {
    @VisibleForTesting
    val ancestorDimensionIdParser = AncestorDimensionIdParser()
    private val separator = ">"
    private val spaceSeparator = " "
    private val typeComparator = RefinementTypeComparator()
    override fun format(input: TaxonomyIdFormatterModel): String {
        if (input.prefix == TopOffersViewModel.taxonomyName) {
            return input.prefix
        }
        var taxonomyTypeIdentifier = ""
        input.taxonomyType?.let { taxonomyType ->
            if (taxonomyType.isNotEmpty()) {
                taxonomyTypeIdentifier = when (taxonomyType) {
                    Refinement.TYPE_AISLE, Refinement.TYPE_DEPT, Refinement.TYPE_CAT, Refinement.TYPE_SHELF -> taxonomyType.substring(0, 1).toLowerCase()
                    else -> taxonomyType
                }
            }
        }
        val inputInfoList = mutableListOf(input.prefix, taxonomyTypeIdentifier)
        input.ancestorList?.let {
            inputInfoList.addAll(ancestorDimensionIdParser.parse(it.sortedWith(typeComparator)).distinct())
        }
        if(!inputInfoList.contains(input.dimensionId)) {
            inputInfoList.add(input.dimensionId)
        }
        return inputInfoList.filter { !it.isNullOrEmpty() }.joinToString(spaceSeparator+separator +spaceSeparator)
    }
}
