package com.asda.android.app.shop.taxonomy.repository

import androidx.annotation.VisibleForTesting
import com.asda.android.taxonomy.features.taxonomy.datasource.ITaxonomyDataSource
import com.asda.android.app.shop.taxonomy.model.*
import com.asda.android.taxonomy.features.taxonomy.repository.TaxonomyPageViewTrackerRepository
import com.asda.android.taxonomy.features.taxonomy.model.IEvent
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyP13NInfoModel
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyPageViewInfoModel
import com.asda.android.taxonomy.features.taxonomy.repository.TaxonomyP13NImpressionTrackerRepository

/**
 * Performs background operations e.g execute pageView, p13nImpression
 */
class TaxonomyRemoteRepository : ITaxonomyDataSource {
    @VisibleForTesting
    val p13NImpressionTrackerRepository = TaxonomyP13NImpressionTrackerRepository()
    @VisibleForTesting
    val pageViewTrackerRepository = TaxonomyPageViewTrackerRepository()
    val ancestorRepository = GetAncestorRepository()
    @VisibleForTesting
    val apiErrorTrackerRepository = ApiErrorTrackerRepository()

    override fun execute(eventInfo: IEvent) {
        when (eventInfo) {
            is TaxonomyPageViewInfoModel -> pageViewTrackerRepository.execute(eventInfo)
            is TaxonomyP13NInfoModel -> p13NImpressionTrackerRepository.execute(eventInfo)
            is GetAncestorEvent -> ancestorRepository.execute(eventInfo)
            is APIErrorEventModel -> apiErrorTrackerRepository.execute(eventInfo)
        }
    }
}
