package com.asda.android.app.shop.taxonomy.view

import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.shop.ShelfRecipesTabbedFragment
import com.asda.android.app.shop.formatter.TaxonomyPathEndFormatter
import com.asda.android.base.interfaces.ISearchManager
import com.asda.android.restapi.search.model.AsdaSearchData

object SearchManager : ISearchManager {

    private val taxonomyPathEndFormatter = TaxonomyPathEndFormatter()

    override fun onSearchRequested(
          searchData: AsdaSearchData,
          taxonomyPath: String
    ): Pair<Fragment, String> {

        val fragmentBase =
            ShelfRecipesTabbedFragment.newInstance(
                searchData,
                false
            )
        fragmentBase.setHeaderText(
            AsdaApplication.getComponent().application.getString(
                R.string.search_title,
                searchData.searchText
            )
        )
        fragmentBase.setTaxonomy(getTaxonomy(taxonomyPath))
        return Pair(fragmentBase, getBreadCrumbTitle(searchData))
    }

    private fun getTaxonomy(taxonomyPath: String): String? {
        return when (val searchPage = "search") {
            taxonomyPathEndFormatter.format(taxonomyPath) -> taxonomyPath
            else -> AsdaAnalyticsEvent.concatStringArgs(taxonomyPath, searchPage)
        }
    }

    private fun getBreadCrumbTitle(searchData: AsdaSearchData): String {
        return when {
            searchData.isBarcodeScanSearch -> AsdaApplication.getComponent().application
                .getString(R.string.scan_barcode_shelf_header_text)
            else -> AsdaApplication.getComponent().application
                .getString(R.string.search_title, searchData.searchText)
        }
    }
}
