package com.asda.android.app.shop.taxonomy.view

import android.app.Activity
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.asda.android.app.shop.MerchandisingHandler
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.interfaces.ITaxonomyManager
import com.asda.android.restapi.cms.model.Refinement
import com.asda.android.restapi.constants.EventConstants
import com.asda.android.taxonomy.AsdaTaxonomyConfig
import com.asda.android.taxonomy.features.taxonomy.TaxonomyConstants.CATEGORY_ID
import com.asda.android.taxonomy.features.taxonomy.TaxonomyConstants.IS_SPECIAL_OFFER
import com.asda.android.taxonomy.features.taxonomy.TaxonomyConstants.TAXONOMY_PATH
import com.asda.android.taxonomy.features.taxonomy.helper.view.TaxonomyViewHelper
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyViewInfoModel
import com.asda.android.taxonomy.features.taxonomy.view.BaseTaxonomyFragment


object TaxonomyManager : ITaxonomyManager {
    override fun handleClickEvent(activity: Activity?, fragmentManager: FragmentManager?, additionalInfo: MutableMap<String, Any?>) {
        if (activity == null || fragmentManager == null) {
            return
        }
        var type: String? = null
        var id: String? = null
        var campaignId: String? = null
        additionalInfo[EventConstants.PAGE_ID].tryCast<String> { type = this  }
        additionalInfo[EventConstants.DIMENSION_ID].tryCast<String> { id = this  }
        additionalInfo[EventConstants.CAMPAIGN_ID].tryCast<String> { campaignId = this  }

        MerchandisingHandler.handleMerchandisingEvent(activity, fragmentManager, type, id, campaignId)
    }

    override fun launchTaxonomy(source: Fragment?, refinements: ArrayList<Refinement>?, additionalInfo: Map<String, String?>?) {
        var fragment: BaseTaxonomyFragment? = null
        var title: String? = null
        refinements?.last()?.let {
            val dimensionId = it.dimensionId?.toLongOrNull()
            val taxonomyId = it.taxonomyId?.toLongOrNull()
            val storeId = AsdaTaxonomyConfig.asdaService.lastStoreIdFromSessionMetadata
            title = it.name
            val categoryId = additionalInfo?.get(CATEGORY_ID)
            val taxonomyPath = getTaxonomyPathFromRefinements(refinements)
            val campaignId = additionalInfo?.get(EventConstants.CAMPAIGN_ID)
            val specialOffer = additionalInfo?.get(IS_SPECIAL_OFFER).orEmpty()
            val taxonomyViewInfoModel = TaxonomyViewInfoModel(dimensionId = dimensionId, taxonomyId = taxonomyId, title = title, storeId = storeId, taxonomyPath = taxonomyPath, bannerCampaignId = campaignId, categoryId = categoryId, isSpecialOffers = specialOffer.toBoolean(), refinements = refinements)
            fragment = TaxonomyViewHelper().getFragment(taxonomyViewInfoModel, it.type)
        }
        source.tryCast<BaseFragment> {
            fragment?.let { taxonomyFragment ->
                this.push(taxonomyFragment, title)
            }
        }
    }

    private fun getTaxonomyPathFromRefinements(refinements: ArrayList<Refinement>?): String? {
        val taxonomyPath = refinements?.map { it.name }?.joinToString(separator = " > ")
        return taxonomyPath?.let {
            return "shop > $it"
        }
    }
}
