package com.asda.android.app.shop.taxonomy.viewmodel

import android.content.Context
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.asda.android.app.main.ApplicationComponent
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.taxonomy.formatter.EventFormatterProvider
import com.asda.android.taxonomy.features.taxonomy.model.IEvent
import com.asda.android.app.shop.taxonomy.repository.TaxonomyRemoteRepository
import com.asda.android.app.shop.taxonomy.utils.getOffersAisleCampaignId
import com.asda.android.app.shop.taxonomy.utils.getOffersAisleVariationId
import com.asda.android.base.core.formatter.BygPageViewInfoFormatter
import com.asda.android.base.core.listener.ITypedListener
import com.asda.android.base.core.listener.TypedLiveDataListener
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.restapi.service.data.cms.Refinement
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.service.base.Authentication
import com.asda.android.taxonomy.AsdaTaxonomyConfig

open class TaxonomyViewModel : ViewModel() {
    @VisibleForTesting
    val dataSource = TaxonomyRemoteRepository()
    val p13nInfoFormatter = BygPageViewInfoFormatter()
    val personalizationResponseLiveData: MutableLiveData<BaseStateResponse<PersonalizationResponse>> = MutableLiveData()
    val personalizationResponseBaseTypeListener: ITypedListener<BaseStateResponse<PersonalizationResponse>> = TypedLiveDataListener(personalizationResponseLiveData)
    @VisibleForTesting
    val eventFormatter = EventFormatterProvider(dataSource)

    private val USER_SERGMENT = "|p13nOffers"

    fun execute(eventInfo: IEvent?) {
        eventInfo?.let {
            eventFormatter.format(it)
            dataSource.execute(it)
        }
    }


    fun getAnalyticsTaxonomyType(catType: String?): String {
        if (catType != null) {
            when (catType) {
                Refinement.TYPE_AISLE -> return "Aisle"
                Refinement.TYPE_CAT -> return "Category"
                Refinement.TYPE_DEPT -> return "Department"
                Refinement.TYPE_SPECIAL_OFFERS -> return "Special Offers"
            }
        }
        return ""
    }

    fun isValidPersonalizationResponse(stateResponse: BaseStateResponse<PersonalizationResponse>, pageId: String?): Boolean {
        return (stateResponse.getState() == State.SUCCESS && stateResponse.metaData != null && stateResponse.metaData == pageId)
    }

    fun getUserSegment(appComponent: ApplicationComponent): String? {
        return if (isP13NOffersOnAisleEnabled(appComponent.applicationContext) && isLoggedIn()) {
            USER_SERGMENT
        } else {
            null
        }
    }

    @VisibleForTesting
    fun isP13NOffersOnAisleEnabled(context: Context): Boolean {
        return AsdaTaxonomyConfig.featureSettingManager.isP13NOffersOnAisleEnabled(context) == true
    }

    fun isLoggedIn(): Boolean {
        return (Authentication.getInstance() as Authentication).isLoggedIn
    }

    fun getCurrentTaxonomyLabel(): String? {
        return dataSource.ancestorRepository.label
    }
}
