package com.asda.android.app.shop.view

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Rect
import android.os.Handler
import android.os.Looper
import android.text.Spannable
import android.text.Spanned
import android.text.style.StrikethroughSpan
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.Group
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.constants.AdConstants.BKPARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.TAPARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.TEPARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.TNPARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.TRACKING_ABK
import com.asda.android.admonetization.criteo.constants.AdConstants.TRACKING_ABK_BTN
import com.asda.android.admonetization.criteo.constants.AdConstants.TRACKING_QTY_CHANGE
import com.asda.android.admonetization.criteo.AdTrackingServiceQuery
import com.asda.android.admonetization.criteo.AdUtils
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.dynamic.DynamicPageFragment
import com.asda.android.app.shop.ShelfListAdapter
import com.asda.android.base.core.view.ui.ItemQuantityController.createCartItem
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.cxo.viewmodel.EXTRA_PAYMENT_AUTH_BANNER_VISIBLE
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.products.ui.promos.view.AsdaPromoBannerView
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.Callback
import com.asda.android.restapi.service.base.Iconography
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import com.asda.android.utils.getBannerViewType
import com.asda.android.utils.view.ViewUtil
import java.math.BigDecimal
import java.util.*


class BannerViewHolder(itemView: View, viewType: Int, var merchandisingClickListener: ShelfListAdapter.OnMerchandisingClickListener?, var onItemClickListener: ShelfListAdapter.OnShelfItemClickListener?) : ShelfListAdapter.ShelfItemViewHolder(itemView, viewType, false), View.OnClickListener {
    constructor(itemView: View, viewType: Int, actionHandler: DynamicPageFragment.OnActionHandler) : this(itemView, viewType, null, null) {
        this.homePageBanner = true
        this.merchandisingClickListener = ShelfListAdapter.OnMerchandisingClickListener { merchandising, merchandisingView ->
            actionHandler.onMerchandisingClicked(merchandising, merchandisingView)
        }
        this.onItemClickListener = ShelfListAdapter.OnShelfItemClickListener { shelfItem, viewHolder ->
            actionHandler.onSimilarClicked(shelfListBannerItem.bannerItem)
        }
    }

    lateinit var imageView: ImageView
    var homePageBanner = false
    private val description: TextView
    private val priceSupplementInfo: TextView
    private val itemPrice: TextView
    private val itemPriceDecimals: TextView
    private val itemPencePrice: TextView
    private val itemUnit: TextView
    private val promoIconOverlay: TextView
    private val bwsItemDisclaimer: TextView
    lateinit var itemButton: Button
    private val bannerPromoView: AsdaPromoBannerView?
    private val promoIcon: View
    lateinit var containerLayout: ConstraintLayout
    private val imageFullView: ImageView = ViewUtil.findViewById(itemView, R.id.banner_image_full_merchandising_view)
    private val promoIconAlcohol: View
    private val currencyView: View
    private var iconography: Iconography? = null
    private val hideIconographyOverlayCallbacks = WeakHashMap<View, Runnable>()
    private val handler = Handler(Looper.getMainLooper())
    lateinit var shelfListBannerItem: Merchandising.Banner
    private var itemViewGroup: Group? = null
    private lateinit var context: Context
    var sendViewTracking: Boolean = false
    var trackingOnAddButton: Boolean = false
    lateinit var internalAdapter: ShelfListAdapter

    private val iconographyCallback = object : Callback<Iconography>() {
        override fun onFinished(iconographyRet: Iconography) {
            iconography = iconographyRet
            handler.post { }
        }
    }


    init {
        imageView = ViewUtil.findViewById(itemView, R.id.banner_merchandising_view)
        description = ViewUtil.findViewById(itemView, R.id.banner_item_description)
        itemPrice = ViewUtil.findViewById(itemView, R.id.banner_item_price)
        itemPriceDecimals = ViewUtil.findViewById(itemView, R.id.banner_item_price_superscript)
        itemUnit = ViewUtil.findViewById(itemView, R.id.banner_item_each)
        bannerPromoView = ViewUtil.findViewById(itemView, R.id.promotion_tag)
        priceSupplementInfo = ViewUtil.findViewById(itemView, R.id.item_price_supplement_info)
        promoIcon = ViewUtil.findViewById(itemView, R.id.promo_icon)
        promoIconAlcohol = ViewUtil.findViewById(itemView, R.id.promo_icon_alcohol)
        itemButton = ViewUtil.findViewById(itemView, R.id.add_button_cart)
        itemViewGroup = ViewUtil.findViewById(itemView, R.id.banner_item_view_group)
        promoIconOverlay = ViewUtil.findViewById(itemView, R.id.promo_icon_overlay)
        bwsItemDisclaimer = ViewUtil.findViewById(itemView, R.id.bws_item_disclaimer)
        currencyView = ViewUtil.findViewById(itemView, R.id.banner_item_currency)
        itemPencePrice = ViewUtil.findViewById(itemView, R.id.banner_item_pence_price)
        containerLayout = ViewUtil.findViewById(itemView, R.id.banner_merchandising_container)
        sendViewTracking = false
        trackingOnAddButton = true
    }

    @SuppressLint("CheckResult")
    fun updateViews(banner: Merchandising.Banner?, contx: Context) {
        context = contx
        var shelfListItem: ShelfListItem? = null
        if (banner != null) {
            shelfListBannerItem = banner
            shelfListItem = shelfListBannerItem.bannerItem
        }

        if (homePageBanner) {
            this.internalAdapter = ShelfListAdapter(context, false, this)
            if (shelfListBannerItem.bannerItem != null) {
                this.internalAdapter.setupAddButton(this, shelfListBannerItem.bannerItem, shelfListBannerItem.buttonShopNow)
            }
        }

        // setting up ids to identify if they are shown on the screen.
        itemViewGroup?.id = R.id.banner_item_view_group
        imageFullView?.id = R.id.banner_image_full_merchandising_view

        if (shelfListItem != null) {
            itemViewGroup?.visibility = View.VISIBLE
            imageFullView?.visibility = View.INVISIBLE
            var shelfListItemCommons = ShelfListAdapterCommons(shelfListItem)
            var promoStr = ""
            if (bannerPromoView != null && !shelfListItemCommons.isBundle()) {
                bannerPromoView.visibility = View.GONE
                bannerPromoView.setOnClickListener(null)
                if ((shelfListItem.promotionInfo != null) && (shelfListItem.promotionInfo?.displayTextForRectanglePromo != null)) {
                    val asdaPromoType = getBannerViewType(shelfListItem.promotionInfo!!)
                    if (asdaPromoType != AsdaPromoBannerView.PromoType.PROMO_OFFER_UNDEFINED)
                        (bannerPromoView as AsdaPromoBannerView).populatePromo(asdaPromoType, shelfListItem.promotionInfo?.displayTextForRectanglePromo)
                    else {
                        (bannerPromoView as AsdaPromoBannerView).setPromoText(shelfListItem.promotionInfo?.displayTextForRectanglePromo!!)
                    }
                    bannerPromoView.setClickableView(false)
                    bannerPromoView.isClickable = false
                    bannerPromoView.visibility = View.VISIBLE
                    promoStr = shelfListItem.promotionInfo?.displayTextForRectanglePromo.toString()
                }
            }
            description.text = shelfListItem.title
            val price = shelfListItemCommons.getPriceToDisplay()
            populatePrice(price?.substringAfter(context.getString(R.string.ad_monetization_pound)))  // overwritten if in next logic, if item is in promotion of type 15

            if (shelfListItem.pricePerWt.equals(context.getString(R.string.ad_monetization_both)))
                itemUnit.text = shelfListItem?.pricePerUOM
            else
                itemUnit.text = shelfListItem?.weight

            // Supplement to Price label
            priceSupplementInfo.visibility = View.GONE
            shelfListItemCommons.populateWasPrice(priceSupplementInfo, null, null, null, null, null, false, context)
            if (!priceSupplementInfo.text.isNullOrEmpty()) {
                priceSupplementInfo.setText(context.getString(R.string.was_price_pretext) + " " + priceSupplementInfo.text, TextView.BufferType.SPANNABLE)
                val spannable = priceSupplementInfo.text as Spannable
                spannable.setSpan(StrikethroughSpan(), 0, spannable.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                priceSupplementInfo.visibility = View.VISIBLE
            }
            // linksave offer like "2 for £3"
            var linkSaveStr = ""
            if ((shelfListItem.promotionInfo?.promoOfferTypeCode?.toInt() == AsdaPromoBannerView.PromoType.PROMO_OFFER_TYPE_15.value) && (shelfListItem.promotionInfo?.displayTextForMeatSticker?.isNotEmpty() == true && shelfListItem.promotionInfo?.displayTextForMeatSticker?.contains("£") == true)) {
                priceSupplementInfo.setText(shelfListItem.promotionInfo?.displayTextForMeatSticker!!.substringBefore(context.getString(R.string.ad_monetization_space_pound)))
                priceSupplementInfo.visibility = View.VISIBLE
                populatePrice(shelfListItem.promotionInfo?.displayTextForMeatSticker!!.substringAfter(context.getString(R.string.ad_monetization_pound)))
                linkSaveStr = priceSupplementInfo.text.toString()
            }

            var iconDescription = ""
            if (Looper.myLooper() == Looper.getMainLooper() && shelfListItem.promoIconId != null) {
                if (iconography == null) {
                    iconography = Iconography.getInstance(context, iconographyCallback, false, AsdaServiceFactory.get())
                }
                if (shelfListItem.promoIconId == context.getString(R.string.promo_icon_alcohol)) {
                    promoIcon.visibility = View.GONE
                    bwsItemDisclaimer.visibility = View.VISIBLE
                    promoIconAlcohol.visibility = View.VISIBLE
                } else {
                    promoIcon.visibility = View.VISIBLE
                    iconography?.let { shelfListItemCommons.loadIconographyIconImage(iconography, promoIcon, context, null, hideIconographyOverlayCallbacks, handler) }
                    promoIconAlcohol.visibility = View.GONE
                    bwsItemDisclaimer.visibility = View.GONE
                    iconDescription = iconography?.getIcon(context, shelfListItem.promoIconId)?.description.toString()
                }
            }
            imageView.setOnClickListener(this)
            imageView.contentDescription = shelfListItem.title
            val description = "$linkSaveStr $promoStr ${description?.text} ${priceSupplementInfo?.text} ${context.getString(R.string.ally_priced)} $price ${itemUnit?.text} $iconDescription ${context.getString(R.string.subject_to_availability)}"
            itemViewGroup?.contentDescription = description
            itemViewGroup?.setOnClickListener(this)
        } else if (shelfListBannerItem?.bannerDisplayType?.contains(AdConstants.STATIC) == true) {
            ImageLoader.load(context, imageFullView, ImageLoaderRequest.Builder().path(banner?.merchandisingimage).build(), null)
            itemViewGroup?.visibility = View.INVISIBLE
            imageFullView?.visibility = View.VISIBLE
            imageFullView?.contentDescription = banner?.let { AdUtils().getDescriptionFromMerchandisingBanner(it) }
            imageFullView.setOnClickListener(this)
        }

        /* Send the tracking details to Criteo. */
        if (trackingOnAddButton) {
            sendViewTracking = true
            AdTrackingServiceQuery().sendBannerViewEvent(shelfListBannerItem.trackingInformation)
        }
    }

    // Invoked from outside
    fun onAttachedToRecyclerView(recyclerView: RecyclerView, count: Int) {
        sendViewTracking = true
        val manager = recyclerView.layoutManager
        if (manager !is LinearLayoutManager || count <= 0) return
        val llm = manager as LinearLayoutManager?
        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {

            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                val visiblePosition = llm!!.findFirstCompletelyVisibleItemPosition()

                if (visiblePosition > -1) {
                    val v = llm.findViewByPosition(visiblePosition)
                    if ((v?.findViewById<View>(R.id.banner_item_view_group) != null || v?.findViewById<View>(R.id.banner_image_full_merchandising_view) != null) && (shelfListBannerItem != null)) {
                        /* Send the tracking details to Criteo. */
                        val scrollBounds = Rect()
                        recyclerView.getHitRect(scrollBounds)
                        if (v.getLocalVisibleRect(scrollBounds) && sendViewTracking) {
                            AdTrackingServiceQuery().sendBannerViewEvent(shelfListBannerItem.trackingInformation)
                            sendViewTracking = false
                        }
                    }
                } else {
                    sendViewTracking = true
                }
            }
        })
    }

    private fun populatePrice(price: String?) {
        if (price == null)
            return
        if ((price.indexOf(".") != -1) && price?.substring(0, price?.indexOf("."))?.contains("0", false)) {
            itemPencePrice.visibility = View.VISIBLE
            itemPrice.visibility = View.INVISIBLE
            itemPriceDecimals.visibility = View.INVISIBLE
            currencyView.visibility = View.INVISIBLE
            itemPencePrice.text = price.substring(price.indexOf(".") + 1) + "p"
        } else {
            itemPencePrice.visibility = View.INVISIBLE
            currencyView.visibility = View.VISIBLE
            if (price != null && price.indexOf(".") != -1) {
                itemPrice.visibility = View.VISIBLE
                itemPriceDecimals.visibility = View.VISIBLE
                itemPrice.text = price.substring(0, price.indexOf("."))
                itemPriceDecimals.text = price.substring(price.indexOf(".") + 1)
            } else {
                itemPrice.visibility = View.INVISIBLE
                itemPriceDecimals.visibility = View.INVISIBLE
                currencyView.visibility = View.INVISIBLE
            }
        }
    }

    override fun onClick(view: View?) {
        when (view?.id) {
            R.id.banner_merchandising_view ->
                bannerClickEvent(view)

            R.id.banner_image_full_merchandising_view ->
                bannerClickEvent(view)

            R.id.banner_item_view_group ->
                bannerClickEvent(view)
        }
    }

    @SuppressLint("CheckResult")
    fun bannerClickEvent(v: View) {
        if (v.id == R.id.add_button_cart && shelfListBannerItem?.bannerDisplayType?.contains(AdConstants.DYNAMIC) == true) {
            showItemPDPView()
        } else if (shelfListBannerItem != null && shelfListBannerItem.link != null) {
            if (!shelfListBannerItem.link?.path.isNullOrEmpty()) {
                if(shelfListBannerItem.type.equals(Merchandising.TYPE_PRODUCT_DETAIL)){
                    shelfListBannerItem.productid = shelfListBannerItem.link?.queryString
                }
                merchandisingClickListener?.onMerchandisingClicked(shelfListBannerItem, imageView)
            } else {
                if (shelfListBannerItem.bannerItem != null) {
                    showItemPDPView()
                }
            }
        }
    }

    private fun showItemPDPView() {
        onItemClickListener?.onShelfItemClicked(shelfListBannerItem.bannerItem, this)
        AdTrackingServiceQuery().sendBannerClickEvent(shelfListBannerItem)
    }

}

