package com.asda.android.app.shop.view

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.StrikethroughSpan
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderCallback
import com.asda.android.imageloader.ImageLoaderListener
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.products.ui.product.tile.view.OnIconographyIconClickListener
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.data.ProductDetails
import com.asda.android.restapi.service.data.ShelfItem
import com.asda.android.restapi.service.base.Iconography
import java.util.*

open class ShelfListAdapterCommons(item: ShelfListItem) {
    var shelfListItem : ShelfListItem = item
    fun isBundle(): Boolean {
        return shelfListItem.availability == ShelfItem.AVAILABLE && (shelfListItem.bundledItemCount > 0)
    }

    fun getPriceToDisplay(): String? {
        return if (shelfListItem.availability == ShelfItem.AVAILABLE && shelfListItem.bundledItemCount == 0 && shelfListItem.onSale) {
            shelfListItem.salePrice
        } else {
            shelfListItem.priceToDisplay
        }
    }

    fun showSaleSticker(): Boolean {
        return shelfListItem.availability == ShelfItem.AVAILABLE && shelfListItem.bundledItemCount == 0 && shelfListItem.onSale && !shelfListItem.isFromCart
    }

    fun populateWasPrice(wasPriceView: TextView, itemCost: TextView?, arrowView: ImageView?,
                                 wasPricePrefixView: View?, pricePrefixView: View?, shelfItemWeight: TextView?, usesCardItems: Boolean?, context: Context?) {
        val isComingSoon = shelfListItem.availability == ShelfItem.COMING_SOON
        val showFutureProductPrice = "Yes" == shelfListItem.showFutureProductPrice

        if (showSaleSticker()) {
            val salePriceStyled = SpannableString(shelfListItem.priceToDisplay)
            salePriceStyled.setSpan(StrikethroughSpan(), 0, salePriceStyled.length, SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE)
            wasPriceView.text = salePriceStyled
            wasPriceView.visibility = View.VISIBLE
        } else {
            var wasPrice: String? = if (shelfListItem.promotionInfo == null) null else shelfListItem.promotionInfo!!.getWasPriceToDisplay(shelfListItem.wasPrice, shelfListItem.priceToDisplay)
            if (shelfListItem.availability == ShelfItem.COMING_SOON && shelfListItem.wasPrice != null) {
                wasPrice = shelfListItem.wasPrice
            }
            val hasPriceChangeInfo = shelfListItem.priceIncreased != null && (usesCardItems!! || shelfListItem.availability == ShelfItem.AVAILABLE)
            if (hasPriceChangeInfo) {
                val priceIncreased = shelfListItem.priceIncreased
                wasPrice = shelfListItem.wasPrice
                if (itemCost != null && context != null)
                    itemCost.setTextColor(ContextCompat.getColor(context, if (priceIncreased == true) R.color.text_red else R.color.asda_dark_green))
                if (arrowView != null && shelfListItem.priceIncreased != null) {
                    arrowView.setImageResource(if (priceIncreased == true) R.drawable.icn_red else R.drawable.icn_green)
                    arrowView.visibility = View.VISIBLE
                }
            }
            if (wasPricePrefixView != null) {
                wasPricePrefixView.visibility = if (hasPriceChangeInfo) View.VISIBLE else View.GONE
            }
            if (pricePrefixView != null) {
                pricePrefixView.visibility = if (hasPriceChangeInfo) View.VISIBLE else View.GONE
            }

            if (!TextUtils.isEmpty(wasPrice) && (shelfListItem.availability == ShelfItem.AVAILABLE || isComingSoon && showFutureProductPrice)) {
                val wasPriceStyled = SpannableString(wasPrice)
                wasPriceStyled.setSpan(StrikethroughSpan(), 0, wasPriceStyled.length, SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE)
                if (ProductDetails.PRICE_PER_WEIGHT_BOTH == shelfListItem.pricePerWt) {
                    if (shelfItemWeight != null) {
                        shelfItemWeight.text = ShelfListItem.concatPricePerUOMWasPrice(shelfItemWeight.text, wasPriceStyled)
                    }
                    wasPriceView.visibility = View.GONE
                } else {
                    wasPriceView.text = wasPriceStyled
                    wasPriceView.visibility = View.VISIBLE
                }
            } else {
                wasPriceView.visibility = View.GONE
            }
        }
    }

    fun loadIconographyIconImage(iconography: Iconography?, auxPromoIcon: View, context: Context, auxPromoIconOverlay: TextView?,
                                 hideIconographyOverlayCallbacks: WeakHashMap<View, Runnable>, handler: Handler = Handler(Looper.getMainLooper())) {
        val iconInfo = iconography?.getIcon(context, shelfListItem.promoIconId)
        if (iconInfo != null) {
            val callback = ImageLoaderCallback(object : ImageLoaderListener {
                override fun onSuccess() {
                    auxPromoIcon.visibility = View.VISIBLE
                    val description = iconInfo.description
                    if (!TextUtils.isEmpty(description) && auxPromoIconOverlay != null) {
                        auxPromoIcon.setOnClickListener(OnIconographyIconClickListener(description, auxPromoIconOverlay, hideIconographyOverlayCallbacks, handler))
                    }
                }

                override fun onError() {
                    auxPromoIcon.visibility = View.INVISIBLE
                }
            })
            ImageLoader.load(context, auxPromoIcon as ImageView, ImageLoaderRequest.Builder().path(iconInfo.imageUrl).width(context.getResources().getDimensionPixelSize(R.dimen.iconography_large)).height(context.getResources().getDimensionPixelSize(R.dimen.iconography_large)).build(), callback)
        }
    }

}
