package com.asda.android.app.shoppinglists


import android.content.DialogInterface
import android.graphics.drawable.BitmapDrawable
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.MenuItem
import android.view.View
import android.widget.*
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.base.core.view.ui.BaseAppCompatActivity
import com.asda.android.app.direct.UserReviewsFragment
import com.asda.android.app.shop.productdetail.view.ProductDetailsFragment.EXTRA_ITEM
import com.asda.android.base.core.view.ViewUtil
import com.asda.android.restapi.utils.parseFloat
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.favourites.features.shoppinglists.view.adapter.ExistingShoppingListAdapter
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.service.base.Authentication
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.ProductDetails
import com.asda.android.restapi.shoppinglists.ShoppingListsAPI
import com.asda.android.base.core.view.ui.ItemQuantityController
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.managing_shopping_list.*
import kotlinx.android.synthetic.main.managing_shopping_list_product_summary.*


class ManagingShoppingListActivity : BaseAppCompatActivity() {
    val screenName = "managing shopping list screen"
    private var rvExistingLists: androidx.recyclerview.widget.RecyclerView? = null
    var item: ShelfListItem? = null
    var adapter: ExistingShoppingListAdapter? = null
    var existingListHeaderLayout: LinearLayout? = null
    var existingListDivider: View? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val actionBar = supportActionBar
        actionBar?.setDisplayHomeAsUpEnabled(true)
        setContentView(R.layout.managing_shopping_list)
        val intent = intent
        item = intent.getParcelableExtra(EXTRA_ITEM)

        var itemQuantityController = ItemQuantityController(
              item?.pricePerUnit,
              item?.avgWeight,
              item?.pricePerWt,
              item?.maxQty.parseFloat(0.01f, 99f),
              true)

        adapter = ExistingShoppingListAdapter(this, itemQuantityController, ArrayList(), item)


        shopping_list_product_title.text = item?.title
        shopping_list_price_to_display.text = item?.priceToDisplay

        existingListHeaderLayout = findViewById(R.id.existing_shopping_list_header)
        existingListDivider = findViewById(R.id.existing_shopping_list_divider)
        rvExistingLists = findViewById<View>(R.id.existing_shopping_list) as androidx.recyclerview.widget.RecyclerView
        rvExistingLists?.adapter = adapter
        rvExistingLists?.layoutManager = androidx.recyclerview.widget.LinearLayoutManager(this)

        val listnameButton = findViewById<EditText>(R.id.edit_list_name)
        val addButton = findViewById<Button>(R.id.shopping_list_add_new_list_button)

        listnameButton.addTextChangedListener(object: TextWatcher{
            override fun afterTextChanged(content: Editable?) { updateWarningText(content) }
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { /* Nothing for now */ }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) { /* Nothing for now */ }
        })
        addButton.setOnClickListener {
            var qty = itemQuantityController.increaseDecreaseSteps.toString()
            if (ProductDetails.PRICE_PER_WEIGHT_WEIGHTED == item?.pricePerWt) {
                qty += ":wt"
            }
            ViewUtil.hideKeyboard(this@ManagingShoppingListActivity.currentFocus)
            addItemToNewShoppingList(listnameButton.text.toString(), qty)
            listnameButton.text.clear()
            listnameButton.clearFocus()
        }

        val rating = item?.avgStarRating
        val reviewsCount = item?.totalReviewCount

        setupOverallRatingLabel(rating, reviewsCount)
        populateWeightInfo(item?.weight, item?.pricePerUOM, item?.bundledItemCount, shopping_list_weight_info)

        if (item?.imageURL.isNullOrEmpty()) {
            shopping_list_product_image.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.icn_nophoto_lg))
        } else {
            ImageLoader.load(this, shopping_list_product_image, ImageLoaderRequest.Builder().path(item?.imageURL).build())
        }

        initExistingShoppingLists(item)

    }

    private fun updateWarningText(content: Editable?) {
        when (content?.isEmpty()) {
            true, null -> {
                add_list_to_save_warning_divider?.visibility = View.GONE
                add_list_to_save_warning_text?.visibility = View.GONE
            }
            false -> {
                add_list_to_save_warning_divider?.visibility = View.VISIBLE
                add_list_to_save_warning_text?.visibility = View.VISIBLE
            }
        }
    }


    private fun addItemToNewShoppingList(listName: String?, qty: String?) {
        val shoppingListsAPI = ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", ShoppingListsAPI::class.java) ?: return


        val requiredParamMap = HashMap<String?, String?>()
        requiredParamMap[ITEM_IDs] = item?.id
        requiredParamMap[LIST_NAME] = listName
        requiredParamMap[TOKEN] = Authentication.getInstance().csrfToken
        requiredParamMap[QTY] = qty

        safeAdd(shoppingListsAPI.addItemToNewShoppingList(requiredParamMap)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    when (response.statusCode) {
                        "0" -> {
                            initExistingShoppingLists(item)
                            Toast.makeText(this, "New list has been added!", Toast.LENGTH_LONG).show()
                        }
                        "500" -> {
                            DialogFactory.createAlertDialog(getString(R.string.error), response.firstErrorMessage, this, null).show()
                        }
                        else -> {
                            DialogFactory.createAlertDialog(getString(R.string.error), getString(R.string.oops_error), this, null).show()
                        }
                    }

                }) { error ->
                    DialogFactory.createAlertDialog(this.getString(R.string.error), this.getString(R.string.oops_error), this, null).show()
                    error.printStackTrace()
                }
        )

    }

    override fun onDestroy() {
        super.onDestroy()
        safeDispose()
    }

    private fun initExistingShoppingLists(item: ShelfListItem?) {

        val shoppingListsAPI = ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", ShoppingListsAPI::class.java) ?: return

        val requiredParamMap = HashMap<String?, String?>()
        requiredParamMap[ITEM_ID] = item?.id
        requiredParamMap[SORT_LIST_BY] = CREATION_DATE

        safeAdd(shoppingListsAPI.getAllShoppingListForItem(requiredParamMap)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (!response.statusCode.equals("0", true)) {
                        //something is wrong so hide the list and show error text
                        val clickListener = DialogInterface.OnClickListener { _, _ ->
                            finish()
                        }
                        com.asda.android.base.core.view.DialogFactory.createAlertDialog(getString(R.string.error), getString(R.string.oops_error), this, clickListener, screenName).show()
                    } else {
                        response.shoppingList?.let {
                            if (it.isNotEmpty()) {
                                existingListHeaderLayout?.visibility = View.VISIBLE
                                existingListDivider?.visibility = View.VISIBLE
                                adapter?.addList(it)
                            } else {
                                existingListHeaderLayout?.visibility = View.GONE
                                existingListDivider?.visibility = View.GONE
                            }

                        } ?: run {
                            existingListHeaderLayout?.visibility = View.GONE
                            existingListDivider?.visibility = View.GONE
                        }


                    }

                }) { error ->
                    DialogFactory.createAlertDialog(this.getString(R.string.error), this.getString(R.string.oops_error), this, null).show()
                    error.printStackTrace()
                })

    }

    private fun populateWeightInfo(weightData: String?, pricePerUOM: String?, bundledItemCount: Int?, weight: TextView) {
        val sb = StringBuilder()
        if (bundledItemCount != null && bundledItemCount > 0) {
            sb.append(this.resources.getQuantityString(R.plurals.bundle_contains, bundledItemCount, bundledItemCount))
        } else {
            if (!TextUtils.isEmpty(weightData)) {
                sb.append(weightData)
            }
            if (!TextUtils.isEmpty(pricePerUOM) && bundledItemCount == 0) {
                if (sb.isNotEmpty()) {
                    sb.append(", ")
                }
                sb.append(pricePerUOM)
            }
        }
        weight.text = sb.toString()
    }

    private fun setupOverallRatingLabel(rating: Float?, reviewsCount: Int?) {
        val res = this.resources
        if (reviewsCount != null) {
            if (reviewsCount <= 0) {
                shopping_list_total_reviews_count_label?.visibility = View.GONE
            } else {
                shopping_list_total_reviews_count_label?.text = res.getQuantityString(R.plurals.total_reviews, reviewsCount, reviewsCount)
                shopping_list_total_reviews_count_label?.setCompoundDrawablesWithIntrinsicBounds(BitmapDrawable(res, UserReviewsFragment.getImageForPrimaryRating(this, rating!!)), null, null, null)
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }

        return super.onOptionsItemSelected(item)
    }


    companion object {
        const val ITEM_ID = "itemid"
        const val ITEM_IDs = "itemids"
        const val QTY = "quantities"
        const val TOKEN = "token"
        const val LIST_NAME = "listname"
        const val SORT_LIST_BY = "sortlistby"
        const val CREATION_DATE = "creationdate asc"
    }
}
