package com.asda.android.app.view

import android.content.Intent
import android.os.Bundle
import android.widget.Toast
import com.asda.android.R
import com.asda.android.base.core.view.actitivies.AsdaAppCompatActivity
import com.asda.android.utils.view.Permissions

private val TAG = "BasePermissionActivity"
const val PERMISSION_FRAGMENT = "push_permission_fragment"

abstract class BasePermissionActivity : AsdaAppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.container_fragment)
        if (savedInstanceState == null) {
            intent?.let {
                pushFragment(intent.getStringExtra(PERMISSION_FRAGMENT), intent)
            }
        }
    }

    override fun getFragmentContainer(): Int {
        return R.id.fragment_container
    }

    abstract protected fun pushFragment(permissionFragment: String?, intent: Intent?)

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        when (requestCode) {
            Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION -> {
                Permissions.onLocationPermissionRequestReceived(this)
                //in android M preview grantResults always returns DENY even if permission was granted.
                //and there is no good way to find out that permission is permanently denied
                if (Permissions.isLocationPermissionGranted(this)) {
                    androidx.localbroadcastmanager.content.LocalBroadcastManager.getInstance(this).sendBroadcast(Intent(Permissions.LocationPermissionGrantedEvent.ACTION))
                } else {
                    Toast.makeText(this, R.string.no_permission_for_location, Toast.LENGTH_SHORT).show()
                }
            }
        }
    }
}
