package com.asda.android.service

import android.content.Context
import android.content.Intent
import androidx.localbroadcastmanager.content.LocalBroadcastManager

import com.asda.android.analytics.info.SessionMetaData

import com.asda.android.service.constants.EventConstants.KEY_SHIP_DATE
import com.asda.android.service.constants.EventConstants.KEY_SIGNIN_OR_OUT
import com.asda.android.service.constants.EventConstants.KEY_STORE_ID
import com.asda.android.service.constants.EventConstants.ON_NEW_CACHE_KEYS_RECEIVED

object EventUtils {
    fun broadcast(context: Context?, action: String) {
        context?.let {
            getBroadcastReceiver(it).sendBroadcast(Intent(action))
        }
    }

    fun broadcast(context: Context?, action: String, key: String, value: Boolean) {
        context?.let {
            getBroadcastReceiver(it).sendBroadcast(Intent(action).putExtra(key, value))
        }
    }

    fun broadcast(context: Context?, cacheKeys: SessionMetaData, signInOrSignOut: Boolean) {
        context?.let {
            getBroadcastReceiver(it).sendBroadcast(getIntentBasedOnSession(cacheKeys, signInOrSignOut))
        }
    }

    fun getIntentBasedOnSession(cacheKeys: SessionMetaData, signInOrSignOut: Boolean): Intent {
        return Intent(ON_NEW_CACHE_KEYS_RECEIVED)
                .putExtra(KEY_SIGNIN_OR_OUT, signInOrSignOut)
                .putExtra(KEY_STORE_ID, cacheKeys.storeId)
                .putExtra(KEY_SHIP_DATE, cacheKeys.shipDate)
    }

    private fun getBroadcastReceiver(context: Context): LocalBroadcastManager {
        return LocalBroadcastManager.getInstance(context)
    }
}
