package com.asda.android.service.base


import android.content.Context
import androidx.annotation.VisibleForTesting
import com.asda.android.restapi.service.base.AsdaServiceImpl
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.service.model.CartChangeLoggerInfo
import io.reactivex.Scheduler
import io.reactivex.observers.ResourceObserver

/**
 * logs cart add impression
 */

class CartAddLogger<T : CartChangeLoggerInfo>(val service: AsdaServiceImpl, val context: Context?, val io: Scheduler, val mainThread: Scheduler) : BaseCartChangeInfoLogger<T>() {

    private val PLACEMENT_ID = "add_to_cart_page.minicart01"
    private val limit = "1"
    private val skuIdSeparator = "|"
    private val separator = ","

    override fun postData(data: T): Boolean {
        val itemId = if (data.ids.isNullOrEmpty()) "" else data.ids.split(separator)[0]
        val cartContents = getCartContent(data)
        var result = false
        service.getPersonalizationPlacements(PLACEMENT_ID, "", limit, cartContents, data.pageId, itemId, "", "", false, Authentication.getInstance().loginResponse, data.isAmending
              ?: false, "", "").subscribeOn(io)
              .observeOn(mainThread)
              .subscribe(object : ResourceObserver<PersonalizationResponse>() {
                  override fun onComplete() {
                      dispose()
                  }

                  override fun onNext(t: PersonalizationResponse) {
                      result = true
                  }

                  override fun onError(e: Throwable) {
                      dispose()
                  }

              })
        return result
    }

    @VisibleForTesting
    fun getCartContent(data: T): String {
        val cartContents = mutableSetOf<String>()
        data.cartItems?.let {
            for (item in it) {
                item.id?.let { itemId ->
                    cartContents.add(itemId)
                }
            }
        }
        data.ids?.let {
            it.split(separator).forEach { itemId -> cartContents.add(itemId) }
        }
        return cartContents.joinToString(skuIdSeparator)
    }


    override fun isValid(data: T): Boolean {
        return data.cmd == CartManager.EditCardRequest.CMD_ADD
    }
}
