package com.asda.android.service.repository

import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotCXORequest
import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotCXOResponse
import com.asda.android.restapi.service.data.cart.Items
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers

class CartRepository {

    private val cxoCartManager = com.asda.android.cxo.CartManager.instance

    fun findCart(): Single<Boolean>? {
        return cxoCartManager?.findCart()?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun addItem(itemList: java.util.ArrayList<Any>, spotlightValue: String?, location: String?): Single<Boolean>? {
        return cxoCartManager?.addItem(itemList, spotlightValue, location)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun removeItem(cartItemId: String, unitPrice: Double, quantity: Double): Single<Boolean>? {
        return cxoCartManager?.removeItem(cartItemId, unitPrice, quantity)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun updateItem(cartItemId: String, unitPrice: Double, newQuantity: Double, items: Items, location: String?): Single<Boolean>? {
        return cxoCartManager?.updateItem(cartItemId, unitPrice, newQuantity, items, location)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun reserveSlot(cartId: String?, deliveryImpact: Boolean, reserveSlotCXORequest: ReserveSlotCXORequest): Single<ReserveSlotCXOResponse>? {
        return cxoCartManager?.reserveSlot(cartId, deliveryImpact, reserveSlotCXORequest)
    }

    fun addUpdateShoppingList(itemList: ArrayList<Any>): Single<Any>? {
        return cxoCartManager?.addUpdateShoppingList(itemList)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }
}
