package com.asda.android.utils

import android.content.Context
import android.text.Html
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.ImageSpan
import android.view.View
import androidx.appcompat.widget.AppCompatTextView
import com.asda.android.R
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.getFormattedDateWithDay
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.ConfirmBasketResponse
import com.asda.android.service.base.CartManager

class ExpressUtils {

    companion object {

        fun showExpressInfo(lineItemCount: String, totalQuantity: String): Boolean {
            return lineItemCount.toInt() <= 0 && totalQuantity.toInt() <= 0
        }

        fun getExpressVisibilityForOrder(confirmBasketResponse: ConfirmBasketResponse?): Boolean {
            return confirmBasketResponse?.isExpress.equals("true");
        }

        fun isExpressSlotBooked(cart: Cart?): Boolean {
            return if (cart?.isExpress == null) {
                false
            } else {
                (cart.isExpress.equals("true") && cart.slotExpiredFlag.equals("false") && !TextUtils.isEmpty(cart.bookedUntilTime))
            }
        }

        fun isExpressSlotBooked(cart: com.asda.android.restapi.service.data.cart.Cart?): Boolean {
            return if (cart?.cartMetadata?.expressCart == null) {
                false
            } else {
                (cart.cartMetadata?.expressCart == true && cart.fulfillmentDetails?.slotInfo?.slotExpired == false && !cart.fulfillmentDetails?.slotInfo?.bookedUntilTime.isNullOrBlank())
            }
        }

        fun getTextForReserveUntilExpressBanner(bookedUntilTime: String?, context: Context, expressImage: Int): SpannableStringBuilder {
            val spannableStringBuilder = SpannableStringBuilder()
            spannableStringBuilder.append(context.getString(R.string.express_reserve_until_beginning)).append("  ")
            spannableStringBuilder.setSpan(ImageSpan(context, expressImage), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            spannableStringBuilder.append(context.getString(R.string.express_reserve_until_end)).append(" ")
            spannableStringBuilder.append(if (FeatureSettings.isCXOMigrationEnabled(context)) {
                Html.fromHtml("<b>${bookedUntilTime?.getFormattedDateWithDay()}</b>")
            } else {
                Html.fromHtml("<b>$bookedUntilTime</b>")
            })
            return spannableStringBuilder
        }

        fun showExpressBanner(context: Context?): Boolean {
            context?.let {
                if (FeatureSettings.isCXOMigrationEnabled(context)) {
                    if (isExpressSlotBooked(com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart)) {
                        return true
                    }
                } else {
                    val cart = CartManager.get(it)?.cart
                    if(cart != null && isExpressSlotBooked(cart)) {
                        return true
                    }
                }
            }
            return false
        }

        fun getExpressIconText(context: Context?): SpannableStringBuilder {
            val spannableStringBuilder = SpannableStringBuilder()
                  .append(context?.getString(R.string.single_space))
            context?.let {
                spannableStringBuilder.setSpan(ImageSpan(it, R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            }
            spannableStringBuilder.append(context?.getString(R.string.single_space))
            return spannableStringBuilder
        }
    }
}
