@file:JvmName("ProductUtils")

package com.asda.android.utils

import android.content.Context
import com.asda.android.base.core.Promos
import com.asda.android.restapi.app.shop.PromotionInfo
import com.asda.android.service.base.CartManager
import com.asda.android.restapi.service.data.ShelfItem
import com.asda.android.products.ui.promos.view.AsdaMeatStickerView
import com.asda.android.products.ui.promos.view.AsdaPromoBannerView


fun getMissingCount(context: Context?, promoId: String?): Int {
    var promoMissingCount = 0
    val cartItems = CartManager.get(context).cartItems
    cartItems?.let {
        it
              .asSequence()
              .filter { promoId == it.promoId && ShelfItem.isAvailable(it.availability) }
              .map { it.getPromoMissingCount() }
              .filter { it > promoMissingCount }
              .forEach { promoMissingCount = it }
    }

    return promoMissingCount
}

fun getMeatStickerViewType(promotionInfo: PromotionInfo): AsdaMeatStickerView.PromoType {

    if (!promotionInfo.data?.promoOfferTypeCode.isNullOrBlank()) {
        when (promotionInfo.data.promoOfferTypeCode) {
            Promos.PROMO_TYPE_15 -> {
                return AsdaMeatStickerView.PromoType.PROMO_OFFER_TYPE_15
            }
            Promos.PROMO_TYPE_9B, Promos.PROMO_TYPE_8 -> {
                return AsdaMeatStickerView.PromoType.PROMO_OFFER_TYPE_9B
            }
            Promos.PROMO_TYPE_13 -> {
                return AsdaMeatStickerView.PromoType.PROMO_OFFER_TYPE_13
            }
        }
    }

    return AsdaMeatStickerView.PromoType.PROMO_OFFER_UNDEFINED
}

fun getBannerViewType(promotionInfo: PromotionInfo): AsdaPromoBannerView.PromoType {
    when (promotionInfo.data.promoOfferTypeCode) {
        Promos.PROMO_TYPE_11 -> {
            return AsdaPromoBannerView.PromoType.PROMO_OFFER_TYPE_11
        }
        Promos.PROMO_TYPE_12 -> {
            return AsdaPromoBannerView.PromoType.PROMO_OFFER_TYPE_12
        }
        Promos.PROMO_TYPE_13 -> {
            return AsdaPromoBannerView.PromoType.PROMO_OFFER_TYPE_13
        }
        Promos.PROMO_TYPE_8 -> {
            return AsdaPromoBannerView.PromoType.PROMO_OFFER_TYPE_8
        }
    }

    if (!promotionInfo.data?.promoType.isNullOrBlank() && !promotionInfo.data?.promoType.equals(Promos.PROMO_TYPE_NO_PROMO, true)) {
        when (promotionInfo.data.promoType.toLowerCase()) {
            Promos.PROMO_ROLLBACK -> { //"promoType":"Rollback"
                return AsdaPromoBannerView.PromoType.PROMO_TYPE_ROLLBACK
            }
            Promos.PROMO_NEW -> { //promoType":"New"
                return AsdaPromoBannerView.PromoType.PROMO_TYPE_NEW
            }
            Promos.PROMO_BUNDLE -> { //"promoType":"No promo", isBundle = true
                return AsdaPromoBannerView.PromoType.PROMO_TYPE_BUNDLE
            }
            Promos.PROMO_OUT_OF_STOCK -> { //  ????
                return AsdaPromoBannerView.PromoType.PROMO_TYPE_OUT_OF_STOCK
            }
            Promos.PROMO_ASDA_PRICE -> { //"promoType": "Asda Price"
                return AsdaPromoBannerView.PromoType.PROMO_TYPE_ASDA_PRICE
            }
        }
    }

    return AsdaPromoBannerView.PromoType.PROMO_OFFER_UNDEFINED
}

