package com.asda.android.utils.view

import android.app.Activity
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import androidx.annotation.StringRes
import com.asda.android.R
import com.asda.android.base.core.*
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.singleprofile.features.login.view.LoginSpActivity

private val SCREEN_NAME = "Sign In"

fun startOnTrolleyModify(activity: Activity, origin: String, bundle: Bundle) {
    val clickListener = DialogInterface.OnClickListener { _, _ ->
        val intent = Intent(activity, LoginSpActivity::class.java)
        intent.putExtra(EXTRA_MESSAGE, activity.getString(R.string.login_screen_login_help_text))
        intent.putExtra(EXTRA_ORIGIN, origin)
        bundle.putBoolean(EXTRA_ADD_TO_TROLLEY, true)
        intent.putExtra(EXTRA_DATA, bundle)
        activity.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_LOGIN)
    }

    val r = activity.resources
    val msg = r.getString(R.string.login_dialog_msg)
    val negativeText = r.getString(R.string.login_dialog_continue_button)
    val positiveText = r.getString(R.string.login_dialog_login_button)

    DialogFactory.createAlertDialog(msg, positiveText, clickListener, negativeText, activity, SCREEN_NAME).show()
}

fun startOnTrolleyModify(activity: Activity, origin: String, itemIdsToAdd: String?, itemQtysToAdd: String?, sourcePageId: String? = null) {
    val clickListener = DialogInterface.OnClickListener { _, _ ->
        val intent = Intent(activity, LoginSpActivity::class.java)
        intent.putExtra(EXTRA_MESSAGE, activity.getString(R.string.login_screen_login_help_text))
        intent.putExtra(EXTRA_ORIGIN, origin)
        if (!TextUtils.isEmpty(itemIdsToAdd) && !TextUtils.isEmpty(itemQtysToAdd)) {
            val dataToPass = Bundle()
            dataToPass.putString(EXTRA_ITEM_IDS_TO_ADD, itemIdsToAdd)
            dataToPass.putString(EXTRA_ITEM_QTYS_TO_ADD, itemQtysToAdd)
            dataToPass.putBoolean(EXTRA_ADD_TO_TROLLEY, true)
            sourcePageId?.let {
                dataToPass.putString(EXTRA_SOURCE_PAGE_ID, it)
            }
            intent.putExtra(EXTRA_DATA, dataToPass)
        }
        activity.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_LOGIN)
    }

    val r = activity.resources

    val msg = r.getString(R.string.login_dialog_msg)
    val negativeText = r.getString(R.string.login_dialog_continue_button)
    val positiveText = r.getString(R.string.login_dialog_login_button)

    DialogFactory.createAlertDialog(msg, positiveText, clickListener, negativeText, activity, SCREEN_NAME).show()
}

fun startOnRestrictedAccess(activity: Activity, @StringRes contentDescription: Int, fragmentTag: String, arguments: Bundle?) {
    val intent = Intent(activity, LoginSpActivity::class.java)
    intent.putExtra(
        EXTRA_MESSAGE,
            activity.getString(R.string.please_signin_to_view, activity.getString(contentDescription)))
    intent.putExtra(EXTRA_ORIGIN, activity.getString(contentDescription))
    val dataToPass = Bundle()
    dataToPass.putString(EXTRA_FRAGMENT_TAG, fragmentTag)
    dataToPass.putBundle(EXTRA_FRAGMENT_ARGS, arguments)
    intent.putExtra(EXTRA_DATA, dataToPass)
    activity.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_LOGIN)
}

fun startOnRestrictedAccess(context: Context,  @StringRes contentDescription: Int, fragmentTag: String, arguments: Bundle?) {
    val intent = Intent(context, LoginSpActivity::class.java)
    intent.putExtra(
        EXTRA_MESSAGE,
            context.getString(R.string.please_signin_to_view, context.getString(contentDescription)))
    intent.putExtra(EXTRA_ORIGIN, context.getString(contentDescription))
    val dataToPass = Bundle()
    dataToPass.putString(EXTRA_FRAGMENT_TAG, fragmentTag)
    dataToPass.putBundle(EXTRA_FRAGMENT_ARGS, arguments)
    intent.putExtra(EXTRA_DATA, dataToPass)
    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
    context.startActivity(intent)
}
