package com.asda.giftcard

import com.asda.android.base.interfaces.ICartManager
import android.os.Bundle
import android.os.Parcelable
import android.text.Spannable
import android.text.SpannableString
import android.text.TextPaint
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.asda.android.R
import com.asda.android.analytics.AnalyticsConfig
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.anivia.model.BulkParams
import com.asda.android.analytics.events.ButtonTapEvent
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.view.ui.SafeRecyclerView
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.cxo.view.activities.SCREEN_NAME
import com.asda.android.restapi.service.data.CheckoutResponse
import com.asda.android.restapi.service.data.giftcard.GiftCardResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import com.asda.giftcard.models.GiftCardViewModel
import com.asda.giftcard.view.DP_ENABLED
import com.asda.giftcard.view.EXTRA_ORDER_ID
import com.asda.giftcard.view.GIFT_CARD_TOTAL
import com.asda.giftcard.view.PREVIOUS_PAGE
import com.asda.android.utils.view.ViewUtil
import io.reactivex.SingleObserver
import io.reactivex.SingleSource
import io.reactivex.observers.ResourceObserver
import kotlinx.android.synthetic.main.gift_card_restricted_item_layout.*

class GiftCardRestrictedItemsFragment : BaseFragment(), SingleSource<Boolean> {

    private var mGiftCardViewModel: GiftCardViewModel? = GiftCardViewModel()
    private var gcAdapter: GiftCardRestrictedItemAdapter? = null
    private var observer: SingleObserver<in Boolean>? = null
    internal var gcRestrictedItems: ArrayList<CheckoutResponse.Message.DelistedItem>? = ArrayList()
    private var mAllowRemovingAllGiftCards :String? = null

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun subscribe(ob: SingleObserver<in Boolean>) {
        observer = ob
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.gift_card_restricted_item_layout, container, false)
        val recyclerView = ViewUtil.findViewById<SafeRecyclerView>(view, R.id.restrictedItems_recycler_view)
        recyclerView.layoutManager = LinearLayoutManager(activity)
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        gcRestrictedItems = CartManager.get(context).gcRestrictedItemsMessage.delistedItems?.toList()?.let { ArrayList(it) }
        mAllowRemovingAllGiftCards = CartManager.get(context).gcRestrictedItemsMessage.allow_removing_all_gift_cards
        gcRestrictedItems.let {
            gcAdapter = context?.let { it1 -> gcRestrictedItems?.let { it2 -> GiftCardRestrictedItemAdapter(it1, it2) } }
        }
        restrictedItems_recycler_view.adapter = gcAdapter
        val actionBar = (activity as AppCompatActivity).supportActionBar

        if (!mAllowRemovingAllGiftCards.isNullOrEmpty() && "true" == mAllowRemovingAllGiftCards) {
            actionBar?.title = context?.getString(R.string.gift_card_restricted_item_activity_title)
            setFormattedTopText()
            remove_gift_cards.text = context?.getString(R.string.gift_card_gift_cards_text)
            remove_gift_cards.setOnClickListener {
                removeAllGiftCards(getGiftCardIds(), arguments?.getString(EXTRA_ORDER_ID).toString())
                addButtonTapEvent(Anivia.GIFT_CARD_RESTRICTED_ITEMS_REMOVE_GIFT_CARD_BUTTON_TEXT, false, true)
            }
            remove_items.visibility = View.VISIBLE
        } else {
            actionBar?.title = context?.getString(R.string.gift_card_restricted_item_activity_title_amending)
            setFormattedTopText()
            remove_items.visibility = View.GONE
            remove_gift_cards.text = context?.getString(R.string.gift_card_amending_restricted_item_remove_item_text)
            remove_gift_cards.setOnClickListener {
                removeGCRestrictedItemsAndContinue()
            }
        }
        remove_items.setOnClickListener {
            removeGCRestrictedItemsAndContinue()
            addButtonTapEvent(Anivia.GIFT_CARD_RESTRICTED_ITEMS_REMOVE_ITEMS_BUTTON_TEXT, true, false)
        }
        val pageViewEvent = PageViewEvent(Anivia.PV_GIFT_CARD_RESTRICTED_ITEMS, Anivia.SECTION_GIFT_CARD_RESTRICTED_ITEMS).also {
            it.putBoolean(Anivia.IS_AMEND, CartManager.get(context).isAmending)
            gcRestrictedItems?.toArray()?.let { it1 -> it.putArray(Anivia.PRODUCTS, it1) }
        }
        Tracker.get().trackEvent(pageViewEvent)
    }

    private fun setFormattedTopText() {
        val topText = context?.getString(R.string.gift_card_restricted_item_page_text)
        val spString = SpannableString(topText)
        val clickableSpan = object : ClickableSpan() {
            override fun onClick(view: View) {
                context?.let {
                    ViewUtils.openWebPage(SharedPreferencesUtil.getGiftCardUrl(context), context, SCREEN_NAME)
                }
            }

            override fun updateDrawState(ds: TextPaint) {
                ds.color = context?.resources?.getColor(R.color.color_ocean_blue_0073b1)!!
                ds.isUnderlineText = true
                ds.isFakeBoldText = true
            }
        }
        spString.setSpan(clickableSpan, topText?.indexOf(context?.getString(R.string.gift_card_restriction_text_cannot).toString())!!, topText?.indexOf(context?.getString(R.string.gift_card_restriction_text_items).toString())!! + context?.getString(R.string.gift_card_restriction_text_items).toString().length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        text_top.text = spString
        text_top.movementMethod = LinkMovementMethod.getInstance()
    }

    private fun getNoOfGiftCards(): Int {
        val noOfGiftCards = arguments?.getParcelableArrayList<GiftCardResponse.GiftCard>(GIFT_CARDS);
        noOfGiftCards?.let {
            return it.size
        } ?: run {
            return 0
        }
    }

    private fun getGiftCardIds(): String {
        val giftCards = arguments?.getParcelableArrayList<GiftCardResponse.GiftCard>(GIFT_CARDS)
        var giftCardIds = ""
        giftCards?.let {
            for (gc in giftCards) {
                if (gc.isEditable == true) {
                    giftCardIds = giftCardIds + gc.giftCardId + ","
                }
            }
            if (giftCardIds.isNotEmpty()) {
                giftCardIds = giftCardIds.substring(0, giftCardIds.length - 1)
            }
        }
        return giftCardIds
    }

    fun getItemIds(): ArrayList<String> {
        val itemIds = ArrayList<String>()
        if (gcRestrictedItems?.isNotEmpty() == true) {
            for (item in gcRestrictedItems!!) {
                itemIds.add(item.id)
            }
        }
        return itemIds
    }

    private fun removeAllGiftCards(giftCardIds: String, orderId: String) {

        if (orderId.isNotEmpty() && giftCardIds.isNotEmpty()) {
            mGiftCardViewModel?.removeGiftCard(Authentication.getInstance().csrfToken, giftCardIds, orderId)?.subscribe(object : ResourceObserver<GiftCardResponse>() {
                override fun onComplete() {
                    dispose()
                }

                override fun onNext(t: GiftCardResponse) {
                    observer?.onSuccess(true)
                }

                override fun onError(e: Throwable) {
                    dispose()
                }

            })
        }
    }

    private fun removeGCRestrictedItemsAndContinue() {

        val itemIds = getItemIds()
        context?.let {
            CartManager.get(it).removeItemsFromCart(it, itemIds.toTypedArray()).subscribe(object : ResourceObserver<ICartManager.CartUpdateResponse>() {
                override fun onComplete() {
                    dispose()
                }

                override fun onNext(t: ICartManager.CartUpdateResponse) {
                    observer?.onSuccess(true)
                }

                override fun onError(e: Throwable) {
                    dispose()
                }
            })
        }
    }

    private fun addButtonTapEvent(buttonName: String, isRemoveItem: Boolean, isRemoveGiftCard: Boolean) {
        val buttonTapEvent = ButtonTapEvent(buttonName, Anivia.GIFT_CARD_RESTRICTED_ITEMS_PAGE_NAME, false)
        val userViewResponse = (Authentication.getInstance() as Authentication).userViewResponse
        buttonTapEvent.putString(Anivia.PREVIOUS_PAGE_NAME, arguments?.getString(PREVIOUS_PAGE))
        buttonTapEvent.putString(Anivia.CUSTOMER_ID_KEY, userViewResponse?.profileId)
        buttonTapEvent.putBoolean(Anivia.DP_ENABLED, arguments?.getBoolean(DP_ENABLED)!!)
        buttonTapEvent.putBoolean(Anivia.PUSH_ENABLED, SharedPreferencesUtil.isPushEnabled(context))
        buttonTapEvent.putBoolean(Anivia.LOCATION_ENABLED, Utils.isLocationEnabled(context))
        buttonTapEvent.putString(Anivia.APP_VERSION_KEY, AnalyticsConfig.appVersion)
        buttonTapEvent.putString(Anivia.OS_KEY, AnalyticsConfig.osVersion)
        buttonTapEvent.putString(Anivia.CARTID_KEY, AnalyticsConfig.cartId)
        buttonTapEvent.putString(Anivia.DEVICE_KEY, AnalyticsConfig.model)
        buttonTapEvent.putString(Anivia.TPID, BulkParams.ANDROID)
        buttonTapEvent.putBoolean(Anivia.IS_REMOVE_ITEM, isRemoveItem)
        buttonTapEvent.putBoolean(Anivia.IS_REMOVE_GIFT_CARD, isRemoveGiftCard)
        buttonTapEvent.putString(Anivia.DATE_TIME, System.currentTimeMillis().toString())
        buttonTapEvent.putString(Anivia.RESTRICTED_NO_OF_GIFT_CARDS, Integer.toString(getNoOfGiftCards()))
        buttonTapEvent.putString(Anivia.GIFT_CARD_AMOUNT, arguments?.getString(GIFT_CARD_TOTAL))
        Tracker.get().trackEvent(buttonTapEvent)
    }
    companion object {
        const val GIFT_CARDS = "gift_cards"
        const val TAG = "GiftCardRestrictedItemsFragment"
        @JvmStatic
        fun newInstance(giftCardDetails: ArrayList<GiftCardResponse.GiftCard>, orderId: String, giftCardTotal: String, dpEnabled: Boolean, previousPageName: String): GiftCardRestrictedItemsFragment {
            val fragment = GiftCardRestrictedItemsFragment()
            val args = Bundle()
            args.putParcelableArrayList(GIFT_CARDS, giftCardDetails as java.util.ArrayList<out Parcelable>)
            args.putString(EXTRA_ORDER_ID, orderId)
            args.putString(GIFT_CARD_TOTAL, giftCardTotal)
            args.putBoolean(DP_ENABLED, dpEnabled)
            args.putString(PREVIOUS_PAGE, previousPageName)
            fragment.arguments = args
            return fragment
        }
    }
}
