package com.asda.giftcard

import com.asda.android.restapi.app.cnc.model.data.StoreConfigResponse
import com.asda.android.restapi.service.data.CheckoutResponse
import com.asda.android.restapi.service.data.giftcard.GiftCardResponse

const val INVALID_GIFT_CARD_NUMBER = "INVALID_GIFT_CARD_NUMBER"
const val INVALID_GIFT_CARD_PIN = "INVALID_GIFT_CARD_PIN"

class GiftCardUtils {


    companion object {
        var errorCode: String? = null

        fun isGiftCardFeatureEnabled(storeConfigResponse: StoreConfigResponse?): Boolean? {
            return storeConfigResponse?.let { response ->
                response.featuresEnabled?.giftCardEnabled ?: false
            } ?: false
        }

        fun isValidGiftCardInfo(giftCardNumber: String, giftCardPin: String): Boolean {
            errorCode = ""
            if ((giftCardNumber.length in 1..15) || !giftCardNumber.matches(Regex("[0-9]+"))) {
                errorCode = INVALID_GIFT_CARD_NUMBER
                return false
            }
            if ((giftCardPin.length in 1..3) || !giftCardPin.matches(Regex("[0-9]+"))) {
                errorCode = INVALID_GIFT_CARD_PIN
                return false
            }
            return true
        }

        fun getGiftCardsList(checkoutResponse: CheckoutResponse?): ArrayList<GiftCardResponse.GiftCard> {
            val glist = java.util.ArrayList<GiftCardResponse.GiftCard>()
            checkoutResponse?.paymentDetails?.giftCards?.let {
                for (gift in it) glist.add(gift)
            }
            return glist
        }

        fun formatGiftCardNumber(giftCardNumber: String): String {
            val formattedCardNumber = StringBuilder()
            if (giftCardNumber.isNotEmpty()) {
                for (i in 0 until giftCardNumber.length) {
                    if (i % 4 == 0 && i != 0) {
                        formattedCardNumber.append(" ")
                    }
                    formattedCardNumber.append(giftCardNumber[i])
                }
            }
            return formattedCardNumber.toString()
        }

    }
}
