package com.asda.giftcard.models

import androidx.annotation.VisibleForTesting
import androidx.lifecycle.ViewModel
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.GiftCardAPI
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.giftcard.ApplyGiftCardRequest
import com.asda.android.restapi.service.data.giftcard.GiftCardResponse
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers


class GiftCardViewModel : ViewModel() {

    private var networkAPI: GiftCardAPI? = null

    fun applyGiftCard(token: String?, giftCardNumber: String?, giftCardCode: String?, orderId: String): Observable<GiftCardResponse>? {
        val applyGiftCardRequest = ApplyGiftCardRequest(token, giftCardNumber, giftCardCode)
        return getNetworkAPI()?.applyGiftCard(orderId, applyGiftCardRequest)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    fun removeGiftCard(token: String?, giftCardId: String?, orderId: String): Observable<GiftCardResponse>? {
        return getNetworkAPI()?.removeGiftCard(orderId, giftCardId, token)?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())
    }

    @VisibleForTesting
    fun setNetworkApi(api: GiftCardAPI) {
        networkAPI = api
    }

    private fun getNetworkAPI() : GiftCardAPI? {
        networkAPI?.let {
            return networkAPI
        } ?: run {
            networkAPI = ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", GiftCardAPI::class.java)
        }
        return networkAPI
    }
}
