package com.asda.giftcard.view

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.appcompat.widget.AppCompatButton
import com.asda.android.R
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.web.model.WebViewModel
import com.asda.android.base.core.web.view.WebViewFragment
import com.asda.android.imageloader.ImageLoader
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.view.ViewUtils

class EGiftCardsFragment : FeaturedFragment() {

    companion object {
        const val SCREEN_NAME = "Gift Cards"
    }

    override fun getScreenName() = SCREEN_NAME
    override fun hasActionBar() = false

    var fragmentContext: Context? = null
    var btnBuyCard: AppCompatButton? = null
    var btnGroupBuyCard: AppCompatButton? = null
    var btnAsdaGroupBuyCard: AppCompatButton? = null
    var btnCheckBalance: AppCompatButton? = null
    var headerImage: ImageView? = null

    var firstCardImage: ImageView? = null
    var secondCardImage: ImageView? = null
    var thirdCardImage: ImageView? = null
    var fourthCardImage: ImageView? = null

    var asdaCardImage: ImageView? = null
    var georgeCardImage: ImageView? = null

    override fun onAttach(context: Context) {
        super.onAttach(context)
        fragmentContext = context
    }


    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.e_gift_card_fragment, fragmentContainer, false)
        setupView(view)
        return view
    }

    fun setupView(view: View?) {
        if (isValidContent()) {
            view?.let { _view ->
                btnBuyCard = _view.findViewById(R.id.btn_buy_card)
                btnGroupBuyCard = _view.findViewById(R.id.btn_group_buy_card)
                btnAsdaGroupBuyCard = _view.findViewById(R.id.btn_asda_group_buy_card)
                btnCheckBalance = _view.findViewById(R.id.btn_check_balance)
                headerImage = _view.findViewById(R.id.header_image)

                firstCardImage = _view.findViewById(R.id.first_card)
                secondCardImage = _view.findViewById(R.id.second_card)
                thirdCardImage = _view.findViewById(R.id.third_card)
                fourthCardImage = _view.findViewById(R.id.fourth_card)

                asdaCardImage = _view.findViewById(R.id.asda_first_card)
                georgeCardImage = _view.findViewById(R.id.asda_second_card)

                fragmentContext?.let { _context ->

                    headerImage?.let { _headerImage ->
                        ImageLoader.load(_context, _headerImage, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getEGiftCardLandingImageUrl(_context)).fit(true).build(), null)
                    }

                    firstCardImage?.let { _headerImage ->
                        ImageLoader.load(_context, _headerImage, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getPrefEGiftCardFirstCard(_context)).fit(true).build(), null)
                    }

                    secondCardImage?.let { _headerImage ->
                        ImageLoader.load(_context, _headerImage, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getPrefEGiftCardSecondCard(_context)).fit(true).build(), null)
                    }

                    thirdCardImage?.let { _headerImage ->
                        ImageLoader.load(_context, _headerImage, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getPrefEGiftCardThirdCard(_context)).fit(true).build(), null)
                    }

                    fourthCardImage?.let { _headerImage ->
                        ImageLoader.load(_context, _headerImage, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getPrefEGiftCardFourthCard(_context)).fit(true).build(), null)
                    }

                    btnBuyCard?.setOnClickListener {
                        ViewUtils.openWebPageWithHeaders(SharedPreferencesUtil.getEGiftCardPurchaseUrl(_context), _context, SCREEN_NAME, null)
                    }

                    btnCheckBalance?.setOnClickListener {
                        ViewUtils.openWebPageWithHeaders(SharedPreferencesUtil.getEGiftCardBalanceUrl(_context), _context, SCREEN_NAME, null)
                    }

                    btnGroupBuyCard?.setOnClickListener {
                        ViewUtils.openWebPageWithHeaders(SharedPreferencesUtil.getThirdPartyEGiftCardUrl(_context), _context, SCREEN_NAME, null)
                    }

                    btnAsdaGroupBuyCard?.setOnClickListener {
                        ViewUtils.openWebPageWithHeaders(SharedPreferencesUtil.getEGiftCardPhysicalCardUrl(_context), _context, SCREEN_NAME, null)
                    }

                    asdaCardImage?.let { _image ->
                        ImageLoader.load(_context, _image, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getEGiftCardAsdaTileImgUrl(_context)).fit(true).build(), null)
                    }

                    georgeCardImage?.let { _image ->
                        ImageLoader.load(_context, _image, ImageLoaderRequest.Builder().path(SharedPreferencesUtil.getPrefEGiftCardGeorgeTileImgUrl(_context)).fit(true).build(), null)
                    }

                }
            }
        } else {
            activity?.finish()
        }
    }

    private fun isValidContent(): Boolean {
        fragmentContext?.let { _context ->
            return ((SharedPreferencesUtil.getEGiftCardBalanceUrl(_context).isNotEmpty() || SharedPreferencesUtil.getEGiftCardBalanceUrl(_context).isNotBlank())
                  && (SharedPreferencesUtil.getEGiftCardPurchaseUrl(_context).isNotEmpty() || SharedPreferencesUtil.getEGiftCardPurchaseUrl(_context).isNotBlank())
                  && (SharedPreferencesUtil.getEGiftCardLandingImageUrl(_context).isNotEmpty() || SharedPreferencesUtil.getEGiftCardLandingImageUrl(_context).isNotBlank())
                  && (SharedPreferencesUtil.getPrefEGiftCardFirstCard(_context).isNotEmpty() || SharedPreferencesUtil.getPrefEGiftCardFirstCard(_context).isNotBlank())
                  && (SharedPreferencesUtil.getPrefEGiftCardSecondCard(_context).isNotEmpty() || SharedPreferencesUtil.getPrefEGiftCardSecondCard(_context).isNotBlank())
                  && (SharedPreferencesUtil.getPrefEGiftCardThirdCard(_context).isNotEmpty() || SharedPreferencesUtil.getPrefEGiftCardThirdCard(_context).isNotBlank())
                  && (SharedPreferencesUtil.getPrefEGiftCardFourthCard(_context).isNotEmpty() || SharedPreferencesUtil.getPrefEGiftCardFourthCard(_context).isNotBlank())
                  && (SharedPreferencesUtil.getPrefEGiftCardGeorgeTileImgUrl(_context).isNotEmpty() || SharedPreferencesUtil.getPrefEGiftCardGeorgeTileImgUrl(_context).isNotBlank())
                  && (SharedPreferencesUtil.getEGiftCardPhysicalCardUrl(_context).isNotEmpty() || SharedPreferencesUtil.getEGiftCardPhysicalCardUrl(_context).isNotBlank())
                  && (SharedPreferencesUtil.getThirdPartyEGiftCardUrl(_context).isNotEmpty() || SharedPreferencesUtil.getThirdPartyEGiftCardUrl(_context).isNotBlank()))
        } ?: run {
            return false
        }
    }


}
