package com.asda.android.lastmile.carcolor.view

import android.content.Context
import android.graphics.PorterDuff
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.asda.android.R
import com.asda.android.lastmile.carcolor.formatter.CarColorItem
import com.asda.android.utils.view.ViewUtil

class CarColorAdapter(private var context: Context?) : androidx.recyclerview.widget.RecyclerView.Adapter<CarColorAdapter.CarViewHolder>() {

    constructor(context: Context?, carColors: ArrayList<CarColorItem>?) : this(context) {
        items = carColors
    }

    constructor(context: Context?, initBaseColors: Boolean) : this(context) {
        items = if (initBaseColors) {
            getBaseCarColors()
        } else {
            ArrayList()
        }
    }

    private var items: ArrayList<CarColorItem>? = null
    private val layoutInflater: LayoutInflater = LayoutInflater.from(context)
    private var carColorListener: OnCarColorSelected? = null

    fun setOnCarColorSelected(onCarColorSelect: OnCarColorSelected?) {
        carColorListener = onCarColorSelect
    }

    private fun getBaseCarColors(): ArrayList<CarColorItem> {
        val items = ArrayList<CarColorItem>()
        items.add(CarColorItem(WHITE_CAR))
        items.add(CarColorItem(BLACK_CAR))
        items.add(CarColorItem(GREY_CAR))
        items.add(CarColorItem(SILVER_CAR))
        items.add(CarColorItem(BLUE_CAR))
        items.add(CarColorItem(RED_CAR))
        items.add(CarColorItem(GREEN_CAR))
        items.add(CarColorItem(OTHER_CAR_COLOR))
        return items
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CarViewHolder {

        val view = layoutInflater.inflate(R.layout.car_color_item, parent, false)
        val viewHolder = CarViewHolder(view)

        viewHolder.container?.setOnClickListener({
            val position = viewHolder.adapterPosition
            if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                val item = items?.get(position)
                carColorListener?.onItemSelected(item)
            }
        })

        return viewHolder
    }

    override fun getItemCount() = items?.size ?: 0

    override fun onBindViewHolder(holder: CarViewHolder, position: Int) {
        val carColorItem = items?.get(position)

        carColorItem?.let {
            val paddingLeft: Int
            if (it.colorTint != -1) {
                holder.carColor?.visibility = View.VISIBLE
                paddingLeft = ViewUtil.dpToPixels(16, context)
                context?.let {
                    holder.carColor?.setColorFilter(ContextCompat.getColor(it, carColorItem.colorTint), PorterDuff.Mode.SRC_IN)
                }
            } else {
                paddingLeft = ViewUtil.dpToPixels(36, context)
                holder.carColor?.visibility = View.GONE
            }
            holder.carColorTitle?.setPadding(paddingLeft, 0, 0, 0)
            holder.carColorTitle?.text = context?.getString(it.carColorTitle)
        }
    }

    fun destroy() {
        context = null
        carColorListener = null
        items = null
    }

    inner class CarViewHolder(itemView: View) : androidx.recyclerview.widget.RecyclerView.ViewHolder(itemView) {
        var container: View? = itemView.findViewById(R.id.car_color_item)
        var carColor: ImageView? = itemView.findViewById(R.id.item_color)
        var carColorTitle: TextView? = itemView.findViewById(R.id.item_color_desc)
    }

    interface OnCarColorSelected {
        fun onItemSelected(carColor: CarColorItem?)
    }

    companion object {
        const val WHITE_CAR = 1
        const val BLACK_CAR = 2
        const val GREY_CAR = 3
        const val SILVER_CAR = 4
        const val BLUE_CAR = 5
        const val RED_CAR = 6
        const val GREEN_CAR = 7
        const val OTHER_CAR_COLOR = -1
    }
}
