package com.asda.lastmile.details.view

import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Color
import android.graphics.Paint
import android.location.Location
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.*
import androidx.annotation.DrawableRes
import androidx.appcompat.app.AppCompatActivity
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.DialogFragment
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.model.createCheckinNotification
import com.asda.android.app.view.WismoProgressView
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.OrderManager
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.service.data.mobilecheckin.AccessPointInfo
import com.asda.android.restapi.service.data.mobilecheckin.GeoPoint
import com.asda.android.restapi.service.data.mobilecheckin.RouteInfo
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils
import com.asda.android.lastmile.*
import com.asda.android.lastmile.carcolor.view.CarColorAdapter
import com.asda.lastmile.parkingspot.view.ParkingSpotAdapter
import com.asda.android.lastmile.carcolor.formatter.CarColorItem
import com.asda.android.lastmile.carcolor.view.CarColorDialogFragment
import com.asda.lastmile.parkingspot.view.ParkingSpotDialogFragment
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.api.GoogleApiClient
import com.google.android.gms.location.LocationServices
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.MapView
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.MarkerOptions
import com.google.android.gms.maps.model.PolylineOptions
import com.google.maps.android.PolyUtil
import com.urbanairship.UAirship.getPackageManager
import com.asda.android.utils.view.Permissions
import com.asda.android.app.storelocator.view.StoreLocationManager
import com.asda.android.lastmile.model.CheckinEligibleOrderStatus
import com.asda.android.lastmile.model.DriverFeatures
import com.asda.android.lastmile.model.OrderStatus
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_lmdetails.*
import java.util.regex.Matcher
import java.util.regex.Pattern

private const val SCREEN_NAME: String = "Mobile Checkin"
const val EXTRA_MESSAGE_ORDERID: String = "ORDERID"
const val EXTRA_MESSAGE_CUSTOMER_ID: String = "CUSTOMER_ID"
private const val TAG = "LMDetailsActivity"
private const val LOCATION_NAME: String = "Collection Details"

fun launchGoogleMapsWithRoute(context: Context, lat: Double?, long: Double?) {
    val gmmIntentUri: Uri = Uri.parse("google.navigation:q=${lat ?: 0},${long ?: 0}");
    val mapIntent = Intent(Intent.ACTION_VIEW, gmmIntentUri)
    mapIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
    mapIntent.setPackage("com.google.android.apps.maps")
    if (mapIntent.resolveActivity(getPackageManager()) != null) {
        context.startActivity(mapIntent);
    } else {
        Log.e(TAG, "Google Maps App does not exist")
    }
}

fun logCheckinDetailsPageView(customreId: String, orderId: String, initialTimeInSec: String) {
    val pageViewEvent = PageViewEvent(Anivia.PV_COLLECTION_DETAILS, Anivia.SECTION_CHECKIN)
    pageViewEvent.putString(Anivia.CUSTOMER_ID_KEY, customreId)
    pageViewEvent.putString(Anivia.ORDER_ID_KEY, orderId)
    pageViewEvent.putString(Anivia.CHECKIN_INITIAL_TIME_KEY, initialTimeInSec)
    Tracker.get().trackPageView(pageViewEvent)
}

fun callStore(context: Context, phoneNumber: String?) {
    //ASDAA-10527
    Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
            .putString(Anivia.BUTTON_NAME_KEY, context.getString(R.string.call_us))
            .putString(Anivia.NAME_KEY, LOCATION_NAME)
            .putString(Anivia.OVERLAY_COPY_KEY, context.getString(R.string.checkin_having_issues)))
    val intent = Intent(Intent.ACTION_DIAL, Uri.parse("tel:" + phoneNumber))
    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
    context.startActivity(intent)
}

class LMDetailsActivity : AppCompatActivity(), OnMapReadyCallback, GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener, ActivityCompat.OnRequestPermissionsResultCallback {
    private var checkinManager: LastMileCheckinManager? = null
    private var orderId: String? = null
    private var customerId: String? = null
    private val compositeDisposable: CompositeDisposable = CompositeDisposable()
    private lateinit var container: ConstraintLayout
    private lateinit var loadingProgress: ProgressBar
    private lateinit var orderStatusTextView: TextView
    private lateinit var headingText: TextView
    private lateinit var line1Text: TextView
    private lateinit var line2Text: TextView
    private lateinit var line3Text: TextView
    private lateinit var wismoProgress: WismoProgressView
    private lateinit var needHelpText: TextView
    private lateinit var mapView: MapView
    private lateinit var storeName: TextView
    private lateinit var storeAddress: TextView
    private lateinit var navigation: FrameLayout
    private lateinit var eta: TextView
    private lateinit var collectionMethod: TextView
    private lateinit var collectionTime: TextView
    private lateinit var mapLoadingProgress: ProgressBar
    private lateinit var onYourWayTextView: TextView
    private lateinit var carColorSelector: ImageView
    private lateinit var carColorSelectorContainer: View
    private lateinit var parkingSpotSelector: View
    private lateinit var parkingSpotSelectorDesc: TextView
    private lateinit var walkingSelector: CheckBox
    private var storePhoneNumber: String? = ""
    private var googleMap: GoogleMap? = null
    private var googleApiClient: GoogleApiClient? = null
    private var lastLocation: Location? = null
    private lateinit var locationManager: StoreLocationManager
    private var permissionDialog: Dialog? = null
    private var locationPermissionDeniedFlag = true
    private var storeLat: Double? = 0.0
    private var storeLong: Double? = 0.0
    private var needHelpAtStoreDialog: Dialog? = null
    private var callHelpAtStoreDialog: Dialog? = null
    private var carSelcorDisposable: Disposable? = null
    private var parkingSpotDisposable: Disposable? = null
    private var walkingDisposable: Disposable? = null
    private var manualCheckinDisposable: Disposable? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_lmdetails)

        if (savedInstanceState == null) {
            orderId = intent.getStringExtra(EXTRA_MESSAGE_ORDERID)
            customerId = intent.getStringExtra(EXTRA_MESSAGE_CUSTOMER_ID)
        } else {
            orderId = savedInstanceState.getString(EXTRA_MESSAGE_ORDERID)
            customerId = savedInstanceState.getString(EXTRA_MESSAGE_CUSTOMER_ID)
        }

        if (SharedPreferencesUtil.isUsgrCheckinEnabled(applicationContext)) {
            AsdaApplication.getInstance()?.let {
                checkinManager = USGRLastMileCheckinManager.getInstance(it)
            } ?: run {
                if (Utils.isDebug()) {
                    Log.e(TAG, "Application instance is null!! You are Doomed...")
                }
            }
        } else {
            AsdaApplication.getInstance()?.let {
                if (SharedPreferencesUtil.isLocalMockCheckinEnabled(applicationContext)) {
                    checkinManager = MockLastMileCheckinManager.getInstance(OrderManager.get().orders)
                } else {
                    checkinManager = ASDALastMileCheckinManager.getInstance(it)
                }
            } ?: run {
                if (Utils.isDebug()) {
                    Log.i(TAG, "Application instance is null!! You are Doomed...")
                }
            }

        }
        initialize(savedInstanceState)
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(outState, "LMDetailsActivity")
        outState.putString(EXTRA_MESSAGE_ORDERID, orderId)
        outState.putString(EXTRA_MESSAGE_CUSTOMER_ID, customerId)
    }

    override fun onConnected(bundle: Bundle?) {
        try {
            lastLocation = LocationServices.FusedLocationApi.getLastLocation(
                    googleApiClient)
        } catch (e: SecurityException) {
            if (Utils.isDebug()) {
                Log.e(TAG, "Exception in onConnected ${e.localizedMessage}")
            }
        }

    }

    override fun onConnectionSuspended(result: Int) {
        if (Utils.isDebug()) {
            Log.e(TAG, "connection suspende with ${result}")
        }
    }

    override fun onConnectionFailed(connectionResult: ConnectionResult) {
        if (Utils.isDebug()) {
            Log.e(TAG, "connection failed with ${connectionResult}")
        }
    }

    override fun onMapReady(gMap: GoogleMap?) {
        if (Utils.isDebug()) {
            Log.d(TAG, "Map is Ready!!")
        }
        googleMap = gMap

        if (!Permissions.isLocationPermissionGranted(this)) {
            setProgressVisibility(false)
            //todo: vmk need to check with PM
            Toast.makeText(applicationContext, getString(R.string.checkin_enable_location_permission), Toast.LENGTH_SHORT).show()
        } else {
            googleMap?.getUiSettings()?.setMyLocationButtonEnabled(false);
            googleMap?.setMyLocationEnabled(true);
            lastLocation = LocationServices.FusedLocationApi.getLastLocation(googleApiClient)
        }
    }

    private fun setProgressVisibility(flag: Boolean) {
        if (!flag) {
            loadingProgress.visibility = View.INVISIBLE
            mapLoadingProgress.visibility = View.INVISIBLE
        } else {
            loadingProgress.visibility = View.VISIBLE
            mapLoadingProgress.visibility = View.VISIBLE
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        if (requestCode == Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION) {
            if (grantResults.size == 2) {
                if (grantResults[0] == PackageManager.PERMISSION_GRANTED || grantResults[1] == PackageManager.PERMISSION_GRANTED) {
                    googleMap?.getUiSettings()?.setMyLocationButtonEnabled(false);
                    googleMap?.setMyLocationEnabled(true);
                    lastLocation = LocationServices.FusedLocationApi.getLastLocation(googleApiClient)
                } else {
                    // Permission was denied. Display an error message.
                    //todo: vmk need to check with PM
                    Toast.makeText(applicationContext, getString(R.string.checkin_enable_location_permission) +
                            "  " +
                            "", Toast.LENGTH_SHORT).show()
                }
            } else {

            }
        }
    }

    private fun initialize(bundle: Bundle?) {
        locationPermissionDeniedFlag = true
        locationManager = StoreLocationManager(this.applicationContext)
        container = findViewById(R.id.container)
        orderStatusTextView = findViewById(R.id.order_status)
        loadingProgress = findViewById(R.id.loading_progress)
        headingText = findViewById(R.id.checkin_detail_heading)
        line1Text = findViewById(R.id.checkin_detail_line1)
        line2Text = findViewById(R.id.checkin_detail_line2)
        line3Text = findViewById(R.id.checkin_detail_line3)
        wismoProgress = findViewById(R.id.wismo_progress)
        carColorSelector = findViewById(R.id.car_color_selector)
        carColorSelectorContainer = findViewById(R.id.car_color_selector_container)
        carColorSelector.setOnClickListener {
            orderId?.toLong()?.let {
                val order = it
                val dialogFragment = CarColorDialogFragment.newInstance()
                dialogFragment.setStyle(DialogFragment.STYLE_NORMAL, R.style.ASDAABaseDialog)
                dialogFragment.setOnCarColorSelected(object : CarColorAdapter.OnCarColorSelected {
                    override fun onItemSelected(carColor: CarColorItem?) {
                        carColor?.carColorTitle?.let {
                            if (it != -1) {
                                checkinManager?.setDriverFeatures(order, DriverFeatures(getString(it)))
                                        ?.distinct()
                                        ?.subscribeOn(Schedulers.single())
                                        ?.observeOn(AndroidSchedulers.mainThread())
                                        ?.subscribe(
                                                object : Observer<DriverFeatures?> {
                                                    override fun onComplete() {
                                                        carSelcorDisposable?.dispose()
                                                    }

                                                    override fun onSubscribe(d: Disposable) {
                                                        carSelcorDisposable = d
                                                    }

                                                    override fun onNext(driverFeatures: DriverFeatures) {
                                                        setCarColor(driverFeatures)
                                                    }

                                                    override fun onError(e: Throwable) {
                                                        if (Utils.isDebug()) {
                                                            Log.e(TAG, e.message)
                                                        }
                                                        carSelcorDisposable?.dispose()
                                                    }

                                                }
                                        )
                            }
                        }

                        dialogFragment.dialog?.isShowing?.let {
                            if (it) {
                                try {
                                    dialogFragment.dismiss()
                                } catch (ignore: Exception) {
                                }
                            }
                        }
                    }
                })
                dialogFragment.show(supportFragmentManager, "TAG")
            }.run {
                BaseAsdaDialogHelper.displayErrorDialog(null, -1, applicationContext, SCREEN_NAME)
            }
        }

        parkingSpotSelector = findViewById(R.id.parking_spot_selector)
        parkingSpotSelector.setOnClickListener({
            orderId?.toLong()?.let {
                val order = it
                val dialogFragment = ParkingSpotDialogFragment.newInstance()
                dialogFragment.setStyle(DialogFragment.STYLE_NORMAL, R.style.ASDAABaseDialog)
                dialogFragment.setOnParkingSpotSelected(object : ParkingSpotAdapter.OnParkingSpotSelected {
                    override fun onItemSelected(parkingSpot: String?) {

                        parkingSpot?.let {
                            checkinManager?.setDriverFeatures(order, DriverFeatures(null, null, parkingSpot))
                                    ?.distinct()
                                    ?.subscribeOn(Schedulers.single())
                                    ?.observeOn(AndroidSchedulers.mainThread())
                                    ?.subscribe(
                                            object : Observer<DriverFeatures?> {
                                                override fun onComplete() {
                                                    parkingSpotDisposable?.dispose()
                                                }

                                                override fun onSubscribe(d: Disposable) {
                                                    parkingSpotDisposable = d
                                                }

                                                override fun onNext(driverFeatures: DriverFeatures) {
                                                    setCarColor(driverFeatures)
                                                }

                                                override fun onError(e: Throwable) {
                                                    if (Utils.isDebug()) {
                                                        Log.e(TAG, e.message)
                                                    }
                                                    parkingSpotDisposable?.dispose()
                                                }

                                            }
                                    )
                        }

                        dialogFragment.dialog?.isShowing?.let {
                            if (it) {
                                try {
                                    dialogFragment.dismiss()
                                } catch (ignore: Exception) {
                                }
                            }
                        }
                    }
                })
                dialogFragment.show(supportFragmentManager, null)
            }.run {
                BaseAsdaDialogHelper.displayErrorDialog(null, -1, applicationContext, SCREEN_NAME)
            }
        })
        parkingSpotSelectorDesc = findViewById(R.id.parking_spot_selector_desc);

        walkingSelector = findViewById(R.id.walking_selector);
        walkingSelector.setOnClickListener({ _ ->
            orderId?.toLong()?.let {
                val order = it
                val transportType = if (walkingSelector.isChecked) {
                    DriverFeatures.TRANSPORT_TYPE_WALK
                } else {
                    DriverFeatures.TRANSPORT_TYPE_CAR
                }
                checkinManager?.setDriverFeatures(order, DriverFeatures(null, transportType, null))
                        ?.subscribeOn(Schedulers.single())
                        ?.distinct()
                        ?.observeOn(AndroidSchedulers.mainThread())
                        ?.subscribe(
                                object : Observer<DriverFeatures?> {
                                    override fun onComplete() {
                                        if (Utils.isDebug()) {
                                            Log.i(TAG, "completed")
                                        }
                                        walkingDisposable?.dispose()
                                    }

                                    override fun onSubscribe(d: Disposable) {
                                        walkingDisposable = d
                                    }

                                    override fun onNext(t: DriverFeatures) {
                                        updateDriverFeaturesUI(t)
                                    }

                                    override fun onError(e: Throwable) {
                                        if (Utils.isDebug()) {
                                            Log.e(TAG, e.message)
                                        }
                                        walkingDisposable?.dispose()
                                    }

                                }
                        )
            }.run {
                BaseAsdaDialogHelper.displayErrorDialog(null, -1, applicationContext, SCREEN_NAME)
            }
        })

        needHelpText = findViewById(R.id.checkin_need_help)
        needHelpText.setOnClickListener(object : View.OnClickListener {
            override fun onClick(p0: View?) {

                if (orderStatusTextView.text.toString()
                                .compareTo(getString(R.string.onYorWay), true) == 0) {
                    if (needHelpAtStoreDialog == null) {
                        needHelpAtStoreDialog = DialogFactory.createAlertDialog(applicationContext.getString(R.string.checkin_having_issues),
                                applicationContext.getString(R.string.i_am_at_store),
                                { _, _ -> manualCheckin() },
                                "",
                                this@LMDetailsActivity, SCREEN_NAME)
                    }
                    //ASDAA-10527
                    Tracker.get().trackPageView(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                            .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata)
                            .putString(Anivia.NAME_KEY, Anivia.SECTION_CHECK_IN)
                            .putString(Anivia.OVERLAY_COPY_KEY, applicationContext.getString(R.string.i_am_at_store)))
                    needHelpAtStoreDialog?.show()
                } else if (orderStatusTextView.text.toString()
                                .compareTo(ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE, true) == 0) {
                    if (callHelpAtStoreDialog == null) {
                        callHelpAtStoreDialog = DialogFactory.createAlertDialog(applicationContext.getString(R.string.checkin_having_issues),
                                applicationContext.getString(R.string.call_us),
                                { _, _ ->
                                    callStore(context = applicationContext, phoneNumber = storePhoneNumber)
                                },
                                "",
                                this@LMDetailsActivity, SCREEN_NAME)
                    }
                    //ASDAA-10527
                    Tracker.get().trackPageView(PageViewEvent(Anivia.PV_OVERLAY, Anivia.SECTION_OVERLAYS)
                            .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata)
                            .putString(Anivia.NAME_KEY, Anivia.SECTION_CHECK_IN)
                            .putString(Anivia.OVERLAY_COPY_KEY, applicationContext.getString(R.string.call_us)))
                    callHelpAtStoreDialog?.show()
                }
            }

        })
        mapView = findViewById(R.id.map_view)
        mapLoadingProgress = findViewById(R.id.map_loading_progress)
        storeName = findViewById(R.id.store_name)
        storeAddress = findViewById(R.id.store_address)
        navigation = findViewById(R.id.navigation)
        navigation.setOnClickListener(object : View.OnClickListener {
            override fun onClick(p0: View?) {
                //ASDAA-10528
                Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                        .putString(Anivia.BUTTON_NAME_KEY, "Directions")
                        .putString(Anivia.NAME_KEY, LOCATION_NAME)
                        .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().lastStoreIdFromSessionMetadata))
                launchGoogleMapsWithRoute(applicationContext, storeLat, storeLong)
            }
        })
        eta = findViewById(R.id.text_eta)
        collectionMethod = findViewById(R.id.collection_method)
        collectionTime = findViewById(R.id.collection_timings)
        onYourWayTextView = findViewById(R.id.on_your_way_msg)
        onYourWayTextView.visibility = View.GONE
        supportActionBar?.setTitle(R.string.labelCollectionDetails)

        if (googleApiClient == null) {
            googleApiClient = GoogleApiClient.Builder(this)
                    .addConnectionCallbacks(this)
                    .addOnConnectionFailedListener(this)
                    .addApi(LocationServices.API)
                    .build()
        }

        mapView.onCreate(bundle)
    }

    private fun manualCheckin() {
        //ASDAA-10527
        Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAPS_EVENT)
                .putString(Anivia.BUTTON_NAME_KEY, applicationContext.getString(R.string.i_am_at_store))
                .putString(Anivia.NAME_KEY, LOCATION_NAME)
                .putString(Anivia.OVERLAY_COPY_KEY, applicationContext.getString(R.string.checkin_having_issues)))
        //perform manual checkin
        customerId?.let {
            val custId = it
            orderId?.let {
                getOrder(it)?.let {
                    checkinManager?.checkin(order = it, customerId = custId, notification = createCheckinNotification(applicationContext),
                            isManualCheckin = true)
                            ?.subscribe(
                                    object : Observer<CheckinEligibleOrderStatus> {
                                        override fun onComplete() {
                                            Log.i(TAG, "manual checkin completed")
                                            manualCheckinDisposable?.dispose()
                                        }

                                        override fun onSubscribe(d: Disposable) {
                                            manualCheckinDisposable = d
                                        }

                                        override fun onNext(t: CheckinEligibleOrderStatus) {
                                            updateLine1and2()
                                            makeCarSpotAndWalkingOptionVisible(true)
                                        }

                                        override fun onError(e: Throwable) {
                                            manualCheckinDisposable?.dispose()
                                        }

                                    }
                            )
                }
            } ?: run {
                if (Utils.isDebug()) {
                    Log.i(TAG, "Could not perform manual checkin as orderId is null")
                }
            }
        } ?: run {
            if (Utils.isDebug()) {
                Log.i(TAG, "Could not perform manual checkin as customerId is null")
            }
        }
    }

    private fun updateLine1and2() {
        orderStatusTextView.text = getString(R.string.youAreHere)
        headingText.text = getString(R.string.checkin_order_title_at_store)
        val paintFlags = Paint.STRIKE_THRU_TEXT_FLAG
        line1Text.setPaintFlags(paintFlags)
        line2Text.setPaintFlags(paintFlags)
        onYourWayTextView.visibility = View.VISIBLE
    }

    private fun makeCarSpotAndWalkingOptionVisible(visible: Boolean) {
        val visibility = when(visible) {
            true -> View.VISIBLE
            false -> View.GONE
        }

        car_color_selector_container.visibility = visibility
        parking_spot_selector.visibility = visibility
        bottom_divider_parking_spot.visibility = visibility
        walkingSelector.visibility = visibility
    }


    override fun onStart() {
        super.onStart()
        googleApiClient?.connect();
        mapView.onStart()

        orderId?.let {
            getOrder(it)?.let {
                //ASDAA-10523
                Tracker.get().trackPageView(PageViewEvent(Anivia.PV_COLLECTION_DETAILS, Anivia.SECTION_CHECK_IN)
                        .putString(Anivia.STOREID_KEY, AsdaServiceFactory.get().getLastStoreIdFromSessionMetadata()))
                storePhoneNumber = it.accessPointInfos?.get(0)?.contactInformation?.contactPhoneNo
                var disposable: Disposable? = checkinManager?.getOrderState(SharedPreferencesUtil.USGR_CUSTOMER_ID, it)
                        ?.startWith(OrderStatus(isLoading = true))
                        ?.doOnNext { response ->
                            when (response) {
                                is OrderStatus -> {
                                    val orderStatus = OrderStatus(response.orderId,
                                          response.total,
                                          response.orderStatus,
                                          response.isLoading,
                                          response.error,
                                          it.accessPointInfos?.get(0),
                                          response.store,
                                          response.etaHumanReadble,
                                          response.etaMillis)
                                    updateUi(orderStatus, it)
                                }
                            }
                        }
                        ?.subscribe({ _ ->
                            updateLocationInfo()
                        }, { error ->
                            if (Utils.isDebug()) {
                                Log.e(TAG, "error with: " + error)
                            }
                        }, {
                            if (Utils.isDebug()) {
                                Log.d(TAG, "completed!!")
                            }
                        })
                disposable?.let {
                    compositeDisposable.add(disposable)
                }
            } ?: run {
                Log.e(TAG, "unable to get the order for the ${orderId}")
            }
        }
    }

    override fun onResume() {
        super.onResume()
        mapView.getMapAsync(this)
        mapView.onResume()
        checkinManager?.hasCustomerArrived()?.let {
            if (it) {
                initDriverFeaturesUI(orderId, it)
                updateLine1and2()
            } else {
                makeCarSpotAndWalkingOptionVisible(false)
            }
        }
    }

    private fun updateLocationInfo() {

    }

    override fun onPause() {
        super.onPause()
        mapView.onPause()
        permissionDialog?.dismiss()
        compositeDisposable.clear()
    }

    private fun updateUi(response: OrderStatus?, order: ViewOrderResponse.Order?) {
        runOnUiThread {
            Log.i(TAG, "ORDER STATE:: ************ ${response.toString()} ***************")
            response?.let {
                if (it.isLoading) {
                    setProgressVisibility(true)
                    container.visibility = View.GONE
                } else {
                    setProgressVisibility(false)
                    container.visibility = View.VISIBLE
                    if (it.error) {
                        orderStatusTextView.text = getString(R.string.error_status)
                        orderStatusTextView.setBackgroundColor(ContextCompat.getColor(applicationContext, R.color.asda_dark_red))
                        orderStatusTextView.setTextColor(ContextCompat.getColor(applicationContext, R.color.white))
                    } else {
                        response.orderId?.toLong()?.let {
                            checkinManager?.getOrderEta(it)
                                    ?.subscribeOn(Schedulers.single())
                                    ?.distinct()
                                    ?.observeOn(AndroidSchedulers.mainThread())
                                    ?.subscribe(object : Observer<String> {
                                        override fun onComplete() {

                                        }

                                        override fun onSubscribe(d: Disposable) {
                                            compositeDisposable.add(d)
                                        }

                                        override fun onError(e: Throwable) {
                                            if (Utils.isDebug()) {
                                                Log.e(TAG, "getETA errored out with ${e.message}")
                                            }
                                        }

                                        override fun onNext(etaStr: String) {
                                            eta.text = etaStr
                                            if (Utils.isDebug()) {
                                                Log.e(TAG, "ETA --> ${etaStr}")
                                            }
                                            customerId?.let {
                                                val customerId = it
                                                orderId?.let {
                                                    logCheckinDetailsPageView(customerId, it, etaStr)
                                                }
                                            }
                                        }
                                    })
                        }

                        line2Text.text = Utils.fromHtml(getLine2(it.accessPointInfo?.locationInstructions))
                        it.orderStatus?.let {

                            val paintFlags = if (it.compareTo(ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE, true) == 0) {
                                line1Text.getPaintFlags() or Paint.STRIKE_THRU_TEXT_FLAG
                            } else {
                                line1Text.getPaintFlags()
                            }
                            line1Text.setPaintFlags(paintFlags)
                            line2Text.setPaintFlags(paintFlags)

                        }
                        mapStatusToProgress(it)
                        updateStoreInfo(it.accessPointInfo)
                        updateMapView(it.accessPointInfo)
                        updateReservationUi(order)
                        makeCarSpotAndWalkingOptionVisible(false)
                    }
                }
            }
        }
    }

    private fun updateReservationUi(order: ViewOrderResponse.Order?) {
        order?.accessPointInfos?.let {
            it[0]?.let {
                //Note: FOR ASDA this is hard coded to Click and Collect in place of it.serviceInfo?.fulfillmentType?.replace("_", " ")
                collectionMethod.text = getString(R.string.book_slot_header_cc_slot_not_booked)
            }
        } ?: run {
            if (Utils.isDebug()) {
                Log.e(TAG, "No serviceInfo available")
            }
        }

        order?.deliveryStartTime?.let {
            collectionTime.text = formatTime(it, order.deliveryEndTime)
        } ?: run {
            if (Utils.isDebug()) {
                Log.e(TAG, "No reservation info available")
            }
        }
    }

    private fun updateDriverFeaturesUI(driverFeatures: DriverFeatures) {
        walkingSelector.visibility = View.VISIBLE
        if (driverFeatures.transportType == DriverFeatures.TRANSPORT_TYPE_WALK) {
            carColorSelectorContainer.visibility = View.GONE
            bottom_divider_parking_spot.visibility = View.GONE
            carColorSelector.visibility = View.GONE
            parkingSpotSelector.visibility = View.GONE
            walkingSelector.isChecked = true
        } else {
            setCarColor(driverFeatures)
            carColorSelectorContainer.visibility = View.VISIBLE
            bottom_divider_parking_spot.visibility = View.VISIBLE
            carColorSelector.visibility = View.VISIBLE
            parkingSpotSelector.visibility = View.VISIBLE
            parkingSpotSelectorDesc.text = driverFeatures.parkBayNumber ?: ""
            walkingSelector.isChecked = false
        }
    }

    var initDisposable: Disposable? = null
    private fun initDriverFeaturesUI(orderId: String?, hasCustomerArrived: Boolean) {
        orderId?.toLong()?.let {
            val order = it
            checkinManager?.getDriverFeatures(order)
                    ?.subscribeOn(Schedulers.single())
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribe(
                            object : Observer<DriverFeatures?> {
                                override fun onComplete() {
                                    initDisposable?.dispose()
                                }

                                override fun onSubscribe(d: Disposable) {
                                    initDisposable = d
                                }

                                override fun onNext(t: DriverFeatures) {
                                    makeCarSpotAndWalkingOptionVisible(hasCustomerArrived)
                                    updateDriverFeaturesUI(t)
                                }

                                override fun onError(e: Throwable) {
                                    if (Utils.isDebug()) {
                                        Log.e(TAG, e.message)
                                    }
                                    initDisposable?.dispose()
                                }

                            }
                    )
        }
    }

    private fun setCarColor(driverFeatures: DriverFeatures?) {
        carColorSelectorContainer.visibility = View.VISIBLE
        bottom_divider_parking_spot.visibility = View.VISIBLE
        if (driverFeatures?.vehicleColor == null || driverFeatures.vehicleColor == DriverFeatures.DEFAULT_CAR_COLOR || driverFeatures.vehicleColor == DriverFeatures.DEFAULT_NO_CAR_COLOR) {
            carColorSelector.colorFilter = null
            carColorSelector.setImageDrawable(ContextCompat.getDrawable(baseContext, R.drawable.ic_white_car))
        } else {
            val carColor = getCarColorByName(driverFeatures.vehicleColor)
            if (carColor != -1) {
                carColorSelector.setImageDrawable(ContextCompat.getDrawable(baseContext, carColor))
            } else {
                carColorSelector.setImageDrawable(ContextCompat.getDrawable(baseContext, R.drawable.ic_white_car))
            }
        }
    }

    @DrawableRes
    private fun getCarColorByName(colorName: String?): Int {
        return when (colorName) {
            getString(R.string.ci_white_color) -> R.drawable.ic_white_car
            getString(R.string.ci_black_color) -> R.drawable.ic_car_black
            getString(R.string.ci_green_color) -> R.drawable.ic_car_green
            getString(R.string.ci_red_color) -> R.drawable.ic_car_red
            getString(R.string.ci_silver_color) -> R.drawable.ic_car_silver
            getString(R.string.ci_grey_color) -> R.drawable.ic_car_gray
            getString(R.string.ci_blue_color) -> R.drawable.ic_car_blue
            else -> -1
        }
    }

    private fun formatTime(begin: String?, end: String?): String {
        if (!TextUtils.isEmpty(begin) && !TextUtils.isEmpty(end)) {
            return String.format("FROM %s - %s", begin, end)
        }
        return ""
    }

    private fun getLine2(s: String?): String {
        s?.let {
            val p = "<li>(.+?)</li>"
            // returns first match of p in s for first group in regular expression
            val m: Matcher = Pattern.compile(p).matcher(s);
            var count = 0
            while (m.find()) {
                count++
                if (count == 2) {
                    return "2. " + m.group(1)
                }
            }
            return getString(R.string.parking_instruction)
        } ?: run {
            return getString(R.string.parking_instruction)
        }

    }

    private fun updateStoreInfo(accessPointInfo: AccessPointInfo?) {
        accessPointInfo?.let {
            storeName.setText("ASDA Store")
            storeAddress.setText(it.serviceAddress?.addressLineOne + "," + it.serviceAddress?.city + ",\n" + it.serviceAddress?.postalCode)
        }
    }

    private fun updateMapView(accessPointInfo: AccessPointInfo?) {
        if (Permissions.isLocationPermissionGranted(this)) {
            lastLocation?.latitude?.let {
                drawMapWithRouteInfo(accessPointInfo, LatLng(it, lastLocation?.longitude!!))
            } ?: run {
                Log.d(TAG, "There is not longitude or latitude values")
                //Show a dialog syaing there is no known last location so it can not show the route to store
                Toast.makeText(this, getString(R.string.checkin_failure_current_location), Toast.LENGTH_SHORT).show()
                mapLoadingProgress.visibility = View.GONE
            }

        } else {
            locationManager.requestLocationWithDialogs(this@LMDetailsActivity, object : StoreLocationManager.LocationCallback {
                override fun onStarted() {

                }

                override fun onFoundLocation(coordinates: LatLng?) {
                    coordinates?.let {
                        drawMapWithRouteInfo(accessPointInfo, coordinates)
                    } ?: run {
                        if (Utils.isDebug()) {
                            Log.e(TAG, "There is not longiture or latitude values in callback onFoundLocation")
                        }
                    }
                }

                override fun onError(error: Int) {
                    if (Utils.isDebug()) {
                        Log.e(TAG, "Error on onError with code ${error}")
                    }
                }

                override fun onError(connectionResult: ConnectionResult?) {
                    if (Utils.isDebug()) {
                        Log.e(TAG, "Error on onError with connectionResult ${connectionResult}")
                    }
                }

            })
        }
    }

    private fun drawMapWithRouteInfo(accessPointInfo: AccessPointInfo?, latLng: LatLng) {
        latLng.let {
            val curLat = it.latitude
            val curLong = it.longitude
            accessPointInfo?.let {
                it.serviceAddress?.geoPoint?.let {
                    val storeCoordinates: GeoPoint = it
                    googleMap?.let {
                        val map = it
                        storeLat = storeCoordinates.latitude?.toDouble()
                        storeLong = storeCoordinates.longitude?.toDouble()
                        storeLat?.let {
                            storeLong?.let {
                                map.moveCamera(CameraUpdateFactory.newLatLng(LatLng(curLat, curLong)))
                                map.addMarker(MarkerOptions().position(latLng));
                                map.addMarker(MarkerOptions().position(LatLng(storeLat as Double, storeLong as Double)))
                                compositeDisposable.add(AsdaServiceFactory.get()
                                        .getMapsRouteInfo("${curLat},${curLong}", "${storeLat},${storeLong}", resources.getString(R.string.checkin_sdk_maps_api_key))
                                        .subscribeWith(object : ResourceObserver<RouteInfo>() {
                                            override fun onNext(routeInfo: RouteInfo) {
                                                val polyLineOptions = PolylineOptions()
                                                routeInfo.routes?.let {
                                                    if (it.size > 0) {
                                                        map.animateCamera(CameraUpdateFactory.zoomTo(13.0f))
                                                        polyLineOptions.addAll(PolyUtil.decode(it.get(0).overviewPolyline?.points))
                                                        map.addPolyline(polyLineOptions.color(Color.BLUE))
                                                    }
                                                } ?: run {
                                                    if (Utils.isDebug()) {
                                                        Log.e(TAG, "No routes available in routeInfo")
                                                    }
                                                }

                                            }

                                            override fun onError(e: Throwable) {
                                                if (Utils.isDebug()) {
                                                    Log.e(TAG, "error in getting route information: ${e.message}")
                                                }
                                            }

                                            override fun onComplete() {

                                            }
                                        })
                                )
                                mapLoadingProgress.visibility = View.GONE
                            } ?: run {
                                Log.e(TAG, "Store Longitude is null")
                            }
                        } ?: run {
                            Log.e(TAG, "Store Latidute is null")
                        }

                    } ?: run {
                        Log.d(TAG, "Map is not initialized yet!!")
                    }
                }
            }
        } ?: run {
            Log.d(TAG, "Not able to get the devices current Location!!")
        }
    }

    fun mapStatusToProgress(orderState: OrderStatus) {
        wismoProgress.setStartDotText(WismoProgressView.PLACED_TEXT)
        wismoProgress.setEndDotText(WismoProgressView.COLLECTED_TEXT)
        wismoProgress.progress = 3

        orderState.let {
            when (it.orderStatus) {
                ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> {
                    wismoProgress.setStartDotText("")
                    wismoProgress.setEndDotText("")
                    wismoProgress.setHintText(ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY)
                    wismoProgress.setHintTextVisibility(true)
                }
                ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> {
                    wismoProgress.setHintText(ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE)
                    wismoProgress.setHintTextVisibility(true)
                    wismoProgress.setStartDotText("")
                    wismoProgress.setEndDotText("")
                }
                ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> {
                    wismoProgress.progress = 4
                    wismoProgress.setHintTextVisibility(false)
                    wismoProgress.setEndDotText(WismoProgressView.COLLECTED_TEXT)
                }
                else -> {
                    wismoProgress.setHintText(getString(R.string.orderReadyForCollection))
                    wismoProgress.setHintTextVisibility(true)
                }
            }
        }
    }

    override fun onStop() {
        super.onStop()
        mapView.onStop()
        googleApiClient?.disconnect();
    }

    override fun onDestroy() {
        super.onDestroy()
        mapView.onDestroy()
    }
}

fun getOrder(orderId: String): ViewOrderResponse.Order? {
    for (order in OrderManager.get().orders) {
        if (orderId.equals(order.orderId)) {
            return order
        }
    }
    return null
}
