package com.asda.lastmile.mock

import android.content.Context
import android.view.LayoutInflater
import android.widget.RadioButton
import android.widget.RadioGroup
import com.asda.android.R
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.Size
import com.facebook.litho.StateValue
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils

@MountSpec
object CheckinEligibilityRadioGroupSpec {

    @OnCreateMountContent
    internal fun onCreateMountContent(c: ComponentContext): RadioGroup? {
        val inflater: LayoutInflater = c.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        val radioGrp: RadioGroup = inflater.inflate(R.layout.checkin_status, null) as RadioGroup

        return radioGrp
    }

    @OnMeasure
    @Suppress("UNUSED_PARAMETER")
    internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size): Unit {
        MeasureUtils.measureWithEqualDimens(widthSpec, heightSpec, size)
    }

    @OnCreateInitialState
    @Suppress("UNUSED_PARAMETER")
    internal fun createInitialState(
            c: ComponentContext,
            checkinState: StateValue<Boolean>,
            @Prop isCheckinEligible:Boolean) {
        checkinState.set(isCheckinEligible)
    }

    @OnUpdateState
    fun updateState(checkinState: StateValue<Boolean>, @Param isChekinParam:Boolean?) {
        isChekinParam?.let {
            checkinState.set(isChekinParam)
        }
    }

    @OnMount
    @Suppress("UNUSED_PARAMETER")
    internal fun onMount(c: ComponentContext, radioGroup: RadioGroup?, @State checkinState: Boolean, @Prop isCheckinEligible: Boolean,
                         @Prop index: Int,
                         @Prop(optional = true) checkinStateChgListener: ((Int, Boolean?) -> Unit)?) : Unit {
        radioGroup?.check(mapIndexToId(checkinState))
        radioGroup?.setOnCheckedChangeListener(object : RadioGroup.OnCheckedChangeListener {
            override fun onCheckedChanged(readioGro: RadioGroup?, id: Int) {
                val radio: RadioButton? = radioGroup.findViewById<RadioButton>(id)
                radio?.let { _ ->
                    if (radio.isPressed) {
                        val check: Boolean? = mapIdToIndex(id)
                        checkinStateChgListener!!(index, check)
                        CheckinEligibilityRadioGroup.updateState(c, check)
                    }
                }

            }

        })
    }

    private fun mapIndexToId(isCheckinEligible: Boolean): Int {
        return if(isCheckinEligible)  R.id.checkingTrue else R.id.checkinFalse
    }

    private fun mapIdToIndex(id: Int): Boolean? {
        when(id) {
            R.id.checkingTrue -> return true
            R.id.checkinFalse -> return false
        }
        return null
    }

}
