@file:Suppress("UNCHECKED_CAST")

package com.asda.lastmile.mock

import android.util.Log
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.facebook.litho.StateValue
import com.facebook.litho.annotations.*
import com.facebook.litho.sections.Children
import com.facebook.litho.sections.SectionContext
import com.facebook.litho.sections.annotations.GroupSectionSpec
import com.facebook.litho.sections.annotations.OnCreateChildren
import com.facebook.litho.sections.common.SingleComponentSection


@GroupSectionSpec
object OrderItemListSectionSpec {

    @OnCreateInitialState
    @Suppress("UNUSED_PARAMETER")
    internal fun createInitialState(
            c: SectionContext,
            orders: StateValue<List<*>>,
            @Prop orderList: List<ViewOrderResponse.Order>) {
        orders.set(orderList)
    }

    @OnUpdateState
    fun updateOrders(orders: StateValue<List<*>>,  @Param status: String?, @Param isCheckinEnabled: Boolean?, @Param index: Int) {
        if (status != null) {
            val orderList = orders.get() as MutableList<ViewOrderResponse.Order>
            orderList[index] = UpdateOrderStatusEvent.deepClone(orderList[index])
            orderList[index].orderStatus = status
            val newList = ArrayList<ViewOrderResponse.Order>()
            newList.addAll(orderList)
            orders.set(newList)
        }

        if (isCheckinEnabled != null) {
            val orderList: List<ViewOrderResponse.Order> = orders.get() as List<ViewOrderResponse.Order>
            orderList[index].isCheckinEligible = isCheckinEnabled
        }
    }

    @OnCreateChildren
    @Suppress("UNUSED_PARAMETER")
    internal fun onCreateChildren(c: SectionContext,
                                  @State orders: List<*>,
                                  @Prop orderList: List<ViewOrderResponse.Order>,
                                  @Prop(optional = true) orderStateChglistener: ((Int, String) -> Unit)?,
                                  @Prop(optional = true) checkinStateChgListener: ((Int, Boolean?) -> Unit)?): Children {
        val builder = Children.create()

        for (i in orders.indices) {
            builder.child(
                    SingleComponentSection.create(c)
                            .key(i.toString())
                            .component(OrderItem.create(c)
                                    .key("key $i")
                                    .order(orders[i] as ViewOrderResponse.Order)
                                    .orderStateChglistener(orderStateChglistener)
                                    .checkinStateChgListener(checkinStateChgListener)
                                    .orderStatusHandler(OrderItemListSection.onOrderUpdated(c))
                                    .index(i)
                                    .build()))

        }
        return builder.build()
    }

    @OnEvent(UpdateOrderStatusEvent::class)
    fun onOrderUpdated(
            c: SectionContext,
            @FromEvent index: Int,
            @FromEvent status: String) {
        Log.d("ChkEligibleOrderStatus", "Order Changed for index ${index} with status ${status}")

        OrderItemListSection.updateOrders(c, status, null, index)

    }


}
