package com.asda.lastmile.mock

import android.graphics.Color
import android.util.Log
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.facebook.litho.*
import com.facebook.litho.annotations.*
import com.facebook.yoga.YogaEdge


/**
 * Created by vn0zpn2 on 3/11/18.
 */
@LayoutSpec(events = [UpdateOrderStatusEvent::class])
object OrderItemSpec {

    @OnCreateLayout
    @Suppress("UNUSED_PARAMETER")
    internal fun onCreateLayout(c: ComponentContext, @Prop index: Int,
                                @Prop(optional = true) orderStateChglistener: ((Int, String) -> Unit)?,
                                @Prop order: ViewOrderResponse.Order,
                                @Prop(optional = true) checkinStateChgListener: ((Int, Boolean?) -> Unit)?,
                                @Prop(optional = true) orderStatusHandler: EventHandler<UpdateOrderStatusEvent>): ComponentLayout {

        return Column.create(c)
                .paddingDip(YogaEdge.ALL, 16f)
                .backgroundColor(Color.WHITE)
                .child(
                        Row.create(c)
                                .child(
                                        OrderStatus.create(c)
                                                .index(index)
                                                .listener(orderStateChglistener)
                                                .orderStatus(order.orderStatus ?: "something wrong")
                                                .orderStatusHandler(OrderItem.onOrderUpdated(c))
                                                .title(order.orderId ?: "empty order id")
                                )
                                .child(
                                        CheckinEligibilityStatus.create(c)
                                                .index(index)
                                                .checkinStateChgListener(checkinStateChgListener)
                                                .isCheckinElegible(order.isCheckinEligible)
                                )

                )
                .build()
    }

    @OnEvent(UpdateOrderStatusEvent::class)
    @Suppress("UNUSED_PARAMETER")
    fun onOrderUpdated(
            c: ComponentContext,
            @FromEvent index: Int,
            @FromEvent status: String,
            @Prop(optional = true) listener: ((Int, String) -> Unit)?,
            @Prop(optional = true) orderStatusHandler: EventHandler<UpdateOrderStatusEvent>) {
        Log.d("CheckinEligibleOrderStatus", "Order Changed for index $index with status $status")
        val event = UpdateOrderStatusEvent()
        event.index = index
        event.status = status
        orderStatusHandler.dispatchEvent(event)
    }


}
