package com.asda.lastmile.mock

import android.content.Context
import android.util.Log
import android.view.LayoutInflater
import android.widget.RadioButton
import android.widget.RadioGroup
import com.asda.android.R
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.EventHandler
import com.facebook.litho.Size
import com.facebook.litho.annotations.*
import com.facebook.litho.utils.MeasureUtils

val TAG: String = "OrderRadioGroup"

@MountSpec(events = [UpdateOrderStatusEvent::class] )
object OrderRadioGroupSpec {

    @OnCreateMountContent
    internal fun onCreateMountContent(c: ComponentContext): RadioGroup? {
        val inflater: LayoutInflater = c.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        val radioGrp: RadioGroup = inflater.inflate(R.layout.order_status, null) as RadioGroup

        return radioGrp
    }

    @OnMeasure
    @Suppress("UNUSED_PARAMETER")
    internal fun onMeasure(c: ComponentContext, layout: ComponentLayout, widthSpec: Int, heightSpec: Int, size: Size): Unit {
        MeasureUtils.measureWithEqualDimens(widthSpec, heightSpec, size)
    }

    @OnMount
    @Suppress("UNUSED_PARAMETER")
    internal fun onMount(c: ComponentContext, radioGroup: RadioGroup?, @Prop index: Int, @Prop status:String,
                         @Prop(optional = true) orderStatusHandler: EventHandler<UpdateOrderStatusEvent>) {
        radioGroup?.check(mapOrderStatusToId(status))
        radioGroup?.setOnCheckedChangeListener(object: RadioGroup.OnCheckedChangeListener {
            override fun onCheckedChanged(radioGroup: RadioGroup?, id: Int) {
                val btn: RadioButton? = radioGroup?.findViewById(id)
                if (btn?.isPressed()!!) {
                    Log.d(TAG, "clicked on ${index}")
                    OrderRadioGroup.dispatchUpdateOrderStatusEvent(orderStatusHandler, index, mapIdToOrderStatus(id))
                }
            }

        })
    }

    private fun mapOrderStatusToId(status: String): Int {
        when(status) {
            ViewOrderResponse.Order.ORDER_STATUS_PLACED -> return R.id.orderPlaced
            ViewOrderResponse.Order.ORDER_STATUS_CANCELLED -> return R.id.orderCancelled
            ViewOrderResponse.Order.ORDER_STATUS_DELIVERED -> return R.id.orderDelivered
            ViewOrderResponse.Order.ORDER_STATUS_PROCESSING -> return R.id.orderProcessing
            ViewOrderResponse.Order.ORDER_STATUS_UNAUTHORIZED -> return R.id.orderUnAuthorized
            ViewOrderResponse.Order.ORDER_STATUS_AUTHORIZED -> return R.id.orderAuthorized
            ViewOrderResponse.Order.ORDER_STATUS_PICK_COMPLETE -> return R.id.orderPickComplete
            ViewOrderResponse.Order.ORDER_STATUS_LOADED_TO_VAN -> return R.id.orderLoadedToVan
            ViewOrderResponse.Order.ORDER_STATUS_READY_COLLECTION -> return R.id.orderReadyForCollection

            ViewOrderResponse.Order.ORDER_STATUS_LOC_PERM_DENIED -> return R.id.locPermDenied
            ViewOrderResponse.Order.ORDER_STATUS_PUSH_PERM_DENIED -> return R.id.pushPermPermDenied
            ViewOrderResponse.Order.ORDER_STATUS_ORDER_READY -> return R.id.checkin
            ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY -> return R.id.onYourWay
            ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE -> return R.id.youAreHere
            ViewOrderResponse.Order.ORDER_STATUS_COLLECTED -> return R.id.orderCollected
        }
        return -1
    }

    private fun mapIdToOrderStatus(id: Int): String {
        when(id) {
            R.id.orderPlaced -> return ViewOrderResponse.Order.ORDER_STATUS_PLACED
            R.id.orderCancelled -> return ViewOrderResponse.Order.ORDER_STATUS_CANCELLED
            R.id.orderDelivered -> return ViewOrderResponse.Order.ORDER_STATUS_DELIVERED
            R.id.orderProcessing -> return ViewOrderResponse.Order.ORDER_STATUS_PROCESSING
            R.id.orderUnAuthorized -> return ViewOrderResponse.Order.ORDER_STATUS_UNAUTHORIZED
            R.id.orderAuthorized -> return ViewOrderResponse.Order.ORDER_STATUS_AUTHORIZED
            R.id.orderPickComplete -> return ViewOrderResponse.Order.ORDER_STATUS_PICK_COMPLETE
            R.id.orderLoadedToVan -> return ViewOrderResponse.Order.ORDER_STATUS_LOADED_TO_VAN
            R.id.orderReadyForCollection -> return ViewOrderResponse.Order.ORDER_STATUS_READY_COLLECTION

            R.id.locPermDenied -> return ViewOrderResponse.Order.ORDER_STATUS_LOC_PERM_DENIED
            R.id.pushPermPermDenied -> return ViewOrderResponse.Order.ORDER_STATUS_PUSH_PERM_DENIED
            R.id.checkin -> return ViewOrderResponse.Order.ORDER_STATUS_ORDER_READY
            R.id.onYourWay -> return ViewOrderResponse.Order.ORDER_STATUS_ON_YOUR_WAY
            R.id.youAreHere -> return ViewOrderResponse.Order.ORDER_STATUS_YOU_ARE_HERE
            R.id.orderCollected -> return ViewOrderResponse.Order.ORDER_STATUS_COLLECTED
        }
        return "Unknown"
    }

}
