package com.asda.lastmile.mock

import android.util.Log
import com.facebook.litho.Column
import com.facebook.litho.ComponentContext
import com.facebook.litho.ComponentLayout
import com.facebook.litho.EventHandler
import com.facebook.litho.annotations.*
import com.facebook.litho.widget.Text
import com.facebook.yoga.YogaEdge

/**
 * Created by vn0zpn2 on 3/13/18.
 */

@LayoutSpec(events = [UpdateOrderStatusEvent::class])
object OrderStatusSpec {

    @OnCreateLayout
    @Suppress("UNUSED_PARAMETER")
    internal fun onCreateLayout(c: ComponentContext, @Prop orderStatus: String, @Prop index: Int,
                                @Prop title: String,
                                @Prop(optional = true) listener: ((Int, String) -> Unit)?,
                                @Prop(optional = true) orderStatusHandler: EventHandler<UpdateOrderStatusEvent>): ComponentLayout {

        return Column.create(c)
                .paddingDip(YogaEdge.ALL, 16f)
                .child(
                        Text.create(c)
                                .text(title)
                                .textSizeSp(20f))
                .child(
                        OrderRadioGroup.create(c)
                                .index(index)
                                .status(orderStatus)
                                .heightDip(580f)
                                .orderStatusHandler(OrderStatus.onOrderUpdated(c)))
                .build()
    }

    @OnEvent(UpdateOrderStatusEvent::class)
    @Suppress("UNUSED_PARAMETER")
    fun onOrderUpdated(
            c: ComponentContext,
            @FromEvent index: Int,
            @FromEvent status: String,
            @Prop(optional = true) listener: ((Int, String) -> Unit)?,
            @Prop(optional = true) orderStatusHandler: EventHandler<UpdateOrderStatusEvent>) {
        Log.d("CheckinEligibleOrderStatus", "Order Changed for index $index with status $status")

        listener?.invoke(index, status)
        OrderStatus.dispatchUpdateOrderStatusEvent(orderStatusHandler, index, status)
    }


}
