package com.asda.lastmile.parkingspot.view

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R

class ParkingSpotAdapter(private var context: Context?) : RecyclerView.Adapter<ParkingSpotAdapter.ParkingSpotViewHolder>() {

    constructor(context: Context?, parkingSpots: Array<String>?) : this(context) {
        items = parkingSpots
    }

    constructor(context: Context?, initBaseParkingSpots: Boolean) : this(context) {
        items = if (initBaseParkingSpots) {
            getBaseParkingSpots()
        } else {
            emptyArray()
        }
    }

    private var items: Array<String>? = null
    private val layoutInflater: LayoutInflater = LayoutInflater.from(context)
    private var parkingSpotSelected: OnParkingSpotSelected? = null

    fun setOnParkingSpotSelected(onParkingSpotSelected: OnParkingSpotSelected?) {
        parkingSpotSelected = onParkingSpotSelected
    }

    private fun getBaseParkingSpots(): Array<String> {
        return arrayOf("Overflow parking", "1", "2", "3", "4", "5", "6")
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ParkingSpotViewHolder {

        val view = layoutInflater.inflate(R.layout.parking_spot_item, parent, false)
        val viewHolder = ParkingSpotViewHolder(view)

        viewHolder.container?.setOnClickListener({
            val position = viewHolder.adapterPosition
            if (position != RecyclerView.NO_POSITION) {
                val item = items?.get(position)
                parkingSpotSelected?.onItemSelected(item)
            }
        })

        return viewHolder
    }

    override fun getItemCount() = items?.size ?: 0

    override fun onBindViewHolder(holder: ParkingSpotViewHolder, position: Int) {
        val parkingSpot = items?.get(position)
        holder.parkingSpotTitle?.text = parkingSpot
    }

    fun destroy() {
        context = null
        parkingSpotSelected = null
        items = null
    }

    inner class ParkingSpotViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        var container: View? = itemView.findViewById(R.id.parking_spot_item)
        var parkingSpotTitle: TextView? = itemView.findViewById(R.id.parking_spot_desc)
    }

    interface OnParkingSpotSelected {
        fun onItemSelected(parkingSpot: String?)
    }
}
