package com.asda.recipes

import android.content.Intent
import android.os.Bundle
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.app.base.FragmentConfig
import com.asda.android.app.dynamic.DynamicPageFragment
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.actitivies.AppCompatActivity
import com.asda.android.base.core.view.fragments.getASDAFragmentTransaction
import com.asda.android.designlibrary.view.bottomNavigation.AsdaBottomNavigationView
import com.asda.android.restapi.service.RestUtils
import com.google.android.material.bottomnavigation.BottomNavigationView


class RecipeActivity : AppCompatActivity() {


    override fun getFragmentContainer() = R.id.fragment_container

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_recipe)
        setTitle(R.string.recipes)
        initBottomNavBar()
        supportFragmentManager.getASDAFragmentTransaction()
              .add(R.id.fragment_container, DynamicPageFragment.newInstance("/pages/mobileapp/recipes/collections", null, if (RestUtils.isTablet(this)) 3 else 2))
              .addToBackStack("recipe")
              .commit()
    }

    // region Bottom Nav
    private fun initBottomNavBar() {
        val bottomNav = findViewById<AsdaBottomNavigationView>(R.id.bottom_nav_bar)
        if (FeatureSettings.isBottomNavOrderTestEnabled(this)) {
            bottomNav.menu.clear()
            bottomNav.inflateMenu(R.menu.bottom_nav_ab_test)
        }
        bottomNav.selectedItemId = R.id.navigate_shop
        bottomNav.setOnNavigationItemSelectedListener(onNavigationItemSelectedListener())
        bottomNav.setOnNavigationItemReselectedListener {
            //do nothing
        }
    }

    private fun onNavigationItemSelectedListener() = BottomNavigationView.OnNavigationItemSelectedListener { item ->
        Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.BUTTON_TAP_EVENT).putString(Anivia.BUTTON_NAME_KEY, item.title.toString())
              .putString(Anivia.LOCATION_KEY, "tabBar"))
        when (item.itemId) {
            R.id.navigate_home -> {
                launchAsdaActivity(FragmentConfig.TAG_HOME); true
            }
            R.id.navigate_shop -> {
                launchAsdaActivity(FragmentConfig.TAG_SHOP); true
            }
            R.id.navigate_search -> {
                launchAsdaActivity(FragmentConfig.TAG_SEARCH); true
            }
            R.id.navigate_favorites -> {
                launchAsdaActivity(FragmentConfig.TAG_FAVORITES); true
            }
            R.id.navigate_orders -> {
                launchAsdaActivity(FragmentConfig.TAG_ORDERS); true
            }
            R.id.navigate_account -> {
                launchAsdaActivity(FragmentConfig.TAG_ACCOUNT); true
            }
            else -> false
        }
    }

    private fun launchAsdaActivity(tag: String?) {
        val intent = Intent(this, AsdaActivity::class.java)
        intent.putExtra(AsdaActivity.EXTRA_FRAGMENT_TO_OPEN, tag)
        startActivity(intent)
    }
    //endregion
}
